#!/usr/bin/env python
# -*- coding:utf-8 -*-


# ////////////////////////////////////////////////////////////////
#                                                                 
#   LINGUAE                                                       
#   Gestionnaire de dictionnaire multiformat multiplateforme      
#                                                                 
#   sous licence CeCILL : http://www.cecill.info/                 
#                                                                 
#   Auteur   : Billig - 2008-2009                                 
#   Contact  : linguae@stalikez.info                              
#   Site Web : http://linguae.stalikez.info                       
#                                                                 
# ////////////////////////////////////////////////////////////////



from Tkinter import*
from ScrolledText import*
import os.path
import tkFileDialog
import tkSimpleDialog
import tkMessageBox
import tkFont
import time
import datetime
import base64
import gzip
import stat
import shutil
import glob
import webbrowser
import smtplib
import xml.dom.minidom
import re
import urllib

from lib.mytools import *             # bibliothèque de fonctions perso
import lib.myencsel                   # Widget perso : sélection d'encodage
import lib.myprogbr                   # Widget perso : barre de progression
import lib.myscrlst                   # Widget perso : liste à barre de progression
import lib.myimg                      # Ensemble de Gifs pour l'interface
import lib.adjutor                    # Gestionnaire d'aide        

import lib.infobulle                  # classe d'infobulle (code récupéré)



class AppData():
    u"""
    Classe de gestion du répertoire des données d'application (emplacement variable suivant la plateforme)
    et de son contenu
    
    On l'initialise avec le nom interne de l'application servant de libellé à ce répertoire
    
    On peut ensuite invoquer ses attributs .path et .cache
    
    """
    
    def __init__(self, appname):
        
        self.path = ""      # Chemin complet du répertoire d'application
        self.cache = ""     # Chemin complet du répertoire de cache (sous-rép. du rép. d'application)
        
        
        # ---------------------------------------------------------------------
        # --- Déterminer le chemin du répertoire en fonction de la plateforme  
        # ---------------------------------------------------------------------
        
        pf = sys.platform.lower()
        
        # --- Windows
        if pf[:3] == 'win':
            
            repAppData = os.path.join(os.path.expanduser('~'), "Application Data", appname)
        
        # --- Unix (MacosX, Linux, )
        elif pf[:6] == 'darwin' or pf[:5] == 'linux':
            
            repAppData = os.path.join(os.path.expanduser('~'), "." + appname)
            
        # --- (autres plateformes)
        else:
            # Pour les autres plateformes, on utilise le répertoire de l'application,
            # en attente de documentation sur les chemins précis pour ces plateformes
            # (autres valeurs possibles : 'mac', 'sunos5', 'PalmOS3', 'hp-ux11')
            repAppData = os.path.dirname(os.path.realpath(sys.argv[0]))
        
        
        if not os.path.isdir(repAppData): os.mkdir(repAppData)  # Si ce répertoire n'existe pas > le créer
        
        
        # ------------------------------
        # --- Répertoire du cache       
        # ------------------------------
        
        repCache = os.path.join(repAppData, "cache")
        
        if not os.path.isdir(repCache): os.mkdir(repCache)  # Si ce répertoire n'existe pas > le créer
        
        
        # ---
        
        self.path = repAppData      # Chemin complet du répertoire d'application
        self.cache = repCache       # Chemin complet du répertoire de cache
        
        return
        


class Inifile():
    u"""
    Classe de gestion d'un fichier de configuration *.ini simplifié
    
    *.ini simplifié : de type clef=valeur SANS titre de sections,
    chaque clef doit donc être unique dans le fichier
    
    """

    
    def __init__(self, path):
        u"""
        Initialise l'objet d'accès au fichier ini
        Le fichier est créé s'il n'existe pas
        Nom et extension du fichier sont indifférents (extension '.ini' non obligatoire)
        """
        
        self.path = ""              # Chemin complet de l'ini
        
        if os.path.isfile(path):
            self.path = path
        else:
            ff = open(path,'w')     # Créer le fichier si inexistant...
            self.path = path
            ff.close()
            
        return
    
    
    
    def get(self, k, default):
        u"""
        Retourne la valeur de la clef 'k' du fichier ini
        Retourne la valeur 'default' si la clé est inexistante
        """
        
        try:
            fichini = open(self.path,'rU')      # Essayer d'ouvrir l'ini en lecture
        except:
            return default                      # Retourner la valeur par défaut si échec
        
        
        # --- Ouverture ok en lecture 
        
        k = k + "="
        lignes = fichini.readlines()            # Copier le fichier dans une liste de lignes
        fichini.close()                         # On peut refermer le fichier
        
        for ligne in lignes:
            if ligne.startswith(k):             # La ligne de la clé est trouvée !
                ligne = ligne.replace('\n',"").replace('\r',"")    # On supprime le caractère de fin de ligne
                ligne = ligne[len(k):]          # On retourne la ligne sans sa clef
                ligne = ligne.decode('utf-8')   # On décode car l'ini est encodé en utf-8
                if ligne == "": ligne = default # Ligne vide > retour de la valeur par défaut
                return ligne
        
        return default                          # Ligne non trouvée > retour de la valeur par défaut



    def write(self, k, v):        
        u"""
        Inscrit la valeur 'v' de la clef 'k' dans le fichier de configuration de type ini simplifié
        """
        
        flagUpdate = False                      # Flag mis à vrai si mise à jour d'une ligne existante
                                                # ... reste faux si la ligne est ajoutée à l'ini
        
        
        # ---------------------------------------------
        # --- Copier l'ini dans une pyListe de lignes  
        # ---------------------------------------------
        
        try:
            fichini = open(self.path,'rU')      # Essayer d'ouvrir le fichier ini en lecture
        except:
            fichini = open(self.path,'w')       # Créer le fichier si inexistant...
            fichini.close()
            fichini = open(self.path,'rU')      # ... puis le réouvrir en lecture
        
        lignes = fichini.readlines()            # Copier le fichier dans une liste de lignes
        fichini.close()                         # On peut refermer le fichier
        
       
        
        # ---------------------------------------------
        # --- Confectionner la nouvelle ligne          
        # ---------------------------------------------
        
        
        # --- Clef
        
        k = k + "="
        
        try:
            k = k.decode('cp1242')
        except:
            try:
                k = k.decode('Latin-1')
            except:
                pass
        
        k = k.encode('utf-8')
        
        # --- Valeur
        
        try:
            v = v.decode('cp1242')
        except:
            try:
                v = v.decode('Latin-1')
            except:
                pass
        
        v = v.encode('utf-8')
        
        # --- Libellé de la ligne à inscrire dans l'ini
        
        newligne = k + v + "\n"                  
        
        
        # -------------------------------------------------
        # --- Vérifier si la clé est déjà présente ou non  
        # -------------------------------------------------
        
        for i in range(len(lignes)):
            if lignes[i].startswith(k):         # La ligne de la clé est trouvée !
                lignes[i] = newligne            # On modifie la ligne avec la nouvelle valeur
                flagUpdate = True               # On marque que la ligne a été mise à jour
                break                           # On peut quitter l'itération
        
        if flagUpdate == False:                 # Aucune ligne n'a été mise à jour...
            lignes.append(newligne)             # ...donc la clef est absente > on l'ajoute
        
        
        # --------------------------------------------------
        # --- Réécrire la liste de lignes dans le fichier   
        # --------------------------------------------------
        
        fichini = open(self.path,'w')
        #fichini.writelines(lignes)
        fichini.write("".join(lignes))
        fichini.close()
        
        return
    



class DicObject():
    
    u"""
    Objet Dictionnaire (tout format)
    """

    def __init__(self, fichPath, lstbox=None, prgbar=None, cp1="", cp2=""):
        
        u"""
        1) Lit le ou les fichiers du dictionnaire :
            
          - charge la liste des entrées dans une ListBox liée '.lstbox', si sa référence est fournie,
            OU sinon dans la pyListe '.innerlst'
          
          - crée la séquence '.datamap' (liste de tuples contenant les décalages et tailles des données)
        
        
        2) Ouvre et laisse ouvert le fichier des données du dico, accessible par l'objet '.file'
        
        
        3) Utilise une barre de progression liée à l'objet : '.progr'
        
        - cp1 et cp2 = encodages langue 1 et langue 2, s'ils sont fournis lors de l'initialisation,
          ils évitent la demande à l'utilisateur des encodages à utiliser ; cp1 et cp2 sont pris
          en compte uniquement pour les formats de dicos dont l'encodage est variable (WB, CSV...)
        
        """
        
        # ======================================================
        # === Initialiser les Attributs de l'objet dictionnaire 
        # ======================================================
        
        # --- Attributs généraux
        
        self.file = None            # (objet fichier) pointeur vers le fichier des données du dictionnaire courant
        self.path = ""              # (string) chemin complet du fichier
        self.type = ""              # (string) type du dictionnaire courant (dict, wb, ...)
        self.subtype = ""           # (string) sous-type du dictionnaire courant (utile surtout pour dict)
        self.innerlst = []          # (pyListe) liste des entrées (complétée uniquement si pas de listbox liée)
        
        # --- Attributs de structure et d'encodage
        
        self.br = ""                # (string) terminateur des enregistrements/lignes du dictionnaire courant (utile pour fichiers texte délimmités)
        self.sep = ""               # (string) séparateur des champs du dictionnaire courant (utile pour fichiers texte délimmités)
        self.subsep = ""            # (string) segmenteur des champs du dictionnaire courant
        self.encoding1 = ""         # (string) encodage langue 1
        self.encoding2 = ""         # (string) encodage langue 2
        
        # --- Images
        
        self.img1 = ""              # (string) codegif/base64 de l'image de la langue 1
        self.img2 = ""              # (string) codegif/base64 de l'image de la langue 2
        
        # ---
        
        self.datamap = []           # (liste) liste d'indexage (offset, taille) du dico courant en mode Dict ou CSV
        self.IDs ={}                # (dictionnaire) liste des IDs pointant sur les index
        self.curID = ""             # (string) ID courant du dictionnaire
        self.curIdx = -1            # (integer) index courant du dictionnaire
        self.previousIdx = 0        # (integer)index précédant du dictionnaire
        
        # --- Objets de l'interface graphique liés au dictionnaire
        
        self.lstbox = None          # (objet Listbox) listbox liée au dictionnaire dans l'interface
        self.progr = None           # (objet ProgressBar) barre de progression liée au dictionnaire dans l'interface
        
        # --- Attributs de cartographie des données du fichier du dictionnaire
        
        self.hdroffset = 0	        # (integer) Offset du bloc de l'en-tête
        self.hdrsize = 0	        # (integer) Taille du bloc de l'en-tête
        self.wordlistoffset = 0     # (integer) Offset du bloc de la liste des entrées
        self.wordlistsize = 0	    # (integer) Taille du bloc de la liste des entrées
        self.idlistoffset = 0	    # (integer) Offset du bloc de la liste des IDs
        self.idlistsize = 0		    # (integer) Taille du bloc de la liste des IDs
        self.datamapoffset = 0	    # (integer) Offset du bloc de cartographie des données
        self.datamapsize = 0	    # (integer) Taille du bloc de cartographie des données
        self.dataoffset = 0		    # (integer) Offset du bloc des données
        self.datasize = 0		    # (integer) Taille du bloc des données
        self.img1offset = 0		    # (integer) Offset du bloc de l'icône1 (0 si absent)
        self.img1size = 0		    # (integer) Taille du bloc de l'icône1 (0 si absent)
        self.img2offset = 0		    # (integer) Offset du bloc de l'icône2 (0 si absent)
        self.img2size = 0		    # (integer) Taille du bloc de l'icône2 (0 si absent)
        
        # --- Attributs du header du format ling
        
        self.minCompatVersion = ""	# (string) Compatibilité minimale du fichier avec les versions du format (sans limite si vide)
        self.maxCompatVersion = ""	# (string) Compatibilité maximale du fichier avec les versions du format (sans limite si vide)
        
        self.dicName = ""		    # (string) Nom convivial du dictionnaire
        self.langName1 = ""		    # (string) Nom convivial de la langue 1 (langue source)
        self.langName2 = ""		    # (string) Nom convivial de la langue 2 (langue cible)
        self.langIso1 = ""		    # (string) Code ISO 639-2 de la langue 1 (langue source)
        self.langIso2 = ""		    # (string) Code ISO 639-2 de la langue 2 (langue cible)
        self.langNameUser=""		# (string) Nom convivial de la langue de l'utilisateur supposé du dictionnaire
        self.langIsoUser=""		    # (string) Code ISO 639-2 de la langue de l'utilisateur supposé du dictionnaire
        self.langFamily1="..."		# (string) Famille linguistique de la langue 1
        self.langFamily2="..."		# (string) Famille linguistique de la langue 2
        self.isReverseDic = False	# (booléen) ce dictionnnaire est un dictionnaire inverse
        self.doReverseDic = False	# (booléen) ce dictionnnaire pilote un dictionnaire inverse
        self.reverseDicFileName=""	# (String) Nom du fichier du dictionnaire inverse (ou du dictionnaire maître)
        self.reverseDicName=""	    # (String) Nom convivial du fichier du dictionnaire inverse (ou du dictionnaire maître)
        self.sortEquPatterns = []   # (tupple de strings) Motifs d'équivalence pour le tri alphabétique du dictionnaire
        self.wordcount = 0		    # (integer) Nombre d'entrées du dictionnaire
        
        self.mainAuthors = []	    # (tupple de strings) Auteurs principaux du dictionnaire
        self.altAuthors = []	    # (tupple de strings) Auteurs accessoires
        self.contactAuthor = ""	    # (string) Contacter les auteurs
        self.dicStatus = ""		    # (string) Statut du dictionnaire (domaine public, freeware, etc.)
        self.showDicStatus = False  # (booléen) Afficher le statut du dictionnaire 
        self.copyright = ""		    # (string) Mention de copyright ou équivalent
        
        self.creationDate = ""	    # (string) Date de création du dictionnaire
        self.versionDate = ""	    # (string) Date de la présente version du dictionnaire
        self.localEditDate = ""	    # (string) Date d'édition locale du fichier (par l'utilisateur)
        
        self.dicID = ""     	    # (string) Identifiant du dictionnaire (nom interne unique dans un ensemble de dictionnaires)
        self.dicVersionNumber = ""	# (string) N° de version du dictionnaire
        self.dicUrl = ""    	    # (string) URL du dictionnaire sur le Web
        self.verUrl = ""			# (string) URL d'un éventuel fichier annexe au dictionnaire (infos de version, etc.)
        self.dicInfo = ""		    # (string) Infos sur le dictionnaire (saut de ligne : '\n')
        self.showDicInfo = False    # (booléen) Afficher les infos du dictionnaire 
        
        self.protected1 = ""        # (string) Type de protection des entrées d'index (langue 1) (vide = pas de protection)
        self.protected2 = ""		# (string) Type de protection des données (langue 2) (vide = pas de protection)
        
        self.displayFontName1 = ""	# (string) Police d'affichage de la langue 1
        self.displayFontName2 = ""	# (string) Police d'affichage de la langue 2
        self.grammarEncoding1 = ""	# (string) Norme de l'encodage grammatical de la langue 1
        
        self.compatPlugins=[]	    # (tupple de strings) Liste de plugins compatibles avec le dictionnaire
        self.noCompatPlugins=[]	    # (tupple de strings) Liste de plugins incompatibles avec le dictionnaire
        self.usePlugins = []	    # (tupple de strings) Liste des noms des plugins à associer au dictionnaire
        
        self.wordGroups=[]		    # (tupple de strings) Noms/IDs des groupes de mots du dictionnaire
        
        self.biblio = []            # (tupple de strings) Liste de références biblio
        self.showBiblio = False     # (booléen) Afficher la biblio du dictionnaire 
        
        self.extFieldCount = 0	    # (integer) Nombre de champs d'extension de la section data de ce dictionnaire
        self.extFieldList =	[]	    # (pyListe) Liste des Noms conviviaux des champs supplémentaires
        
        
        
        # =========================================
        # === Lier la ListBox au dictionnaire      
        # =========================================
        
        # NB : s'il n'y a pas de Listbox fournie (lstbox is None),
        # la pyListe self.innerlst est utilisée en mémoire à sa place
        
        if not lstbox is None:
            self.lstbox = lstbox
            self.lstbox.delete(0, END)              # Vider la listbox liée
            self.lstbox.update()
        
        
        # =========================================
        # === Lier la ProgressBar au dictionnaire  
        # =========================================
        
        try:
            self.progr = prgbar
        except:
            tkMessageBox.showwarning(parent=app,
                title=u"Charger un dictionnaire",
                message=fichPath + u"\n\nImpossible de lier la barre de progression : '"+ prgbar +u"'\n\nEchec du chargement du dictionnaire.")
            return
        
        
        # ===============================================================================
        # === Traitement du chemin fourni, en fonction de la source et de la plateforme  
        # ===============================================================================
        
        #~ try:
          #~ fichPath = fichPath.decode('utf-8')    # (nb : le chemin fourni par l'ini est en utf-8)
        #~ except:
          #~ try:
            #~ fichPath = fichPath.decode('cp1252')
          #~ except:
            #~ try:
              #~ fichPath = fichPath.decode('latin1')
            #~ except:
              #~ pass
        
        #~ if sys.platform.lower()[:3] == 'win':
          #~ fichPath = fichPath.encode('latin-1')     # pour Windows : encoder le chemin en ISO 7-bits
        
        
        fichPath = os.path.normpath(fichPath)       # Normalise le chemin en fonction de la plateforme
        
        if not os.path.isfile(fichPath):            # Vérifie que le chemin existe, sinon quitter
            tkMessageBox.showwarning(parent=app,
                title=u"Charger un dictionnaire",
                message=fichPath + u"\n\nCe fichier est introuvable !")
            self = None
            return
        
        self.path = fichPath                        # Le chemin de fichier est correct
        
        fplow = fichPath.lower()                    # version en minuscule du chemin du fichier
        
        
        # =================================================================
        # === Charger le fichier (aiguillage suivant le format)            
        # =================================================================
        
        if fplow.endswith('.ling'):                     # === Fichier Ling
            
            self.__loadLing(fichPath)
        
        elif fplow.endswith('.ifo') or fplow.endswith('.dict') or fplow.endswith('.dict.dz') or fplow.endswith('.dict.gz'): # === Fichier DICT
            
            self.__loadDict(fichPath)
        
        elif fplow.endswith('.idx') or fplow.endswith('.dict.gz'):   # === Fichier DICT (bis)
            # Nb : les *idx sont normalement masqués par le sélecteur de fichier
            self.__loadDict(fichPath)
        
        elif fplow.endswith('.wb'):                     # === Fichier WB
            
            self.__loadWB(fichPath, cp1=cp1, cp2=cp2)
        
        elif fplow[-4:] in ('.csv', '.tab', '.txt'):    # === Fichier délimité
            
            self.__loadCSV(fichPath, cp=cp1)
        
        elif fplow.endswith('.ini'):                    # === Fichier WB
            
            self.__loadINI(fichPath, cp=cp1)
        
        elif fplow.endswith('.xdxf'):                   # === Fichier XDXF
            
            self.__loadXDXF(fichPath)
        
        else:                                           # Format de fichier non reconnu
            
            tkMessageBox.showwarning(parent=app,
                    title=u"Echec de l'ouverture d'un fichier",
                    message=u"Le fichier '"+ fichPath +"'\nn'a pas un format reconnu par Linguae.")
            self = None
            return
        
        
        return


    def __loadLing(self, fichPath):
        u"""
        Charger un fichier dictionnaire de type Ling
        dans l'objet dictionnaire
        """
        
        self.file = open(fichPath, 'rb')            # Ouvrir le fichier en lecture binaire
        
        # ----------------------------------
        # --- Lire le mappage du fichier    
        # ----------------------------------
        
        filemap = self.file.read(70)                # Copier le bloc de mappage dans une chaîne
        
        
        self.hdroffset = 70
        self.hdrsize = Str32ToNum(filemap[18:22])
        self.wordlistoffset = Str32ToNum(filemap[22:26])
        self.wordlistsize = Str32ToNum(filemap[26:30])
        self.idlistoffset = Str32ToNum(filemap[30:34])
        self.idlistsize = Str32ToNum(filemap[34:38])
        self.datamapoffset = Str32ToNum(filemap[38:42])
        self.datamapsize = Str32ToNum(filemap[42:46])
        self.dataoffset = Str32ToNum(filemap[46:50])
        self.datasize = Str32ToNum(filemap[50:54])
        self.img1offset = Str32ToNum(filemap[54:58])
        self.img1size = Str32ToNum(filemap[58:62])
        self.img2offset = Str32ToNum(filemap[62:66])
        self.img2size = Str32ToNum(filemap[66:70])
        
        
        
        # -----------------------------
        # --- Lire le header           
        # -----------------------------
        
        self.file.seek(70)                                          # Offset du bloc du header
        chps = self.file.read(self.hdrsize).split("\0")             # Lire le bloc et le splitter
        
        for chp in chps:
            exec("self." + chp)
        
        # S'assurer que les propriétés de type tuple
        # sont réellement des tuples et non des chaînes
        # (car ils se chargent en tant que chaînes s'il n'y a qu'un seul élément...)
        
        if type(self.mainAuthors) is type('xxx'):self.mainAuthors=(self.mainAuthors,)
        if type(self.altAuthors) is type('xxx'):self.altAuthors=(self.altAuthors,)
        if type(self.compatPlugins) is type('xxx'):self.compatPlugins=(self.compatPlugins,)
        if type(self.usePlugins) is type('xxx'):self.usePlugins=(self.usePlugins,)
        if type(self.biblio) is type('xxx'):self.biblio=(self.biblio,)
        if type(self.extFieldList) is type('xxx'):self.extFieldList=(self.extFieldList,)
        if type(self.sortEquPatterns) is type('xxx'):self.sortEquPatterns=(self.sortEquPatterns,)
        if type(self.wordGroups) is type('xxx'):self.wordGroups=(self.wordGroups,)
        
        # ---
        prg = 0 ; self.progr.setMax(self.wordcount * 2)             # Paramétrer la ProgressBar
        
        # Accélérateurs locaux
        setprogr = self.progr.setValue
        dmap = self.datamap.append
        
        # -----------------------------
        # --- Lire les entrées         
        # -----------------------------
        
        self.file.seek(self.wordlistoffset)                         # Offset du bloc des entrées
        sz = self.wordlistsize                                      # Taille du bloc des entrées
        
        if sz >0:
            
            chps = self.file.read(sz)                               # Lire le bloc des entrées
            if self.protected1=="Linguae": chps=atrifer(chps)
            chps = chps.split("\0")                                 # Splitter le bloc des entrées
            
            # boucle de lecture
            for chp in chps:
                self.lstAdd(chp)                                    # Charger la liste des entrées
                prg += 1 ; setprogr(prg)                            # Afficher la progression
            
            if not self.lstbox is None:                             # Compter les mots
                self.wordcount = self.lstbox.size()
            else:
                self.wordcount = len(self.innerlst)
        
        
        # ----------------------------------------
        # --- Lire les IDs                        
        # ----------------------------------------
        # pyDictionnaire ID -> index              
        
        self.file.seek(self.idlistoffset)                           # Offset du bloc des IDs
        sz = self.idlistsize                                        # Taille du bloc (nulle si pas d'ID)
        
        if sz > 0:                                                  # Lire le bloc des IDs
            bloc = self.file.read(sz)
            chp = bloc.split("\0")
            
            i=0
            while i < len(chp):
                # Nb : id dans chp[i] et idx dans chp[i+1]
                
                # Contrôle des doublons d'ID
                if self.IDs.has_key(chp[i]):
                    r = tkMessageBox.askyesno(parent=app,
                        title="Erreur dans le dictionnaire",
                        message ="L'identifiant '" + chp[i] + "' est doublonné.\n\nUn identifiant doit être unique dans un dictionnaire !\nVeuillez corriger les données (menu Edition) ou signaler cette erreur à l'auteur du dictionnaire.\n\nContinuer le chargement du dictionnaire ?")
                    if not r:
                        self = None
                        return
                    
                    
                self.IDs[chp[i]] = int(chp[i+1])
                i += 2
        
        
        # --------------------------------------------------
        # --- Lire le mappage des données                   
        # --------------------------------------------------
        
        self.file.seek(self.datamapoffset)              # Offset du bloc de mappage des données
        sz = self.datamapsize                           # Taille du bloc de mappage des données
        
        if sz >0:
            bloc = self.file.read(sz)                   # Lire le bloc de mappage dans une chaîne
            
            # --- Construire .datamap en mémoire
            
            delta = self.dataoffset                     # Offset du bloc des données
            
            for i in range(self.wordcount):
                p = i * 8                               # Position des données correspondant à l'entrée
                decal = delta + Str32ToNum(bloc[p:p+4]) # Lire l'offset (relatif + delta)
                sz = Str32ToNum(bloc[p+4:p+8])          # Lire la taille
                dmap((decal,sz))
                
                prg += 1 ; setprogr(prg)                # Afficher la progression
        
        
        # ------------------------------------------
        # --- Lire les images                       
        # ------------------------------------------
        
        if self.img1offset != 0:
            self.file.seek(self.img1offset)
            self.img1 = self.file.read(self.img1size)   # Lire le bloc (base64) dans une chaîne
        
        if self.img2offset != 0:
            self.file.seek(self.img2offset)
            self.img2 = self.file.read(self.img2size)   # Lire le bloc (base64) dans une chaîne
        
        
        # ---
        
        self.type = "ling"
        self.encoding1 = "utf-8"
        self.encoding2 = "utf-8"
        self.subsep = ";"
        
        return


    def __loadXDXF(self, fichPath):
        
        u"""
        Charger un dictionnaire de type XDXF
        """
        

        self.file = open(fichPath, 'rb')            # Ouvrir le fichier en lecture binaire
        
        strf = self.file.read()
        
        
        # ----------------------------------------------
        # --- Parcours de la chaîne xml du fichier      
        # ----------------------------------------------
        
        p1ar = 0    # position de '<ar>'
        p2ar = 0    # position de '</ar>'
        p1k = 0     # position de '<k>'
        p2k = 0     # position de '</k>'
        
        wc = 0
        
        self.progr.setMax(40000)           # Paramétrer la ProgressBar à la louche car nombre exact inconnu
        
        cleanTag = re.compile(r"<[^>]*?>", re.MULTILINE)
        
        # accélérateurs locaux
        sfind = strf.find
        dmap = self.datamap.append
        setprogr = self.progr.setValue
        
        while 1:
            
            # Bornage
            p1ar = sfind("<ar>", p2ar)
            p1k = sfind("<k>", p1ar)
            p2k = sfind("</k>", p1k)
            p2ar = sfind("</ar>", p1ar)
            
            if p1ar < 0: break
            
            chp1 = strf[p1k+3:p2k]
            chp1 = re.sub(cleanTag, "", chp1)
            
            self.lstAdd(chp1)                           # Charger la liste des entrées
            
            dmap((p1ar, (p2ar+5)-p1ar))  # Conserver décalages et tailles dans .datamap (une liste de tuples)
            
            wc += 1
            
            setprogr(wc)
        
        # ---
        
        self.wordcount = wc
        self.type = "xdxf"
        self.encoding1 = 'utf-8'
        self.encoding2 = 'utf-8'
        
        return


    def __loadDict(self, fichPath):
        u"""
        Charger un dictionnaire de type Dict
        """
        
        fplow = fichPath.lower()                            # version en minuscule du chemin du fichier
        
        
        # --- Oter l'extension > radical du chemin
        # (attention : le fichier fourni peut-être non compressé alors que le dict peut être compressé 
        # ou inversement, donc ne pas se baser ensuite sur ce flag temporaire)
        flagGz = False
        if fplow.endswith('.gz'): flagGz = True              # Le fichier est compressé (oui/non)
        if fplow.endswith('.dz'): flagGz = True              # Le fichier est compressé (oui/non)
        
        if flagGz:
            strPathRadic = fichPath[:fichPath.rfind('.', 0, -3)]
        else:
            strPathRadic = fichPath[:fichPath.rfind('.')]
        
        
        # ----------------------------------
        # --- Ouvrir le fichier *.ifo       
        # ----------------------------------
        
        # --- Ouvrir l'ifo en lecture texte
        #   (NB : l'ifo n'est jamais compressé)
        
        try:
            fichifo = open(strPathRadic+'.ifo', 'rU')
        except:
            tkMessageBox.showwarning(parent=app,
                    title='Charger un dictionnaire Dict',
                    message=u"Impossible de charger le fichier *.ifo")
            self = None
            return    
        
        lignes = fichifo.readlines()                   # Charger dans une liste de lignes
        fichifo.close()
        
        # --- Lecture des champs de propriétés
        
        self.subtype = 'mixte' # ( type par défaut = pas de champ 'sametypesequence' )
        
        for ligne in lignes:
            
            if ligne.startswith("sametypesequence"):
                ligne = ligne.replace("\r","").replace("\n","")
                self.subtype = ligne[ligne.find("=")+1:].strip()
            
            elif ligne.startswith("bookname"):
                ligne = ligne.replace("\r","").replace("\n","")
                self.dicName = ligne[ligne.find("=")+1:].strip()
            
            elif ligne.startswith("author"):
                ligne = ligne.replace("\r","").replace("\n","")
                self.mainAuthors=(ligne[ligne.find("=")+1:].strip(),)
            
            elif ligne.startswith("email"):
                ligne = ligne.replace("\r","").replace("\n","")
                self.contactAuthor=ligne[ligne.find("=")+1:].strip()
            
            elif ligne.startswith("description"):
                ligne = ligne.replace("\r","").replace("\n","")
                self.dicInfo=ligne[ligne.find("=")+1:].strip()
                self.showDicInfo = True
            
            elif ligne.startswith("website"):
                ligne = ligne.replace("\r","").replace("\n","")
                self.dicUrl=ligne[ligne.find("=")+1:].strip()
            
            elif ligne.startswith("date"):
                ligne = ligne.replace("\r","").replace("\n","")
                self.versionDate=ligne[ligne.find("=")+1:].strip()
        
        
        # --- Quitter si le sous-type n'est pas encore pris en charge
        # m : texte Utf-8
        # l : texte 8-bits
        # g : texte balisé Pango
        # x : données XDXF
        # h : données HTML
        
        if not self.subtype in ('m', 'l', 'g', 'x', 'h'):
            tkMessageBox.showinfo(parent=app,
                    title=u"Charger un dictionnaire",
                    message=fichPath + u"\n\nCe dictionnaire est de sous-type '" + self.subtype + u"'. Ce sous-type du format Dict n'est pas encore pris en charge par Linguae.")
            self = None
            return
        
        # --- Lecture du wordcount
        
        for ligne in lignes:
            if ligne.startswith("wordcount"):
                self.wordcount = int(ligne[10:])
                self.progr.setMax(self.wordcount)   # Paramétrer la ProgressBar
                break
        
        
        # ----------------------------------
        # --- Ouvrir le fichier *.idx       
        # ----------------------------------
        
        # Accélérateurs locaux
        setprogr = self.progr.setValue
        dmap = self.datamap.append
        
        # --- Ouvrir l'index en lecture binaire
        try:
            fichidx = open(strPathRadic+'.idx', 'rb')
        except:
            try:
                fichidx = gzip.GzipFile(strPathRadic+'.idx.gz', 'rb')
            except:
                tkMessageBox.showwarning(parent=app,
                    title='Charger un dictionnaire Dict',
                    message=u"Impossible d'ouvrir le fichier index *.idx")
                self = None
                return
        
        strfile = fichidx.read()                                # Charger l'index dans une chaîne
        fichidx.close()
        
        
        # --- Analyser la chaîne contenant l'index
        
        p1, p2 = 0, 0
        while 1:
            p2 = strfile.find("\0",p1)
            if p2 == -1 : break
            
            self.lstAdd(strfile[p1 : p2])                       # charger la liste des entrées
            
            decal = Str32ToNum(strfile[p2 + 1 : p2 + 5])        # extraire l'offset (4 octets)
            sz = Str32ToNum(strfile[p2 + 5 : p2 + 9])           # extraire la taille (4 octets)
            
            p1 = p2 + 9
            
            dmap((decal, sz))                           # conserver décalages et tailles dans une liste de tuples
            
            setprogr(len(self.datamap))                 # afficher progression
        
        
        
        # -------------------------------------------------------------------------------
        # --- Ouvrir le fichier *.dict correspondant (et le laisser ouvert pour la suite)
        # -------------------------------------------------------------------------------
        
        # Si le dictionnaire est ouvert par appel à un dict compressé, on impose l'ouverture de celui-ci
        if fplow.endswith('dict.dz'):
            self.type = "dictdz"
        elif fplow.endswith('dict.gz'):
            self.type = "dictgz"
        else:
        
        # Sinon on essaie d'abord d'ouvrir un *.dict non compressé (s'il est présent)
            try:
                self.file = open(strPathRadic + ".dict", 'rb')          # Pointeur de fichier
                self.type = "dict"
            except:
                
                # On recherche un fichier compressé en cas d'échec de l'ouverture
                if os.path.isfile(strPathRadic + ".dict.dz"):           
                    self.type = "dictdz"
                elif os.path.isfile(strPathRadic + ".dict.gz"):
                    self.type = "dictgz"
                else:
                    tkMessageBox.showwarning(parent=app,
                            title='Charger un dictionnaire DICT',
                            message=u"Erreur lors de la tentative d'ouverture du dictionnaire\n" + strPathRadic +"\n\nImpossible de charger le fichier des données *.dict")
                    self = None
                    return    
                
        
        # Décompresser si besoin le dict.*z dans un fichier dict temporaire
        # qui sera utilisé à la place du dict lui-même
        if self.type in ("dictgz", "dictdz"):
            
            #fichtemp = strPathRadic+".dict.TEMP"    
            # Chemin du fichier dict temporaire dans le cache
            fichtemp = os.path.join(app.appdata.cache, "TEMP.dict")  
            
            # Effacer un fichier précédent (au cas où...)
            try   : os.remove(fichtemp)
            except: pass
            
            if self.type == "dictdz":
                fichgz = gzip.GzipFile(strPathRadic+'.dict.dz', 'rb')
            else:
                fichgz = gzip.GzipFile(strPathRadic+'.dict.gz', 'rb')
            
            try:
                self.file = open(fichtemp, 'wb')
            except:
                tkMessageBox.showwarning(parent=app,
                        title=u'Charger un dictionnaire DICT compressé',
                        message=u"Erreur lors de la tentative d'ouverture du dictionnaire\n" + strPathRadic +"\n\nImpossible d'ouvrir le fichier temporaire.")
                self = None
                return
            
            self.file.write(fichgz.read())          # Enregistrer le contenu décompressé dans le dict temporaire
            self.file.close()
            
            self.file = open(fichtemp, 'rb')        # Réouvrir le dict temporaire en lecture
        
        self.encoding1 = "utf-8"
        self.encoding2 = "utf-8"
        
        
        # ---------------------------------------------------------------
        # --- Ouvrir le fichier *.syn s'il existe                        
        # ---------------------------------------------------------------
        # NB section désactivée car les synonymes Dict ne sont pas utilisables dans l'interface actuelle
        # ces synonymes Dict sont uniquement des synonymes externes (= qui ne sont pas eux-mêmes des entrées du dico)
        # qui pointent sur des entrées du dico
        #
        # --- Ouvrir l'index en lecture binaire
        #flagsyn = True
        #try:
        #    fichsyn = open(strPathRadic+'.syn', 'rb')
        #except:
        #    try:
        #        fichsyn = gzip.GzipFile(strPathRadic+'.syn.gz', 'rb')
        #    except:
        #        flagsyn = False
        #
        #if flagsyn:
        #    strfile = fichsyn.read()                                # Charger le fichier dans une chaîne
        #    fichsyn.close()
        #
        # <<<< Traitement : à réaliser, éventuellement >>>>
        
        return


    def __loadWB(self, fichPath, cp1="", cp2=""):
        u"""
        Charger un dictionnaire de type WB
        si cp1 et cp2 ne sont pas fournis, ils sont demandés dans deux inputBoxes successives
        """
        
        flagErrCp1 = False
        
        # -----------------------------------------------------------------------------
        # ---Confirmer les deux pages de code                                          
        # les *.wb associent deux codages indépendants dans un unique fichier 8-bits   
        # champ1 (codepage:cp1) \0 champ2 (codepage:cp2)                               
        # -----------------------------------------------------------------------------
        
        tcp ="\tcp1252 (Windows - Europe de l'Ouest)\n\tcp1250 (Windows - Europe centrale)\n\tcp1251 (Windows - cyrillique\n\tcp1253 (Windows - grec)\n\tcp1254 (Windows - turc)\n\tetc..."
        
        if cp1 == "":
            cp1 = tkSimpleDialog.askstring(parent=app,
                    title="Ouverture d'un fichier WB (1 de 2)",
                    prompt="Les fichiers WB sont encodés sur 8-bits\n et ne contiennent aucune information sur l'encodage des caractères...\n\nPour un affichage correct, il est donc nécessaire de préciser manuellement\nl'encodage de la langue source ET celui de la langue cible.\n\nLes encodages les plus couramment utilisés pour ce format sont :\n\n" + tcp +"\n\nPrécisez tout d'abord l'encodage de la LANGUE SOURCE ce fichier WB :\n",
                    initialvalue="cp1252")
        if not cp1:
            self = None
            return
        
        if cp2 == "":
            cp2 = tkSimpleDialog.askstring(parent=app,
                    title="Ouverture d'un fichier WB (2 de 2)",
                    prompt=".../...\n\n"+tcp + "\n\nPrécisez ensuite l'encodage de la LANGUE CIBLE dans ce fichier WB :\n",
                    initialvalue="cp1252")
        if not cp2:
            self = None
            return
        
        
        # ----------------------------
        # --- Scanner le fichier WB   
        # ----------------------------
        
        self.file = open(fichPath, 'rb')                # Ouvrir le fichier en lecture binaire
        strfile = self.file.read()                      # Charger le contenu dans une chaîne
        
        self.wordcount = len(strfile)//84               # Déterminer le wordcount
        self.progr.setMax(self.wordcount)               # Paramétrer la ProgressBar
        
        # Accélérateurs locaux
        setprogr = self.progr.setValue
        
        nbmots = 0
        p = 0
        while p < len(strfile)-1:
            nbmots += 1                                 # Compter les mots chargés
            chp1 = strfile[p:p+31]                      # Champ 1 : entrées (31 octets)
            p0 = chp1.find('\0')
            if p0 > -1: chp1 = chp1[:p0]                # Supprimer les \0 en fin de chaîne
            
            try:
                chp1 = chp1.decode(cp1).encode('utf-8') # Décodage et réencodage en utf-8
            except:
                flagErrCp1 = True
            
            self.lstAdd(chp1)                           # Charger la liste des entrées
            
            p+= 84
            
            setprogr(nbmots)
        
        # ---
        
        self.type = "wb"
        self.encoding1 = cp1
        self.encoding2 = cp2
        self.subsep = ","
        
        # NB : on laisse ouvert le fichier pour la suite
        
        # Avertir si erreurs de décodage du champ 1
        if flagErrCp1:
            tkMessageBox.showwarning(parent=app,
                    title=u"Ouverture d'un fichier WB",
                    message=u"Ce fichier WB a pu être ouvert mais l'encodage choisi pour la langue source ('"+cp1+"') est incorrect,\nl'affichage de certains caractères sera probablement erronné.\n\nPour rectifier cela, fermez ce dictionnaire et rouvrez-le en choisissant un autre encodage.")
        
        
        return


    def __loadCSV(self, fichPath, cp=""):
        u"""
        Charger un dictionnaire de type CSV
        """
        
        flagErrCp = False
        
        if cp == "":
            cp = tkSimpleDialog.askstring(parent=app,
                    title="Ouverture d'un fichier délimité",
                    prompt="Les fichiers délimités ne contiennent aucune information\nsur l'encodage des caractères...\n\nPour un affichage correct, il est donc nécessaire de\n préciser manuellement l'encodage originel de ce fichier.\n",
                    initialvalue="utf-8")
        
        if not cp:
            self = None
            return
        
        
        self.file = open(fichPath, 'rU')                    # Ouvrir le fichier en lecture texte
        
        self.progr.setMax( len(self.file.readlines()) )     # Paramétrer la ProgressBar
        
        
        # Charger les deux premières lignes non vides pour tester le séparateur
        lignes = []
        n = 0
        self.file.seek(0)
        while n < 2:
            ligne = self.file.readline()
            if ligne.strip != "":
                lignes.append(ligne)
                n +=1
        
        
        self.file.close()
        
        # Déterminer automatiquement le caractère séparateur
        sp = ""
        if (fichPath.lower()[-4:] == '.tab') and (len(lignes[0].split('\t')) == 2):
            sp='\t'
        else:
            if len(lignes[0].split(';')) == 2 and len(lignes[1].split(';')) == 2:
                sp = ';'
            elif len(lignes[0].split(',')) == 2 and len(lignes[1].split(',')) == 2:
                sp = ','
            elif len(lignes[0].split('\t')) == 2 and len(lignes[1].split('\t')) == 2:
                sp = '\t'
        
        if sp == "":
            tkMessageBox.showwarning(parent=app,
                    title=u"Charger un fichier délimité",
                    message=u"Impossible de déterminer le séparateur des champs de ce fichier.\n\nChargement impossible !")
            self = None
            return
        
        self.file = open(fichPath, 'rb')   # Ouvrir le fichier en lecture binaire
        strfile = self.file.read()         # Charger le contenu dans une chaîne
        
        # Déterminer automatiquement le terminateur
        br = ""
        if "\r\n" in strfile: br = "\r\n"
        elif "\n" in strfile: br = "\n"
        elif "\r" in strfile: br = "\r"
        
        # Homogénéiser les sauts de lignes > remplacer par des zéros binaires (1 ou 2)
        # (modification temporaire en mémoire, le fichier sur disque n'est pas modifié)
        strfile = strfile.replace("\n","\0")
        strfile = strfile.replace("\r","\0")
        strfile += "\0" # Ajouter un zéro binaire en fin de fichier, au cas où il serait absent
        
        
        # accélérateurs locaux de la boucle de scan
        setprogr = self.progr.setValue
        sfind = strfile.find
        dmap = self.datamap.append
        
        nb = 0                                  # nb des lignes
        wc = 0                                  # wordcount
        psp = 0                                 # pointeur des séparateurs
        pnl = -1                                # pointeur des \n
        while psp > -1:
            
            nb += 1
            setprogr(nb)                        # afficher progression
            
            psp = sfind(sp, pnl+1)              # pointeur des séparateurs
            chp1 = strfile[pnl+1 : psp]
            chp1 = chp1.strip('\0')
            chp1 = chp1.strip()                 # Elimine espaces et lignes vides (elles sont accolées en tête de champ)
            chp1 = chp1.strip('"')              # Eliminer éventuels guillemets d'entourage
            
            try   : chp1 = chp1.decode(cp).encode('utf-8')  # Décodage et réencodage en utf-8
            except: flagErrCp = True
            
            pnl = sfind('\0', psp)              # pointeur des \n
            
            if chp1 !="":
                
                self.lstAdd(chp1)               # Charger la liste des entrées
                
                dmap((psp+1, pnl-psp-1))        # Conserver décalages et tailles dans .datamap (une liste de tuples)
                wc += 1
        
        # ---
        
        self.type = "csv"
        self.sep = sp
        self.br = br
        self.encoding1 = cp
        self.encoding2 = cp
        self.wordcount = wc
        
        # Avertir si erreurs de décodage du champ 1
        if flagErrCp:
            tkMessageBox.showwarning(parent=app,
                    title=u"Ouverture d'un fichier délimité",
                    message=u"Ce fichier a pu être ouvert mais l'encodage choisi ('"+cp+"') est incorrect,\nl'affichage de certains caractères sera probablement erronné.\n\nPour rectifier cela, fermez ce dictionnaire et rouvrez-le en choisissant un autre encodage.")
        
        return


    def __loadINI(self, fichPath, cp=""):
        u"""
        Charger un dictionnaire de type INI
        """
        
        flagErrCp = False
        
        if cp == "":
            cp = tkSimpleDialog.askstring(parent=app,
                    title="Ouverture d'un fichier délimité",
                    prompt="Les fichiers INI ne contiennent aucune information\nsur l'encodage des caractères...\n\nPour un affichage correct, il est donc nécessaire de\n préciser manuellement l'encodage originel de ce fichier.\n",
                    initialvalue="cp1252")
        
        if not cp:
            self = None
            return
        
        
        self.file = open(fichPath, 'rU')                    # Ouvrir le fichier en lecture texte
        self.progr.setMax( len(self.file.readlines()) )     # Paramétrer la ProgressBar
        self.file.close()
        
        sp = '='
        
        self.file = open(fichPath, 'rb')   # Ouvrir le fichier en lecture binaire
        strfile = self.file.read()         # Charger le contenu dans une chaîne
        
        # Déterminer automatiquement le terminateur
        br = ""
        if "\r\n" in strfile: br = "\r\n"
        elif "\n" in strfile: br = "\n"
        elif "\r" in strfile: br = "\r"
        
        # Homogénéiser les sauts de lignes > remplacer par des zéros binaires (1 ou 2)
        # (modification temporaire en mémoire, le fichier sur disque n'est pas modifié)
        strfile = strfile.replace("\n","\0")
        strfile = strfile.replace("\r","\0")
        strfile += "\0" # Ajouter un zéro binaire en fin de fichier, au cas où il serait absent
        
        
        # accélérateurs locaux de la boucle de scan
        setprogr = self.progr.setValue
        sfind = strfile.find
        dmap = self.datamap.append
        
        nb = 0                                  # nb des lignes
        wc = 0                                  # wordcount
        psp = 0                                 # pointeur des séparateurs
        pnl = -1                                # pointeur des \n
        cursection = ""                         # section courante
        while psp > -1:
            
            nb += 1
            setprogr(nb)                        # afficher progression
            
            psp = sfind(sp, pnl+1)              # pointeur des séparateurs
            chp1 = strfile[pnl+1 : psp]
            chp1 = chp1.strip('\0')
            chp1 = chp1.strip()                 # Elimine espaces et lignes vides (elles sont accolées en tête de champ)
            chp1 = chp1.strip('"')              # Eliminer éventuels guillemets d'entourage
            
            try   : chp1 = chp1.decode(cp).encode('utf-8')    # Décodage et réencodage en utf-8
            except: flagErrCp = True
            
            oldpnl = pnl
            pnl = sfind('\0', psp)              # pointeur des \n
            
            if chp1 !="":
                
                psec = chp1.find("\0")
                if psec > 0: # i.e. un titre de section est accolé à la ligne suivante
                    cursection = chp1.split("\0")[0]
                    
                    self.lstAdd(cursection)         # Charger la liste des entrées
                    dmap((oldpnl+psec+len(br), 0))  # pas de données associées à cette entrée
                    wc += 1
                    
                    psec = chp1.rfind("\0")
                    chp1= chp1[psec+1:]
                    
                    self.lstAdd(" "*6 + chp1)       # Charger la liste des entrées
                    dmap((psp+1, pnl-psp-1))        # Conserver décalages et tailles dans .datamap
                    wc += 1
            
                else: # ligne "clé=valeur" normale
                    self.lstAdd(" "*6 + chp1)       # Charger la liste des entrées
                    dmap((psp+1, pnl-psp-1))        # Conserver décalages et tailles dans .datamap (une liste de tuples)
                    wc += 1
        
        # ---
        
        self.type = "ini"
        self.sep = sp
        self.br = br
        self.encoding1 = cp
        self.encoding2 = cp
        self.wordcount = wc
        
        # Avertir si erreurs de décodage du champ 1
        if flagErrCp:
            tkMessageBox.showwarning(parent=app,
                    title=u"Ouverture d'un fichier délimité",
                    message=u"Ce fichier a pu être ouvert mais l'encodage choisi ('"+cp+"') est incorrect,\nl'affichage de certains caractères sera probablement erronné.\n\nPour rectifier cela, fermez ce dictionnaire et rouvrez-le en choisissant un autre encodage.")
        
        
        return


    def lstAdd(self, mot):
        u"""
        Ajoute le 'mot' à la liste des entrées du dico
        (à la Listbox si elle existe, sinon à la pyListe interne)
        """
        
        if not self.lstbox is None:
            self.lstbox.insert(END, mot)
        else:
            self.innerlst.append(mot)  
        
        
        return
    
    

    def getdata(self, i):
        u"""
        Retourne la liste des données d'index i, quel que soit le type de dictionnaire chargé.
        Les données sont lues dans le fichier de données.
        Les données sont retournées  dans un pyDictionnaire de 10 champs utf-8
        
        Rapports :
        Champs des données Ling :   Clefs du pyDictionnaire
        -------------------------   -----------------------
                                    ['mot']     entrée du dico
                    chp 0       :   ['short']   traduction courte
                    chp 1       :   ['long']    traduction longue (infos)
                    chp 2       :   ['ID']      ID de l'entrée
                    chp 3       :   ['rac']     ids des racines
                    chp 4       :   ['syn']     ids des synonymes
                    chp 5       :   ['see']     ids des voir-aussi
                    chp 6       :   ['tok']     tokens
                    chp 7       :   ['phon']    phonétique
                    chp 8       :   ['ant']     antonymes
            
        """
        
        
        # ------------------------------------------------
        # --- Initialiser le dictionnaire à retourner     
        # ------------------------------------------------
        
        tx = u"[(non géré par ce format)]"
        tx = tx.encode('utf-8')
        
        data = {}
    
        if not self.lstbox is None:
            w = self.lstbox.get(i)
        else:
            w = self.innerlst[i]
        
        try   : w = w.encode("utf-8")
        except: pass
        
        data['mot'] = w
        
        data['short']=""
        data['long']= tx
        data['ID']=""
        data['rac']= tx
        data['syn']= tx
        data['see']= tx
        data['tok']= tx
        data['phon']= tx
        data['ant']= tx
        
        
        # -----------------------------------------------------
        # --- Extraction des données du fichier (aiguillage)   
        # -----------------------------------------------------
        
        
        # ---------------------------------------------------------------------
        if self.type.startswith("ling"): # --- LING ---------------------------
        # ---------------------------------------------------------------------
        
            self.file.seek(self.datamap[i][0])                   # Décalage dans le fichier de données
            chp = self.file.read(self.datamap[i][1])             # Lire le bloc de données
            if self.protected2=="Linguae": chp=atrifer(chp)
            
            chp = chp.split("\0")                                # Splitter les champs
            
            data['short'] = chp[0]
            data['long'] = chp[1]
            
            try   : data['ID'] = chp[2]
            except: data['ID'] = ""
            
            try   : data['rac']= chp[3]
            except: data['rac']= ""
            
            try   : data['syn']= chp[4]
            except: data['syn']= ""
            
            try   : data['see']= chp[5]
            except: data['see']= ""
            
            try   : data['tok']= chp[6]
            except: data['tok']= ""
            
            try   : data['phon']= chp[7]
            except: data['phon']= ""
            
            try   : data['ant']= chp[8]
            except: data['ant']= ""
            
            # Théoriquement un ling ne devrait recracher que de l'utf-8 mais...
            try: data['short'] = data['short'].encode('utf-8')
            except: pass
            try: data['long'] = data['long'].encode('utf-8')
            except: pass
            try: data['phon'] = data['phon'].encode('utf-8')
            except: pass
        
        
        # ---------------------------------------------------------------------
        elif self.type.startswith("dict"): # --- DICT -------------------------
        # ---------------------------------------------------------------------
            
            self.file.seek(self.datamap[i][0])          # Décalage dans le fichier de données
            short = self.file.read(self.datamap[i][1])  # Lire le bloc de données
            
            
            # Gestion du retour suivant le sous-type
            
            if self.subtype == "x":  # XDXF
                app.clipboard_clear()
                app.clipboard_append(short)
                data = self.__xdxfDataToData(data, short)
                
            else:
                
                # Théoriquement un dict ne devrait recracher que de l'utf-8 mais...
                try   : short = short.encode('utf-8')
                except: pass
                
                data['short'] = short
                data['syn'] = ""
        
        
        # ---------------------------------------------------------------------
        elif self.type.startswith("xdxf"): # --- XDXF -------------------------
        # ---------------------------------------------------------------------
            
            self.file.seek(self.datamap[i][0])                      # Décalage dans le fichier de données
            xmldata = self.file.read(self.datamap[i][1]).strip()    # Lire le bloc de données xml
            
            data = self.__xdxfDataToData(data, xmldata)
            
            
        # ---------------------------------------------------------------------
        elif self.type.startswith("csv"): # --- CSV ---------------------------
        # ---------------------------------------------------------------------
            
            self.file.seek(self.datamap[i][0])                  # Décalage dans le fichier de données
            chp0 = self.file.read(self.datamap[i][1])           # Lire le bloc dans une chaîne
            
            try:
                chp0 = chp0.decode(self.encoding2).encode('utf-8')    # Décodage et réencodage en utf-8
            except:
                tkMessageBox.showwarning(parent=app,
                        title=u"Extraire une traduction/définition à partir du fichier CSV",
                        message=u"Une erreur de décodage/encodage est survenue\npour l'entrée d'index "+ str(i))
            
            data['short'] = chp0
        
        
        # ---------------------------------------------------------------------
        elif self.type.startswith("ini"): # --- INI ---------------------------
        # ---------------------------------------------------------------------
            
            id = ""
            tx = u"[(non géré par format INI)]"
            tx = tx.encode('utf-8')
            
            if data['mot'].startswith("["):                     # Titre de section
                chp0 = "[(titre de section, pas de valeur associée)]"
            else:
                self.file.seek(self.datamap[i][0])              # Décalage dans le fichier de données
                chp0 = self.file.read(self.datamap[i][1])       # Lire la valeur dans une chaîne
                
                try:
                    chp0 = chp0.decode(self.encoding2).encode('utf-8')    # Décodage et réencodage en utf-8
                except:
                    tkMessageBox.showwarning(parent=app,
                            title=u"Extraire une traduction/définition à partir du fichier INI",
                            message=u"Une erreur de décodage/encodage est survenue\npour l'entrée d'index "+str(i))
            
            data['short'] = chp0
        
        
        # ---------------------------------------------------------------------
        elif self.type.startswith("wb"): # --- WB -----------------------------
        # ---------------------------------------------------------------------
            
            p = (i*84)+31
            self.file.seek(p)                                 # Décalage
            chp0 = self.file.read(84)                         # Lire le bloc dans une chaîne
            p0 = chp0.find('\0')
            if p0 > -1: chp0 = chp0[:p0]                      # Supprimer les \0 en fin de chaîne
            
            try:
                chp0 = chp0.decode(self.encoding2).encode('utf-8')    # Décodage et réencodage en utf-8
            except:
                tkMessageBox.showwarning(parent=app,
                        title=u"Extraire une traduction/définition à partir du fichier WB",
                        message=u"Une erreur de décodage/encodage est survenue\npour l'entrée d'index "+str(i))
            
            data['short'] = chp0
        
        
        
        # ---------------------------------------------------------
        # --- Retourner le pyDictionnaire des données (en utf-8)   
        # ---------------------------------------------------------
        
        
        return data

    
    def __xdxfDataToData(self, data, xmldata):
        u"""
        Complète le pyDictionaire d'extraction des données d'une entrée,
        à partir d'un bloc de données XDXF (issues d'un *.xdxf ou d'un *.dict/x)
        - data : pyDictionnaire de retour des données
        - xmldata : bloc xml
        """
        
#        xmldata = xmldata.strip()
#        
#        # -----------------------------------------
#        # --- Construire l'arbre DOM des données   
#        # -----------------------------------------
#        
#        try:
#            xmldom = xml.dom.minidom.parseString(xmldata)       
#        except:
#            
#            if ("<ar>" not in xmldata) and (not xmldata.startswith("<ar>")):
#                xmldata = "<ar>" + xmldata
#            if ("</ar>" not in xmldata) and (not xmldata.endswith("</ar>")):
#                xmldata = xmldata + "</ar>"
#            
#            try:
#                xmldom = xml.dom.minidom.parseString(xmldata)    # Réessayer
#            except :
#                tkMessageBox.showwarning(parent=app,
#                        title=u"Extraire les données d'un dictionnaire XDXF",
#                        message=u"Impossible de créer l'arborescence DOM des données !")
#                return
#        
#        # ---
#        cleanTag = re.compile(r"<[^>]*?>", re.MULTILINE)
#                 
#        data['short'] = ""
#        data['long'] = ""
#        data['phon'] = ""
#        
#        
#        for nd in xmldom.getElementsByTagName('ar')[0].childNodes:
#        
##            if nd.nodeType == 3:        # noeud de texte
##                t = nd.nodeValue.strip()
##                t = t.replace("\n","<br>")
##                #t = re.sub(cleanTag, "", t) 
##                data['short'] += t
##            
##            elif nd.nodeName == 'def':
##                pass    # A COMPLETER...
##            
##            elif nd.nodeName == 'dtrn': # (direct translation)
##                t = nd.firstChild.nodeValue.strip()
##                t = t.replace("\n","<br>")
##                #t = re.sub(cleanTag, "", t)
##                data['short'] += " " + t
##                nd.unlink()
#            if nd.nodeName == 'k': 
#                #nd.unlink()
#                pass
#                
#            elif nd.nodeName == 'co':   # (commentaire)
#                t = nd.firstChild.nodeValue.strip()
#                t = t.replace("\n","<br>")
#                #t = re.sub(cleanTag, "", t)
#                data['long'] += " " + t
#                #nd.unlink()
#                
#            elif nd.nodeName == 'ex':   # (exemple)
#                t = nd.firstChild.nodeValue.strip()
#                t = t.replace("\n","<br>")
#                #t = re.sub(cleanTag, "", t)
#                data['long'] += " " + t
#                #nd.unlink()
#            
#            elif nd.nodeName == 'trn':  # (phonétique)
#                t = nd.firstChild.nodeValue.strip()
#                #t = re.sub(cleanTag, "", t)
#                data['phon'] += " " + t
#                #nd.unlink()
#                
#            data['short'] = xmldom.getElementsByTagName('ar')[0].toxml()
        
        
        t = xmldata.strip()
        cleanK = re.compile(r"<k>.*?</k>", re.MULTILINE)
        t = re.sub(cleanK, "", t) # supprime <k> et son contenu
        
        data['short'] = t
        data['long'] = ""
        data['phon'] = ""
        
        
        # Théoriquement un document xdxf ne devrait contenir que de l'utf-8 mais...
        try   : data['short'] = data['short'].encode('utf-8')
        except: pass
        try   : data['long'] = data['long'].encode('utf-8')
        except: pass
        try   : data['phon'] = data['phon'].encode('utf-8')
        except: pass
        
        return data
    
    

    def idxfind(self, motif, dom, decal, debut, ignorecase, ignorelig=False, reverse=False):
        
        u"""
        Cherche 'motif' dans la listbox liée (ou la pyListe interne),
            - à partir de l'index 'decal'
            - dans le domaine 'dom' ( idx | notice | both |IDs )
            - uniquement en début d'élément si 'debut' est True
            - sans tenir compte de la casse si 'ignorecase' est True
            - sans tenir compte des ligatures si 'ignorelig' est True
        
        ... et retourne l'index correspondant
        retourne 'None' si rien de trouvé
        """
        
        # /// Sous-routines imbriquées ///
        def getnotice(i):
            data = self.getdata(i)
            return data['short'] + " " + data['long']
        def getwnotice(i):
            data = self.getdata(i)
            return data['mot'] + " " + data['short'] + " " + data['long']
        def getid(i):
            data = self.getdata(i)
            return data['ID']
        # ///////////////////////////////
        
        
        iback = None        # index qui sera retourné (None par défaut)
        
        # --------------------------------------------------------------
        # --- Définir la procédure d'extraction en fonction du domaine  
        # --------------------------------------------------------------
        
        
        if dom == "idx":
            
            # Accélérateur local (moins lourd que self.getdata)
            if not self.lstbox is None:
                wordget = self.lstbox.get   
            else:
                wordget = self.innerlst.__getitem__
        
        elif dom == "notice" :
            
            wordget = getnotice
        
        elif dom == "both" :
            
            wordget = getwnotice
        
        elif dom == "IDs" :
            
            wordget = getid
            
        
        # ---
        
        if ignorelig:
            motif = unlig(motif)
        
        # ----------------------------------------------------
        # --- Définir le pas et le "range" de la recherche    
        # ----------------------------------------------------
        
        if reverse: # Recherche à rebours
            step = -1
            rg1 = decal 
            rg2 = 0
        else:       # Recherche en avant
            step = 1
            rg1 = decal
            rg2 = self.wordcount
        
        # ----------------------------------------------
        if ignorecase:      # Casse non significative   
        # ----------------------------------------------
        
            motif = motif.lower()
            
            if debut:   # --- Casse non significative + seulement le début de mot
                
                for i in range(rg1, rg2, step):
                    
                    meule = wordget(i).lower()
                    if ignorelig:
                        meule = unlig(meule)
                    
                    if meule.startswith(motif):
                        iback = i
                        break
                
            else :      # --- Casse non significative + n'importe où dans le mot
                
                for i in range(rg1, rg2, step):
                    
                    meule = wordget(i).lower()
                    if ignorelig:
                        meule = unlig(meule)
                    
                    if motif in meule:
                        iback = i
                        break
            
            
        # -------------------------------------------
        else :              # Casse significative    
        # -------------------------------------------
            
          if debut:   # --- Casse non significative + seulement le début de mot
            
            for i in range(rg1, rg2, step):
                
                meule = wordget(i)
                if ignorelig:
                    meule = unlig(meule)
                
                if meule.startswith(motif):
                    iback = i
                    break
            
          else :      # --- Casse significative + n'importe où dans le mot
            
            for i in range(rg1, rg2, step):
                
                meule = wordget(i)
                if ignorelig:
                    meule = unlig(meule)
                
                if motif in meule:
                    iback = i
                    break
        
        return iback
        


    @staticmethod
    def strPrelingToStrLing(strPreling="", cp=""):
        
        u"""
        Convertit la chaîne 'strPreling' (chaîne pur texte au format preLing)
        en une chaîne binaire au format Ling et retourne cette chaîne
        
        cp : encodage des données sources preLing ("" = "utf-8")
            Si un encodage est inscrit en dur en tête de la chaîne preLing, cet encodage
            a la préséance sur l'encodage éventuellement fournit en paramètre
        
        En cas de problème de conversion, retourne un message explicatif préfixé par "!"
        en lieu et place de la chaîne Ling
        
        """
        
        
        
        # ----------------------------------------------
        # --- Convertir en une liste de lignes sources  
        # ----------------------------------------------
        
        lignes = strPreling.splitlines()
        
        # chercher si l'encodage est inscrit dans le fichier
        if lignes[0].startswith("%"):
            try:
                cp = lignes[0].split("/")[1]
            except:
                pass
            
            del lignes[0] # On efface la ligne (économise un test à chaque itération du scannage)
        
        
        # -------------------------------------------------------------------------------
        # Initaliser les listes/chaînes qui vont constituer les blocs du fichier ling    
        # -------------------------------------------------------------------------------
        
        propr = {}  # pyDictionnaire temporaire recevant les données du Header
        strhdr = ""
        strwordlist = []
        stridlist = []
        strdatamap = []
        strdata = []
        strgif1 = ""
        strgif2 = ""
        offsetdata = 0
        flaggif1 = False
        flaggif2 = False
        
        # -----------------------------------------------------------
        # --- Scanner les lignes sources et compléter les blocs      
        # -----------------------------------------------------------
        
        i = -1
        
        for ligne in lignes:
        
            ligne = ligne.strip()
            
            if ligne == "":                                 # Sauter les lignes vides
                continue
            elif ligne[0] == "_":                           # Sauter les lignes débutant par "_"
                continue
            
            
            # --- Champ de propriété > traitement spécifique
            #     on complète le pyDictionnaire temporaire propr que l'on convertira par la suite en bloc Header
            
            if ligne[0:2] == "::":
                
                if cp != "":
                    try:
                        ligne = ligne.decode(cp)            # Décodage de l'encodage du CSV
                    except:
                        return "! Erreur de décodage " + cp
                    ligne = ligne.encode('utf-8')           # Encodage en utf-8 de la ligne
                
                chp = ligne.split('=')                      # Splitter le champ de propriétés
                propr[chp[0][2:]] = chp[1]                  # Compléter le pyDictionnaire temporaire des propriétés
                continue
            
            # ---- Images
            
            elif ligne[0:11] == "**img1begin":
                flaggif1 = True
                continue
            elif ligne[0:11] == "**img2begin":
                flaggif2 = True
                continue
            elif ligne[0:9] == "**img1end":
                flaggif1 = False
                continue
            elif ligne[0:9] == "**img2end":
                flaggif2 = False
                continue
            elif flaggif1:
                strgif1 += ligne
                continue
            elif flaggif2:
                strgif2 += ligne
                continue
            
            # --- Ligne de donnée :
            
            if cp != "":
                try:
                    ligne = ligne.decode(cp)                # Décodage de l'encodage du CSV
                except:
                    return "! Erreur de décodage " + cp
                ligne = ligne.encode('utf-8')               # Encodage en utf-8 de la ligne
            
            
            i += 1                                          # Incrémenter l'index de l'entrée
            
            chp = ligne.split('\t')                         # Splitter les champs de la ligne de données
            if len(chp) < 2:
                return u'! PreLing > Ling : Ligne sans traduction : \n\n"' + ligne + '"'
            
            strwordlist.append(chp[0])                      # Libellé de l'entrée
            
            id = ""
            try   : id = chp[3]
            except: pass
            if id != "":
                stridlist.append(id + "\0" + str(i))        # L'entrée a un ID
            
            
            # Chaîne des données de l'entrée :
            
            txt=[]
            try:
                txt.append(chp[1] + "\0")                       # Traductions courtes
            except:
                return '! PreLing > Ling : l\'ajout de la traduction courte a échoué : \n\n"' + ligne + '"'
            
            try   : txt.append(chp[2] + "\0")                   # Infos
            except: txt.append("\0")
            
            txt.append(id + "\0")                               # ID
            
            try   : txt.append(chp[4].strip(";") + "\0")        # Racines
            except: txt.append("\0")
            
            try   : txt.append(chp[5].strip(";") + "\0")        # Synonymes
            except: txt.append("\0")
            
            try   : txt.append(chp[6].strip(";") + "\0")        # Voir-aussi
            except: txt.append("\0")
            
            try   : txt.append(chp[7].strip(";") + "\0")        # Tokens
            except: txt.append("\0")
            
            try   : txt.append(chp[8] + "\0")                   # Phonétique
            except: txt.append("\0")
            
            try   : txt.append(chp[9].strip(";"))               # Antonymes
            except: pass
            
            txt="".join(txt)
            
            
            strdata.append(txt)
            
            strdatamap.append(NumToStr32(offsetdata) + NumToStr32(len(txt))) # Ajouter l'offset et la taille
            
            offsetdata += len(txt)    # Ajouter la taille pour obtenir l'offset des données suivantes
        
        
        # --- (Next ligne // fin du scannage des lignes)
        
        
        # ---------------------------------------------
        # --- Construire la chaîne du bloc Header      
        # ---------------------------------------------
        # Conversion du pyDictionnaire temporaire propr
        
        # NB1 : construction progressive d'une chaîne pour fixer l'ordre des champs                 
        # car concaténer avec items() retourne un ordre aléatoire                                   
        # NB2 : guillemets obligatoires dans le fichier Ling pour encadrer les chaînes de propriétés
        # car les propriétés sont ensuite chargées par 'exec'                                       
        
        strhdr = ""         # Chaîne du Header
        
        if propr.has_key('minCompatVersion'): strhdr += 'minCompatVersion="' + propr['minCompatVersion'].strip('"') + '"\0'
        else: strhdr += 'minCompatVersion="01.00.00"\0'
        
        if propr.has_key('maxCompatVersion'): strhdr += 'maxCompatVersion="' + propr['maxCompatVersion'].strip('"') + '"\0'
        else: strhdr += 'maxCompatVersion=""\0'
        
        if propr.has_key('dicName'): strhdr += 'dicName="' + propr['dicName'].strip('"') + '"\0'
        else: strhdr += 'dicName=""\0'
        
        if propr.has_key('langName1'): strhdr += 'langName1="' + propr['langName1'].strip('"') + '"\0'
        else: strhdr += 'langName1=""\0'
        
        if propr.has_key('langName2'): strhdr += 'langName2="' + propr['langName2'].strip('"') + '"\0'
        else: strhdr += 'langName2=""\0'
        
        if propr.has_key('langIso1'): strhdr += 'langIso1="' + propr['langIso1'].strip('"') + '"\0'
        else: strhdr += 'langIso1=""\0'
        
        if propr.has_key('langIso2'): strhdr += 'langIso2="' + propr['langIso2'].strip('"') + '"\0'
        else: strhdr += 'langIso2=""\0'
        
        if propr.has_key('langNameUser'): strhdr += 'langNameUser="' + propr['langNameUser'].strip('"') + '"\0'
        else: strhdr += 'langNameUser=""\0'
        
        if propr.has_key('langIsoUser'): strhdr += 'langIsoUser="' + propr['langIsoUser'].strip('"') + '"\0'
        else: strhdr += 'langIsoUser=""\0'
        
        if propr.has_key('langFamily1'): strhdr += 'langFamily1="' + propr['langFamily1'].strip('"') + '"\0'
        else: strhdr += 'langFamily1=""\0'
        
        if propr.has_key('langFamily2'): strhdr += 'langFamily2="' + propr['langFamily2'].strip('"') + '"\0'
        else: strhdr += 'langFamily2=""\0'
        
        if propr.has_key('isReverseDic'): strhdr += 'isReverseDic=' + propr['isReverseDic'].strip('"') + '\0'
        else: strhdr += 'isReverseDic=False\0'
        
        if propr.has_key('doReverseDic'): strhdr += 'doReverseDic=' + propr['doReverseDic'].strip('"') + '\0'
        else: strhdr += 'doReverseDic=False\0'
        
        if propr.has_key('reverseDicFileName'): strhdr += 'reverseDicFileName="' + propr['reverseDicFileName'].strip('"') + '"\0'
        else: strhdr += 'reverseDicFileName=""\0'
        
        if propr.has_key('reverseDicName'): strhdr += 'reverseDicName="' + propr['reverseDicName'].strip('"') + '"\0'
        else: strhdr += 'reverseDicName=""\0'
        
        if propr.has_key('sortEquPatterns'): strhdr += 'sortEquPatterns="' + propr['sortEquPatterns'].strip('"') + '"\0'
        else: strhdr += 'sortEquPatterns=""\0'
        
        strhdr += 'wordcount=' + str(i+1) + '\0'
        
        if propr.has_key('mainAuthors'): strhdr += 'mainAuthors="' + propr['mainAuthors'].strip('"') + '"\0'
        else: strhdr += 'mainAuthors=""\0'
        
        if propr.has_key('altAuthors'): strhdr += 'altAuthors="' + propr['altAuthors'].strip('"') + '"\0'
        else: strhdr += 'altAuthors=""\0'
        
        if propr.has_key('contactAuthor'): strhdr += 'contactAuthor="' + propr['contactAuthor'].strip('"') + '"\0'
        else: strhdr += 'contactAuthor=""\0'
        
        if propr.has_key('dicStatus'): strhdr += 'dicStatus="' + propr['dicStatus'].strip('"') + '"\0'
        else: strhdr += 'dicStatus=""\0'
        
        if propr.has_key('showDicStatus'): strhdr += 'showDicStatus=' + propr['showDicStatus'].strip('"') + '\0'
        else: strhdr += 'showDicStatus=False\0'
        
        if propr.has_key('copyright'): strhdr += 'copyright="' + propr['copyright'].strip('"') + '"\0'
        else: strhdr += 'copyright=""\0'
        
        if propr.has_key('creationDate'): strhdr += 'creationDate="' + propr['creationDate'].strip('"') + '"\0'
        else: strhdr += 'creationDate=""\0'
        
        if propr.has_key('versionDate'): strhdr += 'versionDate="' + propr['versionDate'].strip('"') + '"\0'
        else: strhdr += 'versionDate=""\0'
        
        if propr.has_key('localEditDate'): strhdr += 'localEditDate="' + propr['localEditDate'].strip('"') + '"\0'
        else: strhdr += 'localEditDate=""\0'
        
        if propr.has_key('dicID'): strhdr += 'dicID="' + propr['dicID'].strip('"') + '"\0'
        else: strhdr += 'dicID=""\0'
        
        if propr.has_key('dicVersionNumber'): strhdr += 'dicVersionNumber="' + propr['dicVersionNumber'].strip('"') + '"\0'
        else: strhdr += 'dicVersionNumber="1"\0'
        
        if propr.has_key('dicUrl'): strhdr += 'dicUrl="' + propr['dicUrl'].strip('"') + '"\0'
        else: strhdr += 'dicUrl=""\0'
        
        if propr.has_key('verUrl'): strhdr += 'verUrl="' + propr['verUrl'].strip('"') + '"\0'
        else: strhdr += 'verUrl=""\0'
        
        if propr.has_key('dicInfo'): strhdr += 'dicInfo="' + propr['dicInfo'].strip('"') + '"\0'
        else: strhdr += 'dicInfo=""\0'
        
        if propr.has_key('showDicInfo'): strhdr += 'showDicInfo="' + propr['showDicInfo'].strip('"') + '"\0'
        else: strhdr += 'showDicInfo=False\0'
        
        if propr.has_key('protected1') and propr['protected1'] != "":
          flgc1 = True ; strhdr += 'protected1="Linguae"\0'
        else: flgc1 = False ; strhdr += 'protected1=""\0'
        
        if propr.has_key('protected2') and propr['protected2'] != "":
          flgc2 = True ; strhdr += 'protected2="Linguae"\0'
        else: flgc2 = False ; strhdr += 'protected2=""\0'
        
        if propr.has_key('displayFontName1'): strhdr += 'displayFontName1="' + propr['displayFontName1'].strip('"') + '"\0'
        else: strhdr += 'displayFontName1=""\0'
        
        if propr.has_key('displayFontName2'): strhdr += 'displayFontName2="' + propr['displayFontName2'].strip('"') + '"\0'
        else: strhdr += 'displayFontName2=""\0'
        
        if propr.has_key('grammarEncoding1'): strhdr += 'grammarEncoding1="' + propr['grammarEncoding1'].strip('"') + '"\0'
        else: strhdr += 'grammarEncoding1=""\0'
        
        if propr.has_key('compatPlugins'): strhdr += 'compatPlugins="' + propr['compatPlugins'].strip('"') + '"\0'
        else: strhdr += 'compatPlugins=""\0'
        
        if propr.has_key('noCompatPlugins'): strhdr += 'noCompatPlugins="' + propr['noCompatPlugins'].strip('"') + '"\0'
        else: strhdr += 'noCompatPlugins=""\0'
        
        if propr.has_key('usePlugins'): strhdr += 'usePlugins="' + propr['usePlugins'].strip('"') + '"\0'
        else: strhdr += 'usePlugins=""\0'
        
        if propr.has_key('wordGroups'): strhdr += 'wordGroups="' + propr['wordGroups'].strip('"') + '"\0'
        else: strhdr += 'wordGroups=""\0'
        
        if propr.has_key('biblio'): strhdr += 'biblio="' + propr['biblio'].strip('"') + '"\0'
        else: strhdr += 'biblio=""\0'
        
        if propr.has_key('showBiblio'): strhdr += 'showBiblio=' + propr['showBiblio'].strip('"') + '\0'
        else: strhdr += 'showBiblio=False\0'
        
        if propr.has_key('extFieldCount'): strhdr += 'extFieldCount=' + propr['extFieldCount'].strip('"') + '\0'
        else: strhdr += 'extFieldCount=0\0'
        
        if propr.has_key('extFieldList'): strhdr += 'extFieldList="' + propr['extFieldList'].strip('"') + '"'
        else: strhdr += 'extFieldList=""'
        # Attention : ne pas ajouter de '\0' pour terminer le dernier champ du bloc
        
        
        
        # ----------------------------------------
        # --- Construire les chaînes des blocs    
        # ----------------------------------------
        
        strwordlist = "\0".join(strwordlist)
        if flgc1: strwordlist = atrifer(strwordlist)
        
        stridlist = "\0".join(stridlist)
        
        strdatamap = "".join(strdatamap)
        
        if flgc2:
            for i in range(len(strdata)):
                strdata[i] = atrifer (strdata[i])
        strdata = "".join(strdata)
        
        
        
        # -----------------------------------------
        # --- Construire le bloc de cartographie   
        # -----------------------------------------
        strmap = "%ling/01.00.00"                                   # Identifiant de type de fichier
        
        decal=70                                                    # offset du bloc hdr
        strmap += NumToStr32(decal) + NumToStr32(len(strhdr))       # ajouter offset et taille du bloc hdr
        
        decal += len(strhdr)                                        # offset du bloc wordlist
        strmap += NumToStr32(decal) + NumToStr32(len(strwordlist))  # ajouter offset et taille du bloc wordlist
        
        decal += len(strwordlist)                                   # offset du bloc idlist
        strmap += NumToStr32(decal) + NumToStr32(len(stridlist))    # ajouter offset et taille du bloc idlist
        
        decal += len(stridlist)                                     # offset du bloc datamap
        strmap += NumToStr32(decal) + NumToStr32(len(strdatamap))   # ajouter offset et taille du bloc datamap
        
        decal += len(strdatamap)                                    # offset du bloc datam
        strmap += NumToStr32(decal) + NumToStr32(len(strdata))      # ajouter offset et taille du bloc data
        
        decal += len(strdata)                                       # offset du bloc image1
        if strgif1 != "":
            strmap += NumToStr32(decal) + NumToStr32(len(strgif1))  # ajouter offset et taille du bloc image1
        else:
            strmap += "\0" * 8                                      # ajouter offset(0) et taille(0) du bloc image1
        
        decal += len(strgif1)                                       # offset du bloc image2
        if strgif2 != "":
            strmap += NumToStr32(decal) + NumToStr32(len(strgif2))  # ajouter offset et taille du bloc image2
        else:
            strmap += "\0" * 8                                      # ajouter offset(0) et taille(0) du bloc image2
        
        
        
        
        
        # ----------------------------------------
        # --- Construire la chaîne globale        
        # ----------------------------------------
        
        strLing = strmap + strhdr + strwordlist + stridlist + strdatamap + strdata + strgif1 + strgif2
        
        
        return strLing




    def toStrPreling(self, progr=None, iEdit=None, contentEdit=None, sortedList=None, protected1=None, protected2=None):
        
        u"""
        Convertit le dictionnaire courant en une chaîne formattée "preLing" utf-8
        et retourne cette chaîne
        
        - 'protected1' et 'protected2' : nom des standards de protection à inscrire dans la chaîne
        
        - On peut passer dans 'progr' la référence facultative à une barre de progression
        
        
        # FONCTIONNALITE DE TRI (facultative):
        
        - si sortedList est renseigné, l'ordre des entrées est repris
          de cette liste formattée ainsi : 'entrée'|index originel
        
        
        
        # FONCTIONNALITE D'EDITION (facultative) :
        
        - 'iEdit' : index des données dont le contenu est à remplacer par 'contentEdit'
        
        - si contentEdit == "---" l'entrée d'index 'iEdit' est supprimée
        
        - si iEdit == -1, une entrée de contenu 'contentEdit' est ajoutée en fin de chaîne
        
        """
        
        
        strPreling = ["%preling",]
        
        # -----------------------------------------------------
        # --- En-tête                                          
        # -----------------------------------------------------
        
        strPreling.append("::minCompatVersion=" + self.minCompatVersion)
        strPreling.append("::maxCompatVersion=" + self.maxCompatVersion)
        strPreling.append("::dicName=" + self.dicName)
        strPreling.append("::langName1=" + self.langName1)
        strPreling.append("::langName2=" + self.langName2)
        strPreling.append("::langIso1=" + self.langIso1)
        strPreling.append("::langIso2=" + self.langIso2)
        strPreling.append("::langNameUser=" + self.langNameUser)
        strPreling.append("::langIsoUser=" + self.langIsoUser)
        strPreling.append("::langFamily1=" + self.langFamily1)
        strPreling.append("::langFamily2=" + self.langFamily2)
        strPreling.append("::isReverseDic=" + str(self.isReverseDic))
        strPreling.append("::doReverseDic=" + str(self.doReverseDic))
        strPreling.append("::reverseDicFileName=" + self.reverseDicFileName)
        strPreling.append("::reverseDicName=" + self.reverseDicName)
        strPreling.append('::sortEquPatterns="' + '","'.join(self.sortEquPatterns) + '"')
        
        if iEdit == -1:             # Ajout d'une nouvelle entrée
            strPreling.append("::wordcount=" + str(self.wordcount + 1))
        elif contentEdit == "---":  # Suppression d'une entrée
            strPreling.append("::wordcount=" + str(self.wordcount - 1))
        else:
            strPreling.append("::wordcount=" + str(self.wordcount))
        
        strPreling.append('::mainAuthors="' + '","'.join(self.mainAuthors) + '"')
        strPreling.append('::altAuthors="' + '","'.join(self.altAuthors) + '"')
        strPreling.append("::contactAuthor=" + self.contactAuthor)
        strPreling.append("::dicStatus=" + self.dicStatus)
        strPreling.append("::showDicStatus=" + str(self.showDicStatus))
        strPreling.append("::copyright=" + self.copyright)
        strPreling.append("::creationDate=" + self.creationDate)
        strPreling.append("::versionDate=" + self.versionDate)
        strPreling.append("::localEditDate=" + self.localEditDate)
        strPreling.append("::dicID=" + self.dicID)
        strPreling.append("::dicVersionNumber=" + self.dicVersionNumber)
        strPreling.append("::dicUrl=" + self.dicUrl)
        strPreling.append("::verUrl=" + self.verUrl)
        strPreling.append("::dicInfo=" + self.dicInfo)
        strPreling.append("::showDicInfo=" + str(self.showDicInfo))
        
        # infos de protection
        if protected1 is None : protected1= self.protected1
        if protected2 is None : protected2= self.protected2
        strPreling.append("::protected1=" + protected1)
        strPreling.append("::protected2=" + protected2)
        
        strPreling.append("::displayFontName1=" + self.displayFontName1)
        strPreling.append("::displayFontName2=" + self.displayFontName2)
        strPreling.append("::grammarEncoding1=" + self.grammarEncoding1)
        strPreling.append('::compatPlugins="' + '","'.join(self.compatPlugins) + '"')
        strPreling.append('::noCompatPlugins="' + '","'.join(self.noCompatPlugins) + '"')
        strPreling.append('::usePlugins="' + '","'.join(self.usePlugins) + '"')
        strPreling.append('::wordGroups="' + '","'.join(self.wordGroups) + '"')
        strPreling.append('::biblio="' + '","'.join(self.biblio) + '"')
        strPreling.append("::showBiblio=" + str(self.showBiblio))
        strPreling.append("::extFieldCount=" + str(self.extFieldCount))
        strPreling.append('::extFieldList="' + '","'.join(self.extFieldList) + '"')
        
        
        # -----------------------------------------------------
        # --- Bloc des images                                  
        # -----------------------------------------------------
        
        if app.dic.img1 != "":
            strPreling.append("**img1begin\n" + app.dic.img1 + "\n**img1end")
        
        if app.dic.img2 != "":
            strPreling.append("**img2begin\n" + app.dic.img2 + "\n**img2end")
        
        # -----------------------------------------------------
        # --- Corps des données                                
        # -----------------------------------------------------
        
        lignePreL = ""
        
        getdata = self.getdata           # (référence en variable locale pour accélérer la boucle)
        
        for i in range(self.wordcount):
            
            if not progr is None: progr.setValue(i)        # Afficher la progression
            
            # modifier l'index à extraire si tri de la liste
            if not sortedList is None:
                iExtract = int(sortedList[i].split('|')[-1])       # lire l'index originel avant le tri
            else:
                iExtract =i
            
            if i != iEdit:          # Entrée standard (c.à.d. non éditée ni ajoutée)
                
                data = getdata(iExtract)                    # Lire les données sur le disque > pyDictionnaire utf-8
                
                word = data['mot']                          # Libellé de l'entrée
                
                chp0 = data['short']                        # Traductions courtes
                
                chp1 = data['long']                         # Traduction longue
                if chp1.startswith("[("): chp1 = ""         # (non géré par le format courant)
                
                chp2 = data['ID']                           # ID de l'entrée
                
                chp3 = data['rac']                          # ids des racines
                if chp3.startswith("[("): chp3 = ""         # (non géré par le format courant)
                
                chp4 = data['syn']                          # ids des synonymes
                if chp4.startswith("[("): chp4 = ""         # (non géré par le format courant)
                
                chp5 = data['see']                          # ids des voir-aussi
                if chp5.startswith("[("): chp5 = ""         # (non géré par le format courant)
                
                chp6 = data['tok']                          # tokens
                if chp6.startswith("[("): chp6 = ""         # (non géré par le format courant)
                
                chp7 = data['phon']                         # phonétique
                if chp7.startswith("[("): chp7 = ""         # (non géré par le format courant)
                
                chp8 = data['ant']                          # ids des Antonymes
                if chp8.startswith("[("): chp8 = ""         # (non géré par le format courant)
                
                # Concaténer la ligne de données de l'entrée
                lignePreL = word + '\t' + chp0  + '\t' + chp1 + '\t' + chp2 + '\t' + chp3 + '\t' + chp4 + '\t' + chp5 + '\t' + chp6 + '\t' + chp7 + '\t' + chp8
            
            
            elif i == iEdit:   # Editer cette entrée
                
                if contentEdit == "---":        # Supprimer l'entrée (la sauter)
                    word=""
                    continue
                
                else:           # Modifier l'entrée
                    lignePreL = contentEdit
                
            
            # Ajouter la ligne de donnée
            try:
                strPreling.append(lignePreL)
            except:
                print "plouf :"+ligne
            
            lignePreL = ""
        
        # (fin boucle de scannage des données)
        
        # Ajouter éventuellement une nouvelle entrée (indiqué par index == -1)
        
        if iEdit == -1:
            strPreling.append(contentEdit)
        
        
        # -----------------------------------------
        # --- Concaténer la chaîne preling         
        # -----------------------------------------
        
        try:
            strPreling = "\n".join(strPreling)
        except:
            return "! Erreur concaténation preLing !"
        
        
        return strPreling



    def toStrsDict(self, progr=None, iEdit=None, contentEdit=None, sortedList=None):
        
        u"""
        Convertit le dictionnaire courant
        en un tupple de 3 chaines :(stridx, strdict, wordcount)
        correspondant à un dictionnaire Dict sans son fichier *.ifo
        
        En cas d'échec, retourne une chaîne simple contenant un message d'erreur
        
        - On peut passer dans 'progr' la référence facultative à une barre de progression
        
        
        # FONCTIONNALITE DE TRI (facultative):
        
        - si sortedList est renseigné, l'ordre des entrées est repris
          de cette liste formattée ainsi : 'entrée'|index originel
        
        
        # FONCTIONNALITE D'EDITION (facultative) :
        
        - 'iEdit' : index des données dont le contenu est à remplacer par 'contentEdit'
        
        - si contentEdit == "---" l'entrée d'index 'iEdit' est supprimée
        
        - si iEdit == -1, une entrée de contenu 'contentEdit' est ajoutée en fin de chaîne
        
        
        """
        
        strifo=[]   # Future chaîne du *.ifo
        stridx=[]   # Future chaîne du *.idx
        strdict=[]  # Future chaîne du *.dict
        
        
        # --------------------------------------
        # --- Scanner la liste des entrées      
        # --------------------------------------
        
        decal = 0
        chp1 = ""
        chp2 = ""
        wordcount = 0
        
        # accélérateurs locaux
        setprogr = self.progr.setValue
        
        
        for i in range(self.wordcount):
            
            if not progr is None: setprogr(i)      # Afficher la progression
            
            # modifier l'index à extraire si tri de la liste
            if not sortedList is None:
                iExtract = int(sortedList[i].split('|')[-1])       # lire l'index originel avant le tri
            else:
                iExtract = i
            
            
            decal = decal + len(chp2)               # Décalage (offset) de la traduction
            
            if i != iEdit:          # Entrée standard (c.à.d. non éditée ni ajoutée)
                
                data = self.getdata(iExtract)       # Extraire les données (> pyDictionnaire)
                chp1 = data['mot']                  # Libellé de l'entrée (en utf-8)
                chp2 = data['short']                # Texte de la traduction (en utf-8)
            
            elif i == iEdit:    # Editer cette entrée
                
                if contentEdit == "---": # Supprimer l'entrée (la sauter)
                    decal = decal - len(chp2)       # annuler l'incrémentation précédente
                    continue
                
                else: # Modifier l'entrée
                    chp1 = contentEdit.split("\0")[0]
                    chp2 = contentEdit.split("\0")[1]
            
            
            # --- Compléter les chaînes
            wordcount += 1
            strdict.append(chp2)                    # Ajouter la traduction dans le *.dict
            stridx.append(chp1 + '\0')              # Ajouter l'entrée dans le *.idx
            stridx.append(NumToStr32(decal))        # Ajouter le décalage (image chaîne en 32 bits) dans le *.idx
            stridx.append(NumToStr32(len(chp2)))    # Ajouter la longueur de la traduction (image chaîne en 32 bits) dans le *.idx
        
        
        # --- Ajouter éventuellement une nouvelle entrée (indiqué par index == -1)
        
        if iEdit == -1:
            decal = decal + len(chp2)
            chp1 = contentEdit.split("\0")[0]
            chp2 = contentEdit.split("\0")[1]
            wordcount += 1
            strdict.append(chp2)
            stridx.append(chp1 + '\0')
            stridx.append(NumToStr32(decal))
            stridx.append(NumToStr32(len(chp2)))
        
        
        
        # --- Concaténer les chaines
        stridx = "".join(stridx)
        strdict = "".join(strdict)
        wordcount = str(wordcount)
        
        return (stridx, strdict, wordcount)


    def moveindic(self, isource, icible):
        
        u"""
        Déplace une entrée dans le dictionnaire
        
        Pour cela on réutilise les fonctions de tri propres à chaque format,
        en générant une pyListe "triée" dans laquelle le "tri" a simplement
        consisté à déplacer l'entrée concernée
        
        Retourne l'index réel de l'insertion ou None si échec
        """
        
        # --- Tests préalables
        
        if self.type in ("dictdz", "dictgz"):
            app.hideProgr()
            tkMessageBox.showinfo(parent=app,
                    title=u"Trier le dictionnaire",
                    message=u"Ce dictionnaire est au format DICT compressé, il ne peut être modifié.\n\nPour déplacer une de ses entrées, vous devrez d'abord le convertir au format DICT non compressé\nou tout autre format géré par Linguae (Menu 'Fichier' > 'Exporter').")
            return
        
        
        # -----------------------------------------------------------
        # --- Confectionner une pyListe temporaire [entrée]|[index]  
        # -----------------------------------------------------------
        
        ltri=[]
        
        # Accélérateur local (moins lourd que self.getdata)
        if not self.lstbox is None:
            wordget = self.lstbox.get
        else:
            wordget = self.innerlst.__getitem__
        
        
        # --- On construit la pyListe
        
        
        for i in range(self.wordcount):
            t = wordget(i).lower().encode('utf-8') # Libellé de l'entrée, mis en minuscule
            ltri.append(t + "|" + str(i))
        
        # --- On déplace l'entrée dans la pyListe
        # (supprimer/insérer modifie les index de la pyListe
        #  mais la pyListe contenant les index originaux > pas de correction à effectuer)
        
        machin = ltri.pop(isource)
        
        ltri.insert(icible, machin)  # (icible sera retourné par la procédure)
        
        
        # ----------------------------------------------------------------
        # --- Aiguillage                                                  
        # --- (> rebatir le dico en fonction de l'ordre de la pyListe)    
        # ----------------------------------------------------------------
        
        memoLocalEditDate = self.localEditDate 
        self.localEditDate = str(datetime.date.today())
        
        ret="" # Message de retour des modules de tri
        
        if self.type.startswith("ling"):
            ret = self.__sortdicLing(ltri)
        
        elif self.type.startswith("wb"):
            ret = self.__sortdicWB(ltri)
        
        elif self.type.startswith("dict"):
            ret = self.__sortdicDict(ltri)
        
        elif self.type.startswith("csv"):
            ret = self.__sortdicCSV(ltri)
        
        elif self.type.startswith("xdxf"):
            ret = self.__sortdicXDXF(ltri)
        
        else:
            tkMessageBox.showwarning(parent=app,
                title=u"Déplacer une entrée",
                message=u"Les entrées de ce type de dictionnaire ne sont pas déplacables")
        
        if ret != "":
            tkMessageBox.showwarning(parent=app,
                    title=u"Echec du déplacement",
                    message=ret)
            self.localEditDate = memoLocalEditDate
            return
        
        
        return icible




    def sortdic(self, equs=None):
        u"""
        Tri alphabétique du dictionnaire courant
        
        'equs' est un pyDictionnaire des équivalences de caractères pour le tri
        s'il n'est pas fourni, l'attribut .sortEquPatterns du dictionnaire est utilisé (s'il est complété)
        """
        
        # --- Tests préalables
        
        if self.type.startswith("ini"):
            app.hideProgr()
            tkMessageBox.showinfo(parent=app,
                    title=u"Trier le dictionnaire",
                    message=u"Du fait de sa structure hiérarchique, un dictionnaire au format INI\nne peut être trié")
            return
        
        elif self.type in ("dictdz", "dictgz"):
            app.hideProgr()
            tkMessageBox.showinfo(parent=app,
                    title=u"Trier le dictionnaire",
                    message=u"Ce dictionnaire est au format DICT compressé, il ne peut être modifié.\n\nPour le trier, vous devrez d'abord le convertir au format DICT non compressé\nou tout autre format géré par Linguae (Menu 'Fichier' > 'Exporter').")
            return
        
        
        # -----------------------------------------------------------
        # --- Confectionner une pyListe temporaire [entrée]|[index]  
        # --- qui sera ensuite triée                                 
        # -----------------------------------------------------------
        
        ltri=[]
        flagUseSortDefault = True
        
        # Accélérateur local (moins lourd que self.getdata)
        if not self.lstbox is None:
            wordget = self.lstbox.get
        else:
            wordget = self.innerlst.__getitem__
        
        # Déterminer si on va utiliser des équivalences de caractères
        # ou le tri par défaut de la plateforme
        if not equs is None:
            flagUseSortDefault = False
        elif len(self.sortEquPatterns) > 0:
            if self.sortEquPatterns[0] != "":
                flagUseSortDefault = False
        
        
        # --- On construit la pyListe en utilisant des équivalences de caractères
        if flagUseSortDefault == False: 
            
            # Batir, si besoin, le pyDictionnaire des équivalences de caractères
            if equs is None:
                equs = {}
                for n in range(len(self.sortEquPatterns)):
                    t = self.sortEquPatterns[n].split(":")
                    equs[t[0]] = t[1]
            
            for i in range(self.wordcount):
                
                t = wordget(i).lower().encode('utf-8') # Libellé de l'entrée, mis en minuscule
                
                # Appliquer les équivalences de caractères
                for equ in equs:
                    try:
                        t = t.replace(equ, equs[equ])
                    except:
                        print "! ! ! ERREUR EQUIVALENCE CARACTERES POUR LE TRI ! ! !"
                
                ltri.append(t + "|" + str(i))
        
        
        # --- On construit la pyListe avec les critères de tri par défaut de la plateforme
        else:
            
            for i in range(self.wordcount):
                
                t = wordget(i).lower().encode('utf-8') # Libellé de l'entrée, mis en minuscule
                
                ltri.append(t + "|" + str(i))
        
        
        # ----------------------------------
        # --- Trier la pyListe temporaire   
        # ----------------------------------
        
        # On applique un tri alphabétique standard, mais celui-ci est en fait personnalisé
        # du fait de l'application antérieure des équivalenes de caractères
        
        ltri.sort() 
        
        
        # -------------------------------------------------------------------
        # --- Aiguillage                                                     
        # --- (> rebatir le dico en fonction de l'ordre de la pyListe triée) 
        # -------------------------------------------------------------------
        
        memoLocalEditDate = self.localEditDate 
        self.localEditDate = str(datetime.date.today())
        
        ret="" # Message de retour des modules de tri
        
        if self.type.startswith("ling"):
            ret = self.__sortdicLing(ltri)
        
        elif self.type.startswith("wb"):
            ret = self.__sortdicWB(ltri)
        
        elif self.type.startswith("dict"):
            ret = self.__sortdicDict(ltri)
        
        elif self.type.startswith("csv"):
            ret = self.__sortdicCSV(ltri)
        
        elif self.type.startswith("xdxf"):
            ret = self.__sortdicXDXF(ltri)
        
        else:
            tkMessageBox.showwarning(parent=app,
                title=u"Trier le dictionnaire",
                message=u"Ce type de dictionnaire n'est pas triable")
        
        if ret != "":
            tkMessageBox.showwarning(parent=app,
                    title=u"Echec du tri du dictionnaire",
                    message=ret)
            self.localEditDate = memoLocalEditDate
            return
        
        
        return


    def __sortdicLing(self, ltri):
        u"""
        Trier un dictionnaire de type Ling
        """
        
        # Créer une chaîne preLing en fonction du tri antérieur
        strPreling = self.toStrPreling(sortedList=ltri)
        
        # Convertir la chaîne preling en chaîne Ling
        strLing = DicObject.strPrelingToStrLing(strPreling=strPreling)
        
        # Enregistre le nouveau fichier
        fichpath = self.path
        self.file.close()
        ff = open(fichpath, 'wb')
        ff.write(strLing)
        ff.close()
        
        # Recharge le dictionnaire avec le nouveau fichier trié
        lbox = self.lstbox
        prg = self.progr
        self.__init__(fichpath, lbox, prg)
        
        
        return ""


    def __sortdicDict(self, ltri):
        u"""
        Trier un dictionnaire de type Dict
        """
        
        
        # Créer les chaînes Dict en fonction du tri antérieur
        strs = self.toStrsDict(sortedList=ltri)
        
        
        # si strs est une chaine et non un tupple => problème !
        if type(strs) is type("xxx"):
            return strs
        
        
        # ------------------------------------------------------
        # --- Enregistrer les nouveaux fichiers *.idx et *.dict 
        # ------------------------------------------------------
        
        radicfich = os.path.splitext(self.path)[0]     # Radical des différents fichiers Dict
        
        self.file.close()
        
        ff = open(radicfich + '.idx', 'wb')   # Enregistrer le *idx
        ff.write(strs[0])
        ff.close()
        
        ff = open(radicfich + '.dict', 'wb')  # Enregistrer le *dict
        ff.write(strs[1])
        ff.close()
        
        # (NB : le *ifo reste le même après le tri)
        
        
        # ----------------------------------------------------------------
        # --- Recharger le dictionnaire avec le fichier Dict trié         
        # ----------------------------------------------------------------
        
        
        lbox = self.lstbox
        prg = self.progr
        self.__init__(radicfich + '.dict', lbox, prg)
        
        
        return ""



    def __sortdicXDXF(self, ltri):
        u"""
        Trier un dictionnaire de type XDXF
        """
        
        strxdxf = ""
        
        # Accélérateurs locaux
        sk = self.file.seek
        rd = self.file.read
        
        # début du fichier : du début au début du premier article
        sk(0)
        strxdxf = rd(self.datamap[0][0]) 
        
        # ensemble des articles
        for ligne in ltri:
            i = int(ligne.split('|')[-1])       # lire l'index originel avant le tri
            sk(self.datamap[i][0])
            strxdxf += rd(self.datamap[i][1]) + "\n"
        
        strxdxf += "</xdxf>\n"
        
        # Enregistre le nouveau fichier
        fichpath = self.path
        self.file.close()
        ff = open(fichpath, 'wb')
        ff.write(strxdxf)
        ff.close()
        
        # Recharge le dictionnaire avec le nouveau fichier trié
        lbox = self.lstbox
        prg = self.progr
        self.__init__(fichpath, lbox, prg)
        
        
        return ""


    def __sortdicWB(self, ltri):
        u"""
        Trier un dictionnaire de type WB
        """
        
        strwb = ""
        
        for ligne in ltri:
            i = int(ligne.split('|')[-1])       # lire l'index originel avant le tri
            self.file.seek(84*i)
            strwb += self.file.read(84)
        
        # Enregistre le nouveau fichier
        fichpath = self.path
        self.file.close()
        ff = open(fichpath, 'wb')
        ff.write(strwb)
        ff.close()
        
        # Recharge le dictionnaire avec le nouveau fichier trié
        lbox = self.lstbox
        prg = self.progr
        cp1 = self.encoding1
        cp2 = self.encoding2
        self.__init__(fichpath, lbox, prg, cp1=cp1, cp2=cp2)
        
        
        return ""



    def __sortdicCSV(self, ltri):
        u"""
        Trier un dictionnaire de type CSV
        """
        
        strcsv = ""
        
        
        for ligne in ltri:
            i = int(ligne.split('|')[-1])       # lire l'index originel avant le tri
            
            data = self.getdata(i)
            
            chp1 = data['mot'].decode('utf-8')
            chp1 = chp1.encode(self.encoding1)
            
            chp2 = data['short'].decode('utf-8')
            chp2= chp2.encode(self.encoding2)
            
            strcsv += chp1 + self.sep + chp2 + self.br
        
        # Enregistre le nouveau fichier
        fichpath = self.path
        self.file.close()
        ff = open(fichpath, 'wb')
        ff.write(strcsv)
        ff.close()
        
        # Recharge le dictionnaire avec le nouveau fichier trié
        lbox = self.lstbox
        prg = self.progr
        cp = self.encoding1
        self.__init__(fichpath, lbox, prg, cp1=cp)
        
        
        return ""



class WindowInfo(Toplevel):
    u"""
    Fenêtre d'affichage des infos et statut du dictionnaire
    
    Cette fenêtre est affichée en splash au chargement d'un dico,
    elle n'apparaît que si elle contient quelque chose
    
    Cette fenêtre est appelable par le menu Aide (commande dégrisée uniquement si
    la fenêtre contient quelque chose)
    """
    
    def __init__(self):
        
        u"""
        Mise en place de l'interface de la fenêtre d'affichage
        des infos et statut du dictionnaire
        """
        
        flagContent = False
        
        Toplevel.__init__(self, padx=5, pady=3)
        
        self.title("A propos de ce dictionnaire...")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+40)
        top = str(int(app.geometry().split('+')[2])+60)
        self.geometry('+' + left + '+' + top)
        
        wtxt=65
        
        # --- Nom convivial du dictionnaire
        if app.dic.dicName != "":
            fr_ttr=Frame(self)
            fr_ttr.grid()
            Label(fr_ttr,image=app.ic1).grid(row=0, column=0)
            Label(fr_ttr, text=app.dic.dicName, font=("Helvetica", 12, "bold"), background='black', foreground='white').grid(row=0, column=1, padx=10, ipadx=8)
            Label(fr_ttr,image=app.ic2).grid(row=0, column=2)
            flagContent = True
        
        # --- Date et version du Dictionnaire
        t = ""
        if app.dic.dicVersionNumber !="":
            pass
            t = "Version " + app.dic.dicVersionNumber  + "  "
        if app.dic.versionDate != "":
            t = t + app.dic.versionDate
        if t != "":
            Label(self, text=t, font=("Helvetica", 8, "normal")).grid() 
            flagContent = True
            
        # --- Information du dictionnaire
        if app.dic.showDicInfo and app.dic.dicInfo != "":
            Label(self, text="Informations :", font=("Helvetica", 9, "bold")).grid(sticky="W")
            txt_info=ScrolledText(self, width=wtxt, height=7, background='white', font=("Helvetica", 8, "normal"))
            txt_info.tag_config("normal", wrap=WORD, lmargin1=5, lmargin2=5, spacing1=5) 
            txt_info.insert(END, app.dic.dicInfo.replace("<br>", "\n"), "normal")
            txt_info.grid()
            flagContent = True
        
        # --- Statut et Licence
        if app.dic.showDicStatus and app.dic.dicStatus != "":
            Label(self, text="Statut et Licence :", font=("Helvetica", 9, "bold")).grid(sticky="W")
            txt_status=ScrolledText(self, width=wtxt, height=7, background='white', font=("Helvetica", 8, "normal"))
            txt_status.tag_config("normal", wrap=WORD, lmargin1=5, lmargin2=5, spacing1=5)
            txt_status.insert(END, app.dic.dicStatus.replace("<br>", "\n"), "normal")
            txt_status.grid()
            flagContent = True
        
        # --- Auteur(s) et Copyright
        t = "\n".join(app.dic.mainAuthors)
        if t != "":
            Label(self, text="Auteur(s) :", font=("Helvetica", 9, "bold")).grid(sticky="W")
            Message(self, text=t, font=("Helvetica", 9, "normal"), width=500).grid(sticky="W")
            flagContent = True
        
        t = "\n".join(app.dic.altAuthors)
        if t != "":
            Label(self, text="Autres auteur(s) :", font=("Helvetica", 8, "bold")).grid(sticky="W")
            Label(self, text=t).grid(sticky="W")
            flagContent = True
        
        if app.dic.contactAuthor != "":
            Label(self, text="Contact : " + app.dic.contactAuthor, font=("Helvetica", 8, "normal")).grid(sticky="WE")
            flagContent = True
            
        # --- Biblio
        if app.dic.showBiblio and len(app.dic.biblio) > 0:
            Label(self, text="Références bibliographiques :", font=("Helvetica", 9, "bold")).grid(sticky="W")
            txt_biblio=ScrolledText(self, width=wtxt, height=6, background='white', font=("Helvetica", 8, "normal"))
            txt_biblio.tag_config("normal", wrap=WORD, lmargin1=5, lmargin2=5, spacing1=5)
            for r in  app.dic.biblio:
                txt_biblio.insert(END, r + "\n", "normal")
            txt_biblio.grid()
            flagContent = True
                
        # --- Bouton fermer
        
        Button(self, text="Ok", width=20, command=self.destroy).grid(pady=5)
        
        # --- Copyright
        if app.dic.copyright != "":
            t = "© " + app.dic.copyright
            Label(self, text=t, font=("Helvetica", 8, "normal")).grid(sticky="W")
            flagContent = True
        
        # Quitter immédiatement si rien à afficher
        if not flagContent :
            app.m_hlp.entryconfigure(5, state='disabled') # Désactiver le menu "à propos de ce dictionnaire"
            self.destroy()
        else:
            # Activer la commande du menu permettant de réafficher ultérieurement la fenêtre d'info
            app.m_hlp.entryconfigure(5, state='normal')
        
        return



class WindowOptions(Toplevel):
    u"""
    Fenêtre d'options et paramètres
    """

    def __init__(self):
        u"""
        Mise en place de l'interface de la fenêtre des options
        """
        
        Toplevel.__init__(self, padx=10, pady=10)
        
        self.title("Préférences")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        
        wOngl = 160
        hOngl = 20
        hPan = 20 # nb caractères
        wPan = 100 # nb caractères
        
        # ------------------------------------
        # --- Frame des Boutons/faux-onglets  
        # ------------------------------------
        
        
        r=0
        fr_ongl = Frame(self)
        fr_ongl.grid(row=r, sticky="WENS")
        
        
        ongl_0 = Button(fr_ongl, text=' Polices', image=app.imglst.puceS, compound=LEFT, height=hOngl, width=wOngl, borderwidth=0, foreground='black', background='white')
        ongl_0.pack(side=LEFT, anchor=W, padx=4)
        
        ongl_1 = Button(fr_ongl, text=' Chemins', image=app.imglst.puceX, compound=LEFT, height=hOngl, width=wOngl, borderwidth=0, foreground='white', background='#B4B4B4')
        ongl_1.pack(side=LEFT, padx=4)
        
        ongl_2 = Button(fr_ongl, text=' Divers', image=app.imglst.puceX, compound=LEFT, height=hOngl, width=wOngl, borderwidth=0, foreground='white', background='#B4B4B4')
        ongl_2.pack(side=LEFT, padx=4)
        
        # Dimensionneur de l'espace des frames
        
        Label(self, text="", width=wPan, height=hPan).grid(row=1)
        
        
        # ========================================
        # === Frame des polices (onglet 0)        
        # ========================================
        
        r=1
        fr_font = Frame(self, borderwidth=3, relief='ridge')
        fr_font.grid(row=r, ipadx=20, ipady=20, sticky="WENS")
        
        
        # --------------------------------------
        # --- Police de la liste de dico        
        # --------------------------------------
        
        r=0
        Label(fr_font, text="Police actuelle des dictionnaires :").grid(row=r, column=0, sticky=E)
        Label(fr_font, text=app.lst_dico['font']).grid(row=r, column=1, sticky=W)
        
        Label(fr_font, text="Nouvelle Police des dictionnaires :").grid(row=r+1, column=0, sticky=E)
        self.txt_dicofontName=Entry(fr_font, width=30, background='white')
        self.txt_dicofontName.grid(row=r+1, column=1)
        
        Label(fr_font, text="Taille :").grid(row=r+1, column=3)
        self.txt_dicofontSize=Entry(fr_font, width=3, background='white')
        self.txt_dicofontSize.grid(row=r+1, column=4, padx=6)
        
        self.lab_exdicofont=Label(fr_font, text="0123456789\nABCDEFabcdef", borderwidth=1, background='white', relief='solid', padx=10, pady=5)
        self.lab_exdicofont.grid(row=r+2, column=0)
        
        # --- Afficher le nom/taille de la police courante
        
        f = app.lst_dico['font'].split("}")
        
        if len(f) >1:
            sz = f[1].split()[0]
            f = f[0].strip("{}")
        else:
            f = f[0].split()
            sz = f[1]
            f = f[0]
        
        self.txt_dicofontName.insert(END, f)
        self.txt_dicofontSize.insert(END, sz)
        
        # --- Appliquer la police courante à la case-exemple
        
        self.lab_exdicofont['font'] = app.lst_dico['font']
        
        
        # --- Liste des polices
        
        r=2
        self.lst_font1=Listbox(fr_font,height=5, width=30)
        self.lst_font1.grid(row=r, column=1)
        scr1=Scrollbar(fr_font, command=self.lst_font1.yview)
        scr1.grid(row=r, column=2, sticky=NS)
        self.lst_font1['yscrollcommand'] = scr1.set
        self.lst_font1['font']=self.txt_dicofontName['font']
        
        fntlst=list(tkFont.families())
        fntlst.sort()
        for t in fntlst: self.lst_font1.insert(END, t)
        self.lst_font1.bind('<ButtonRelease-1>', lambda event : self.fontSel(1))
        
        r=3
        Label(fr_font, text="", height=1).grid(row=r, columnspan=10) # espaceur
        
        
        # ----------------------------------------------
        # --- Police de la case de la notice            
        # ----------------------------------------------
        
        r=4
        Label(fr_font, text="Police actuelle des traductions :").grid(row=r, column=0, sticky=E)
        Label(fr_font, text=app.txt_notice['font']).grid(row=r, column=1, sticky=W)
        
        Label(fr_font, text="Nouvelle Police des traductions :").grid(row=r+1, column=0, sticky=E)
        self.txt_traducfontName=Entry(fr_font, width=30, background='white')
        self.txt_traducfontName.grid(row=r+1, column=1)
        
        Label(fr_font, text="Taille :").grid(row=r+1, column=3)
        self.txt_traducfontSize=Entry(fr_font, width=3, background='white')
        self.txt_traducfontSize.grid(row=r+1, column=4, padx=6)
        
        self.lab_extraducfont=Label(fr_font, text="0123456789\nABCDEFabcdef", borderwidth=1, background='white', relief='solid', padx=10, pady=5)
        self.lab_extraducfont.grid(row=r+2, column=0)
        
        # --- Afficher le nom/taille de la police courante
        
        f = app.txt_notice['font'].split("}")
        
        if len(f) >1:
            sz = f[1].split()[0]
            f = f[0].strip("{}")
        else:
            f = f[0].split()
            sz = f[1]
            f = f[0]
        
        self.txt_traducfontName.insert(END, f)
        self.txt_traducfontSize.insert(END, sz)
        
        # --- Appliquer la police courante à la case-exemple
        
        self.lab_extraducfont['font'] = app.txt_notice['font']
        
        
        # --- Liste des polices
        
        r=6
        self.lst_font2=Listbox(fr_font,height=5, width=30)
        self.lst_font2.grid(row=r, column=1)
        scr2=Scrollbar(fr_font, command=self.lst_font2.yview)
        scr2.grid(row=r, column=2, sticky=NS)
        self.lst_font2['yscrollcommand'] = scr2.set
        for t in fntlst: self.lst_font2.insert(END, t)
        self.lst_font2.bind('<ButtonRelease-1>', lambda event : self.fontSel(2))
        self.lst_font2['font']=self.lst_font1['font']
        
        # Bouton "Appliquer les polices par défaut"
        Button(fr_font, text="Polices par défaut", command=self.setDefFonts).grid(row=7, ipadx=3)
        
        
        # ---------------------------------
        # --- Frame des chemins (onglet 1) 
        # ---------------------------------
        
        r=1
        fr_dir = Frame(self, borderwidth=3, relief='ridge')
        fr_dir.grid(row=r, ipadx=20, ipady=20, sticky="WENS")
        
        r=0
        Label(fr_dir, text="", width=wPan, height=2).grid(row=r, columnspan=2) # dimensionneur
        
        r=1
        Label(fr_dir, text="Chemin par défaut de recherche des dictionnaires :").grid(row=r, column=0, sticky='W')
        self.txt_pathDic = Entry(fr_dir, width=70, background='white')
        dp = app.ini.get('DicDefaultPath','')
        try   : dp = dp.decode('utf-8').encode('latin-1')     # L'ini retourne le chemin en utf-8
        except: pass
        if os.path.isdir(dp): self.txt_pathDic.insert(END, dp)
        self.txt_pathDic.grid(row=r+1, column=0, sticky='E')
        Button(fr_dir, text="Parcourir...", command= lambda: browseFolder(self, self.txt_pathDic)).grid(row=r+1, column=1, sticky='W')
        
        Label(fr_dir, text=" ").grid(row=3)
        
        r=4
        Label(fr_dir, text="Chemin du dossier lié au menu \"Dictionnaires\"\n(Tous les dictionnaires contenus dans ce dossier seront affichés dans le menu) :", justify=LEFT).grid(row=r, column=0, sticky='W')
        self.txt_pathMenuDic = Entry(fr_dir, width=70, background='white')
        dp = app.ini.get('DicDefaultFolder','')
        try   : dp = dp.decode('utf-8').encode('latin-1')     # L'ini retourne le chemin en utf-8
        except: pass
        if os.path.isdir(dp): self.txt_pathMenuDic.insert(END, dp)
        self.txt_pathMenuDic.grid(row=r+1, column=0, sticky='E')
        Button(fr_dir, text="Parcourir...", command= lambda: browseFolder(self, self.txt_pathMenuDic)).grid(row=r+1, column=1, sticky='W')
        
        
        fr_dir.grid_remove()
        
        
        # --------------------------------
        # --- Frame des divers (onglet 2) 
        # --------------------------------
        
        fr_div = Frame(self, borderwidth=3, relief='ridge', padx=10)
        fr_div.grid(row=1, ipadx=20, ipady=20, sticky="WENS")
        
        # ---
        
        Label(fr_div, text="Actions au démarrage :", font="Helvetica 10 bold", height=2, justify=LEFT).pack(anchor='w')
        
        self.vChkNewver = IntVar()
        Checkbutton(fr_div, text="Vérifier la disponibilité d'une nouvelle version.", variable=self.vChkNewver).pack(anchor='w')
        self.vChkNewver.set(int(app.ini.get("CheckNewVersion","1")))
        
        self.vOpenLastFile = IntVar()
        Checkbutton(fr_div, text="Rouvrir le dernier fichier ouvert.", variable=self.vOpenLastFile).pack(anchor='w')
        self.vOpenLastFile.set(int(app.ini.get("OpenLastFile","1")))
        
        self.vOpenLastPlugin = IntVar()
        Checkbutton(fr_div, text="Rouvrir le dernier greffon ouvert.", variable=self.vOpenLastPlugin).pack(anchor='w')
        self.vOpenLastPlugin.set(int(app.ini.get("OpenLastPlugin","1")))
        
        # ---
        
        fr_div.grid_remove()
        
        
        # ------------------------------------------
        # --- Frame des Boutons de pied de page     
        # ------------------------------------------
        r = 2
        fr_butOpt=Frame(self)
        fr_butOpt.grid(row=r, column=0, columnspan=4, pady=8, sticky="WENS")
        
        Label(fr_butOpt, text="", width=wPan).grid(row=0, columnspan=3) # dimensionneur
        Button(fr_butOpt, image=app.imglst.ok, compound=LEFT, text="  Ok", width=100, padx= 10, command=self.ValiderOptions).grid(row=0, column=0, pady=8, padx=10)
        Button(fr_butOpt, image=app.imglst.exit, compound=LEFT, text="  Fermer", width=100, padx= 10, command=self.destroy).grid(row=0, column=1, pady=8, padx=10)
        Button(fr_butOpt, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=100, padx= 10,command=lambda: app.showHelp("pref")).grid(row=0, column=2, pady=8, padx=10)
        
        
        # --------------------------
        # --- Gestion des onglets   
        # --------------------------
        listOngl = [ (ongl_0, fr_font), (ongl_1, fr_dir), (ongl_2, fr_div )]
        ongl_0['command'] = lambda: onglClick(0, listOngl, app.imglst, self, 'g')
        ongl_1['command'] = lambda: onglClick(1, listOngl, app.imglst, self, 'g')
        ongl_2['command'] = lambda: onglClick(2, listOngl, app.imglst, self, 'g')
        
        return


    
    def setDefFonts(self):
        u"""
        Affecte les polices par défaut dans les cases
        """
        
        self.txt_dicofontName.delete(0,END)
        self.txt_dicofontName.insert(END, "<defaut>")
        self.txt_dicofontSize.delete(0,END)
        self.txt_dicofontSize.insert(END, "9")
        
        # Appliquer la police à la case-exemple
        self.lab_exdicofont['font'] = (self.txt_dicofontName.get(), int(self.txt_dicofontSize.get()))
        
        # ---
        
        self.txt_traducfontName.delete(0,END)
        self.txt_traducfontName.insert(END, "<defaut>")
        self.txt_traducfontSize.delete(0,END)
        self.txt_traducfontSize.insert(END, "9")
        
        # Appliquer la police à la case-exemple
        self.lab_extraducfont['font'] = (self.txt_traducfontName.get(), int(self.txt_traducfontSize.get()))
        
        
        return


    def fontSel(self, n):
        u"""
        Clic sur un nom de police dans la liste
        Affiche le nom de police et met à jour la case-exemple
        """
        
        if n == 1:
            self.txt_dicofontName.delete(0,END)
            self.txt_dicofontName.insert(END,self.lst_font1.get(self.lst_font1.curselection()[0]))
            
            # Appliquer la police à la case-exemple
            self.lab_exdicofont['font'] = (self.txt_dicofontName.get(), int(self.txt_dicofontSize.get()))
        
        else:
            self.txt_traducfontName.delete(0,END)
            self.txt_traducfontName.insert(END,self.lst_font2.get(self.lst_font2.curselection()[0]))
            
            # Appliquer la police à la case-exemple
            self.lab_extraducfont['font'] = (self.txt_traducfontName.get(), int(self.txt_traducfontSize.get()))
        
        return


    def ValiderOptions(self):
        u"""
        Valider les options et quitter la fenêtre d'options
        """
        
        # ===================
        # === Polices        
        # ===================
        
        # Récupérer les noms de police en cours
        nmDico = self.txt_dicofontName.get()
        nmTraduc = self.txt_traducfontName.get()
        if nmDico == ""  : nmDico='<defaut>'
        if nmTraduc == "": nmTraduc='<defaut>'
        
        # Récupérer les tailles de police en cours
        try   : szDico=int(self.txt_dicofontSize.get())
        except: szDico=9
        try   : szTraduc=int(self.txt_traducfontSize.get())
        except: szTraduc=9
        
        # Regrouper le tout dans des chaînes uniques "splittables"
        strFontDico = nmDico + "?" + str(szDico) + "?normal"
        strFontTraduc = nmTraduc + "?" + str(szTraduc) + "?normal"
        
        app.appliquerFont(strFontDico, strFontTraduc)
        
        
        # ======================
        # === Chemins           
        # ======================
        
        app.ini.write('DicDefaultPath', self.txt_pathDic.get())
        app.ini.write('DicDefaultFolder', self.txt_pathMenuDic.get())
        
        app.updateMenuDic()
        
        
        # =======================
        # === Divers             
        # =======================
        
        if self.vChkNewver.get():
            app.ini.write('CheckNewVersion', '1')
        else:
            app.ini.write('CheckNewVersion', '0')
        
        if self.vOpenLastFile.get():
            app.ini.write('OpenLastFile', '1')
        else:
            app.ini.write('OpenLastFile', '0')
        
        if self.vOpenLastPlugin.get():
            app.ini.write('OpenLastPlugin', '1')
        else:
            app.ini.write('OpenLastPlugin', '0')
        
        # ===
        
        self.destroy()
        
        return




class WindowNewDic(Toplevel):
    u"""
    Fenêtre d'ouverture d'un nouveau dictionnaire vide
    """

    def __init__(self):
        
        Toplevel.__init__(self)
        
        self.title("Créer un nouveau dictionnaire")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        ww = 80
        
        # -------------------------------
        # --- Frame du type de fichier   
        # -------------------------------
        
        fr_type=LabelFrame(self, text="Format du nouveau dictionnaire :")
        fr_type.grid(row=0, column=0, columnspan=10)
        
        Label(fr_type,width=ww).grid(row=0, column=0) #dimensionneur
        
        self.vFileType = StringVar()
        Radiobutton(fr_type, text="Ling", variable=self.vFileType, value="ling").grid(row=0,column=0, padx=5, sticky='W')
        Radiobutton(fr_type, text="Dict", variable=self.vFileType, value="dict").grid(row=1,column=0, padx=5, sticky='W')
        Radiobutton(fr_type, text="XDXF", variable=self.vFileType, value="xdxf").grid(row=2,column=0, padx=5, sticky='W')
        Radiobutton(fr_type, text="WB", variable=self.vFileType, value="wb").grid(row=3,column=0, padx=5, sticky='W')
        Radiobutton(fr_type, text="INI", variable=self.vFileType, value="ini").grid(row=4,column=0, padx=5, sticky='W')
        Radiobutton(fr_type, text="CSV (fichier texte tabulé)", variable=self.vFileType, value="csv").grid(row=5,column=0, padx=5, sticky='W')
        self.vFileType.set('ling')
        
        
        # ---------------------------
        # --- Frame du répertoire    
        # ---------------------------
        
        fr_rep=LabelFrame(self, text="Dossier du nouveau dictionnaire :")
        fr_rep.grid(row=1, column=0, columnspan=10, padx=4, pady=2)
        
        Label(fr_rep, width=ww, height=2).grid(row=0, column=0, columnspan=10) #dimensionneur
        
        self.txt_folder=Entry(fr_rep, width=60, background="white")
        self.txt_folder.grid(row=0, column=0, padx=5, sticky='WN', pady=5)
        
        self.bt_browse=Button(fr_rep,text=' Parcourir... ', command= lambda: browseFolder(self, self.txt_folder))
        self.bt_browse.grid(row=0, column=1, sticky="WN", pady=4)
        
        
        self.vChkDicFolder = IntVar()
        ChkDicFolder=Checkbutton(fr_rep, text=u'Créer dans le dossier des dictionnaires', variable=self.vChkDicFolder, command=self.onclicChkDicFolder)
        ChkDicFolder.grid(row=1, column=0, sticky='WN')
        
        
        # Présélectionner le répertoire
        dp = app.ini.get('DicDefaultFolder','')
        if dp != "":
            ChkDicFolder.select()
            self.txt_folder.insert(END, dp)
            self.txt_folder['state']='disabled'
            self.bt_browse['state']='disabled'
        else:
            ChkDicFolder['state']='disabled'
        
        
        # ------------------------------
        # --- Frame du nom de fichier   
        # ------------------------------
        
        fr_name=LabelFrame(self, text="Nom du fichier du nouveau dictionnaire :")
        fr_name.grid(row=2, column=0, columnspan=10, padx=4, pady=2)
        
        Label(fr_name,width=ww, height=4).grid(row=0, column=0) #dimensionneur
        
        self.txt_name=Entry(fr_name, width=30, background="white")
        self.txt_name.grid(row=0, column=0)
        self.txt_name.insert(END, "newdic")
        
        
        # -------------------------
        # --- Frame des Boutons    
        # -------------------------
        fr_but=Frame(self)
        fr_but.grid(row=3, column=0, columnspan=10, padx=4, pady=2)
        
        Button(fr_but, image=app.imglst.ok, compound=LEFT, text="  Créer", width=100, padx= 10, command=self.makeNewDic).grid(row=0, column=0, pady=8, padx=10)
        Button(fr_but, image=app.imglst.exit, compound=LEFT, text="  Fermer", width=100, padx= 10, command=self.destroy).grid(row=0, column=1, pady=8, padx=10)
        Button(fr_but, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=100, padx= 10,command=lambda: app.showHelp("newdic")).grid(row=0, column=2, pady=8, padx=10)
        
        return


    def onclicChkDicFolder(self):
        u"""
        Clic sur la case 'utiliser le dossier des dictionnaires'
        """
        
        if self.vChkDicFolder.get() == 1:
            self.txt_folder.delete(0, END)
            self.txt_folder.insert(END, app.ini.get('DicDefaultFolder',''))
            self.txt_folder['state']='disabled'
            self.bt_browse['state']='disabled'
            self.bt_browse['state']='disabled'
        else:
            self.txt_folder['state']='normal'
            self.bt_browse['state']='normal'
            self.bt_browse['state']='normal'
        
        
        return


    def makeNewDic(self):
        u"""
        Créer un nouveau dictionnaire vide
        """
        
        newDicType = self.vFileType.get()
        
        # -----------------------------------------------------------------
        # --- Confectionner le chemin du fichier du nouveau dictionnaire   
        # -----------------------------------------------------------------
        
        if newDicType == 'xdxf':
            
            repxdxf = os.path.join(self.txt_folder.get(),self.txt_name.get())
            fichpath = os.path.join( repxdxf, "dict.xdxf")
            
        else :
            
            fichpath = os.path.join(self.txt_folder.get(),self.txt_name.get()) + "." + newDicType
        
        # Vérifier qu'aucun fichier n'existe sous ce nom
        if os.path.isfile(fichpath):
            tkMessageBox.showwarning(parent=self,
                    title=u"Echec de la création du dictionnaire",
                    message=u"Un fichier existe déjà sous ce nom !")
            return
        
        # ------------------------------------------------------
        # --- Initialiser le nouveau dictionnaire               
        # ------------------------------------------------------
        
        cp1 = ""
        cp2 = ""
        if newDicType == 'ling': # -----------------------------
            
            strFich = DicObject.strPrelingToStrLing(strPreling="%preling")
            
            # Echec > avertir et quitter
            if strFich.startswith("!"):
                tkMessageBox.showwarning(parent=self,
                    title=u"Echec de la création du dictionnaire",
                    message=strLing)
                return
            
        elif newDicType == 'dict': # -----------------------------
            
            radicfich = os.path.splitext(fichpath)[0]
            
            # le *.dict est enregistré dans la section commune plus bas
            strFich=""
            
            # Enregistrer le *.idx
            ff = open(radicfich + ".idx",'wb')
            ff.write("")
            ff.close()
            
            # Enregistrer le *.ifo
            t ="StarDict's dict ifo file\nversion=2.4.2\n[options]\nbookname=" + os.path.basename(radicfich) +\
            "\nwordcount=0" +\
            "\nidxfilesize=" + str(os.path.getsize(radicfich + ".idx")) +\
            "\nsynwordcount=\nsametypesequence=m\nidxoffsetbits=32\nauthor=\nemail=\nwebsite=\ndescription=\ndate=\n"
            
            try:
                t = t.encode('utf-8')
            except:
                tkMessageBox.showwarning(parent=self,
                title = u"Echec de la création du nouveau dictionnaire",
                message = "Erreurs d'encodage utf-8 dans le fichier *.ifo")
                return
            
            ff = open(radicfich + ".ifo",'wb')
            ff.write(t)
            ff.close()
            
        elif newDicType == 'xdxf': # -----------------------------
            
            if not os.path.isdir(repxdxf):
                os.mkdir(repxdxf) # Créer le répertoire du dico
            
            strFich = '<?xml version="1.0" encoding="UTF-8" ?>\n<xdxf lang_from="" lang_to="" format="visual">\n<ar><k>Mot1</k>\nTraduction1</ar>\n</xdxf>\n'
            
        elif newDicType == 'wb': # -----------------------------
            
            strFich=""
            
        elif newDicType == 'csv': # -----------------------------
            
            strFich="Mot1\tTraduction1\nMot2\tTraduction2\n"
            
        elif newDicType == 'ini': # -----------------------------
            
            strFich="[Section1]\nClef1=Valeur1\nClef2=Valeur2\n[Section2]\nClef1=Valeur1\nClef2=Valeur2\n"
            cp1 = "latin-1"
            cp2 = "latin-1"
            
        else:
            print "Format non reconnu : " + newDicType
            
        
        # ---------------------------------------
        # --- Enregistre le nouveau fichier      
        # ---------------------------------------
        
        try   : app.dic.file.close()
        except: pass
        ff = open(fichpath, 'wb')
        ff.write(strFich)
        ff.close()
        
        # -------------------------------------------------------
        # -- Recharge le dictionnaire avec le nouveau fichier    
        # -------------------------------------------------------
        
        app.loadDico(fichpath, cp1=cp1, cp2=cp2, showSplash=False)
        
        self.destroy()
        
        return
    
    
    

class WindowEdit(Toplevel):
    u"""
    Fenêtre d'édition d'une entrée de dictionnaire
    """

    def __init__(self, i):
        u"""
        Mise en place de la fenêtre d'édition d'une entrée
        """
        
        if app.dic.type in ("dictdz", "dictgz"):
            app.hideProgr()
            tkMessageBox.showinfo(parent=self,
                    title=u"Editer le dictionnaire",
                    message=u"Ce dictionnaire est au format DICT compressé, il ne peut être modifié.\n\nPour l'éditer, vous devrez d'abord le convertir au format DICT non compressé\nou tout autre format géré par Linguae (Menu 'Fichier' > 'Exporter').")
            return
        
        # ---
        
        
        self.mysound = None # son Snack
        
        wTxt=110
        
        Toplevel.__init__(self, padx=3)
        
        if i == -1 :
            self.title("Ajouter un nouvel élément de dictionnaire " + app.dic.type.upper())
        else:
            self.title("Editer un élément de dictionnaire " + app.dic.type.upper())
        
        self.resizable(width=NO, height=NO)
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # --- Frame de tête
        
        fr_head=Frame(self)
        fr_head.grid(row=0, column=0, columnspan=3, sticky='W')
        
        lab_chp1 = Label(fr_head) # le texte est complété plus bas
        lab_chp1.grid(row=0, column=0, sticky='W')
        
        self.lab_idx=Label(fr_head, text=" "*20 + "Index : " + str(i)+ " "*20, borderwidth=2) # Affichage de l'index
        self.lab_idx.grid(row=0, column=1)
        
        Button(fr_head, compound=LEFT, text=" Coller", image=app.imglst.paste, command=self.__pasteToEdit).grid(row=0, column=2, ipadx=3)
        
        
        # --- Case de l'entrée
        
        self.txt_chp1=Entry(self, background='white')
        self.txt_chp1.grid(row=1, column=0, columnspan=3, sticky='W')
        self.txt_chp1['font']=app.lst_dico['font']
        
        if app.dic.type.startswith('wb'):
          lab_chp1['text']="Entrée de dictionnaire (31 caractères maximum) :"
          self.txt_chp1['width']=31
        else:
          lab_chp1['text']="Entrée de dictionnaire :"
          self.txt_chp1['width']=wTxt
        
        
        # --- Case des traductions courtes
        
        lab_editShort=Label(self, justify=LEFT)
        lab_editShort.grid(row=2, column=0, columnspan=10, sticky='W')
        
        self.txt_editShort=ScrolledText(self, height=1, background='white')
        self.txt_editShort.grid(row=3, column=0, columnspan=10, sticky='W')
        self.txt_editShort['font']=app.txt_notice['font']
        
        if app.dic.type.startswith('wb'):
            lab_editShort['text']="Traductions/définitions courtes (53 car. maxi au total, 31 car. chacune et séparées par ' , ' )\n(seront utilisées pour inverser le dictionnaire) :"
            self.txt_editShort['width']=53
        else:
            lab_editShort['text']="Traductions/définitions courtes (séparées par ' ; ' )  (seront utilisées pour inverser le dictionnaire) :"
            self.txt_editShort['width']=wTxt-3
            
            if app.dic.type[:4] in ("dict", "xdxf"):
                self.txt_editShort['height'] = 10
        
        if app.dic.type[:4] in ("ling", "xdxf"):
            
            # Case multiligne des infos
            Label(self, text='Infos ou traductions/définitions longues :').grid(column=0, sticky='W')
            self.txt_editLong=ScrolledText(self, width=wTxt-3, height=5, background='white')
            self.txt_editLong.grid(column=0, columnspan=10, sticky='W')
            self.txt_editLong['font']=app.txt_notice['font']
        
        if app.dic.type[:4] in ("ling"):
            
            # Frame de l'ID (identifiant)
            fr_ID = Frame (self, pady=2)
            fr_ID.grid(column=0, columnspan=10, sticky="W")
            Label(fr_ID, text="ID :").grid(row=0, column=0)
            self.txt_ID=Entry(fr_ID, width=10, background='white')
            self.txt_ID.grid(row=0, column=1)
            Label(fr_ID, text="(identifiant facultatif, unique dans le dictionnaire)").grid(row=0, column=2)
            self.bt_unlockID=Button(fr_ID, text="Déverrouiller l'ID", width=30, command=self.__unlockID)
            self.bt_unlockID.grid(row=0, column=3, padx=10)
            self.bt_makeID=Button(fr_ID, text="Ajouter un ID automatique",width=30, command=self.__makeID)
            self.bt_makeID.grid(row=0, column=3, padx=10)
            
            
            # Case des racines
            Label(self, text="IDs des Racines (séparés par ' ; ' ) :").grid(column=0, sticky='W')
            self.txt_rac=Entry(self, width=wTxt, background='white', font="courier 9")
            self.txt_rac.grid(column=0, columnspan=10, sticky='W')
            
            # Case des synonymes
            Label(self, text="IDs des Synonymes (séparés par ' ; ' ) :").grid(column=0, sticky='W')
            self.txt_syn=Entry(self, width=wTxt, background='white', font="courier 9")
            self.txt_syn.grid(column=0, columnspan=10, sticky='W')
            
            # Case des antonymes
            Label(self, text="IDs des Antonymes (séparés par ' ; ' ) :").grid(column=0, sticky='W')
            self.txt_ant=Entry(self, width=wTxt, background='white', font="courier 9")
            self.txt_ant.grid(column=0, columnspan=10, sticky='W')
            
            # Case des "voir aussi"
            Label(self, text="IDs des \" Voir-aussi \" (séparés par ' ; ' ) :" ).grid(column=0, sticky='W')
            self.txt_see=Entry(self, width=wTxt, background='white', font="courier 9")
            self.txt_see.grid(column=0, columnspan=10, sticky='W')
            
            # Case des tokens
            Label(self, text="Symboles et commandes : (séparés par ' ; ' ) :" ).grid(column=0, sticky='W')
            self.txt_tok=Entry(self, width=wTxt, background='white', font="courier 9")
            self.txt_tok.grid(column=0, columnspan=10, sticky='W')
        
        if app.dic.type[:4] in ("ling", "xdxf"):
            
            # Case de la phonétique
            Label(self, text="Phonétique de l'entrée :" ).grid(column=0, sticky='W')
            self.txt_phon=Entry(self, width=wTxt-15, background='white')
            self.txt_phon.grid(column=0, columnspan=10, sticky='W')
            self.txt_phon['font']=app.txt_notice['font']
        
        if app.dic.type[:4] in ("ling"):
            
            # Frame de l'enregistreur
            fr_rec = Frame (self, padx=5, pady=3, borderwidth=3, relief=SUNKEN, background='black')
            fr_rec.grid(row=100,column=0, sticky='W', padx=3, pady=3)
            Label(fr_rec, text="Enregistreur de la prononciation ", font="Helvetica 8 bold", foreground='white', background='black').grid(row=0, column=0)
            self.bt_rec=Button(fr_rec, image=app.imglst.rec, compound=LEFT, text=" Rec ", command=self.__recWord)
            self.bt_rec.grid(row=0, column=1)
            Button(fr_rec, image=app.imglst.stop, compound=LEFT, text=" Stop ", command=self.__soundStop).grid(row=0, column=2)
            self.bt_play=Button(fr_rec, image=app.imglst.audio, compound=LEFT,text=" Play ", command=self.__playWord)
            self.bt_play.grid(row=0, column=3)
            self.lab_recon=Label(fr_rec, text=" ", width=1, background='black')
            self.lab_recon.grid(row=0, column=4, padx=10)
            self.lab_time=Label(fr_rec,text="0.00", foreground='white', background='black', font='courier 9')
            self.lab_time.grid(row=0, column=5)
            
            Label(fr_rec, text="Périphérique d'enregistrement :", foreground='white', background='black').grid(row=1, column=0, sticky='W')
            self.vAudioinput = StringVar(self)
            opt=[]
            opt.append("0: ( par défaut )")
            n = 0
            for d in tkSnack.audio.inputDevices():
                n += 1 ; opt.append(str(n) +": "+d)
            opt = tuple(opt)
            self.opt_audioinput=apply(OptionMenu, (fr_rec, self.vAudioinput) + opt)
            t = app.ini.get("audioInputDevice","")
            if t == "":
                self.vAudioinput.set(opt[0])
            else:
                self.vAudioinput.set(t)
            self.opt_audioinput.grid(row=1,column=1, columnspan=10, pady=5, sticky='W')
            
            Button(fr_rec, text=" Importer ", command=self.__importAudio).grid(row=1, column=3, columnspan=10)
            
            
            # Frame des images
            fr_img = LabelFrame (self, text="Illustrations (gif)", padx=5, pady=3, borderwidth=2, relief=GROOVE)
            fr_img.grid(row=100,column=1, columnspan=10, padx=3, pady=3, sticky='W')
            self.lst_img=lib.myscrlst.ScrolledList(fr_img, height=4, ondblclic=self.__seeImg)
            self.lst_img.grid(row=0, rowspan=10,column=0, padx=3, pady=3)
            Button(fr_img, text="Importer", width=11, command=self.__loadImg).grid(row=0, column=1)
            Button(fr_img, text="Supprimer", width=11, command=self.__delImg).grid(row=1, column=1)
            Button(fr_img, text="Voir", width=11, command=self.__seeImg).grid(row=2, column=1)
            
            
            
        # --- Frame des boutons
        fr_but=Frame(self, pady=10)
        fr_but.grid(column=0, columnspan=10)
        Button(fr_but, image=app.imglst.ok, compound=LEFT, text='  Ok', width=100, command=self.doEdit).grid(row=0, column=0, padx=25)
        Button(fr_but, image=app.imglst.exit, compound=LEFT, text='  Annuler', width=100, command=self.cancelEdit).grid(row=0, column=1, padx=25)
        Button(fr_but, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=100, padx= 10,command=lambda: app.showHelp("editnotice")).grid(row=0, column=2, pady=8, padx=10)
        
        
        # ------------------------------------
        # --- Remplir les cases d'édition     
        # ------------------------------------
        
        if i == -1:     # Ajout d'une nouvelle entrée
            txt=u"( ... )"
            txt = txt.encode(app.dic.encoding1)
            self.txt_chp1.insert(END, txt)
            
            txt=u"( ... )"
            txt = txt.encode(app.dic.encoding2)
            self.txt_editShort.insert(END, txt)
        
        else :
            
            # --- Récupérer toutes les données (>pyDictionnaire)
            data = app.dic.getdata(i)
            
            # --- Entrée du dictionnaire
            self.txt_chp1.insert(END, data['mot']) 
            
            # --- Traductions courtes
            t = data['short']                                 
            
            if app.dic.type.startswith('xdxf'):
                # Oter les balises d'article
                t = t.replace("<ar>","")
                t = t.replace("</ar>","")
            
            self.txt_editShort.insert(END, t)
            
            if t.startswith("[("):
                self.txt_editShort["background"]=self["background"]
                self.txt_editShort["state"]="disabled"
            
            # --- Traduction longue
            try   : self.txt_editLong.insert(END, data['long'])   
            except: pass
            
            # --- ID
            id = data['ID']
            if id != "":
                self.txt_ID.insert(END, id)
                
                # verrouiller l'ID              
                self.txt_ID['state'] = 'disabled'
                self.txt_ID['background'] = self['background']
                self.bt_makeID.grid_remove()
            
            # --- Racines
            try   : self.txt_rac.insert(END, data['rac'])        
            except: pass
            
            # --- Synonymes
            try:self.txt_syn.insert(END, data['syn'])        
            except:pass
            
            # --- Antonymes
            try   : self.txt_ant.insert(END, data['ant'])        
            except: pass
            
            # --- Voir-aussi
            try   : self.txt_see.insert(END, data['see'])        
            except: pass
            
            # --- Tokens
            try   : self.txt_tok.insert(END, data['tok'])        
            except: pass
            
            # --- Phonétique
            try   : self.txt_phon.insert(END, data['phon'])       
            except: pass
            
            # --- Charger la liste des images
            lf = glob.glob(os.path.join(app.dic.path + "_res", "gif", id + "_*.gif"))
            for f in lf:
                self.lst_img.add(os.path.basename(f))
        
        
        return


    def __unlockID(self):
        u"""
        Dévérrouiller l'ID ; permettre sa modification
        """
        
        # Avertir
        r = tkMessageBox.askyesno(parent=self,
            title=u"Dévérrouiller l'Identifiant de l'entrée",
            message=u"ATTENTION !\n\nToute modification secondaire de l'identifiant d'une entrée brise les pointages éventuels vers cette entrée (racines, synonymes...)\nainsi que ses liens vers les ressources sonores et graphiques associées.\n\nContinuer ?")
        if r == False : return
        
        
        self.txt_ID['state']='normal'
        self.txt_ID['background']='white'
        self.bt_unlockID['state']='disabled'
        
        
        return


    def __importAudio(self):
        u"""
        Importer un fichier mp3 comme ressource associée à une entrée de dictionnaire
        """
        
        id = self.txt_ID.get().strip()
        i = int(self.lab_idx['text'].split(":")[1]) # Index de l'entrée éditée
        
        # ----------------------------------
        # --- Contrôles préalables          
        # ----------------------------------
        
        # Contrôle de l'unicité de l'ID éventuel
        # pour éviter l'écrasement de fichiers audios
        if app.dic.IDs.has_key(id):
            if app.dic.IDs[id] != i:
                tkMessageBox.showinfo(parent=self,
                    title=u"Importer une image",
                    message=u"L'identifiant '" + id + "' existe déjà dans le dictionnaire.\n\nL'identifiant doit être unique dans le dictionnaire.")
                return
        
        if id == "":
            tkMessageBox.showinfo(parent=self,
                title=u"Importer un fichier audio",
                message=u"L'importation d'un fichier audio n'est possible que si le mot est muni d'un identifiant.")
            self.txt_ID.focus_set()
            return
        
        
        
        # ------------------------------------------------------
        # --- Choix du fichier ressource source                 
        # ------------------------------------------------------
        
        fichsourcepath = tkFileDialog.askopenfilename(parent=self,
                            title=u"Importer un fichier audio MP3",
                            filetypes=(('Fichiers MP3','*.mp3'),)
                            )
        
        if not fichsourcepath: return   # L'utilisateur a cliqué 'Annuler' dans la boîte de dialogue
        
        
        
        # -----------------------------------------------------------
        # --- Créer le répertoire de ressource s'il n'existe pas     
        # -----------------------------------------------------------
        
        repRes = os.path.join(app.dic.path + "_res","mp3")
        
        if not os.path.isdir(repRes):
            try:
                os.makedirs(repRes)
            except:
                tkMessageBox.showwarning(parent=self,
                    title=u"Importer un fichier audio",
                    message=u"Le sous-répertoire de ressource ne peut être créé,\nvérifiez les droits en écriture sur le répertoire du dictionnaire.")
                return
          
        
        # ------------------------------------------
        # --- Effacer l'éventuel fichier antérieur  
        # ------------------------------------------
        
        
        mp3 = os.path.join(repRes, id + ".mp3")     # Chemin du fichier
        
        try   : os.chmod(mp3, stat.S_IWRITE)        # Supprimer l'attribut lecture seule
        except: pass
        
        try   : os.remove(mp3)                      # effacer
        except: pass
        
        
        # ------------------------------------------------------
        # --- Enregistrer le fichier ressource                  
        # ------------------------------------------------------
        
        shutil.copy(fichsourcepath, mp3)
        
        
        return



    def __loadImg(self):
        u"""
        Importer une image gif comme ressource associée à une entrée de dictionnaire
        """
        
        
        id = self.txt_ID.get().strip()
        i = int(self.lab_idx['text'].split(":")[1]) # Index de l'entrée éditée
        
        # ----------------------------------
        # --- Contrôles préalables          
        # ----------------------------------
        
        # Contrôle de l'unicité de l'ID éventuel
        # pour éviter l'écrasement de fichiers images
        if app.dic.IDs.has_key(id):
            if app.dic.IDs[id] != i:
                tkMessageBox.showinfo(parent=self,
                    title=u"Importer une image",
                    message=u"L'identifiant '" + id + "' existe déjà dans le dictionnaire.\n\nL'identifiant doit être unique dans le dictionnaire.")
                return
        
        if id == "":
            tkMessageBox.showinfo(parent=self,
                title=u"Importer une image",
                message=u"L'importation d'images n'est possible que si le mot est muni d'un identifiant.")
            self.txt_ID.focus_set()
            return
        
        
        
        # ------------------------------------------------------
        # --- Choix du fichier ressource source                 
        # ------------------------------------------------------
        
        fichsourcepath = tkFileDialog.askopenfilename(parent=self,
                            title=u"Importer une image GIF",
                            filetypes=(
                            ('Fichiers GIF','*.gif'),
                            ('Tous les fichiers','*.*'))
                            )
        
        if not fichsourcepath: return   # L'utilisateur a cliqué 'Annuler' dans la boîte de dialogue
        
        
        
        # -----------------------------------------------------------
        # --- Créer le répertoire de ressource s'il n'existe pas     
        # -----------------------------------------------------------
        
        repRes = os.path.join(app.dic.path + "_res","gif")
        
        if not os.path.isdir(repRes):
            try:
                os.makedirs(repRes)
            except:
                tkMessageBox.showwarning(parent=self,
                    title=u"Importer une image",
                    message=u"Le sous-répertoire de ressource ne peut être créé,\nvérifiez les droits en écriture sur le répertoire du dictionnaire.")
                return
        
        
        # ------------------------------------------------------
        # --- Chemin du fichier ressource à enregistrer (cible) 
        # ------------------------------------------------------
        
        gifname = id + "_1.gif"
        gifpath = os.path.join(repRes, gifname) 
        
        n = 1
        while 1:
            if os.path.isfile(gifpath): # le fichier existe déjà
                n += 1
                gifname = id + "_" + str(n) + ".gif"
                gifpath = os.path.join(repRes, gifname)
            else:
                break    
        
        # ------------------------------------------------------
        # --- Enregistrer le fichier ressource                  
        # ------------------------------------------------------
        
        shutil.copy(fichsourcepath, gifpath)
        
        # Compléter la liste
        
        self.lst_img.add(gifname)
        
        
        return

    
    def __delImg(self):
        u""" Supprimer le fichier ressource image """
        
        fichname = self.lst_img.getselecteditem()
        
        if fichname == "":
            tkMessageBox.showwarning(parent=self,
                    title=u"Supprimer une image",
                    message=u"Aucune image n'est sélectionnée !")
            return
        
        
        fichpath = os.path.join(app.dic.path + "_res","gif", fichname)

    
    def __seeImg(self, i=None, widget=None):
        u"""
        Afficher la fenêtre de visualisation des images (Bouton 'Voir' ou dblClic)
        avec l'image sélectionnée
        """
        
        idximg = self.lst_img.getselectedindex()
        
        if idximg == -1:
            if self.lst_img.listcount()>0:
                idximg = 0
            else:
                tkMessageBox.showwarning(parent=self,
                        title=u"Voir l'image",
                        message=u"Aucune image n'est sélectionnée !")
                return
        
        
        self.f_img = WindowImage(id=self.txt_ID.get().strip(), idximg=idximg)
        
        return
        

    def __playWord(self):
        u"""
        Ecouter la prononciation du mot en édition
        """
        
        # Librairie audio Snack activée ?
        if not app.snackisactive:
            tkMessageBox.showwarning(parent=self,
                    title=u"Librairie audio absente",
                    message=u"Les fonctions audio de Linguae sont désactivées.\nVérifiez l'installation correcte de la librairie audio Snack.")
            return
        # ---
        
        id = self.txt_ID.get().strip()
        
        if id == "":
            tkMessageBox.showinfo(parent=self,
                title=u"Ecouter",
                message=u"L'enregistrement et l'écoute de la prononciation\nne sont possibles que si le mot est muni d'un identifiant.")
            self.txt_ID.focus_set()
            return
        
        
        # ---------------------------
        # --- Chemin du fichier      
        # ---------------------------
        
        mp3 = os.path.join(app.dic.path + "_res","mp3",id + ".mp3")
        
        # Vérifier si existe
        if not os.path.isfile(mp3):
            tkMessageBox.showinfo(parent=self,
                title=u"Impossible d'écouter la prononciation",
                message=u"Aucune prononciation n'est encore enregistrée.\n\nPressez 'Rec' pour l'enregistrer puis 'Stop' pour arrêter l'enregistrement.")
            return
        
        
        # -------------------
        # --- Ecouter        
        # -------------------
        
        try:
            #self.mysound = tkSnack.Sound(fileformat="MP3")
            self.mysound = tkSnack.Sound()
            self.mysound.read(mp3) 
            self.mysound.play()
        except:
            tkMessageBox.showwarning(parent=self,
                title=u"Ecouter",
                message=u"Echec de la lecture du fichier audio !")
            self.mysound.stop()
            return
        
        self.bt_play['state']='disabled'
        self.bt_rec['state']='disabled'
        self.lab_recon['background']='green'
        
        
        # ------------------------
        # --- Afficher le temps   
        # ------------------------
        
        lensound= float(self.mysound.length(unit="SECONDS"))
        while 1:
            playtime = tkSnack.audio.elapsedTime()
            self.lab_time['text']= str(playtime)[:4]
            self.lab_time.update()
            if playtime >= lensound: break
        
        self.bt_play['state']='normal'
        self.bt_rec['state']='normal'
        self.lab_recon['background']='black'
        self.update()
        
        return


    def __recWord(self):
        u"""
        Enregistrer la prononciation du mot en édition (clic sur le bouton)
        dans un fichier de ressource mp3
        """
        
        # Librairie audio Snack activée ?
        if not app.snackisactive:
            tkMessageBox.showwarning(parent=self,
                    title=u"Librairie audio absente",
                    message=u"Les fonctions audio de Linguae sont désactivées.\nVérifiez l'installation correcte de la librairie audio Snack.")
            return
        # ---
        
        id = self.txt_ID.get().strip()
        i = int(self.lab_idx['text'].split(":")[1]) # Index de l'entrée éditée
        
        # ----------------------------------
        # --- Contrôles préalables          
        # ----------------------------------
        
        # Contrôle de l'unicité de l'ID éventuel
        # pour éviter l'écrasement de fichiers sons
        if app.dic.IDs.has_key(id):
            if app.dic.IDs[id] != i:
                tkMessageBox.showinfo(parent=self,
                    title=u"Enregistrer la prononciation",
                    message=u"L'identifiant '" + id + "' existe déjà dans le dictionnaire.\n\nL'identifiant doit être unique dans le dictionnaire.")
                return
        
        if id == "":
            tkMessageBox.showinfo(parent=self,
                title=u"Enregistrer la prononciation",
                message=u"L'enregistrement et l'écoute de la prononciation\nne sont possibles que si le mot est muni d'un identifiant.")
            self.txt_ID.focus_set()
            return
        
        
        # --------------------------------------------
        # --- Chemin du fichier à enregistrer         
        # --------------------------------------------
        
        # Créer le répertoire de ressource s'il n'existe pas
        
        repRes = os.path.join(app.dic.path + "_res","mp3")
        
        if not os.path.isdir(repRes):
            try:
                os.makedirs(repRes)
            except:
                tkMessageBox.showwarning(parent=self,
                    title=u"Enregistrer la prononciation",
                    message=u"Le sous-répertoire de ressource ne peut être créé,\nvérifiez les droits en écriture sur le répertoire du dictionnaire.")
                return
        
        # Chemin du fichier
        mp3 = os.path.join(repRes,id + ".mp3")  
        
        
        # ------------------------------------------
        # --- Effacer l'éventuel fichier antérieur  
        # ------------------------------------------
        
        try   : os.chmod(mp3, stat.S_IWRITE)       # Supprimer l'attribut lecture seule
        except: pass
        
        try   : os.remove(mp3)                     # effacer
        except: pass
        
        
        # -----------------------------------------------------------------
        # --- Sélectionner le périphérique d'entrée pour l'enregistrement  
        # -----------------------------------------------------------------
        
        n = int(self.vAudioinput.get()[0])-1 # index du périphérique dans la liste retournée par inputDevices()
        if n > -1:
            try:
                tkSnack.audio.selectInput(tkSnack.audio.inputDevices()[n])
            except:
                tkMessageBox.showwarning(parent=self,
                    title=u"Périphérique d'enregistrement",
                    message=u"Echec de la sélection !\n\n Vous devez sélectionner le périphérique sur lequel est connecté votre microphone.")
                return
        
        
        # -------------------
        # --- Enregistrer    
        # -------------------
        
        try:
            self.mysound = tkSnack.Sound(fileformat="MP3", file=mp3,)
            self.mysound.record() 
        except:
            tkMessageBox.showwarning(parent=self,
                    title=u"Enregistrer",
                    message=u"Echec de l'enregistrement du fichier audio !")
            self.mysound.stop()
            return
        
        self.bt_play['state']='disabled'
        self.bt_rec['state']='disabled'
        self.lab_recon['background']='red'
        
        
        # -------------------------
        # --- Afficher le temps    
        # -------------------------
        
        while 1:
            if self.mysound is None: break # mis à None par le bouton 'Stop'
            self.lab_time['text']= str(tkSnack.audio.elapsedTime())[:4]
            self.lab_time.update()
            
        self.bt_play['state']='normal'
        self.bt_rec['state']='normal'
        self.lab_recon['background']='black'
        self.update() # Permet l'affichage du temps et l'action du bouton Stop
        
        return


    def __soundStop(self):
        
        try:
            self.mysound.stop() # Stoppe la lecture ou l'enregistrement
            #tkSnack.audio.stop()
            #self.mysound.destroy()
        except:
            pass
        
        # (NB: les objets suivants ne sont pas instanciés dans tous les formats, d'où les Try)
        try   : self.bt_play['state']='normal'
        except: pass
        
        try   : self.bt_rec['state']='normal'
        except: pass
        
        try   : self.lab_recon['background']='black'
        except: pass
        
        return


    def __pasteToEdit(self):
        u"""
        Copie le contenu du presse-papiers dans la case du focus
        """

        try:
            w = self.focus_displayof() # widget ayant le focus
        except:
            return
        
        try:    w.delete(SEL_FIRST, SEL_LAST)
        except: pass
        
        try:
            w.insert(INSERT, self.clipboard_get())
        except:
            return
        
        return


    def cancelEdit(self):
        
        self.__soundStop()  # stoppe un éventuel enregistrement/écoute en cours
        
        self.destroy()


    def doEdit(self):
        u"""
        Valide les modifications d'une entrée de dictionnaire
        (aiguillage vers les modules de validation propres à chaque format)
        """
        
        
        app['cursor'] = "watch"
        self['cursor'] = "watch"
        
        
        self.__soundStop()  # stoppe un éventuel enregistrement/écoute en cours
        
        
        # ------------------------------------------------
        # --- Aiguillage                                  
        # ------------------------------------------------
        
        memoLocalEditDate = app.dic.localEditDate 
        app.dic.localEditDate = str(datetime.date.today())
        
        ret = ""
        
        if app.dic.type.startswith("ling"):
            ret = self.__doEditLing()
        
        elif app.dic.type.startswith("wb"):
            ret = self.__doEditWB()
        
        elif app.dic.type.startswith("dict"):
            ret = self.__doEditDict()
            
        elif app.dic.type.startswith("xdxf"):
            ret = self.__doEditXDXF()
        
        elif app.dic.type.startswith("csv"):
            ret = self.__doEditCSVorINI()
        
        elif app.dic.type.startswith("ini"):
            ret = self.__doEditCSVorINI()
        
        
        # -------------------------------------------------
        
        app['cursor'] = ""
        self['cursor'] = ""
        
        # Message d'échec et quitter sans fermer la fenêtre d'édition
        if ret != "":
            tkMessageBox.showwarning(parent=self,
                    title=u"Modifications non applicables",
                    message=ret)
            app.dic.localEditDate = memoLocalEditDate
            return
        
        # Mémoriser le périphérique d'enregistrement ('try' car non utilisé par tous les formats)
        try   : app.ini.write("audioInputDevice", self.vAudioinput.get())
        except: pass
        
        self.destroy() # Ferme la fenêtre d'édition si édition ok
        
        return


    def __doEditLing(self):
        u"""
        Valide les modifications d'une entrée de dictionnaire Ling
        Retourne un message en cas d'échec
        """
        
        i = int(self.lab_idx['text'].split(":")[1]) # Index de l'entrée éditée
        
        # Contrôle de l'unicité de l'ID éventuel
        id = self.txt_ID.get().strip()
        if app.dic.IDs.has_key(id):
            if app.dic.IDs[id] != i:
                return "L'identifiant '" + id + "' existe déjà dans le dictionnaire.\n\nL'identifiant est facultatif mais doit être unique dans le dictionnaire.\nUn identifiant est nécessaire seulement si vous voulez ensuite pouvoir\nvous référer à cette entrée comme racine, synonyme, etc.\nou pour lui adjoindre des ressources (sons, images)."
        
        
        # Formatage des éventuels sauts de lignes dans short et infos >> <br>
        
        tShort = self.txt_editShort.get("0.0",END).strip()
        tShort = tShort.replace("\r\n","<br>")
        tShort = tShort.replace("\r","<br>")
        tShort = tShort.replace("\n","<br>")
        
        tLong = self.txt_editLong.get("0.0",END).strip()
        tLong = tLong.replace("\r\n","<br>")
        tLong = tLong.replace("\r","<br>")
        tLong = tLong.replace("\n","<br>")
        
        
        # Confectionner la chaîne de l'entrée (compatible preling)
        # avec le contenu des cases d'édition de la feuille d'édition
        
        newStr = ""
        
        newStr += self.txt_chp1.get().strip().encode('utf-8') + "\t"            # [0] Entrée du dictionnaire
        newStr += tShort.encode('utf-8') + "\t"                                 # [1] traduc courte
        newStr += tLong.encode('utf-8') + "\t"                                  # [2] traduc longue
        newStr += self.txt_ID.get().strip() + "\t"                              # [3] ID
        newStr += self.txt_rac.get().strip() + "\t"                             # [4] racines
        newStr += self.txt_syn.get().strip() + "\t"                             # [5] synonymes
        newStr += self.txt_see.get().strip() + "\t"                             # [6] voir-aussi
        newStr += self.txt_tok.get().strip() + "\t"                             # [7] tokens
        newStr += self.txt_phon.get().strip().encode('utf-8') + "\t"            # [8] phonétique
        newStr += self.txt_ant.get().strip()                                    # [9] antonymes
        
        # Confectionner la nouvelle chaîne preling du dictionnaire en incorporant
        # la nouvelle chaîne de l'entrée éditée
        
        strPreling = app.dic.toStrPreling(progr=None, iEdit=i, contentEdit=newStr)
        
        # Confectionner la nouvelle chaîne Ling
        
        strLing = DicObject.strPrelingToStrLing(strPreling=strPreling)
        
        if strLing.startswith("!"):
            return strLing  # retourner le message d'erreur
        
        # Enregistrer le nouveau fichier
        fichpath = app.dic.path
        app.dic.file.close()
        
        try:
            ff = open(fichpath, 'wb')
        except IOError, msg:        # Impossible d'ouvrir le fichier du dico en écriture (droits d'accés ou lecture seule)
            app.dic.file = open(fichpath, 'rb')# le rouvrir en lecture et quitter
            return "Impossible d'accéder au fichier en écriture, vérifiez que vous avez les droits d'accès nécessaires et que le fichier n'est pas en lecture seule.\n\n" + str(msg)
        ff.write(strLing)
        ff.close()
        
        # Recharger le dictionnaire avec le nouveau fichier
        app.loadDico(fichpath, showSplash=False)
        
        # Rafraîchir l'affichage des données
        
        if i == -1:
            i = app.lst_dico.size()-1    # Dernier index de la liste si édition d'une nouvelle entrée
        
        
        app.dic.lstbox.activate(i)
        app.dic.lstbox.selection_set(i)
        app.dic.lstbox.see(i)
        app.selDisplayData(None)
        
        return ""
    

    def __doEditWB(self):
        u"""
        Valide les modifications d'une entrée de dictionnaire WB
        Retourne un message en cas d'échec
        """
        
        i = int(self.lab_idx['text'].split(":")[1]) # index du mot édité
        
        # ---------------------------------------------------
        # --- Confectionner le bloc WB à modifier ou ajouter 
        # ---------------------------------------------------
        
        chp1 = self.txt_chp1.get().strip()
        chp1 = chp1.replace(",", ";")
        
        try:
            chp1 = chp1.encode (app.dic.encoding1)   # Appliquer l'encodage originel du fichier
        except:
            return "Problème d'encodage du champ 1 pour l'index "+ str(i) +"\n\nNB : utilisez la table de caractères intégrée pour inscrire des caractères non accessibles par le clavier"
        
        chp2 = self.txt_editShort.get("0.0",END).strip()
        
        try:
            chp2 = chp2.encode (app.dic.encoding2)   # Appliquer l'encodage originel du fichier
        except:
            return "Problème d'encodage du champ 2 pour l'index "+ str(i) +"\n\nNB : utilisez la table de caractères intégrée pour inscrire des caractères non accessibles par le clavier"
        
        
        newBloc = chp1[:30].ljust(31,'\0') + chp2[:50].ljust(53,'\0')
        
        # ----------------------------------------------------------
        # --- Mise à jour de la liste des entrées dans l'interface  
        # ----------------------------------------------------------
        
        if i == -1:     # Ajouter une entrée
            app.lst_dico.insert(END, self.txt_chp1.get())
        else:           # Remplacer une entrée
            app.lst_dico.delete(i)
            app.lst_dico.insert(i, self.txt_chp1.get())
        
        # ----------------------------------------------------
        # --- Confectionner la nouvelle chaîne du fichier WB  
        # ----------------------------------------------------
        
        fichpath = app.dic.path
        
        if i == -1:     # Ajouter une entrée
            pass
        else:           # Remplacer une entrée
            app.dic.file.seek(0)
            part1 = app.dic.file.read(84*i)
            app.dic.file.seek(84*(i+1))
            part2 = app.dic.file.read(os.path.getsize(fichpath))
            
            strfile = part1 + newBloc + part2
        
        # ----------------------------------------
        # --- Enregistrer le nouveau fichier WB   
        # ----------------------------------------
        
        app.dic.file.close()
        
        if i == -1:     # Ajouter une entrée
            
            try:
                app.dic.file = open(fichpath, 'ab') # ouvrir en "append"
            except IOError, msg:        # Impossible d'ouvrir le fichier du dico en écriture (droits d'accés ou lecture seule)
                app.dic.file = open(fichpath, 'rb')# le rouvrir en lecture et quitter
                return "Impossible d'accéder au fichier en écriture, vérifiez que vous avez les droits d'accès nécessaires et que le fichier n'est pas en lecture seule.\n\n" + str(msg)
            
            app.dic.file.write(newBloc)
            app.dic.file.close()
            app.dic.file = open(fichpath, 'rb')
        
        else:           # Remplacer une entrée
            
            try:
                app.dic.file = open(fichpath, 'wb')
            except IOError, msg:        # Impossible d'ouvrir le fichier du dico en écriture (droits d'accés ou lecture seule)
                app.dic.file = open(fichpath, 'rb')# le rouvrir en lecture et quitter
                return "Impossible d'accéder au fichier en écriture, vérifiez que vous avez les droits d'accès nécessaires et que le fichier n'est pas en lecture seule.\n\n" + str(msg)
            
            app.dic.file.write(strfile)
            app.dic.file.close()
            app.dic.file = open(fichpath, 'rb')
        
        
        # ------------------------------------------
        # --- Rafraîchit l'affichage des données    
        # ------------------------------------------
        
        
        if i == -1:
            i = app.lst_dico.size()-1    # Dernier index de la liste si édition d'une nouvelle entrée
        
        app.dic.lstbox.activate(i)
        app.dic.lstbox.selection_set(i)
        app.dic.lstbox.see(i)
        app.selDisplayData(None)
        
        
        return ""
    
    
    def __doEditXDXF(self):
        u"""
        Valide les modifications d'une entrée de dictionnaire XDXF
        Retourne un message en cas d'échec
        """
        
        i = int(self.lab_idx['text'].split(":")[1]) # index du mot édité
        
        # -----------------------------------------------------
        # --- Confectionner les blocs du fichier, à conserver  
        # -----------------------------------------------------
        
        fichpath = app.dic.path
        
        if i == -1:     # nouvelle entrée
            
            app.dic.file.seek(0)
            part1 = app.dic.file.read()
            part1 = part1[:part1.rfind("</xdxf>")]
            part1 = part1.rstrip("\n")
            part2 = "\n</xdxf>\n"
        
        else:
            ii = i-1
            if ii<0: ii=0
            
            app.dic.file.seek(0)
            part1 = app.dic.file.read(app.dic.datamap[ii][0] + app.dic.datamap[ii][1])
                        
            app.dic.file.seek(app.dic.datamap[i][0] + app.dic.datamap[i][1])
            part2 = app.dic.file.read(os.path.getsize(fichpath)) # lit jusqu'au bout du fichier
        
        
        # --------------------------------------------------
        # --- Confectionner le bloc de l'entrée à modifier  
        # --------------------------------------------------
        tchp1 = self.txt_chp1.get().strip()
        tshort = self.txt_editShort.get("0.0",END).strip()
        tlong = self.txt_editLong.get("0.0",END).strip()
        tphon = self.txt_phon.get().strip()
        
        newBloc = "\n<ar><k>" + tchp1 + "</k>\n" + tshort 
        
        if tphon != "":
            newBloc += "<trn>" + tphon + "</trn>\n"
        
        if tlong != "":
            newBloc += "<co>" + tlong + "</co>\n"
        
        newBloc += "</ar>"    
        
        # --- Appliquer l'encodage originel du fichier
        
        try:
            newBloc = newBloc.encode('utf-8')
        except:
            return "Problème d'encodage UTF-8 pour l'index "+ str(i)
        
        
        # -------------------------------------------------
        # --- Mise à jour du fichier                       
        # -------------------------------------------------
        
        # Confectionner la nouvelle chaîne du fichier
        strfile = part1 + newBloc + part2
        
        # Enregistrer le fichier
        app.dic.file.close()
        
        try:
            app.dic.file = open(fichpath, 'wb')
        except IOError, msg:        # Impossible d'ouvrir le fichier du dico en écriture (droits d'accés ou lecture seule)
            app.dic.file = open(fichpath, 'rb')# le rouvrir en lecture et quitter
            return "Impossible d'accéder au fichier en écriture, vérifiez que vous avez les droits d'accès nécessaires et que le fichier n'est pas en lecture seule.\n\n" + str(msg)
        
        app.dic.file.write(strfile)
        app.dic.file.close()
        app.dic.file = open(fichpath, 'rb')
        
        
        # Recharge le dictionnaire avec le nouveau fichier
        app.loadDico(fichpath, showSplash=False)
        
        # Rafraîchit l'affichage des données
        
        if i == -1:
            i = app.lst_dico.size()-1    # Dernier index de la liste si édition d'une nouvelle entrée
        
        app.dic.lstbox.activate(i)
        app.dic.lstbox.selection_set(i)
        app.dic.lstbox.see(i)
        app.selDisplayData(None)
        
        
        return ""
        

    def __doEditCSVorINI(self):
        u"""
        Valide les modifications d'une entrée de dictionnaire CSV ou INI
        Retourne un message en cas d'échec
        """
        
        i = int(self.lab_idx['text'].split(":")[1]) # index du mot édité
        
        
        # -----------------------------------------------------
        # --- Confectionner les blocs du fichier, à conserver  
        # -----------------------------------------------------
        
        fichpath = app.dic.path
        
        if i == -1:     # nouvelle entrée
            
            app.dic.file.seek(0)
            part1 = app.dic.file.read() + app.dic.br
            part2 = ""
        
        else:
            ii = i-1
            if ii<0: ii=0
            
            app.dic.file.seek(0)
            part1 = app.dic.file.read(app.dic.datamap[ii][0]+app.dic.datamap[ii][1])
            if part1 != "" : part1 = part1 + app.dic.br
            
            app.dic.file.seek(app.dic.datamap[i][0]+app.dic.datamap[i][1])
            part2 = app.dic.file.read(os.path.getsize(fichpath)) # lit jusqu'au bout du fichier
        
        
        # --------------------------------------------------
        # --- Confectionner le bloc de l'entrée à modifier  
        # --------------------------------------------------
        chp1 = self.txt_chp1.get().strip()
        chp2 = self.txt_editShort.get("0.0",END).strip()
        
        if app.dic.type.startswith("ini") and chp1.startswith("["): # section d'ini
            newBloc = chp1
        else:
            newBloc = chp1 + app.dic.sep + chp2
        
        # --- Appliquer l'encodage originel du fichier
        
        try:
            newBloc = newBloc.encode (app.dic.encoding1)
        except:
            return "Problème d'encodage pour l'index "+ str(i) +"\n\nNB : utilisez la table de caractères intégrée pour inscrire des caractères non accessibles par le clavier"
        
        
        # -------------------------------------------------
        # --- Mise à jour du fichier                       
        # -------------------------------------------------
        
        # Confectionner la nouvelle chaîne du fichier
        strfile = part1 + newBloc + part2
        
        # Enregistrer le fichier
        app.dic.file.close()
    
        try:
            ff = open(fichpath, 'wb')
        except IOError, msg:        # Impossible d'ouvrir le fichier du dico en écriture (droits d'accés ou lecture seule)
            app.dic.file = open(fichpath, 'rb')# le rouvrir en lecture et quitter
            return "Impossible d'accéder au fichier en écriture, vérifiez que vous avez les droits d'accès nécessaires et que le fichier n'est pas en lecture seule.\n\n" + str(msg)
        
        ff.write(strfile)
        ff.close()
        
        # Recharger le dictionnaire avec le nouveau fichier
        cp = app.dic.encoding1
        app.loadDico(fichpath, cp1=cp, showSplash=False)
        
        # Rafraîchit l'affichage des données
        
        if i == -1:
            i = app.lst_dico.size()-1    # Dernier index de la liste si édition d'une nouvelle entrée
        
        app.dic.lstbox.activate(i)
        app.dic.lstbox.selection_set(i)
        app.dic.lstbox.see(i)
        app.selDisplayData(None)
        
        return ""


    def __doEditDict(self):
        u"""
        Valide les modifications d'une entrée de dictionnaire Dict
        Retourne un message en cas d'échec
        """
        
        i = int(self.lab_idx['text'].split(":")[1]) # Index de l'entrée éditée
        
        # Confectionner la chaîne de l'entrée éditée
        # avec le contenu des cases d'édition de la feuille d'édition
        
        try:
            newStr = self.txt_chp1.get().strip().encode('utf-8') + "\0"         # [0] Entrée du dictionnaire
            newStr += self.txt_editShort.get("0.0",END).strip().encode('utf-8') # [1] traduc courte
        except:
            return "Problème d'encodage Utf-8 des données éditées" 
        
        # Obtenir un tuple des chaines Dict : (stridx, strdict, strwordcount)
        strs = app.dic.toStrsDict(iEdit=i, contentEdit=newStr)
        
        # si strs est une chaîne et non un tupple => problème !
        if type(strs) is type("xxx"):
            return strs
        
        
        # -------------------------------------------------------
        # --- Enregistrer les nouveaux fichiers *.idx et *.dict  
        # -------------------------------------------------------
        
        radicfich = os.path.splitext(app.dic.path)[0]     # Radical des différents fichiers Dict
        
        
        # --- Enregistrer le *idx
        try:
            ff = open(radicfich + '.idx', 'wb')
        except IOError, msg:        # Impossible d'ouvrir le fichier du dico en écriture (droits d'accés ou lecture seule)
            return "Impossible d'accéder au fichier en écriture, vérifiez que vous avez les droits d'accès nécessaires et que le fichier n'est pas en lecture seule.\n\n" + str(msg)        
        
        ff.write(strs[0])
        ff.close()
        
        
        # --- Enregistrer le *dict
        app.dic.file.close()
        try:
            ff = open(radicfich + '.dict', 'wb')
        except IOError, msg:        # Impossible d'ouvrir le fichier du dico en écriture (droits d'accés ou lecture seule)
            app.dic.file = open(radicfich + '.dict', 'rb')
            return "Impossible d'accéder au fichier en écriture, vérifiez que vous avez les droits d'accès nécessaires et que le fichier n'est pas en lecture seule.\n\n" + str(msg)        
        
        ff.write(strs[1])
        ff.close()
        
        
        # ------------------------------------------------
        # --- Reconstruire le fichier *.ifo               
        # ------------------------------------------------
        lg = ""
        strifo = ""
        ff = open(radicfich + '.ifo', 'rU')
        
        for ligne in ff:
            
            if ligne.startswith("wordcount="):
                lg = "wordcount=" + strs[2] + "\n"
            elif ligne.startswith("idxfilesize="):
                lg = "idxfilesize=" + str(os.path.getsize(radicfich + ".idx")) + "\n"
            else:
                lg = ligne
                
            strifo += lg 
        
        ff.close()
        ff = open(radicfich + '.ifo', 'wb')   # Enregistrer le *ifo
        ff.write(strifo)
        ff.close()
        
        
        # --------------------------------------------------------------
        # --- Recharger le dictionnaire avec le nouveau fichier Dict    
        # --------------------------------------------------------------
        
        app.loadDico(radicfich + '.dict', showSplash=False)
        
        # --- Rafraîchit l'affichage des données
        
        if i == -1:
            i = app.lst_dico.size()-1    # Dernier index de la liste si édition d'une nouvelle entrée
        
        
        app.dic.lstbox.activate(i)
        app.dic.lstbox.selection_set(i)
        app.dic.lstbox.see(i)
        app.selDisplayData(None)
        
        
        return ""


    def __makeID(self):
        u"""
        Confectionne un ID automatique pour l'entrée en édition
        et l'affiche dans la case ad hoc
        """
        
        # Confectionner le radical de trois lettres
        id = self.txt_chp1.get()[0:3]
        
        ords = map(ord,id)
        for n in range(len(ords)):
            if ords[n] > 128: ords[n]=ord('_')
        
        id = map(chr, ords)
        id = "".join(id)
        
        # Ajouter le chiffre final
        
        n=1
        while 1:
            testid = id + str(n)
            if not app.dic.IDs.has_key(testid):
                id = testid
                break
            n += 1
        
        self.txt_ID.delete(0,END)
        id = self.txt_ID.insert(END, id)
        
        return



class WindowTranslate(Toplevel):
    u"""
    Fenêtre d'aide à la traduction commune à toutes les langues
    Le principe est de feuillter le dictionnaire et d'afficher les mots de plus grande concordance
    et non forcément le mot exact s'il est absent.
    """
    
    def __init__(self):
        u""" Interface graphique de la fenêtre """
        
        Toplevel.__init__(self)
        
        self.title("Aide à la traduction")
        self.resizable(width=NO, height=NO)
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        
        fr_tool=Frame(self)
        fr_tool.grid(row=0, column=0, columnspan=10, sticky="W")
        Label(fr_tool, text="Tapez ou collez le texte à traduire").grid(row=0, column=0, sticky='W')
        self.bt_copy=Button(fr_tool, text=" Coller ", compound=LEFT, image=app.imglst.paste, command=self.copyToTransl)
        self.bt_copy.grid(row=0, column=1, pady=2, padx=5)
        self.bt_open=Button(fr_tool, text=" Ouvrir un fichier texte ", compound=LEFT, image=app.imglst.open, command=self.openToTransl)
        self.bt_open.grid(row=0, column=2, pady=2, padx=5)
        Button(fr_tool, image=app.imglst.hlp, compound=LEFT, text=" Aide ", padx= 10,command=lambda: app.showHelp("trad")).grid(row=0, column=3, pady=2, padx=5)
        
        Label(self, text="(Double-cliquez sur un mot pour afficher ses correspondances)").grid(row=1, column=0, columnspan=10, sticky='W')
        
        # zone du texte à traduire
        self.txt_source=ScrolledText(self,width=80,height=10, background='white')
        self.txt_source['font']=app.lst_dico['font']
        self.txt_source.grid(row=2, column=0, columnspan=10, sticky='W')
        self.txt_source.bind("<Double-ButtonRelease-1>", self.dbClicOnWord)
        
        Label(self, text="Correspondances possibles :").grid(row=3, column=0, sticky='W')
        
        # répétiteurs du mot et de la meilleure correspondance
        self.repwd=Text(self, width=20, height=1, borderwidth=0, background=self['background'])
        self.repwd.insert(END, "...")
        self.repwd.tag_config("vert", foreground="#008000")
        self.repwd.tag_config("rouge", foreground="red")
        self.repwd.grid(row=3, column=1, sticky='W')
        
        self.repbestwd=Text(self, width=20, height=1, borderwidth=0, background=self['background'])
        self.repbestwd.insert(END, "...")
        self.repbestwd.tag_config("vert", foreground="#008000")
        self.repbestwd.tag_config("rouge", foreground="red")
        self.repbestwd.grid(row=3, column=2, sticky='W')
        
        self.repwd['font'] = app.txt_notice['font']
        self.repbestwd['font'] = app.lst_dico['font']
        
        # liste des correspondances
        self.lst_match=lib.myscrlst.ScrolledList(self, width=82, height=10, font=app.dic.lstbox['font'], ondblclic=self.dblClicOnMatch, background='#ffffee')
        self.lst_match.grid(row=4, column=0, columnspan=10, sticky='W')
        
        Label(self, text="Double-cliquez sur une correspondance pour afficher sa notice...").grid(row=5, column=0, columnspan=10, sticky='W')
        
        self.txt_source.focus_set()
        
        return
    
    
    def openToTransl(self):
        u"""
        Ouvrir un fichier texte à traduire
        """
        
        fichpath = tkFileDialog.askopenfilename(parent=self,
                                title=u"Ouvrir un fichier texte à traduire",
                                filetypes=(
                                ('Fichiers texte','*.txt'),
                                ('Tous les fichiers','*.*'))
                                )
            
        if not fichpath: return   # L'utilisateur a cliqué 'Annuler' dans la boîte de dialogue
        
        ff = open(fichpath,"rU")
        t = ff.read()
        ff.close()
        
        try:
            t = t.decode('cp1252')
        except:
            pass
        
        self.txt_source.delete("1.0", END)
        self.txt_source.insert(END, ff.read())
        
        
        return


    def copyToTransl(self):
        u"""
        Copie le contenu du presse-papiers dans le panneau de traduction
        """
        
        try:    self.txt_source.delete(SEL_FIRST, SEL_LAST)
        except: pass
        self.txt_source.insert(INSERT, self.clipboard_get())
            
        return

    

    def dbClicOnWord(self, event):
        u"""
        Double-Clic sur un mot à traduire
        > affiche les correspondances approchantes
        """
        
        self['cursor']="watch"
        app['cursor']="watch"
        self.update()
        
        wordsel = self.txt_source.get(SEL_FIRST, SEL_LAST)  # mot à traduire
        wordsel = wordsel.lower().strip(" ,;:!?.()[]{}'\"") # nettoyage des ponctuations
        
        wordstart = wordsel.lower()[:3]
        
        # accélérateurs locaux
        idxfind = app.dic.idxfind
        get = app.dic.lstbox.get
        getdata = app.dic.getdata
        
        # ------------------------------------------
        # --- Chercher la correspondance optimale   
        # ------------------------------------------
        
        motif = wordsel
        
        while len(motif)>0:
            ibest = idxfind(motif, dom='idx', decal=0, debut=True, ignorecase=True)
            if ibest is None:
                motif = motif[:-1]
            else:
                break
        
        # Nettoyage des données antérieures
        self.lst_match.clear()
        self.repwd.delete("1.0", END)
        self.repbestwd.delete("1.0", END)
        
        if ibest is None:
            app['cursor']=""
            self['cursor']=""
            return
        
        bestword = get(ibest).encode('utf-8').split()[0]
        
        
        # --------------------------------------------
        # --- Compléter la liste des correspondances  
        # --------------------------------------------
        
        
        # Ajouter les correspondances approchantes (3 caractères mini) précédentes
        i = ibest
        while 1:
            i -= 1
            word = get(i).encode('utf-8')
            if word[:3].lower() == wordstart:
                self.lst_match.add(word + "  >>>  " + getdata(i)['short'].replace("\n", "//").replace("\t", " ") + "  [" + str(i) + "]", p=0)
            else:
                break
        
        # Correspondance optimale
        self.lst_match.add(get(ibest).encode('utf-8') + "  >>>  " + getdata(ibest)['short'].replace("\n", "//").replace("\t", " ") + "  [" + str(ibest) + "]")
        memidx = self.lst_match.listcount()-1
        
        # Ajouter les correspondances approchantes (3 caractères mini) suivantes
        
        i = ibest
        while 1:
            i += 1
            word = get(i).encode('utf-8')
            if word[:3].lower() == wordstart:
                self.lst_match.add(word + "  >>>  " + getdata(i)['short'].replace("\n", "//").replace("\t", " ") + "  [" + str(i) + "]")
            else:
                break
        
        # Sélectionner la correspondance optimale
        self.lst_match.setSel(memidx)
        self.lst_match.see(memidx)
        self.lst_match.activate(memidx)
        self.lst_match.focus_set()
        
        
        # -----------------------------------------------------------------------
        # --- Afficher les répétiteurs : mot cliqué et meilleure correspondance  
        # -----------------------------------------------------------------------
        
        if len(bestword) > len(wordsel):
            nmax = len(bestword)
            nmin = len(wordsel)
        else:
            nmax = len(wordsel)
            nmin = len(bestword)
        
        for l in range(nmin):
            if wordsel[l] == bestword[l] :
                self.repwd.insert(END, wordsel[l], "vert")
                self.repbestwd.insert(END, bestword[l], "vert")
            else:
                self.repwd.insert(END, wordsel[l], "rouge")
                self.repbestwd.insert(END, bestword[l], "rouge")
        
        for l in range(nmin, nmax):
            try:
                self.repwd.insert(END, wordsel[l], "rouge")
            except :
                pass
            try:
                self.repbestwd.insert(END, bestword[l], "rouge")
            except:
                pass
            
        # ---
        
        app['cursor']=""
        self['cursor']=""
        
        return



    def dblClicOnMatch(self, i, id):
        u"""
        Clic sur une correspondance
        """
        
        imatch = self.lst_match.getselectedindex()
        
        # Extraire l'index originel de la correspondance
        t = self.lst_match.getitem(i).split()[-1]
        i = int(t[1:len(t)-1])
        
        # Afficher la notice correspondante
        app.dic.lstbox.activate(i)
        app.dic.lstbox.selection_set(i)
        app.dic.lstbox.see(i)
        app.selDisplayData(None)
        
        self.lst_match.activate(imatch)
        self.lst_match.setSel(imatch)
        
        return


class WindowExport(Toplevel):
    u"""
    Fenêtre d'exportation du dictionnaire courant
    vers divers formats
    """


    def __init__(self):
        u""" Interface graphique de la fenêtre """
        
        # Si le fichier est protégé > avertir et quitter
        # (message dépendant de app car avant instanciation de Toplevel)
        if app.dic.protected1 !="" or app.dic.protected2 !="":
            tkMessageBox.showinfo(parent=app,
                    title=u"Exportation du dictionnaire",
                    message=u"Les données de ce dictionnaire sont protégées,\nl'exportation vers un autre format est donc impossible.")
            return
        
        Toplevel.__init__(self)
        
        self.title("Exporter le dictionnaire")
        self.resizable(width=NO, height=NO)
        self.grab_set()      # fenêtre modale
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        hPan = 20 # nb caractères
        wPan = 50 # nb caractères
        padxPan = 5
        
        
        # ---------------------------------------------
        # --- ListBox des formats cible de conversion  
        # ---------------------------------------------
        
        self.lst_fmt=Listbox(self, height=hPan-7, width=10, background='white', selectborderwidth=3)
        self.lst_fmt.grid(row=0, column=0, padx=5, pady=5)
        self.lst_fmt.bind('<ButtonRelease-1>', self.lst_fmt_select)
        
        self.lst_fmt.insert(END, 'LING')
        self.lst_fmt.insert(END, 'preling')
        self.lst_fmt.insert(END, 'DICT')
        self.lst_fmt.insert(END, 'XDXF')
        self.lst_fmt.insert(END, 'WB')
        self.lst_fmt.insert(END, 'CSV')
        self.lst_fmt.insert(END, 'INI')
        self.lst_fmt.insert(END, 'Binaire')
        
        
        # --------------------------------------------------
        # Welcome / Dimensionneur de l'espace des panneaux  
        # --------------------------------------------------
        
        Label(self, text="* * * * * * * * * * * * * * *\n\nChoisissez un format d'exportation\ndans la liste de gauche\n\n* * * * * * * * * * * * * * *", width=wPan, height=hPan).grid(row=0, column=1, padx=padxPan)
        
        # -------------------------
        # --- Panneau Ling         
        # -------------------------
        
        self.fr_ling=Frame(self, borderwidth=2, relief='ridge')
        self.fr_ling.grid(row=0,column=1,padx=padxPan)
        
        Label(self.fr_ling, text=" ", width=wPan, height=hPan).grid(row=0, column=0, rowspan=10, columnspan=10) #dimensionneur)
        
        Label(self.fr_ling, text='Exportation au format Ling', borderwidth=4, foreground="white", background="black", relief='ridge').grid(row=0, column=0, columnspan=10, ipady=2, ipadx=5)
        
        Label(self.fr_ling, text="L'encodage des données textuelles se fera en Utf-8,\n quel que soit l'encodage du dictionnaire source.", justify=LEFT). grid(row=1, column=0)
        
        self.vProtec1 = IntVar()
        chk_lingprotected1 = Checkbutton(self.fr_ling, text="Protéger les entrées", variable=self.vProtec1)
        chk_lingprotected1.grid(row=2, padx=10, column=0, sticky='W')
        self.vProtec2 = IntVar()
        chk_lingprotected2 = Checkbutton(self.fr_ling, text="Protéger les traductions/définitions", variable=self.vProtec2)
        chk_lingprotected2.grid(row=3, padx=10, column=0, sticky='W')
        
        
        # -------------------------
        # --- Panneau PreLing      
        # -------------------------
        
        self.fr_preling=Frame(self, borderwidth=2, relief='ridge')
        self.fr_preling.grid(row=0,column=1,padx=padxPan)
        
        Label(self.fr_preling, text=" ", width=wPan, height=hPan).grid(row=0, column=0, rowspan=10, columnspan=10) #dimensionneur)
        
        Label(self.fr_preling, text='Exportation au format preling', borderwidth=4, foreground="white", background="black", relief='ridge').grid(row=0, column=0, columnspan=10, ipady=2, ipadx=5)
        
        Label(self.fr_preling, text="Le format preling est un format texte passerelle avec le format binaire Ling\nil permet l'édition du dictionnaire dans simple un éditeur de texte\net l'accès à toutes les possibilités du format Ling.", justify=LEFT).grid(row=1, column=0)
        
        Label(self.fr_preling, text="L'encodage des données textuelles se fera en Utf-8,\n quel que soit l'encodage du dictionnaire source.", justify=LEFT). grid(row=2, column=0)
        
        r=3
        self.vCrlf2 = StringVar()
        Label(self.fr_preling, text="Saut de ligne :").grid(row=r, sticky='W')
        Radiobutton(self.fr_preling, text="CRLF : \\r\\n (Windows)", variable=self.vCrlf2, value='\r\n').grid(row=r+1, sticky='W', padx=10)
        Radiobutton(self.fr_preling, text="LF : \\n (Unix)", variable=self.vCrlf2, value='\n').grid(row=r+2, sticky='W', padx=10)
        Radiobutton(self.fr_preling, text="CR : \\r (Mac os)", variable=self.vCrlf2, value='\r').grid(row=r+3, sticky='W', padx=10)
        self.vCrlf2.set("\r\n")
        
        # -------------------------
        # --- Panneau Wb           
        # -------------------------
        
        self.fr_wb=Frame(self, borderwidth=2, relief='ridge')
        self.fr_wb.grid(row=0,column=1,padx=padxPan)
        
        Label(self.fr_wb, text=" ", width=wPan, height=hPan).grid(row=0, column=0, rowspan=10, columnspan=10) #dimensionneur)
        
        Label(self.fr_wb, text='Exportation au format WB', borderwidth=4, foreground="white", background="black", relief='ridge').grid(row=0, column=0, columnspan=10, ipady=2, ipadx=5)
        
        r=1
        self.w_wbcp1=lib.myencsel.EncodingSelector(self.fr_wb, text='latin1', title='Encodage langue source :', height=5)
        self.w_wbcp1.grid(row=r, column=0, padx=3)
        self.w_wbcp2=lib.myencsel.EncodingSelector(self.fr_wb, text='latin1', title='Encodage Langue cible :', height=5)
        self.w_wbcp2.grid(row=r, column=1, padx=3)
        
        Label(self.fr_wb, text="WB est un format 8-bits (Ascii étendu), l'encodage doit donc\nêtre compatible. Ex. : 'latin-1', 'cp1242' pour l'Europe de l'W.", justify=LEFT, anchor='w').grid(row=r+3, column=0, columnspan=10, ipady=2, ipadx=7, sticky='W')
        
        Label(self.fr_wb, text="ATTENTION, les champs du format WB sont limités\nà 30 caractères pour la langue source\n et à 50 caractères pour la langue cible.\nLes données dépassant ces longueurs seront tronquées...", justify=LEFT, anchor='w').grid(row=r+4, column=0, columnspan=10, ipady=2, ipadx=7, sticky='W')
        
        
        
        # -------------------------
        # --- Panneau Dict         
        # -------------------------
        
        self.fr_dict=Frame(self, borderwidth=2, relief='ridge')
        self.fr_dict.grid(row=0, column=1, padx=padxPan)
        
        Label(self.fr_dict, width=wPan, height=hPan).grid(row=0, column=0, rowspan=10) #dimensionneur)
        
        Label(self.fr_dict, text='Exportation au format DICT', borderwidth=4, foreground="white", background="black", relief='ridge').grid(row=0, column=0, ipady=2, ipadx=5)
        
        Message(self.fr_dict, text="Le dictionnaire Dict sera composé de trois fichiers, encodés en Utf-8 :\n\tNomDuDico.ifo\n\tNomDuDico.idx\n\tNomDuDico.dict\nL'extension que vous choisirez pour le nom du fichier d'exportation est sans importance, ces trois fichiers seront créés dans le répertoire avec le même nom de base.", justify=LEFT).grid(row=1, column=0)
        
        Label(self.fr_dict, text='______________').grid(row=2, column=0)
        
        self.vTransGram = IntVar()
        Checkbutton(self.fr_dict, text="Transférer les contenus entre crochets du champ des\nentrées en tête du champ traduction (facilite l'usage\nde la lecture d'écran par les logiciels le permettant)", justify=LEFT, variable=self.vTransGram).grid(row=3, column=0, pady=3)
        
        
        # -------------------------
        # --- Panneau XDXF         
        # -------------------------
        
        self.fr_xdxf=Frame(self, borderwidth=2, relief='ridge', padx=2)
        self.fr_xdxf.grid(row=0,column=1,padx=padxPan)
        
        Label(self.fr_xdxf, text=" ", width=wPan, height=hPan).grid(row=0, column=0, rowspan=10, columnspan=10) #dimensionneur)
        
        Label(self.fr_xdxf, text='Exportation au format XDXF', borderwidth=4, foreground="white", background="black", relief='ridge').grid(row=0, column=0, columnspan=10, ipady=2, ipadx=5)
        
        Label(self.fr_xdxf, text="Un dictionnaire XDXF est composé d'un fichier XML\nnommé obligatoirement 'dict.xdxf' et placé dans\nun sous-dossier propre à chaque dictionnaire.\n\nLes données seront encodées en Utf-8", justify=LEFT).grid(row=1, column=0, padx=5)
        
        Label(self.fr_xdxf, text="Dossier qui contiendra le dictionnaire :").grid(row=2, column=0, sticky='W')
        self.txt_repxdxf = Entry(self.fr_xdxf, width=40, background='white')
        self.txt_repxdxf.grid(row=3, column=0, sticky='W')
        Button(self.fr_xdxf, text="Parcourir...", command= lambda: browseFolder(self, self.txt_repxdxf)).grid(row=3, column=1, sticky='W')
        
        Label(self.fr_xdxf, text="Nom du dictionnaire (sera le nom du nouveau sous-dossier) :").grid(row=4, column=0, columnspan=10, padx=3, sticky='W')
        self.txt_namexdxf = Entry(self.fr_xdxf, width=20, background='white')
        self.txt_namexdxf.grid(row=5, column=0, sticky='W')
        
        
        # -------------------------
        # --- Panneau CSV          
        # -------------------------
        
        self.fr_csv=Frame(self, borderwidth=2, relief='ridge')
        self.fr_csv.grid(row=0,column=1,padx=padxPan)
        
        Label(self.fr_csv, text=" ", width=wPan, height=hPan).grid(row=0, column=0, rowspan=10, columnspan=10) #dimensionneur
        
        Label(self.fr_csv, text='Exportation au format CSV (texte délimité)', borderwidth=4, foreground="white", background="black", relief='ridge').grid(row=0, column=0, columnspan=10, ipady=2, ipadx=5)
        
        r=1
        self.vSp = StringVar()
        Label(self.fr_csv, text="Caractère séparateur :").grid(row=r, sticky='W')
        Radiobutton(self.fr_csv, text="(tabulation)", variable=self.vSp, value='\t').grid(row=r+1, sticky='W', padx=10)
        Radiobutton(self.fr_csv, text="(virgule)", variable=self.vSp, value=',').grid(row=r+2, sticky='W', padx=10)
        Radiobutton(self.fr_csv, text="(point-virgule)", variable=self.vSp, value=';').grid(row=r+3, sticky='W', padx=10)
        self.vSp.set("\t")
        
        r=1
        self.w_csvenc=lib.myencsel.EncodingSelector(self.fr_csv, text='latin1', height=12)
        self.w_csvenc.grid(row=r, column=1, rowspan=10, sticky='W')
        
        r=6
        self.vCrlf = StringVar()
        Label(self.fr_csv, text="Saut de ligne :").grid(row=r, sticky='W')
        Radiobutton(self.fr_csv, text="CRLF : \\r\\n (Windows)", variable=self.vCrlf, value='\r\n').grid(row=r+1, sticky='W', padx=10)
        Radiobutton(self.fr_csv, text="LF : \\n (Unix)", variable=self.vCrlf, value='\n').grid(row=r+2, sticky='W', padx=10)
        Radiobutton(self.fr_csv, text="CR : \\r (Mac os)", variable=self.vCrlf, value='\r').grid(row=r+3, sticky='W', padx=10)
        self.vCrlf.set("\r\n")
        
        
        # -------------------------
        # --- Panneau INI          
        # -------------------------
        
        self.fr_ini=Frame(self, borderwidth=2, relief='ridge')
        self.fr_ini.grid(row=0,column=1,padx=padxPan)
        
        Label(self.fr_ini, text=" ", width=wPan, height=hPan).grid(row=0, column=0, rowspan=10, columnspan=10) #dimensionneur
        
        Label(self.fr_ini, text='Exportation au format INI', borderwidth=4, foreground="white", background="black", relief='ridge').grid(row=0, column=0, columnspan=10, ipady=2, ipadx=5)
        
        r=1
        self.vBrIni = StringVar()
        Label(self.fr_ini, text="Saut de ligne :").grid(row=r, sticky='W')
        Radiobutton(self.fr_ini, text="CRLF : \\r\\n (Windows)", variable=self.vBrIni, value='\r\n').grid(row=r+1, sticky='W', padx=10)
        Radiobutton(self.fr_ini, text="LF : \\n (Unix)", variable=self.vBrIni, value='\n').grid(row=r+2, sticky='W', padx=10)
        Radiobutton(self.fr_ini, text="CR : \\r (Mac os)", variable=self.vBrIni, value='\r').grid(row=r+3, sticky='W', padx=10)
        self.vBrIni.set("\r\n")
        
        
        self.w_inienc=lib.myencsel.EncodingSelector(self.fr_ini, text='cp1252', title="Encodage :", height=12)
        self.w_inienc.grid(row=1, column=1, rowspan=10, sticky='W')
        
        
        # -------------------------
        # --- Panneau Binaire      
        # -------------------------
        
        self.fr_bin=Frame(self, borderwidth=2, relief='ridge')
        self.fr_bin.grid(row=0,column=1,padx=padxPan)
        
        Label(self.fr_bin, text=" ", width=wPan, height=hPan).grid(row=0, column=0, rowspan=10, columnspan=10) #dimensionneur)
        
        Label(self.fr_bin, text='Exportation en format binaire personnalisé', borderwidth=4, foreground="white", background="black", relief='ridge').grid(row=0, column=0, columnspan=10, ipady=2, ipadx=5)
        
        r=1
        Label(self.fr_bin, text="Octet(s) de séparation :", padx=3).grid(row=r, column=0, sticky='W')
        txt_spbin=Entry(self.fr_bin, width=8, background="white")
        txt_spbin.grid(row=r, column=1, sticky='W', padx=5)
        txt_spbin.insert(END, "00")
        
        r=2
        Label(self.fr_bin, text="Octet(s) de terminaison :", padx=3).grid(row=r, column=0, sticky='W')
        txt_crlfbin=Entry(self.fr_bin, width=8, background="white")
        txt_crlfbin.grid(row=r, column=1, sticky='W', padx=5)
        txt_crlfbin.insert(END, "00 00")
        
        r=3
        w_binenc=lib.myencsel.EncodingSelector(self.fr_bin, text='utf-8', height=6)
        w_binenc.grid(row=r, column=0, columnspan=2, rowspan=10)
        
        
        # -------------------------
        # --- Frame des Boutons    
        # -------------------------
        r = 1
        fr_butOpt=Frame(self)
        fr_butOpt.grid(row=r, column=0, columnspan=4, pady=8, sticky="WENS")
        
        Label(fr_butOpt, text="", width=wPan+self.lst_fmt['width']).grid(row=0, columnspan=3) # dimensionneur
        self.bt_Do=Button(fr_butOpt, image=app.imglst.export, compound=LEFT, text="  Exporter...", width=90, padx= 7, command=self.doExport, state='disabled')
        self.bt_Do.grid(row=0, column=0, pady=8, padx=10)
        Button(fr_butOpt, image=app.imglst.exit, compound=LEFT, text='  Fermer', width=90, command=self.destroy, padx= 7).grid(row=0, column=1, pady=8, padx=10)
        Button(fr_butOpt, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=90, padx= 7,command=lambda: app.showHelp("export")).grid(row=0, column=2, pady=8, padx=10)
        
        # Barre de progression
        self.progr = lib.myprogbr.ProgressBar(self, width=380)
        self.progr.grid(row=r+1, column=0, columnspan=4, pady=5)
        self.progr.grid_remove()
        
        
        # --- Masquer tous les panneaux au démarrage
        self.fr_ling.grid_remove()
        self.fr_preling.grid_remove()
        self.fr_wb.grid_remove()
        self.fr_dict.grid_remove()
        self.fr_csv.grid_remove()
        self.fr_ini.grid_remove()
        self.fr_bin.grid_remove()
        self.fr_xdxf.grid_remove()
        
        return



    def lst_fmt_select(self, event):
        u"""
        Sélection d'un format dans la liste des formats
        """
        
        self.lst_fmt.update()
        fmt = self.lst_fmt.get(int(self.lst_fmt.curselection()[0])) # texte de la ligne en sélection
        
        self.fr_ling.grid_remove()
        self.fr_preling.grid_remove()
        self.fr_wb.grid_remove()
        self.fr_dict.grid_remove()
        self.fr_csv.grid_remove()
        self.fr_ini.grid_remove()
        self.fr_bin.grid_remove()
        self.fr_xdxf.grid_remove()
        
        # Sélectionner le panneau de format
        if fmt == "LING": self.fr_ling.grid()
        elif fmt == "preling": self.fr_preling.grid()
        elif fmt == "WB": self.fr_wb.grid()
        elif fmt == "DICT": self.fr_dict.grid()
        elif fmt == "CSV": self.fr_csv.grid()
        elif fmt == "INI": self.fr_ini.grid()
        elif fmt == "Binaire": self.fr_bin.grid()
        elif fmt == "XDXF": self.fr_xdxf.grid()
        
        self.bt_Do['state'] = 'active'
        
        return


    def doExport(self):
        u"""
        Exporter dans un fichier d'un autre format
        (suite à action sur le bouton 'Exporter')
        """
        
        fmt = self.lst_fmt.get(ACTIVE) # format : texte de la ligne en sélection dans la liste des formats
        
        dirSearch = app.ini.get("DicDefaultPath",os.path.expanduser('~'))
        
        # ===============================================================================
        # === Interroger (si format source = Ling et cible != Ling ou preling et != WB)                                       
        # ===============================================================================
        
        flagfusion = False
        if app.dic.type.startswith("ling") and fmt != 'LING' and fmt != 'preling' and fmt != 'WB':
            flagfusion = tkMessageBox.askyesno(parent=self,
                    title="Exporter à partir d'un dictionnaire Ling",
                    message="Le format Ling différencie un champ de traductions dites \"courtes\" et un champ de traductions dites \"longues\" (incluant infos, exemples, etc.).\n\nVoulez-vous fusionner ces deux champs dans le nouveau dictionnaire ?\nSi vous choisissez \"Non\", seules les traductions courtes seront intégrées au nouveau dictionnaire.")
        
        
        # ===============================================================================
        if fmt == "XDXF": # === Création des répertoires (XDXF uniquement)               
        # ===============================================================================
            
            repxdxf = self.txt_repxdxf.get().strip()
            namexdxf = self.txt_namexdxf.get().strip()
        
            if repxdxf == "" or namexdxf == "":
                tkMessageBox.showwarning(parent=self,
                        title="Exporter vers XDXF",
                        message="Vous devez définir un dossier ET un nom.")
                return
        
        
            fullrep = os.path.join(repxdxf, namexdxf)
        
            # Créer le répertoire du dico si besoin
            if not os.path.isdir(fullrep):
                try:
                    os.makedirs(fullrep)
                except:
                    return "Echec de la création du dossier du dictionnaire XDXF"
            
            # Enregistrer
            fich = os.path.join(fullrep, "dict.xdxf")
        
        
        
        # ======================================================================
        else: # === Affichage du sélecteur de fichier (tous formats sauf XDXF)  
        # ======================================================================
            
            fich = os.path.splitext(app.dic.path)[0] + "." + fmt.lower() # nom par défaut
            
            ft=""
            if fmt=="LING": ft =(('Fichiers Ling','*.ling'),('Tous les fichiers','*.*'))
            elif fmt=="preling": ft =(('Fichiers texte','*.txt'),('Tous les fichiers','*.*'))
            elif fmt=="WB": ft =(('Fichiers WB','*.wb'),('Tous les fichiers','*.*'))
            elif fmt=="DICT": ft =(('Fichiers Stardict','*.ifo *.dict'),('Tous les fichiers','*.*'))
            elif fmt=="CSV": ft =(('Fichiers délimités','*.csv *.tab *.txt'),('Tous les fichiers','*.*'))
            elif fmt=="INI": ft =(('Fichiers INI','*.ini'),('Tous les fichiers','*.*'))
            elif fmt=="Binaire": ft =(('Tous les fichiers','*.*'),)
            else: ft =(('Tous les fichiers','*.*'),)
            
            fich = tkFileDialog.asksaveasfilename(parent=self,
                        title=u"Exporter un dictionnaire sous...",
                        filetypes=ft,
                        initialdir = dirSearch,
                        initialfile = fich
                        )
            
            if not fich: return
            
            app['cursor'] = "watch"
            self['cursor'] = "watch"
            
            # S'assurer de la conformité de l'extension avec le format d'exportation
            # l'ajouter le cas échéant
            if not fich.endswith('.'+ fmt.lower()):
                fich += '.'+ fmt.lower()
        
        
        # -------------------------------------------
        # --- Afficher et paramétrer la progressbar  
        # -------------------------------------------
        
        self.progr.grid()
        self.progr.setMax(app.dic.lstbox.size())
        self.update()
        
        
        # ============================================
        # === Aiguillage en fonction du format        
        # ============================================
        
        if fmt == "LING":                           # Conversion vers Ling
            
            ret = self.doExportToLing(fich)
        
        if fmt == "preling":                        # Conversion vers preling
            
            ret = self.doExportToPreling(fich)
        
        elif fmt == "CSV" or fmt == "Binaire" or fmt == "INI": # Conversion vers CSV, INI ou binaire
            
            ret = self.doExportToBinIniCsv(fich, fmt, flagfusion)
        
        elif fmt == "DICT":                         # Conversion vers Dict
        
            flagtransgram=self.vTransGram.get()
            ret = self.doExportToDict(fich, flagfusion, flagtransgram=flagtransgram)
        
        elif fmt == "WB":                           # Conversion vers WB
            
            ret = self.doExportToWB(fich)
        
        elif fmt == "XDXF":                         # Conversion vers XDXF
            
            ret = self.doExportToXDXF(fich, flagfusion)
        
        
        # ===============================================================
        # === Gestion du message de retour de la conversion/exportation  
        # === (le retour est une chaîne vide si tout est OK)             
        # ===============================================================
        
        app['cursor'] = ""
        self['cursor'] = ""
        self.progr.grid_remove()
        
        # --- Un problème est survenu : avertir et quitter sans fermer la fenêtre 
        
        if ret != "":       
            tkMessageBox.showwarning(parent=self,
                    title="Linguae",
                    message=u"L'exportation a rencontré le problème suivant :\n\n" + ret)
            return
        
        
        # --- Conversion Ok : avertir et quitter en fermant la fenêtre
        
        app.updateMenuDic() # Mise à jour du menu des dicos
        
        if fmt == "Binaire":
            tkMessageBox.showinfo(parent=self,
                title=u"Linguae",
                message=u"Le dictionnaire a été exporté avec succès\nen format binaire personnalisé.")
            self.destroy()
            return
        elif fmt == "preling":
            tkMessageBox.showinfo(parent=self,
                title=u"Linguae",
                message=u"Le dictionnaire a été exporté avec succès\nen format texte preling.")
            self.destroy()
            return
        
        r = tkMessageBox.askyesno(parent=self,
                title=u"Linguae",
                message=u"Le dictionnaire a été exporté avec succès.\n\nVoulez-vous ouvrir ce nouveau dictionnaire ?")
        
        if r == False:
            self.destroy()
            return
        
        # --- Charger le nouveau dico
        
        app.loadDico(fich)
        
        self.destroy()
        
        return



    def doExportToLing(self, fichpath):
        u"""
        Exporte le dictionnaire courant
        dans le fichier Ling de chemin 'fichpath'
        """
        
        # -----------------------------------------
        # --- Confectionner la chaîne pré-ling     
        # -----------------------------------------
        
        
        # Protection du fichier ?
        protected1 = ""
        protected2 = ""
        if self.vProtec1.get() == 1: protected1="Linguae"
        if self.vProtec2.get() == 1: protected2="Linguae"
        
        strPreling = app.dic.toStrPreling(progr=self.progr, protected1=protected1, protected2=protected2)
        
        if strPreling[0] == "!":       # Echec de la conversion en preling
            return strPreling
        
        # -------------------------------------------------
        # --- Convertir la chaine pre-ling en chaîne Ling  
        # -------------------------------------------------
        
        strLing = DicObject.strPrelingToStrLing(strPreling=strPreling, cp="")
        
        if strLing[0] == "!":       # Echec de la conversion preling>ling
            return strLing
        
        
        
        # -------------------------------------------------
        # --- Enregistrer le fichier Ling                  
        # -------------------------------------------------
        
        try:
            ff = open(fichpath,'wb')
        except:
            return "Echec de l'enregistrement du fichier Ling"
        
        ff.write(strLing)
        ff.close()
        
        return ""



    def doExportToPreling(self, fichpath):
        u"""
        Exporte le dictionnaire courant
        dans le fichier preling de chemin 'fichpath'
        """
        
        # -----------------------------------------
        # --- Confectionner la chaîne pré-ling     
        # -----------------------------------------
        
        
        # Protection du fichier ?
        protected1 = ""
        protected2 = ""
        if self.vProtec1.get() == 1: protected1="Linguae"
        if self.vProtec2.get() == 1: protected2="Linguae"
        
        strPreling = app.dic.toStrPreling(progr=self.progr, protected1=protected1, protected2=protected2)
        
        if strPreling[0] == "!":       # Echec de la conversion en preling
            return strPreling
        
        # ------------------------------------------------
        # --- Conversion des sauts de lignes              
        # ------------------------------------------------
        
        br = self.vCrlf2.get()
        
        if br != "\n" :
            strPreling = strPreling.replace('\n', br)
            
        
        # -------------------------------------------------
        # --- Enregistrer le fichier Preling               
        # -------------------------------------------------
        
        try:
            ff = open(fichpath,'wb')
        except:
            return "Echec de l'enregistrement du fichier Preling"
        
        ff.write(strPreling)
        ff.close()
        
        
        return ""


    def doExportToXDXF(self, fichpath, flagfusion):
        u"""
        Exporte le dictionnaire courant
        dans le fichier XDXF de chemin 'fichpath'
        
        - flagfusion : fusion traductions courtes + longues (en cas de source Ling)
        """
        
        # --------------------------------------------
        # --- Initialiser la chaîne du fichier XDXF   
        # --------------------------------------------
        
        strfich = ['<?xml version="1.0" encoding="UTF-8" ?>',]
        
        strfich.append('<xdxf lang_from="" lang_to="" format="visual">')
        
        t= app.dic.dicName
        if t == "": t ="(dictionnaire sans nom)"
        strfich.append("<full_name>" + t + "</full_name>")
        
        t = app.dic.dicInfo
        if t != "" : t += "<br>"
        t += app.dic.dicStatus
        if t != "" : t += "<br>"
        t += app.dic.copyright
        if t == "": t ="(dictionnaire sans description)"
        strfich.append("<description>" + t + "</description>")
        
        
        # --------------------------------------
        # --- Scanner la liste des entrées      
        # --------------------------------------
        
        getdata = app.dic.getdata # accélérateur
        
        for i in range(app.lst_dico.size()):
            
            self.progr.setValue(i)                              # Afficher la progression
            
            data = getdata(i)                                   # Récupérer les données (> pyDictionnaire utf-8)
            
            strfich.append("<ar><k>" + data['mot'] + "</k>")    # Ajouter l'entrée
            
            chp2 = data['short'].strip()
            
            if flagfusion:
                t = data['long'].strip()
                if t != "":
                    chp2 += " ;--- " + t 
            
            chp2 = chp2.replace("<br>", "\n")
            
            strfich.append(chp2 + "</ar>")                      # Ajouter la traduction
        
        strfich.append('</xdxf>\n')
        
        
        # -------------------------------------
        # --- Enregistrer le fichier           
        # -------------------------------------
        
        try:
            ff = open(fichpath,'w')
        except:
            return "Echec de l'enregistrement du fichier XDXF"
        
        ff.write("\n".join(strfich))
        ff.close()
        
        
        return ""


    def doExportToBinIniCsv(self, fichpath, fmt, flagfusion):
        u"""
        Exporte le dictionnaire courant
        dans le fichier 'fichpath' binaire, INI ou CSV
        """
        
        flagEncodeError=False
        strfile = []
        sp = ""             # Séparateur de champ
        br = ""             # Terminateur de ligne
        
        
        # ---------------------------------------------------------------------------
        # --- Déterminer séparateur, terminateur et encodage en fonction du format   
        # ---------------------------------------------------------------------------
        
        if fmt == "CSV":
            
            sp = self.vSp.get()
            br = self.vCrlf.get()
            cp = self.w_csvenc.get().strip()
            
            if cp == "": cp ='latin-1'

        elif fmt == "INI":
            
            sp = '='
            br = self.vBrIni.get()
            cp = self.w_inienc.get().strip()
        
        else: # binaire
            
            for c in txt_spbin.get().split():
                sp += chr(int(c))
            
            for c in txt_crlfbin.get().split():
                br += chr(int(c))
            
            cp = w_binenc.get().strip()
            if cp == "": cp ='utf-8'
        
        
        # ----------------------------------------------
        # --- Scanner la liste des entrées              
        # ----------------------------------------------
        
        getdata = app.dic.getdata # accélérateur local
        
        for i in range(app.lst_dico.size()):
            
            self.progr.setValue(i)      # Afficher la progression
            
            data = getdata(i)           # Données (pyDictionnaire Utf-8)
            
            # --- Entrée de dico
            
            chp1 = data['mot'].strip().decode('utf-8')
            
            try :
                chp1 = chp1.encode(cp)
            except:
                flagEncodeError = True
            
            if fmt == "INI" and chp1[0] == '[': # Titre de section
                strfile.append(chp1 + br)         # on concatène et on passe à la ligne suivante
                continue
            
            
            # --- Traductions
            
            chp2 = data['short'].strip().decode('utf-8')
            
            if flagfusion:
                t = data['long'].strip()
                if t != "":
                    t = t.decode('utf-8') 
                    chp2 += " ;--- " + t 
            
            try:
                chp2 = chp2.encode(cp)
            except:
                flagEncodeError = True
            
            # Quitter si encodage impossible
            if flagEncodeError:
                return u"Ce dictionnaire ne peut être converti\navec l'encodage choisi : '"+ cp + u"'\n\nChoisissez un autre encodage et réessayez...\n\nSi Unicode est supporté par le(s) programme(s) destiné(s) à lire ces données,\npréférez 'utf-8' comme encodage."
            
            
            # --- Epurer les champs de leurs caractères séparateurs parasites
            if fmt == "CSV":
                if sp=="\t":
                    chp1 = chp1.replace('\t', '  ')
                    chp2 = chp2.replace('\t', '  ')
                elif sp==";":
                    chp1 = chp1.replace(';', ',')
                    chp2 = chp2.replace(';', ',')
                elif sp==",":
                    chp1 = chp1.replace(',', ';')
                    chp2 = chp2.replace(',', ';')
            elif fmt == "INI":
                chp1 = chp1.replace('=', ':')
                chp2 = chp2.replace('=', ':')
            
            # --- Concaténer champs, séparateur et terminateur
            strfile.append(chp1 + sp + chp2 + br)
        
        
        # -------------------------------------
        # --- Enregistrer le fichier           
        # -------------------------------------
        
        try:
            ff = open(fichpath,'wb')
        except:
            return "Echec de l'enregistrement du fichier"
        
        ff.write("".join(strfile))
        ff.close()
        
        return ""


    def doExportToDict(self, fichpath, flagfusion, flagtransgram=False):
        u"""
        Exporte le dictionnaire courant
        dans le fichier DICT de chemin 'fichpath'
        
        - flagfusion : fusion traductions courtes + longues (en cas de source Ling)
        - flagtransgram : tranfert de la grammaire entre crochet du champ "entrée" vers le champ "traduction"
        """
        
        strifo=[]   # Future chaîne du *.ifo
        stridx=[]   # Future chaîne du *.idx
        strdict=[]  # Future chaîne du *.dict
        #strsyn=[]  # Future chaîne du *.syn (désactivé, voir plus bas)
        
        
        # --------------------------------------
        # --- Scanner la liste des entrées      
        # --------------------------------------
        
        decal = 0
        chp2 = ""
        gram = []
        
        getdata = app.dic.getdata                   # Accélérateur local
        
        exGram = re.compile(r"\[.*\]")             # Motif regex pour extraire la grammaire entre crochets
        
        for i in range(app.lst_dico.size()):
            
            self.progr.setValue(i)                  # Afficher la progression
            
            data = getdata(i)                       # Récupérer les données (> pyDictionnaire utf-8)
            
            # --- Transfert optionnel de la grammaire
            if flagtransgram:
                
                try:
                    gram = re.findall(exGram, data['mot'])[0]
                
                except: # grammaire absente
                    pass
                
                else:   # grammaire présente
                    
                    data['mot'] = data['mot'].replace(gram, "").strip()
                    data['short'] = gram + "\n\n" + data['short'] 
                    
            
            
            
            # --- Données du *Dict
            
            decal = decal + len(chp2)               # Décalage (offset) de la traduction
            
            chp2 = data['short'].strip()
            
            if flagfusion:
                t = data['long'].strip()
                if t != "":
                    chp2 += "<br>--- " + t 
            
            
            strdict.append(chp2)                    # Ajouter la traduction dans le *.dict
            
            # --- Données du *Idx
            
            stridx.append(data['mot'] + '\0')       # Ajouter l'entrée dans le *.idx
            
            stridx.append(NumToStr32(decal))        # Ajouter le décalage (image chaîne en 32 bits) dans le *.idx
            stridx.append(NumToStr32(len(chp2)))    # Ajouter la longueur de la traduction (image chaîne en 32 bits) dans le *.idx
            
            # --- Données du *Syn
            
            #if data['syn'] != "":
            #    if not data['syn'].startswith("[("):
            #        
            #        chps = data['syn'].split(";")   # Chaîne des IDs des synonymes
            #        
            #        for chp in chps:
            #            try:
            #                idx = app.dic.IDs[chp]                     # ID > Index
            #                strsyn.append(app.dic.lstbox.get(idx).encode('utf8') + "\0" + NumToStr32(i)) 
            #            except KeyError: # l'index pointe sur du vide > on le saute
            #                pass               
            
            
        
        
        # ------------------------------------------------------
        # --- Enregistrer les fichiers *.idx, *.dict et *.syn   
        # ------------------------------------------------------
        
        radicfich = os.path.splitext(fichpath)[0]   # Radical des différents fichiers Dict
        
        ff = open(radicfich + ".idx",'wb')          # Enregistrer le fichier *.idx
        ff.write("".join(stridx))
        ff.close()
        
        ff = open(radicfich + ".dict",'wb')         # Enregistrer le fichier *.dict
        ff.write("".join(strdict))
        ff.close()
        
        
        # NB : partie  désactivée car les synonymes Dict ne correpondent pas aux synonymes Ling
        # les synonymes ling sont uniquement des synonymes externes (= qui ne sont pas eux-mêmes des entrées du dico)
        # qui pointent sur des entrées du dico
        #
        #if len (strsyn) > 0:
        #    strsyn.sort()                           # Trier le fichier *.syn
        #    ff = open(radicfich + ".syn",'wb')      # Enregistrer le fichier *.syn
        #    ff.write("".join(strsyn))
        #    ff.close()
        
        
        # ------------------------------------------------
        # --- Construire et enregistrer le fichier *.ifo  
        # ------------------------------------------------
        
        lignes = []
        
        ligne = "StarDict's dict ifo file".encode ('utf-8')
        lignes.append(ligne)
        
        ligne = "version=2.4.2".encode ('utf-8')
        lignes.append(ligne)
        
        ligne = "[options]".encode ('utf-8')
        lignes.append(ligne)
        
        ligne = "bookname=" + os.path.basename(radicfich)
        try   : ligne = ligne.encode ('utf-8')
        except: pass
        lignes.append(ligne)
        
        ligne = "wordcount=" + str(app.lst_dico.size())
        try   : ligne = ligne.encode ('utf-8')
        except: pass
        lignes.append(ligne)
        
        ligne = "idxfilesize=" + str(os.path.getsize(radicfich + ".idx"))
        try   : ligne = ligne.encode ('utf-8')
        except: pass
        lignes.append(ligne)
        
        ligne ="synwordcount=".encode ('utf-8')
        lignes.append(ligne)
        
        ligne = "sametypesequence=m".encode ('utf-8')
        lignes.append(ligne)
        
        ligne = "idxoffsetbits=32".encode ('utf-8')
        lignes.append(ligne)
        
        ligne = "author=" + ", ".join(app.dic.mainAuthors)
        try   : ligne = ligne.encode ('utf-8')
        except: pass
        lignes.append(ligne)
        
        ligne = "email=" + app.dic.contactAuthor
        try   : ligne = ligne.encode ('utf-8')
        except: pass
        lignes.append(ligne)
        
        ligne = "website=" + app.dic.dicUrl
        try   : ligne = ligne.encode ('utf-8')
        except: pass
        lignes.append(ligne)
        
        ligne = "description=" + app.dic.dicInfo
        try   : ligne = ligne.encode ('utf-8')
        except: pass
        lignes.append(ligne)
        
        ligne = "date=" + app.dic.versionDate
        try   : ligne = ligne.encode ('utf-8')
        except: pass
        lignes.append(ligne)
        
        t = "\n".join(lignes)
        t += "\n"
        
        # --- Enregistrer
        
        ff = open(radicfich + ".ifo",'wb')
        ff.write(t)
        ff.close()
        
        return ""



    def doExportToWB(self, fichpath):
        u"""
        Exporte le dictionnaire courant
        dans le fichier WB de chemin 'fichpath'
        """
        
        flagEncode1Error = False
        flagEncode2Error = False
        flagCut = False   # flag mis à True si des mots ont été tronqués durant la conversion
        
        strfile = []
        
        # ---------------------------------------------------------------
        # --- Encodages sélectionnés par l'utilisateur pour le WB cible  
        # ---------------------------------------------------------------
        
        cp1 = self.w_wbcp1.get().strip()        # Encodage langue 1
        if cp1 == "": cp1 ='cp1242'             # (par défaut)
        
        cp2 = self.w_wbcp2.get().strip()        # Encodage langue 1
        if cp2 == "": cp2 ='cp1242'             # (par défaut)
        
        
        # -----------------------------------------
        # --- Scannage de la liste des entrées     
        # -----------------------------------------
        
        getdata = app.dic.getdata   # accélérateur local
        
        for i in range(app.lst_dico.size()):
            
            self.progr.setValue(i)
            
            data = getdata(i)       # Extraire les données (pyDictionnaire Utf-8)
            
            # --- Entrée de dico
            
            chp1 = data['mot'].decode('utf-8')
            try :
                chp1 = chp1.encode(cp1)
            except:
                flagEncode1Error = True
            
            # --- Traduction
            
            chp2 = data['short'].decode('utf-8')
            try:
                chp2 = chp2.encode(cp2)
            except:
                flagEncode2Error = True
            
            # --- Si echec de l'encodage : retourner un message et quitter
            
            terr = u""
            if flagEncode1Error:
                app['cursor'] = ""
                self['cursor'] = ""
                terr += u"Encodage impossible de la langue source avec l'encodage choisi : "+ cp1 + "\n\n"
            if flagEncode2Error:
                app['cursor'] = ""
                self['cursor'] = ""
                terr += u"Encodage impossible de la langue cible avec l'encodage choisi : "+ cp2 + "\n\n"
            if terr != "":
                terr += u"Choisissez d'autres encodages et réessayez..."
                terr = u"Echec de l'exportation au format WB \n\n" + terr
                return terr
            
            # --- Epurer le champ1 de ses virgules (car virgule = séparateur WB)
            
            chp1 = chp1.replace(",", ";")
            
            # --- Formater les champs en longueur fixe WB (31 + 53)
            
            if len(chp1) > 30: flagCut = True
            if len(chp2) > 50: flagCut = True
            
            chp1 = chp1[:30].ljust(31,'\0')
            chp2 = chp2[:50].ljust(53,'\0')
            
            # Convertir les champs en leurs valeurs d'octets, puis reconvertir en caractères
            # pour éviter les messages d'erreurs d'encodage de Python                       
            # lors de la concaténation de champs d'encodages différents                     
            # (c'est chronophage et stupide mais je n'ai pas trouvé de meilleure combine..) 
            
            chp1 = map(ord, chp1)
            chp1 = map(chr, chp1)
            chp1 = "".join(chp1)
            
            chp2 = map(ord, chp2)
            chp2 = map(chr, chp2)
            chp2 = "".join(chp2)
            
            strfile.append(chp1 + chp2)
        
        # -----------------------------------
        # --- Enregistrer le fichier WB      
        # -----------------------------------
        
        try:
            ff = open(fichpath,'wb')
        except:
            return "Echec de l'enregistrement du fichier WB"
        
        ff.write("".join(strfile))
        ff.close()
        
        if flagCut:
            t = u"Le dictionnaire a été exporté au format WB,\nmais certaines données ont été tronquées."
        else:
            t = ""
        
        return t




class WindowImage(Toplevel):
    u"""
    Fenêtre de visualisation des images-ressources
    id : ID de l'entrée dont les images sont à viualiser
    idximg : index de l'image dans la liste des images
    """
    
    def __init__(self, id="", idximg=-1):
        
        Toplevel.__init__(self)
        
        if id == "": id = app.dic.curID
    
        self.title("Voir les illustrations")
        self.resizable(width=NO, height=NO)
        self.grab_set()      # fenêtre modale
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        # ---
        
        self.lab_see = Label(self, image=app.imglst.photo, width=300, height=300, borderwidth=2, relief=GROOVE)
        self.lab_see.grid(row=0, rowspan=10, column=0, padx=3, pady=3, ipadx=3, ipady=3)
        
        Label(self,text="Liste des images :").grid(row=0, column=1, ipadx=3, ipady=3)
        
        self.lst_img = lib.myscrlst.ScrolledList(self, height=10, onclic=self.seeImage)
        self.lst_img.grid(row=1, column=1, padx=2)
        
        Button(self, width=90, image=app.imglst.exit, compound=LEFT,text=" Fermer", command=self.destroy).grid(row=2, column=1, sticky="S")
        
        
        # Charger la liste des images
        lf = glob.glob(os.path.join(app.dic.path + "_res", "gif", id + "_*.gif"))
        for f in lf:
            self.lst_img.add(os.path.basename(f))
        
        # Afficher éventuellement une image à l'ouverture
        if idximg > -1:
            self.lst_img.selClear()
            self.lst_img.setSel(idximg)
            self.seeImage(idximg)
        
        
        
        return
    
    
    def seeImage(self, i, widget=None):
        u"""
        Clic sur une ligne de la liste d'images
        > 
        """
        
        # Chemin du fichier de l'image
        f =  os.path.join(app.dic.path + "_res", "gif", self.lst_img.getitem(i))
        
        # Convertir en objet Tkinter.PhotoImage
        self.curimage=PhotoImage(file=f)
        
        # Affiche l'image dans le Label
        self.lab_see['image'] = self.curimage   
        
        return
        


class WindowProperties(Toplevel):
    u"""
    Fenêtre d'affichage et d'édition des propriétés du dictionnaire courant
    """

    def __init__(self):
        
        Toplevel.__init__(self)
        
        self.title("Propriétés du dictionnaire courant")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        self['padx']=5
        
        fich = app.dic.path
        
        # Chemin du fichier
        Label(self, text="Chemin du fichier : "+ fich).grid(row=0, sticky=W, pady=4)
        
        # Taille du fichier
        sz = os.path.getsize(fich)
        if sz >=1024:
            sz=sz/1024
            sz=str(sz)+" Ko"
        else:
            sz=str(sz)+" oct."
        Label(self, text="Taille du fichier : "+ sz).grid(row=1, sticky=W, pady=4)
        
        # Date de modification
        t = time.localtime(os.path.getmtime(fich))
        t= str(t[2]).zfill(2) + "/" + str(t[1]).zfill(2) + "/" + str(t[0]) + "  " + str(t[3]).zfill(2) + ":" + str(t[4]).zfill(2)
        Label(self, text="Modifié le : " + t).grid(row=2, sticky=W, pady=4)
        
        # Nombre d'entrées du dictionnaire
        Label(self, text="Nombre d'entrées : " + str(app.dic.wordcount)).grid(row=3, sticky=W, pady=4)
        
        
        flagAllowEdit = False
        
        r=4
        # =====================================================================================
        if app.dic.type.startswith("ling"):       # === FORMAT LING                            
        # =====================================================================================
            
            Label(self, text="Format du dictionnaire : LING").grid(row=r, sticky=W, pady=4)
            
            Label(self, text=" ").grid(row=r+1, sticky=W, pady=4)
            Label(self, text="Contenu de l'en-tête du fichier :").grid(row=r+2, sticky=W)
            
            self.txt_hdr = ScrolledText(self, height=18)
            self.txt_hdr['font']=('Courier',10,'normal')
            self.txt_hdr.tag_config("normal", wrap=WORD, lmargin1=5, lmargin2=5, spacing1=5)
            self.txt_hdr['background']=self['background']
            
            self.txt_hdr.grid(row=r+3)
            
            # Charger l'en-tête
            app.dic.file.seek(app.dic.hdroffset)
            self.txt_hdr.insert(END, app.dic.file.read(app.dic.hdrsize).replace("\0", "\n"), "normal")
            
            self.txt_hdr['state'] = 'disabled'
            
            flagAllowEdit = True
            
        
        # =====================================================================================
        elif app.dic.type.startswith("dict"):       # === FORMAT DICT                          
        # =====================================================================================
            
            Label(self, text="Format du dictionnaire : DICT").grid(row=r, sticky=W, pady=4)
            
            Label(self, text=" ").grid(row=r+1, sticky=W, pady=4)
            Label(self, text="Contenu du fichier ifo :").grid(row=r+2, sticky=W)
            
            self.txt_hdr = ScrolledText(self, height=12)
            self.txt_hdr['font']=('Courier',10,'normal')
            self.txt_hdr.tag_config("normal", wrap=WORD, lmargin1=5, lmargin2=5, spacing1=5)
            self.txt_hdr['background'] = self['background']
            self.txt_hdr.grid(row=r+3, pady=4, padx=4)
            
            # Charger l'ifo
            fich = fich.replace(".dict.gz", ".ifo")
            fich = fich.replace(".dict.dz", ".ifo")
            fich = fich.replace(".dict", ".ifo")
            ff = open(fich, 'r')
            self.txt_hdr.insert(END,ff.read(), "normal")
            ff.close()
            self.txt_hdr['state'] = 'disabled'
            
            flagAllowEdit = True
        
        
        # =======================================================================
        elif app.dic.type.startswith("wb"):       # === FORMAT WB                
        # =======================================================================
            
            Label(self, text="Format du dictionnaire : WB").grid(row=r, sticky=W, pady=4)
            
            flagAllowEdit = False
        
        # =======================================================================
        elif app.dic.type.startswith("csv"):      # === FORMAT délimité          
        # =======================================================================
            
            Label(self, text="Format du dictionnaire : fichier texte délimité").grid(row=r, sticky=W, pady=4)
            
            flagAllowEdit = False
        
        # =======================================================================
        elif app.dic.type.startswith("ini"):      # === FORMAT ini               
        # =======================================================================
            
            Label(self, text="Format du dictionnaire : fichier INI").grid(row=r, sticky=W, pady=4)
            
            flagAllowEdit = False
        
        
        else:
            
            pass
        
        
        # === Frame des boutons
        fr_but=Frame(self)
        fr_but.grid()
        if flagAllowEdit:
            Button(fr_but, image=app.imglst.ok, compound=LEFT, text='  Valider', width=100, command=self.validClic).grid(row=0, column=0, pady=5, padx=20)
            self.bt_edit=Button(fr_but, image=app.imglst.edit, compound=LEFT, text='  Editer', width=100, command=self.editClic)
            self.bt_edit.grid(row=0, column=0, pady=5, padx=20)
        self.bt_cancel=Button(fr_but, image=app.imglst.exit, compound=LEFT, text='  Fermer', width=100, command=self.destroy)
        self.bt_cancel.grid(row=0, column=1, pady=5, padx=20)
        Button(fr_but, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=100, padx= 10,command=lambda: app.showHelp("propr")).grid(row=0, column=2, pady=8, padx=10)
        
        return


    def editClic(self):
        u"""
        Clic sur le bouton 'Editer les propriétés'
        """
        
        # Avertir
        r = tkMessageBox.askyesno(parent=self,
                    title=u"Editer les propriétés du dictionnaire",
                    message=u"L'édition des propriétés est destinée aux utilisateurs avancés\nconnaissant bien les spécifications du format concerné (voir Aide).\n\nDes propriétés incorrectes ou incorrectement formatées peuvent rendre le dictionnaire illisible.\n\nPour cette raison, une copie de sauvegarde du dictionnaire sera automatiquement enregistrée.\n\nContinuez ?")
        if r == False : return
        
        self.txt_hdr['background'] = 'white' 
        self.txt_hdr['state'] = 'normal'
        self.bt_cancel['text'] = '  Annuler'
        self.bt_edit.grid_remove() # Masquer le bouton > démasque le bouton 'Valider' au-dessous
        
        return
    
    def validClic(self):
        u"""
        Valider les modifs des propriétés
        """
       
        # ----------------------------------------------
        # --- Aiguillage suivant format                 
        # ----------------------------------------------
        
        if app.dic.type.startswith("ling"):
            
            self.doValidLing()
        
        elif app.dic.type.startswith("dict"): 
            
            self.doValidDict()
        
        # -------------------------------------------------------
        # --- Recharger le dictionnaire avec le nouveau fichier  
        # -------------------------------------------------------
        memi = app.dic.curIdx
        app.dic.file.close()
        app.loadDico(app.dic.path, showSplash=False)
        
        # --------------------------------------------------
        # --- Réafficher lemot antérieurement sélectionné   
        # --------------------------------------------------
            
        app.dic.lstbox.activate(memi)
        app.dic.lstbox.selection_set(memi)
        app.dic.lstbox.see(memi)
        app.selDisplayData(None)    
            
        # --- Fermer le fenêtre des propriétés
        
        self.destroy()
        
        return
        
        
    def doValidDict(self):
        u"""
        Valider les modifs des propriétés d'un dico Dict
        """
        # Chemin de l'ifo
        fichpath = app.dic.path
        fichifo = fichpath.replace(".dict.gz", ".ifo")
        fichifo = fichpath.replace(".dict.dz", ".ifo")
        fichifo = fichifo.replace(".dict", ".ifo")
        
        # Enregistrer la copie de sauvegarde du fichier ifo
        # au cas ou l'édition des propriétés soit invalide
        shutil.copy(fichifo, fichifo + ".back")
        
        # Chaîne de l'ifo
        strfile = self.txt_hdr.get("0.0", END)
        strfile = strfile.encode('utf-8')
        if not strfile.endswith("\n"):
            strfile += "\n"
        
        # Enregistrer l'ifo
        ff = open(fichifo, 'wb')
        ff.write(strfile)
        ff.close()
        
        
        return
    
    def doValidLing(self):
        u"""
        Valider les modifs des propriétés d'un dico Ling
        """
        
        # Enregistrer la copie de sauvegarde du fichier dictionnaire
        # au cas ou l'édition des propriétés soit invalide
        fichpath = app.dic.path
        shutil.copy(fichpath, fichpath + ".back")
        
        
        # Construire le nouveau bloc header
        
        strHdr = self.txt_hdr.get("0.0", END).strip()
        lignes = strHdr.splitlines()
        strHdr=[]
        for ligne in lignes:
            ligne=ligne.encode('utf-8')
            strHdr.append(ligne)
        
        strHdr="\0".join(strHdr)
        
        # Reconstruire le bloc de cartographie
        
        strMap=[]
        
        strMap.append("%ling/01.00.00")
        strMap.append(NumToStr32(70))               # offset en-tête
        strMap.append(NumToStr32(len(strHdr)))      # taille en-tête
        
        dlt = len(strHdr) - app.dic.hdrsize         # delta des offsets suivant
        
        strMap.append(NumToStr32(app.dic.wordlistoffset + dlt)) # offset des entrées
        strMap.append(NumToStr32(app.dic.wordlistsize))         # taille des entrées
        strMap.append(NumToStr32(app.dic.idlistoffset + dlt))   # etc...
        strMap.append(NumToStr32(app.dic.idlistsize))
        strMap.append(NumToStr32(app.dic.datamapoffset + dlt))
        strMap.append(NumToStr32(app.dic.datamapsize))
        strMap.append(NumToStr32(app.dic.dataoffset + dlt))
        strMap.append(NumToStr32(app.dic.datasize))
        strMap.append(NumToStr32(app.dic.img1offset + dlt))
        strMap.append(NumToStr32(app.dic.img1size))
        strMap.append(NumToStr32(app.dic.img2offset + dlt))
        strMap.append(NumToStr32(app.dic.img2size))
        
        strMap = "".join(strMap)
        
        # Reconstruire le fichier bloc par bloc
        
        strfich = []
        
        strfich.append(strMap)
        strfich.append(strHdr)
        
        app.dic.file.seek(app.dic.wordlistoffset)
        bloc = app.dic.file.read(app.dic.wordlistsize)
        strfich.append(bloc)
        
        app.dic.file.seek(app.dic.idlistoffset)
        bloc = app.dic.file.read(app.dic.idlistsize)
        strfich.append(bloc)
        
        app.dic.file.seek(app.dic.datamapoffset)
        bloc = app.dic.file.read(app.dic.datamapsize)
        strfich.append(bloc)
        
        app.dic.file.seek(app.dic.dataoffset)
        bloc = app.dic.file.read(app.dic.datasize)
        strfich.append(bloc)
        
        app.dic.file.seek(app.dic.img1offset)
        bloc = app.dic.file.read(app.dic.img1size)
        strfich.append(bloc)
        
        app.dic.file.seek(app.dic.img2offset)
        bloc = app.dic.file.read(app.dic.img2size)
        strfich.append(bloc)
        
        strfich = "".join(strfich)
        
        
        # Enregistrer le nouveau fichier Ling
        
        app.dic.file.close()
        ff = open(app.dic.path, 'wb')
        ff.write(strfich)
        ff.close()
        
        
        return
    


class WindowEncodeOpen(Toplevel):
    u"""
    Boîte de dialogue de choix des encodages de caractères, à l'ouverture d'un dico
    Puis :
    - Lance l'ouverture du dico
    - Demande si les encodages doivent être mémorisés
    """
    
    
    def __init__(self, cp1="", cp2=""):
        u"""
        Interface de la fenêtre
        """
        
        Toplevel.__init__(self)
        
        self.title("Choix des encodages de caractères")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        
        # Données de sortie de la fenêtre
        
        self.cp1 = ""           # encodage langue 1
        self.cp2 = ""           # encodage langue 2
        self.annuler = False    # flag d'annulation du chargment du dico
        
        
        # --- Sélecteurs des encodages
        
        if cp1 == "": cp1 = "cp1252"
        if cp2 == "": cp2 = "cp1252"
        
        self.txt_cp1=lib.myencsel.EncodingSelector(self, text=cp1, title="Encodage langue 1 :")
        self.txt_cp1.grid(row=0, column=0, padx=5)
        
        self.txt_cp2=lib.myencsel.EncodingSelector(self, text=cp2, title="Encodage langue 2 :")
        self.txt_cp2.grid(row=0, column=1, padx=5)
        
        
        # -------------------------
        # --- Frame des Boutons    
        # -------------------------
        fr_but=Frame(self)
        fr_but.grid(row=1, column=0, columnspan=4, pady=8, sticky="WENS")
        
        bt_Do=Button(fr_but, image=app.imglst.open, compound=LEFT, text=" Ouvrir", width=70, padx= 10, command=self.doOpen)
        bt_Do.grid(row=0, column=0, pady=8, padx=10)
        Button(fr_but, image=app.imglst.exit, compound=LEFT, text=' Annuler', width=70, command=self.doCancel, padx= 10).grid(row=0, column=1, pady=8, padx=10)
        Button(fr_but, image=app.imglst.hlp, compound=LEFT, text=" Aide", width=70, padx= 10,command=lambda: app.showHelp("opendic")).grid(row=0, column=2, pady=8, padx=10)
        
        self.mainloop() # fenêtre bloquante
        
        return
    
    
    def doOpen(self):
        u"""
        Fermer la boîte de dialogue
        pour ouvrir le fichier avec les encodages sélectionnés
        """
        
        self.cp1 = self.txt_cp1.get()
        self.cp2 = self.txt_cp2.get()
        
        self.quit()     # sortie du mainloop de la fenêtre bloquante
        
        # Nb : la fenêtre sera détruite par la procédure appelante
        # après récupération de ses données
        
        return


    def doCancel(self):
        u"""
        Fermer la boîte de dialogue sans ouvrir le dico
        """
        
        self.annuler = True
        
        self.quit()     # sortie du mainloop de la fenêtre bloquante
        
        # Nb : la fenêtre sera détruite par la procédure appelante
        # après récupération de ses données
        
        return



class WindowImport(Toplevel):
    u"""
    Fenêtre d'importation d'un fichier CSV
    au format "Preling" vers un fichier "Ling"
    """

    def __init__(self):
        u"""
        Interface de la fenêtre
        """
        
        Toplevel.__init__(self, padx=10)
        
        self.title("Importer un pré-dictionnaire formaté")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+10)
        self.geometry('+' + left + '+' + top)
        
        # --- Etiquette d'info
        Label(self, text="(voir l'aide pour les spécifications du format d'importation)").grid(row=0, column=0, columnspan=4, padx=5, pady=10)
        
        
        # ----------------------------------------------
        # --- Frame du chemin de la source + encodage   
        # ----------------------------------------------
        
        fr_path=LabelFrame(self, text="Fichier source", padx=10, pady=10 )
        fr_path.grid(row=2, column=0, columnspan=4, pady=10, sticky="WENS")
        
        # Chemin du fichier CSV/preling
        Label(fr_path, text="Chemin du fichier texte formaté à importer :").grid(row=0, column=0, sticky="W")
        self.txt_path=Entry(fr_path,width=60, background="white")
        self.txt_path.grid(row=1, column=0)
        Button(fr_path, text="Parcourir", command=self.selFile).grid(row=1, column=1, sticky="W")
        
        # Encodage du fichier CSV/preling
        self.txt_cp=lib.myencsel.EncodingSelector(fr_path, text="cp1252", title="Encodage du fichier à importer :")
        self.txt_cp.grid(row=2, column=0)
        Label(fr_path, text="Cet encodage ne sera pris en compte que si le\nfichier ne contient aucune directive d'encodage").grid(row=3, column=0)
        
        
        # ----------------------------------------------
        # --- Frame du chemin de la cible               
        # ----------------------------------------------
        
        fr_cible=LabelFrame(self, text="Fichier cible", pady=10)
        fr_cible.grid(row=4, column=0, columnspan=4, pady=8, sticky="WENS")
        
        # Nom de la cible
        Label(fr_cible, text="Nom du fichier cible : ").grid(row=2, column=0)
        self.txt_namecible=Entry(fr_cible, width=25, background="white")
        self.txt_namecible.grid(row=2, column=1)
        self.txt_namecible.insert(END, "import.ling")
        
        
        # Options de la cible
        self.vOptCibleLing = StringVar()
        Radiobutton(fr_cible, text="Créer le fichier *.ling dans le dossier du fichier à importer", variable=self.vOptCibleLing, value="repsrc").grid(row=3, column=0, columnspan=4, padx=5, sticky=W)
        self.optRepdic=Radiobutton(fr_cible, text="Créer le fichier *.ling dans le dossier des dictionnaires", variable=self.vOptCibleLing, value="repdic")
        self.optRepdic.grid(row=4, column=0, columnspan=4, padx=5, sticky=W)
        self.vOptCibleLing.set("repsrc")
        
        # Désactiver l'option si le répertoire des dicos n'est pas encore défini
        repdic = app.ini.get('DicDefaultFolder','')
        
        if repdic == "" or not os.path.isdir(repdic):
            self.optRepdic['state'] = 'disabled'
            self.optRepdic['text'] = self.optRepdic['text'] + " [NON ENCORE DEFINI]"
        else:
            self.vOptCibleLing.set("repdic")
        
        
        # -----------------------------------
        # --- Frame inférieur des Boutons    
        # -----------------------------------
        r = 5
        fr_butImp=Frame(self)
        fr_butImp.grid(row=r, column=0, columnspan=4, pady=8, sticky="WENS")
        
        bt_Do=Button(fr_butImp, image=app.imglst.imprt, compound=LEFT, text="  Importer", width=100, padx= 10, command=self.DoImport)
        bt_Do.grid(row=0, column=0, pady=8, padx=10)
        Button(fr_butImp, image=app.imglst.exit, compound=LEFT, text='  Fermer', width=100, command=self.destroy, padx= 10).grid(row=0, column=1, pady=8, padx=10)
        Button(fr_butImp, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=100, padx= 10,command=lambda: app.showHelp("import")).grid(row=0, column=2, pady=8, padx=10)
        
        # Barre de progression
        self.progr = lib.myprogbr.ProgressBar(self, width=380)
        self.progr.grid(row=r+1, column=0, columnspan=4, pady=5)
        self.progr.grid_remove()
        
        return


    def selFile(self):
        u"""
        Sélection du fichier Preling à importer
        """
        
        dirSearch = app.ini.get("DicDefaultPath",os.path.expanduser('~'))
        
        fich = tkFileDialog.askopenfilename(parent=self,
                            title=u"Ouvrir un pré-dictionnaire formaté",
                            filetypes=(
                            ('Fichiers délimités','*.csv *.tab *.txt *.preling'),
                            ('Tous les fichiers','*.*')),
                            initialdir=dirSearch
                            )
        
        if not fich: return   # L'utilisateur a cliqué 'Annuler' dans la boîte de dialogue
        
        # Afficher le chemin du fichier source dans la case dédiée
        self.txt_path.delete(0,END)
        self.txt_path.insert(END, fich)
        
        # Afficher le nom du fichier cible dans la case dédiée
        self.txt_namecible.delete(0,END)
        self.txt_namecible.insert(END,os.path.splitext(os.path.basename(fich))[0] + ".ling")
        
        return



    def DoImport(self):
        u"""
        Importe un fichier texte Preling
        en appelant la méthode statique de l'objet Dictionnaire
        """
        
        # Fichier source preling
        fichscrpath = self.txt_path.get()
        
        if fichscrpath.strip() == "":
            tkMessageBox.showwarning(parent=self,
                    title=u"Importer un fichier",
                    message=u"Vous devez choisir un fichier à importer !")
            return
        
        
        # --------------------------------------------------------
        # --- Construire et vérifier le chemin du fichier cible   
        # --------------------------------------------------------
        
        
        # Chemin : enregistrer dans le répertoire des dictionnaires
        if self.vOptCibleLing.get() == "repdic":
            fichciblepath = os.path.join(app.ini.get('DicDefaultFolder',''), self.txt_namecible.get())
        
        # Chemin : enregistrer dans le même répertoire que le fichier source
        else:
            fichciblepath = os.path.join(os.path.dirname(fichscrpath), self.txt_namecible.get())
        
        # Forcer l'extension ling
        if not fichciblepath.endswith(".ling"):
            fichciblepath += ".ling" 
        
        # Vérifier si le fichier existe déjà
        if os.path.isfile(fichciblepath):
            r = tkMessageBox.askyesno(parent=self,
                    title=u"Importer un fichier",
                    message=u"Le fichier cible '" + fichciblepath + u"' existe déjà.\n\nVoulez-vous le remplacer ?")
            if r == False: return
        
        
        # -------------------------------------------------
        # --- Ouvrir le fichier source Preling             
        # --- et le recopier dans une chaîne               
        # -------------------------------------------------
        
        cp = self.txt_cp.get() # Encodage du fichier source Preling
        
            
        ff = ""
        lignes = ""
        try:
            ff = open(fichscrpath,'rb')
        except:
            if fichscrpath =="":
                msg = u"Entrez le chemin d'un fichier source à importer !\n\nPressez le bouton 'Parcourir' pour rechercher sur le disque.\n\nTout fichier texte tabulé, CSV ou au format preling est valide."
            else:
                msg = u"Fichier source introuvable !"
                    
            tkMessageBox.showwarning(parent=self,
                    title=u"Linguae",
                    message=msg)
            return
        
        strPreling = ff.read()
        ff.close()
        
        # ------------------------------------------------
        # --- Convertir la chaîne Preling en chaîne Ling  
        # ------------------------------------------------
        
        strLing = DicObject.strPrelingToStrLing(strPreling=strPreling, cp=cp)
        
        
        if strLing.startswith("!"): # Echec de la conversion : la chaîne Ling est remplacée par un message
            tkMessageBox.showwarning(parent=self,
                    title=u"Linguae",
                    message=u"La conversion à échouée, pour la raison suivante :\n\n" + strLing)
            self.destroy()
            return
        
        # -----------------------------------------
        # --- Enregistrer le nouveau fichier Ling  
        # -----------------------------------------
        
        try:
            ff = open(fichciblepath,'wb')
            ff.write(strLing)
            ff.close()
        except:
            tkMessageBox.showwarning(parent=self,
                    title=u"Importer un fichier",
                    message=u"Echec de l'enregistrement du fichier,\nvérifiez la validité du nom de fichier.")
            return
        
        
        
        # --------------------------------------------------------
        # --- Message "conversion ok" et demander pour ouverture  
        # --------------------------------------------------------
        
        r = tkMessageBox.askyesno(parent=self,
                title=u"Linguae",
                message=u"L'importation avec conversion au format Ling a été effectuée avec succès.\n\nVoulez-vous ouvrir le nouveau dictionnaire ?")
        
        if r == False:
            self.destroy()
            return
        
        # --- Charger le nouveau dico
        
        app.loadDico(fichciblepath)
        
        self.destroy()
        
        return



class WindowSort(Toplevel):
    u"""
    Fenêtre de tri du dictionnaire
    (permet le paramétrage du tri)
    """
    
    def __init__(self):
        
        
        if app.dic.type in ("dictdz", "dictgz"):
            app.hideProgr()
            tkMessageBox.showinfo(parent=self,
                    title=u"Trier le dictionnaire",
                    message=u"Ce dictionnaire est au format DICT compressé, il ne peut être modifié.\n\nPour le trier, vous devrez d'abord le convertir au format DICT non compressé\nou tout autre format géré par Linguae (Menu 'Fichier' > 'Exporter').")
            return
        
        
        Toplevel.__init__(self, pady=5, padx=5)
        
        self.title("Trier le dictionnaire")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+80)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        
        # --- Zone texte des équivalences de tri             
        self.txt_equ=ScrolledText(self, width=9, height=20, padx=4, background="white")
        self.txt_equ.grid(row=0, rowspan=4, column=0, sticky="W")
        self.txt_equ['font'] = app.lst_dico['font']
        
        
        # ---
        Label(self, text="Vous pouvez personnaliser le tri du dictionnaire\nen ajoutant dans la liste de gauche des équivalences\nde caractères pour le tri alphabétique, suivant\nla syntaxe suivante :\n\n <motif de caractères du dico>=<motif de caractères équivalents>\nUne équivalence par ligne.\n\nExemple : ö = oe\n(signifie que le caractère ö sera trié comme s'il s'agissait de 'oe')", justify=LEFT).grid(row=0, column=1, padx=3)
        Label(self, text="Si la liste reste vide, les critères de tri alphabétique par défaut\nde votre plateforme seront appliqués au dictionnaire.", justify=LEFT).grid(row=1, column=1, padx=3, sticky="W")
        
        bt_saveEqu=Button(self, text=" Sauvegarder les paramètres de tri ", command=self.saveEqu)
        bt_saveEqu.grid(row=3, column=1, sticky="N")
        
        
        # -------------------------
        # --- Frame des Boutons    
        # -------------------------
        r = 4
        fr_butOpt=Frame(self)
        fr_butOpt.grid(row=r, column=0, columnspan=4, pady=8, sticky="WENS")
        
        self.bt_sort=Button(fr_butOpt, image=app.imglst.sortcr, compound=LEFT, text="  Trier", width=100, padx= 10, command=self.okSort)
        self.bt_sort.grid(row=0, column=0, pady=8, padx=10)
        self.bt_esc=Button(fr_butOpt, image=app.imglst.exit, compound=LEFT, text="  Annuler", width=100, padx= 10, command=self.destroy)
        self.bt_esc.grid(row=0, column=1, pady=8, padx=10)
        Button(fr_butOpt, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=100, padx= 10,command=lambda: app.showHelp("sortdic")).grid(row=0, column=2, pady=8, padx=10)
        
        
        # -----------------------------------------------
        # --- Charger les motifs d'équivalence de tri    
        # -----------------------------------------------
        
        if app.dic.type.startswith("ling"):
            
            if len(app.dic.sortEquPatterns) > 0:
                if app.dic.sortEquPatterns[0] != "":
                    for ptrn in app.dic.sortEquPatterns:
                        ptrn = ptrn.replace(":"," = ")
                        self.txt_equ.insert(END, ptrn + "\n")
        
        else:
            
            dicpath = app.dic.path
            
            # Homogénéiser les chemins dict (ifo, dict ...)
            if app.dic.type.startswith("dict"):
                dicpath = os.path.splitext(app.dic.path)[0]+ ".dict"
                
            
            try:
                ff = open(os.path.join(dicpath + "_res", "sort.txt"), 'rb')
                self.txt_equ.insert(END, ff.read())
                ff.close()
            except:
                pass
            
        
        return
    
    
    
    def saveEqu(self):
        u"""
        Sauvegarder les équivalences de tri dans le fichier du dictionnaire (format Ling)
        ou dans le dossier des ressources du dico (autres formats)
        
        Sauvegarde sur demande explicite de l'utilisateur (bouton de commande)
        """
        
        self['cursor'] = "watch"
        app['cursor'] = "watch"
        self.update()
        app.update()
        
        
        # =========================================================
        if app.dic.type.startswith("ling"):   #                    
        # =========================================================
        
            # Confectionner la nouvelle valeur de l' attribut (un tupple de string)
            lignes = self.txt_equ.get('1.0',END).encode('utf-8')
            lignes = lignes.splitlines()
            strequs = []
            for ligne in lignes:
                if '=' in ligne:
                    chp = ligne.split('=')
                    strequs.append (chp[0].strip() + ":" + chp[1].strip())
            
            app.dic.sortEquPatterns = tuple(strequs)
            
            # Confectionner la chaîne du fichier modifié
            strling = DicObject.strPrelingToStrLing(strPreling=app.dic.toStrPreling())
            
            # Enregistrer le fichier modifié
            app.dic.file.close()
            ff = open(app.dic.path, "wb")
            ff.write(strling)
            ff.close ()
            
            # Réouvrir le fichier
            app.dic.file=open(app.dic.path, "rb")
        
        
        
        # ===========================================================
        else:   # autres formats                                     
        # ===========================================================
            
            
            # -----------------------------------------------------------
            # --- Créer le répertoire de ressource s'il n'existe pas     
            # -----------------------------------------------------------
            
            dicpath = app.dic.path
            
            # Homogénéiser les chemins dict (ifo, dict ...)
            if app.dic.type.startswith("dict"):
                dicpath = os.path.splitext(app.dic.path)[0]+ ".dict"
                    
            
            repRes = dicpath + "_res"
            
            if not os.path.isdir(repRes):
                try:
                    os.makedirs(repRes)
                except:
                    tkMessageBox.showwarning(parent=self,
                        title=u"Importer une image",
                        message=u"Le répertoire de ressources ne peut être créé,\nvérifiez les droits en écriture sur le répertoire du dictionnaire.")
                    return
            
            
            # ------------------------------------------------------
            # --- Enregistrer le fichier ressource                  
            # ------------------------------------------------------
            
            ff = open(os.path.join(repRes,"sort.txt"), 'wb')
            ff.write(self.txt_equ.get('1.0', END).strip())
            ff.close()
            
            
            
            
        
        # ---
        
        self['cursor'] = ""
        app['cursor'] = ""
        
        return
    
    
    def okSort(self):
        
        self.bt_sort['state'] = "disabled"
        self.bt_esc['state'] = "disabled"
        
        self['cursor'] = "watch"
        app['cursor'] = "watch"
        self.update()
        app.update()
        
        # --- Construire le pyDictionnaire des équivalences
        lignes = self.txt_equ.get('1.0',END).encode('utf-8')
        lignes = lignes.splitlines()
        equs={}
        for ligne in lignes:
            if '=' in ligne:
                chp = ligne.split('=')
                equs[chp[0].strip()] = chp[1].strip()
        
        app.showProgr()                 # Afficher la barre de progression de la fenêtre principale
        
        app.dic.sortdic(equs=equs)      # Trier en utilisant les équivalences
        
        app.hideProgr()                 # Masquer la barre de progression de la fenêtre principale
        
        self['cursor'] = ""
        app['cursor'] = ""
        
        self.destroy()
        
        return



class WindowFind(Toplevel):
    
    u"""
    Fenêtre de recherche dans la liste des mots
    """

    def __init__(self):
        
        Toplevel.__init__(self, padx=5, pady=3)
        
        self.title("Chercher")
        self.resizable(width=NO, height=NO)
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        
        # ----------------------------
        # --- Interface graphique     
        # ----------------------------
        
        
        # --- Frame de la case et des boutons
        fr_top = Frame(self)
        fr_top.pack()
        
        # Case de recherche
        self.txt_motif = Entry(fr_top, width=30, background='white')
        self.txt_motif.pack(side=LEFT)
        self.txt_motif.bind('<Return>', self.findClick)
        
        # Boutons
        self.bt_findrev = Button(fr_top, text="<< Précédant", width =12, command= lambda : self.findClick(None, reverse=True))
        self.bt_findrev.pack(side=LEFT)
        self.bt_find = Button(fr_top, text="Suivant >>", width =12, command= lambda : self.findClick(None))
        self.bt_find.pack(side=LEFT)
        Button(fr_top, text="Fermer", width =12, command=self.destroy).pack(side=LEFT)
        Button(fr_top, image=app.imglst.hlp, command=lambda: app.showHelp("findword")).pack(side=LEFT)
        
        # --- Frame des options
        fr_opt = Frame(self, pady=2)
        fr_opt.pack(side=LEFT, expand=True, fill=BOTH)
        
        self.vOptCase = IntVar()
        self.vOptStart = IntVar()
        self.vOptLig = IntVar()
        self.vOptWords = IntVar()
        self.vOptWords.set(1)
        self.vOptNotices = IntVar()
        self.vOptIDs = IntVar()
        
        fr_crit=LabelFrame(fr_opt, text=" Options ", relief=GROOVE,)
        fr_crit.pack(side=LEFT)
        Checkbutton(fr_crit, text="Recherche sensible à la casse", variable=self.vOptCase).grid(row=0, column=0, sticky="W")
        self.chk_start=Checkbutton(fr_crit, text="Recherche en début des mots", variable=self.vOptStart)
        self.chk_start.grid(row=1, column=0,sticky="W")
        Checkbutton(fr_crit, text="Recherche sensible aux ligatures (œ, æ, ...)", variable=self.vOptLig).grid(row=2, column=0,sticky="W")
        
        fr_dom=LabelFrame(fr_opt, text=" Domaine de recherche ", relief=GROOVE,)
        fr_dom.pack(side=LEFT, expand=True, fill=BOTH)
        
        self.chk_wd=Checkbutton(fr_dom, text="Entrées", variable=self.vOptWords)
        self.chk_wd.grid(row=0, column=0, sticky="W")
        
        self.chk_not=Checkbutton(fr_dom, text="Notices", variable=self.vOptNotices)
        self.chk_not.grid(row=1, column=0, sticky="W")
        self.chk_not.bind('<ButtonRelease-1>', self.onClickNotices)
        
        chk_IDs=Checkbutton(fr_dom, text="Identifiants (IDs)", variable=self.vOptIDs)
        chk_IDs.grid(row=2, column=0, sticky="W")
        chk_IDs.bind('<ButtonRelease-1>', self.onClickIDs)
        
        
        return


    def onClickIDs(self, event):
        u"""
        Clic sur la case à cocher 'Recherche dans les IDs'
        """
        
        # Nb : le changement de valeur de la variable Tkinter a lieu après le ButtonRelease
        # on teste donc son état antérieur !
        
        if self.vOptIDs.get() == 0:
            # désactiver la recherche des entrées et notices
            self.vOptNotices.set(0)
            self.chk_not['state'] = 'disabled'
            self.vOptWords.set(0)
            self.chk_wd['state'] = 'disabled'
            # réactiver la recherche en début de mot
            self.chk_start['state'] = 'normal'
        else:
            # réactiver la recherche des entrées et notices
            self.chk_not['state'] = 'normal'
            self.vOptWords.set(1)
            self.chk_wd['state'] = 'normal'
        
        return



    def onClickNotices(self, event):
        u"""
        Clic sur la case à cocher 'Recherche dans les notices'
        """
        
        # Nb : le changement de valeur de la variable Tkinter a lieu après le ButtonRelease
        # on teste donc son état antérieur !
        
        if self.vOptNotices.get() == 0:
            # désactiver la recherche en début de mot
            self.vOptStart.set(0)
            self.chk_start['state'] = 'disabled'
        else:
            # réactiver la recherche en début de mot
            self.chk_start['state'] = 'normal'
        
        
        return
    

    def findClick(self, event, reverse=False, ):
        u"""
        Chercher un motif n'importe où dans la liste,
        à partir du mot en sélection
        """
        
        motif = self.txt_motif.get().strip()            # motif à rechercher
        
        app.dic.lstbox.selection_clear(0, END)          # Désélectionner tout dans la listbox des entrées
        
        
        # --------------------------------------------
        # --- Index de départ de la recherche         
        # --------------------------------------------
        
        if reverse:
            decal = int(app.dic.curIdx)-1 # décalage = index du mot précédant celui en sélection
        else:
            decal = int(app.dic.curIdx)+1 # décalage = index du mot suivant celui en sélection
            
        if decal > app.dic.lstbox.size()-1:
            decal = 0
        elif decal < 0:
            decal = 0
        
        # ---------------------------------
        # --- Options de la recherche      
        # ---------------------------------
        
        # Sens de la recherche
        #   (passé dans le paramètre-argument 'reverse')
        
        
        # Domaine de la recherche
        if self.vOptWords.get() and self.vOptNotices.get():
            dom = 'both'
        elif self.vOptWords.get():
            dom = 'idx'
        elif self.vOptNotices.get():
            dom = 'notice'
        elif self.vOptIDs.get():
            dom = "IDs"
        
        # Sensibilité à la casse
        if self.vOptCase.get() == 1:
            ignorecase = False
        else:
            ignorecase = True
        
        # Recherche des débuts de mots
        if self.vOptStart.get() == 1:
            debut = True
        else:
            debut = False
        
        # Sensibilité aux ligatures
        if self.vOptLig.get() == 1:
            ignorelig = False
        else:
            ignorelig = True
        
        
        # ---------------------
        # --- Chercher         
        # ---------------------
        
        i = app.dic.idxfind(motif=motif, dom=dom, decal=decal, debut=debut, ignorecase=ignorecase, ignorelig=ignorelig, reverse= reverse)
        
        
        # ----------------------------------------------
        # --- Résultat de la recherche > sélectionner   
        # ----------------------------------------------
        
        if not i is None:
            app.dic.lstbox.activate(i)
            app.dic.lstbox.selection_set(i)
            app.dic.lstbox.see(i)
            app.selDisplayData(event)
            app.txt_search['foreground']='black'
        else:
            if reverse : t = u"le début du dictionnaire est atteint."
            else       : t = u"la fin du dictionnaire est atteinte."
            tkMessageBox.showinfo(parent=self,
                    title=u'Chercher',
                    message= t + u"\n\nLa chaîne '" + motif + "' est introuvable")
        
        
        return




class WindowSetIcon(Toplevel):
    u"""
    Fenêtre de modification des icônes de langue
    """

    def __init__(self):

        Toplevel.__init__(self, padx=5, pady=5)
        
        self.title("Modifier les icônes")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        # ---
        
        Label(self, text="Image Langue source : ").grid(row=0, column=0, sticky="W")
        self.lab_ic1=Label(self, pady=5)
        self.lab_ic1.grid(row=0, column=2)
        if app.dic.img1 != "" :
            self.ic1 = PhotoImage(data=app.dic.img1)
            self.lab_ic1['image'] = self.ic1
        else:
            self.lab_ic1['text'] = "(pas d'image)"
        
        Label(self, text="Image Langue cible : ").grid(row=1, column=0, sticky="W")
        self.lab_ic2=Label(self, pady=5)
        self.lab_ic2.grid(row=1, column=2)
        if app.dic.img2 != "" :
            self.ic2 = PhotoImage(data=app.dic.img2)
            self.lab_ic2['image'] = self.ic2
        else:
            self.lab_ic2['text'] = "(pas d'image)"
        
        
        # -------------------------
        # --- Frame des Chemins    
        # -------------------------
        
        fr_path=Frame(self)
        fr_path.grid(row=2, column=0, columnspan=4, padx=5, pady=10, sticky="WENS")
        
        # Chemin du gif1
        Label(fr_path, text="Nouvelle image gif pour la langue source (facultatif) :").grid(row=3, column=0, sticky="W")
        self.txt_pathgif1=Entry(fr_path,width=60, background="white")
        self.txt_pathgif1.grid(row=4, column=0, sticky="W")
        Button(fr_path, text="Parcourir", command=lambda:self.selGif(1)).grid(row=4, column=1, sticky="W")
        
        # Chemin du gif2
        Label(fr_path, text="Nouvelle image gif pour la langue cible (facultatif) :").grid(row=5, column=0, sticky="W")
        self.txt_pathgif2=Entry(fr_path,width=60, background="white")
        self.txt_pathgif2.grid(row=6, column=0, sticky="W")
        Button(fr_path, text="Parcourir", command=lambda:self.selGif(2)).grid(row=6, column=1, sticky="W")
        
        # -------------------------
        # --- Frame des Boutons    
        # -------------------------
        r = 3
        fr_but=Frame(self)
        fr_but.grid(row=r, column=0, columnspan=4, pady=8)
        
        self.bt_Do=Button(fr_but, image=app.imglst.ok, compound=LEFT, text="  Modifier", width=100, padx= 10, command=self.doModifIc)
        self.bt_Do.grid(row=0, column=0, pady=8, padx=10)
        Button(fr_but, image=app.imglst.exit, compound=LEFT, text='  Fermer', width=100, command=self.destroy, padx= 10).grid(row=0, column=1, pady=8, padx=10)
        Button(fr_but, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=100, padx= 10,command=lambda: app.showHelp("editic")).grid(row=0, column=2, pady=8, padx=10)
        
        return


    def selGif(self, n):
        u"""
        Sélection du fichier gif à inclure
        Affichage du chemin dans la case et prévisualisation de l'image
        """
        
        fich = tkFileDialog.askopenfilename(parent=self,
                        title=u"Inclure une image Gif",
                        filetypes=(
                        ('Fichiers gif','*.gif'),
                        ('Tous les fichiers','*.*'))
                        )
        
        if not fich: return   # L'utilisateur a cliqué 'Annuler' dans la boîte de dialogue
        
        if n == 1:
            
            try:
                self.txt_pathgif1.delete(0,END)
                self.txt_pathgif1.insert(END, fich)
                self.ic1 = PhotoImage(file=fich)
            except:
                self.txt_pathgif1.delete(0,END)
                self.ic1 = None
            
            self.lab_ic1['image'] =  self.ic1
        
        else:
            try:
                self.txt_pathgif2.delete(0,END)
                self.txt_pathgif2.insert(END, fich)
                self.ic2 = PhotoImage(file=fich)
            except:
                self.txt_pathgif2.delete(0,END)
                self.ic2 = None
            
            self.lab_ic2['image'] =  self.ic2
        
        return


    def doModifIc(self):
        u"""
        Modifier les icônes dans le fichier dictonnaire
        """
        
        gif1path = self.txt_pathgif1.get().strip()
        gif2path = self.txt_pathgif2.get().strip()
        
        if (gif1path=="") and (gif2path==""):
            tkMessageBox.showinfo(parent=self,
                    title=u"Modifier les icônes du dictionnaire",
                    message=u"Inscrivez au moins un chemin valide vers un fichier gif !")
            return
        
        # ------------------------------------------------------
        # --- Construire les chaînes base64 des blocs d'images  
        # --- et les intégrer au dictionnaire                   
        # ------------------------------------------------------
        
        gifBin = ""
        
        if gif1path != "":
            try:
                ff = open(gif1path, "rb")
            except:
                return "Fichier gif 1 introuvable !"
            
            gifBin = ff.read()
            ff.close()
            
            if gifBin[:4] != "GIF8":
                tkMessageBox.showwarning(parent=self, message= gif1path + " n'est pas un fichier GIF standard")
            else:
                app.dic.img1 = base64.encodestring(gifBin)
        
        
        gifBin = ""
        
        if gif2path != "":
            try:
                ff = open(gif2path, "rb")
            except:
                return "Fichier gif 2 introuvable !"
            
            gifBin = ff.read()
            ff.close()
            
            if gifBin[:4] != "GIF8":
                tkMessageBox.showwarning(parent=self, message= gif2path + " n'est pas un fichier GIF standard")
            else:
                app.dic.img2 = base64.encodestring(gifBin)
        
        # ------------------------------------------------
        # --- Mettre à jour le fichier dictionnaire       
        # ------------------------------------------------
        
        # Mémoriser l'index en cours
        try:
            i = int(app.lst_dico.curselection()[0]) # index du mot en sélection
        except:
            i = -1
        
        # Confectionner la nouvelle chaîne preling du dictionnaire
        
        strPreling = app.dic.toStrPreling(progr=None)
        
        # Confectionner la nouvelle chaîne Ling
        
        strLing = DicObject.strPrelingToStrLing(strPreling=strPreling)
        
        # Enregistre le nouveau fichier
        fichpath = app.dic.path
        app.dic.file.close()
        ff = open(fichpath, 'wb')
        ff.write(strLing)
        ff.close()
        
        # Recharge le dictionnaire avec le nouveau fichier
        app.loadDico(fichpath)
        
        # Réaffiche le mot sélectionné
        
        if i != -1:
            app.dic.lstbox.activate(i)
            app.dic.lstbox.selection_set(i)
            app.dic.lstbox.see(i)
            app.selDisplayData(None)
        
        self.destroy()
        
        return



class WindowHerit(Toplevel):
    u"""
    Fenêtre de construction d'un dictionnaire par héritage
    """
    
    def __init__(self):
        
        # Nb : Message dépendant de app et non self car avant instanciation de Toplevel
        tkMessageBox.showinfo(parent=app,
            title ="Construire un dictionnaire par héritage",
            message="Ce module permet de dériver un dictionnaire dit \"par héritage\" à partir de deux dictionnaires sources, suivant le schéma suivant :\n\n\t[ Dictionnaire 1 > 2 ] + [ Dictionnaire 2 > 3 ] => [ Dictionnaire 1 > 3 ]\n\nLa qualité du dictionnaire hérité sera très variable en fonction de la compatibilité de la rédaction des dictionnaires de départ, et ira de l'acceptable au quasi-inutilisable. Un tel processus automatique ne remplace pas un véritable dictionnaire rédigé humainement, ce n'est qu'un pis-aller palliant l'absence d'un dictionnaire existant.")
        
        
        Toplevel.__init__(self, padx=3)
        
        self.title("Construire un dictionnaire par héritage")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        # Frame des dicos disponibles
        wl=40
        fr_dic=Frame(self)
        fr_dic.grid(row=0, column=0, ipadx=5, ipady=5, sticky='W')
        Label(fr_dic, image=app.imglst.dicini, compound=LEFT, text=" Dictionnaire 1<>2").grid(column=0)
        self.lab_dic1=Label(fr_dic, width=wl, borderwidth=1, relief=SOLID, background='white', anchor='w')
        self.lab_dic1.grid(column=0, pady=2)
        self.lst_dic1=lib.myscrlst.ScrolledList(fr_dic, height=23, width=wl, onclic=self.onclicDico)
        self.lst_dic1.grid(column=0)
        Label(fr_dic, image=app.imglst.dicwb, compound=LEFT, text=" Dictionnaire 2<>3").grid(row=0, column=1)
        self.lab_dic2=Label(fr_dic, width=wl, borderwidth=1, relief=SOLID, background='white', anchor='w')
        self.lab_dic2.grid(row=1, column=1, pady=2)
        self.lst_dic2=lib.myscrlst.ScrolledList(fr_dic, height=23, width=wl, onclic=self.onclicDico)
        self.lst_dic2.grid(row=2, column=1)
        
        # Frame droit
        fr_D=Frame(self)
        fr_D.grid(row=0, column=1, ipadx=5)
        Label(fr_D, image=app.imglst.dicling, compound=LEFT, text=" Nom du dictionnaire hérité 1<>3").grid(row=0, column=0)
        self.txt_name=Entry(fr_D, width=25, background="white")
        self.txt_name.grid(row=1, column=0)
        self.txt_name.insert(END, "heritdic")
        
        self.bt_make=Button(fr_D, width=90, image=app.imglst.herit, compound=LEFT, text=" Construire", command=self.makeHeritDic, state='disabled')
        self.bt_make.grid(column=0, pady=8)
        self.bt_cancel=Button(fr_D, width=90, image=app.imglst.exit, compound=LEFT, text=" Fermer", command=self.destroy)
        self.bt_cancel.grid(column=0, pady=8)
        Button(fr_D, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=90, command=lambda: app.showHelp("herit")).grid(column=0, pady=8)
        self.lab_step=Label(fr_D, borderwidth=2, text=u"(Avancement)", relief=SUNKEN, background='white', width=40, height=14, justify=LEFT, anchor="n", font=("Helvetica",8,"normal"))
        self.lab_step.grid(column=0)
        
        # Options : choix du séparateur de signifiance et sensibilité à la casse
        fr_sep = Frame(self)
        fr_sep.grid(row=1, column=0, columnspan=10, ipady=3)
        
        Label(fr_sep, text="Groupe de caractère(s) marquant la fin de la zone à prendre en compte lors du croisement des données (par défaut : espace) :").grid(row=0, column=0)
        self.txt_sep=Entry(fr_sep,width=8, background="white")
        self.txt_sep.grid(row=0, column=1)
        
        self.vCasse=IntVar()
        Checkbutton(fr_sep, text="Sensible à la casse", variable=self.vCasse).grid(row=1, column=0, sticky="w")
        
        
        # Barre de progression
        self.progr=lib.myprogbr.ProgressBar(self, width= 750)
        self.progr.grid(row=2, column=0, columnspan=10, sticky='w')
        
        
        # ----------------------------------
        # --- Remplir les listes de dicos   
        # ----------------------------------
        
        dicosPath = app.ini.get('DicDefaultFolder','')
        
        self.listDirDic(dicosPath, self.lst_dic1)
        self.listDirDic(dicosPath, self.lst_dic2)
        
        return
    
    
    
    def onclicDico(self, i, idwidget):
        u"""
        Sélection d'un dico dans la liste
        """
        
        if idwidget == id(self.lst_dic1):
            self.lab_dic1['text']=self.lst_dic1.getselecteditem()
        else:
            self.lab_dic2['text']=self.lst_dic2.getselecteditem()
        
        if self.lab_dic1['text'] != "" and self.lab_dic2['text'] != "":
            self.bt_make['state'] = 'normal'
        
        return
    

    def listDirDic(self,dicosPath, lstbox):
        u"""
        Remplit une liste de dictionnaires avec le contenu du répertoire des dicos
        """
        
        lstdic=[]
        
        for rootDir, dirs, files in os.walk(dicosPath): 
            for f in files: 
                if os.path.splitext(f)[1].lower() in (".ling",".dict",".wb",".ini",".csv",".txt",".tab"):
                    lstdic.append(os.path.join(rootDir[len(dicosPath):],f)) 
        
        lstdic.sort(lambda x,y: cmp(x.lower(),y.lower())) # tri alphabétique insensible à la casse
        
        for f in lstdic:
            lstbox.add(f)
        
        return


    def makeHeritDic(self):
        u"""
        Construit le dictionnaire hérité
        """
        
        # ------------------------------------
        # --- Contrôle du chemin du fichier   
        # ------------------------------------
        
        dicrep = app.ini.get('DicDefaultFolder','')
        
        fichpath = os.path.join(dicrep, self.txt_name.get())
        
        if not fichpath.lower().endswith(".ling"):
            fichpath += ".ling"
        
        if os.path.isfile(fichpath):
            r = tkMessageBox.askyesno(parent=self,
                    title=u"Contruire un dictionnaire par héritage",
                    message=u"Le fichier '" + fichpath + u"' existe déjà.\n\nVoulez-vous le remplacer ?")
            if r == False: return
        
        self.bt_make['state']='disabled'
        self.bt_cancel['state']='disabled'
        
        
        # --------------------------------------------------
        # --- Instancier les deux objets dicos à hériter    
        # --------------------------------------------------
        
        self.lab_step['text'] += u"\n[ --- Démarrage du traitement global --- ]"
        
        self.lab_step['text'] += "\n(1) Instanciation du dictionnaire 1..." 
        dic1 = DicObject(os.path.join(dicrep, self.lab_dic1['text']), prgbar=self.progr)
        self.lab_step['text'] += u"\n>Terminé !"
        
        # Contrôler la protection
        # Si le fichier est protégé > avertir et quitter
        if dic1.protected1 !="" or dic1.protected2 !="":
            self.progr.setValue(-1)
            tkMessageBox.showinfo(parent=self,
                title=u"Construire un dictionnaire par héritage",
                message=self.lab_dic1['text'] + u"\n\nLes données de ce dictionnaire sont protégées,\nil ne peut donc pas être utilisé comme dictionnaire source.")
            self.bt_make['state']='normal'
            self.bt_cancel['state']='normal'
            self.lab_step['text']=""
            return    
        
        # ----
        
        self.lab_step['text'] += "\n(2) Instanciation du dictionnaire 2..." 
        dic2 = DicObject(os.path.join(dicrep, self.lab_dic2['text']), prgbar=self.progr)
        self.lab_step['text'] += u"\n> Terminé !"
        
        # Contrôler la protection
        # Si le fichier est protégé > avertir et quitter
        if dic2.protected1 !="" or dic2.protected2 !="":
            self.progr.setValue(-1)
            tkMessageBox.showinfo(parent=self,
                title=u"Construire un dictionnaire par héritage",
                message=self.lab_dic2['text'] + u"\n\nLes données de ce dictionnaire sont protégées,\nil ne peut donc pas être utilisé comme dictionnaire source.")
            self.bt_make['state']='normal'
            self.bt_cancel['state']='normal'
            return    
        
        # ------------------------------------------
        
        
        sepsign = self.txt_sep.get().strip()
        if sepsign == "":
            sepsign = " " # Espace, par défaut
        
        flagcasse = self.vCasse.get()
        
        # -------------------------------------------------------------------------------
        # --- Construire une liste minimale du dico 2 (accélère la recherche ultérieure) 
        # -------------------------------------------------------------------------------
        
        self.lab_step['text'] += u"\n(3) Construire un index temporaire..."
        
        words2=[]
        
        # Ne garder que la partie signifiante de l'entrée = jusqu'au premier espace par défaut
        # et mettre en minuscule (optionnel)
        if flagcasse:
            for w in dic2.innerlst:
                words2.append(w.split(sepsign)[0].strip())
        else:
            for w in dic2.innerlst:
                words2.append(w.split(sepsign)[0].strip().lower())
        
        self.lab_step['text'] += u"\n> Terminé !"
        
        
        # ----------------------------------------------------------------------
        # --- Construire une liste recevant les données du dictionnaire hérité  
        # ----------------------------------------------------------------------
        
        self.lab_step['text'] += u"\n(4) Analyse des données...\n(processus long, veuillez patienter)" 
        
        strHerit=[]
        
        subsep1 = dic1.subsep
        if subsep1 == "" : subsep1=";"
        
        # accélérateurs locaux
        word1get = dic1.innerlst.__getitem__
        data1get = dic1.getdata
        data2get = dic2.getdata
        dic1wcount = dic1.wordcount
        dic2wcount = dic2.wordcount
        
        self.progr.setMax(dic1wcount)
        
        #for i1 in range(200): # durant phase de mise au point 
        for i1 in range(dic1wcount):     # scanner les traductions du dico 1
            
            self.progr.setValue(i1)
            
            traducs1 = data1get(i1)['short'].split(subsep1) # champ des traductions du dico 1 > liste
            
            for traduc in traducs1: # Scanner les traductions élémentaires de chaque champ "traduction"     
                
                # Partie signifiante de la traduction = jusqu'au premier espace par défaut
                if flagcasse:
                    traduc = traduc.split(sepsign)[0].strip()
                else:
                    traduc = traduc.split(sepsign)[0].strip().lower()
                
                # rechercher une concordance de la traduction du dico1
                # dans les entrées du dico2
                
                try:
                    i2 = words2.index(traduc)
                except:
                    pass
                else:# Concordance trouvée ! 
                    strHerit.append(word1get(i1) + "\t" + data2get(i2)['short'])
        
        self.lab_step['text'] += u"\n> Terminé !"
        
        
        # -------------------------------------------------------
        # --- Convertir la liste en nouveau dictionnaire Ling    
        # -------------------------------------------------------
        
        self.lab_step['text'] += u"\n(5) Construction du dictionnaire hérité..."
        
        strHerit = "\n".join(strHerit)
        
        strLing = DicObject.strPrelingToStrLing(strPreling=strHerit)
        
        # --- Enregistrer le nouveau fichier
        ff = open(fichpath,"wb")
        ff.write(strLing)
        ff.close()
        
        self.lab_step['text'] += u"\n> Terminé !\n[ --- Fin du traitement global --- ]"
        self.update()
        
        # -----------------------------------------------------
        # --- Demander pour charger le nouveau dico            
        # -----------------------------------------------------
        
        r = tkMessageBox.askyesno(parent=self,
                    title=u"Contruire un dictionnaire par héritage",
                    message=u"Le dictionnaire a été créé avec succès.\n\nVoulez-vous le charger ?")
        
        if r == False:
            self.destroy()
            return
        
        # --- Charger
        
        try   : app.dic.file.close()
        except: pass
        
        app.loadDico(fichpath)
        
        app.update()
        
        # app.dic.sortdic()  # Trier le nouveau dictionnaire (inutile si dic 1 est trié au départ)
        
        app.updateMenuDic()
        
        self.destroy()
        
        
        return
    


class WindowAlpha(Toplevel):
    
    u"""
    Fenêtre de réglage du niveau de transparence (canal alpha)
    """
    
    def __init__(self):
        
        Toplevel.__init__(self)
        
        self.title("Réglage de la transparence")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        self.sl_trans = Scale(self, from_=0, to=20, resolution=1, orient=HORIZONTAL, borderwidth=1, sliderlength=40, length=450, tickinterval=1, width=12)
        self.sl_trans.bind('<ButtonRelease-1>', self.setAlpha)
        self.sl_trans.pack()
        
        Label(self, font='Helvetica 8 normal', text="Nb : la commande de transparence ne fonctionne pas sur toutes les plateformes\n et produit des résultats d'intensité variable suivant les cartes graphiques", justify=LEFT).pack()
        
        return
    
    
    def setAlpha(self,event):
        
        a = (20 - self.sl_trans.get())/20.0
        try:
            app.wm_attributes('-alpha', a)
        except:
            tkMessageBox.showwarning(parent=self,
                    title="Linguae",
                    message="Echec de l'activation de la transparence")
            return
        
        app.lift()
        
        # Nb : ne pas mémoriser la valeur dans l'ini,
        # car une fenêtre trop transparente est dure à récupérer au lancement...
        
        return
    


class WindowReverse(Toplevel):
    u"""
    Fenêtre de construction du dictionnaire inverse
    avec choix du séparateur des traductions et du chemin/nom de fichier
    """
    
    def __onclicFileType(self):
        u"""
        Clic sur les options de format de fichier cible
        """
        # activer/désactiver les cases du mode d'inversion
        if self.vFileType.get() in ("ling", "xdxf", "dict"):
            self.RB_std['state'] = "normal"
            self.RB_lg['state'] = "normal"
        else:
            self.vInv.set('std')
            self.RB_std['state'] = "disabled"
            self.RB_lg['state'] = "disabled"
        return
    
    
    def __init__(self):
        
        
        # ------------------------------------------------------
        # --- Contrôles préliminaires des flags :               
        # ---       flag isReversedic (False par défaut)        
        # ---       flag doReversedic (True par défaut)         
        # ------------------------------------------------------
        
        msg = ""
        if app.dic.isReverseDic:
            msg = u"Ce dictionnaire est déjà un dictionnaire issu d'une inversion automatique !\n\n"
        if not app.dic.doReverseDic:    
            msg += u"Les propriétés de ce dictionnaire mentionnent que ses données ne sont pas destinées à être inversées.\n\n"
            
        if msg != "":    
            msg += u"Son inversion risque donc d'aboutir à un résultat très médiocre.\n\nContinuer quand même ?"
            r = tkMessageBox.askyesno(parent=app,
                    title="Construire le dictionnaire inverse",
                    message=msg)
            if r == False: return
        
        
        # ----------------------------------
        # --- Mise en place de la fenêtre   
        # ----------------------------------
        
        Toplevel.__init__(self)
        
        self.title("Construire le dictionnaire inverse")
        self.resizable(width=NO, height=NO)
        self.grab_set()             # fenêtre modale
        self.transient(master=app)  # fenêtre dépendante
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        
        ww = 80
        r = 0 # rangée pour grid
        
        # -------------------------------
        # --- Frame du type de fichier   
        # -------------------------------
        
        fr_type=LabelFrame(self, text="Format du dictionnaire inverse :")
        fr_type.grid(row=r,column=0)
        
        Label(fr_type,width=ww).grid(row=0, column=0) #dimensionneur
        
        self.vFileType = StringVar()
        RB_ling = Radiobutton(fr_type, text="Ling", variable=self.vFileType, value="ling", command=self.__onclicFileType)
        RB_ling.grid(row=0,column=0, padx=5, sticky='W')
        RB_dict = Radiobutton(fr_type, text="Dict", variable=self.vFileType, value="dict", command=self.__onclicFileType)
        RB_dict.grid(row=1,column=0, padx=5, sticky='W')
        RB_xdxf = Radiobutton(fr_type, text="XDXF", variable=self.vFileType, value="xdxf", command=self.__onclicFileType)
        RB_xdxf.grid(row=2,column=0, padx=5, sticky='W')
        RB_wb = Radiobutton(fr_type, text="WB", variable=self.vFileType, value="wb", command=self.__onclicFileType)
        RB_wb.grid(row=3,column=0, padx=5, sticky='W')
        RB_csv = Radiobutton(fr_type, text="CSV (fichier texte tabulé)", variable=self.vFileType, value="csv", command=self.__onclicFileType)
        RB_csv.grid(row=4,column=0, padx=5, sticky='W')
        
        
        # Si le fichier est protégé > avertir
        if app.dic.protected1 !="" or app.dic.protected2 !="":
            tkMessageBox.showinfo(parent=self,
                title=u"Construire le dictionnaire inverse",
                message=u"Les données de ce dictionnaire sont protégées,\nle format du dictionnaire inverse sera obligatoirement le format Ling\net le dictionnaire inverse sera lui aussi protégé.")
            self.vFileType.set('ling')
            RB_dict['state']='disabled'
            RB_xdxf['state']='disabled'
            RB_wb['state']='disabled'
            RB_csv['state']='disabled'
        elif app.dic.type.startswith('wb'):
            self.vFileType.set('wb')
        elif app.dic.type.startswith('dict'):
            self.vFileType.set('dict')
        elif app.dic.type.startswith('xdxf'):
            self.vFileType.set('xdxf')
        elif app.dic.type.startswith('csv'):
            self.vFileType.set('csv')
        else:
            self.vFileType.set('ling')
        
        
        # ---------------------------
        # --- Frame du séparateur    
        # ---------------------------
        
        r +=1
        fr_sp=LabelFrame(self, text='Séparateur des traductions individuelles dans le dictionnaire source :')
        fr_sp.grid(row=r, column=0, padx=5, pady=5)
        
        Label(fr_sp, width=ww, height=2).grid(row=0, column=0, columnspan=10) #dimensionneur
        
        self.vSp = StringVar()
        Radiobutton(fr_sp, text="Point-virgule", variable=self.vSp, value=";").grid(row=0, column=0, sticky='W')
        Radiobutton(fr_sp, text="Virgule", variable=self.vSp, value=",").grid(row=1, column=0, sticky='W')
        if app.dic.type.startswith('wb'):
            self.vSp.set(",")
        else:
            self.vSp.set(";")
        
        
        # --------------------------------
        # --- Frame du mode d'inversion   
        # --------------------------------
        
        r +=1
        self.fr_md=LabelFrame(self, text="Mode d'inversion :")
        self.fr_md.grid(row=r, column=0, padx=5, pady=5)
        
        Label(self.fr_md, width=ww, height=2).grid(row=0, column=0, columnspan=10) #dimensionneur
        
        self.vInv = StringVar()
        self.RB_std=Radiobutton(self.fr_md, text="Inverser en négligeant les infos et traductions longues", variable=self.vInv, value="std")
        self.RB_std.grid(row=0, column=0, sticky='W')
        self.RB_lg=Radiobutton(self.fr_md, text="Inverser en conservant, sans inversion, les infos et traductions longues", variable=self.vInv, value="lg")
        self.RB_lg.grid(row=1, column=0, sticky='W')
        self.vInv.set("std")
        
        
        # ---------------------------
        # --- Frame du répertoire    
        # ---------------------------
        
        r +=1
        fr_rep=LabelFrame(self, text="Dossier du dictionnaire inverse :")
        fr_rep.grid(row=r, column=0, padx=4, pady=2)
        
        Label(fr_rep, width=ww, height=2).grid(row=0, column=0, columnspan=10) #dimensionneur
        
        self.txt_folder=Entry(fr_rep, width=60, background="white")
        self.txt_folder.grid(row=0, column=0, padx=5, sticky='WN', pady=5)
        
        self.bt_browse=Button(fr_rep,text=' Parcourir... ', command= lambda: browseFolder(self, self.txt_folder))
        self.bt_browse.grid(row=0, column=1, sticky="WN", pady=4)
        
        
        self.vChkDicFolder = IntVar()
        ChkDicFolder=Checkbutton(fr_rep, text=u'Créer dans le dossier des dictionnaires', variable=self.vChkDicFolder, command=self.onclicChkDicFolder)
        ChkDicFolder.grid(row=1, column=0, sticky='WN')
        
        
        # Présélectionner le répertoire
        dp = app.ini.get('DicDefaultFolder','')
        if dp != "":
            ChkDicFolder.select()
            self.txt_folder.insert(END, dp)
            self.txt_folder['state']='disabled'
            self.bt_browse['state']='disabled'
        else:
            ChkDicFolder['state']='disabled'
        
        
        # ------------------------------
        # --- Frame du nom de fichier   
        # ------------------------------
        
        r +=1
        fr_name=LabelFrame(self, text="Nom du fichier du dictionnaire inverse :")
        fr_name.grid(row=r, column=0, padx=4, pady=2)
        
        Label(fr_name,width=ww, height=4).grid(row=0, column=0) #dimensionneur
        
        self.txt_name=Entry(fr_name, width=35, background='white')
        self.txt_name.grid(row=0, column=0)
        
        
        if app.dic.reverseDicFileName != "":
            # Le dico-maître contient le nom de fichier du dico inverse
            self.txt_name.insert(END, os.path.splitext(app.dic.reverseDicFileName)[0])
        else:
            # On construit un nom de dico inverse à partir du nom du dico maître + "_rev"
            self.txt_name.insert(END, os.path.splitext(os.path.basename(app.dic.path))[0] + "_rev")
        
        
        # -------------------------
        # --- Frame des Boutons    
        # -------------------------
        
        r +=1
        fr_but=Frame(self)
        fr_but.grid(row=r, column=0, columnspan=10, padx=4, pady=2)
        
        Button(fr_but, image=app.imglst.reverse, compound=LEFT, text="  Construire", width=100, padx= 10, command=self.makeRevDic).grid(row=0, column=0, pady=8, padx=10)
        Button(fr_but, image=app.imglst.exit, compound=LEFT, text="  Fermer", width=100, padx= 10, command=self.destroy).grid(row=0, column=1, pady=8, padx=10)
        Button(fr_but, image=app.imglst.hlp, compound=LEFT, text="  Aide", width=90, padx= 7,command=lambda: app.showHelp("inverse")).grid(row=0, column=2, pady=8, padx=10)
        
        # --- Barre de progression
        r +=1
        self.progr = lib.myprogbr.ProgressBar(self, width=380)
        self.progr.grid(row=r, column=0, columnspan=4, pady=5)
        self.progr.grid_remove()
        
        
        return


    def onclicChkDicFolder(self):
        u"""
        Clic sur la case 'utiliser le dossier des dictionnaires'
        """
        
        if self.vChkDicFolder.get() == 1:
            self.txt_folder.delete(0, END)
            self.txt_folder.insert(END, app.ini.get('DicDefaultFolder',''))
            self.txt_folder['state']='disabled'
            self.bt_browse['state']='disabled'
            self.bt_browse['state']='disabled'
        else:
            self.txt_folder['state']='normal'
            self.bt_browse['state']='normal'
            self.bt_browse['state']='normal'
        
        
        return

    

    def makeRevDic(self):
        u"""
        Construire un dictionnaire inverse
        """
        
        newDicType = self.vFileType.get()
        
        # ----------------------------------------------------------------
        # --- Confectionner le chemin du fichier du dictionnaire inverse  
        # ----------------------------------------------------------------
        
        if newDicType == 'xdxf':
            
            repxdxf = os.path.join(self.txt_folder.get(),self.txt_name.get())
            fichpath = os.path.join(repxdxf, "dict.xdxf")
        
        else :
            
            fichpath = os.path.join(self.txt_folder.get(),self.txt_name.get()) + "." + newDicType
        
        
        # Avertir si un fichier existe déjà sous ce nom
        if os.path.isfile(fichpath):
            r = tkMessageBox.askyesno(parent=self,
                    title=u"Création d'un dictionnaire",
                    message=u"Un fichier existe déjà sous ce nom. Désirez-vous le remplacer ?")
            if r == False: return
        
        
        
        # -------------------------------------------------------
        # --- Construire un pyDictionnaire inverse temporaire    
        # --- à partir des traductions courtes                   
        # --- (clefs et données du pyDictionnaire en Utf-8)      
        # -------------------------------------------------------
        
        self['cursor']='watch'
        app['cursor']='watch'
        
        self.progr.grid()
        self.progr.setMax(app.dic.lstbox.size())
        
        # (variables locales pour accélérer la boucle)
        getdata = app.dic.getdata 
        lstget = app.dic.lstbox.get
        
        sp = self.vSp.get()
        if sp == ',': sp = ', '
        if sp == ';': sp = ' ; '
        
        if self.vInv.get() == "lg":
            flagAddInfo = True
        else:
            flagAddInfo = False
        
        dicRev = {}     # pyDictionnaire inverse
        dicInfo = {}    # pyDictionnaire des infos (optionnel)
        
        for i in range(app.dic.lstbox.size()):      # Scanner la liste des entrées du dico source
            
            self.progr.setValue(i+1)
            
            trads = getdata(i)['short'].strip(sp)   # Nettoyer éventuels séparateurs trainant en tête ou queue
            trads = trads.split(sp)                 # Traductions courtes de l'entrée (séparées par 'sp')
            
            for trad in trads:                      # Scanner les traductions individuelles de l'entrée
                
                trad = trad.strip()
                try:
                    trad = trad.encode('utf-8')     # 'Try+pass' car le retour est parfois ascii et non utf-8
                except:
                    pass
                
                if dicRev.has_key(trad):            # La traduction est présente > on complète avec l'entrée du dico source
                    wd = sp + lstget(i)
                    wd = wd.encode('utf-8') 
                    dicRev[trad] += wd
                else:                               # La traduction est absente > on ajoute l'entrée du dico source
                    wd = lstget(i).encode('utf-8')
                    dicRev[trad] = wd
                
                
                if flagAddInfo:                      # Prise en compte optionelle des traductions longues de l'entrée
                    
                    if dicInfo.has_key(trad): 
                        dicInfo[trad] += "\n......\n" + getdata(i)['long']
                    else:
                        dicInfo[trad] = getdata(i)['long']
        
        
        # ------------------------------------------------------------------------------
        # --- Convertir le pyDictionnaire inverse en chaîne Dictionnaire (aiguillage)   
        # ------------------------------------------------------------------------------
        
        cp1 = app.dic.encoding1
        cp2 = app.dic.encoding2
        
        
        strfich = ""
        
        if newDicType == "ling":
        
            strFich = self.__doLing(dicRev, dicInfo, flagAddInfo)
        
        elif newDicType == "csv":
            
            strFich = self.__doCSV(dicRev, cp=cp1)
        
        elif newDicType == "xdxf":
            
            strFich = self.__doXDXF(dicRev, repxdxf, dicInfo, flagAddInfo)
        
        elif newDicType == "wb":
            
            # Demander un nouvel encodage si la source est en UTF
            if cp1.lower().startswith('utf') or cp2.lower().startswith ('utf'):
                cp2 = tkSimpleDialog.askstring(
                    title="Création d'un nouveau fichier WB (1 de 2)",
                    prompt="Les fichiers WB sont encodés sur 8-bits\n et ne contiennent aucune information sur l'encodage des caractères...\n\nPour un affichage correct, il est donc nécessaire de préciser manuellement\nl'encodage de la langue source ET celui de la langue cible.\n\nPrécisez tout d'abord l'encodage désiré pour la LANGUE SOURCE du fichier WB à créer :\n",
                    initialvalue="cp1252")
                if not cp2:
                    return
            
                cp1 = tkSimpleDialog.askstring(parent=self,
                        title="Création d'un nouveau fichier WB (2 de 2)",
                        prompt="Précisez ensuite l'encodage désiré pour la LANGUE CIBLE dans ce nouveau fichier WB :\n",
                        initialvalue="cp1252")
                if not cp1:
                    return
            
            strFich = self.__doWB(dicRev, cp1=cp1, cp2=cp2)
        
        elif newDicType == "dict":
            
            strFich = self.__doDict(dicRev, fichpath, dicInfo, flagAddInfo)
        
        
        if strFich.startswith("!"):
            self['cursor']=''
            app['cursor']=''
            tkMessageBox.showwarning(parent=self,
                    title=u"Echec de la construction du dictionnaire inverse",
                    message=strFich)
            return
        
        
        
        # ----------------------------------------------
        # --- Enregistre le nouveau fichier inversé     
        # ----------------------------------------------
        
        ff = open(fichpath, 'wb')
        ff.write(strFich)
        ff.close()
        
        
        # ----------------------------------------------------------------
        # --- Mémoriser le nom du fichier inverse dans le fichier source  
        # ----------------------------------------------------------------
        
        app.dic.reverseDicFileName = self.txt_name.get().encode('utf-8') + "." + newDicType
        
        if self.vFileType.get() == "ling":
            
            strMaster = app.dic.toStrPreling()
            
            if strMaster[0] == "!": # Echec
                tkMessageBox.showwarning(parent=self,
                    title=u"Construction du dictionnaire inverse",
                    message=u"Impossible d'inscrire le nom du dictionnaire inverse dans le dictionnaire-source\npour la raison suivante :\n\n" + strMaster)
            
            else: # ok, on continue
                
                strLingMaster = DicObject.strPrelingToStrLing(strPreling=strMaster)
                
                # NB il n'est pas nécessaire de recharger le dico source
                app.dic.file.close()
                
                ff = open(app.dic.path,'wb')
                ff.write(strLingMaster)
                ff.close()
                
                app.dic.file = open(app.dic.path, 'rb')
        
        
        

        # ---------------------------------------------------------------
        # -- Recharge le dictionnaire avec le nouveau fichier inversé    
        # ---------------------------------------------------------------
        
        try   : app.dic.file.close()
        except: pass
        
        app.loadDico(fichpath, cp1=cp2, cp2=cp1)
        
        app.update()
        
        app.dic.sortdic()       # Trier le nouveau dictionnaire inversé
        
        app.updateMenuDic()     # Mettre à jour le menu des dictionnaires
        
        
        tkMessageBox.showinfo(parent=self,
                title=u"Linguae",
                message=u"Le dictionnaire a été inversé avec succès")
        
        self.destroy()
        
        return
    
    
    
    def __doCSV(self, dicRev, cp):
        u"""
        Convertit le pyDictionnaire inverse en chaîne csv
        """
        
        strFich = ""
        
        for k, x in dicRev.items():
        # k = entrée, x = traduction
            
            k = k.decode('utf-8')
            try:
                k = k.encode(cp)
            except:
                return "! Echec d'encodage !"
            
            x = x.decode('utf-8')
            try:    
                x = x.encode(cp) 
            except:
                return "! Echec d'encodage !"
                
            strFich += k + "\t" + x + "\n"
        
        
        return strFich
    
    
    
    def __doLing(self, dicRev, dicInfo, flagAddInfo=False):
        u"""
        Convertit le pyDictionnaire inverse en chaîne Ling
        (conversion en chaîne preLing puis > Ling)
        
        dicRev      : pyDictionnaire inverse
        dicInfo     : pyDictionnaire des infos/traductions longues associées (non inversées)
        flagAddInfo : flag permettant la prise en compte ou non de dicInfo
        """
        
        # --------------------------------------------
        # --- Confection de la chaîne Preling brute   
        # --------------------------------------------
        
        strPreling = ""
        
        for k, x in dicRev.items():
        # k = entrée, x = traduction
            
            if flagAddInfo:
                strPreling += k + "\t" + x + "\t" + dicInfo[k].replace("\n", "<br>") + "\n"
            else:
                strPreling += k + "\t" + x + "\n"
           
        #app.clipboard_clear()
        #app.clipboard_append(strPreling)
        
        
        # --------------------------------------------
        # --- Ajout du Header à la chaîne Preling     
        # --------------------------------------------
        
        hdr = "::isReverseDic=True\n::doReverseDic=False\n"
        hdr = hdr.encode('utf-8')
        
        # Nom du nouveau dico
        if app.dic.reverseDicName != "":
            hdr += "::dicName=" + app.dic.reverseDicName + "\n"
        elif app.dic.langName1 != "":
            hdr += "::dicName=" + app.dic.langName2 + " - " + app.dic.langName1 + "\n"
        else:
            hdr += "::dicName=Nouveau dictionnaire inverse" + "\n"
        
        # Nom de fichier du dico "inverse" ( = nom du dico-source)
        hdr += "::reverseDicFileName=" + os.path.basename(app.dic.path).encode('utf-8') + "\n"
        
        # N° de version du nouveau dico
        if app.dic.dicVersionNumber != "":
            hdr += "::dicVersionNumber=" + app.dic.dicVersionNumber + " (inverse)\n"
        else:
            hdr += "::dicVersionNumber=1.0 (inverse)\n"
        
        # Autres données d'en-tête
        hdr += "::dicUrl=" + app.dic.dicUrl + "\n"
        hdr += "::langName1=" + app.dic.langName2 + "\n"
        hdr += "::langName2=" + app.dic.langName1 + "\n"
        hdr += "::langIso1=" + app.dic.langIso2 + "\n"
        hdr += "::langIso2=" + app.dic.langIso1 + "\n"
        hdr += "::langNameUser=" + app.dic.langNameUser + "\n"
        hdr += "::langIsoUser=" + app.dic.langIsoUser + "\n"
        hdr += "::langFamily1=" + app.dic.langFamily2 + "\n"
        hdr += "::langFamily2=" + app.dic.langFamily1 + "\n"
        hdr += "::dicStatus=" + "(Dictionnaire inversé automatique - consultez le dictionnaire original pour les informations de licence)" + "\n"
        hdr += "::copyright=" + "(Dictionnaire inversé automatique)" + "\n"
        hdr += "::dicInfo=" + "Ce dictionnaire inverse a été créé par un processus automatique et ne peut prétendre à la précision et à la correction d'un dictionnaire rédigé manuellement." + "\n"
        hdr += "::displayFontName1=" + app.dic.displayFontName2 + "\n"
        hdr += "::displayFontName2=" + app.dic.displayFontName2 + "\n"
        
        
        # Protection des données du fichier ?
        if app.dic.protected1 != "":
            hdr += '::protected2=Linguae' +"\n"
        if app.dic.protected1 != "":
            hdr += '::protected1=Linguae' +"\n"
        
        
        # Ajouter le header
        strPreling = hdr + strPreling
        
        
        # ------------------------------------------------
        # --- Ajout des Icônes à la chaîne Preling        
        # ------------------------------------------------
        
        if app.dic.img2 !="" :
            strPreling += "**img1begin\n" + app.dic.img2 + "\n**img1end\n"
        
        if app.dic.img1 !="" :
            strPreling += "**img2begin\n" + app.dic.img1 + "\n**img2end\n"
        
        
        # ----------------------------------------------------
        # --- Convertir en chaîne ling et retourner celle-ci  
        # ----------------------------------------------------
        
        
        strFich = DicObject.strPrelingToStrLing(strPreling=strPreling)
        
        return strFich
    
    
    def __doXDXF(self, dicRev, repxdxf, dicInfo, flagAddInfo=False):
        u"""
        Convertit le pyDictionnaire inverse en chaîne XDXF
        
        dicRev      : pyDictionnaire inverse
        repxdxf     : répertoire du dico xdxf cible
        dicInfo     : pyDictionnaire des infos/traductions longues associées (non inversées)
        flagAddInfo : flag permettant la prise en compte ou non de dicInfo
        """
        
        strFich = '<?xml version="1.0" encoding="UTF-8" ?>\n<xdxf lang_from="" lang_to="" format="visual">\n'
        
        self.progr.setValue(0)
        self.progr.setMax(len(dicRev))
        
        wc = 1
        for k, x in dicRev.items():
        # k = entrée, x = traduction
            
            self.progr.setValue(wc)
            
            if flagAddInfo and dicInfo[k] != "":
                try:
                    strFich += "<ar><k>" + k + "</k>\n" + x + "\n-------\n" + dicInfo[k] + "</ar>\n"
                except:
                    return "! Erreur d'encodage !"
            
            else:
                
                try:
                    strFich += "<ar><k>" + k + "</k>\n" + x + "</ar>\n"
                except:
                    return "! Erreur d'encodage !"
            
            wc += 1
            
        
        strFich += "</xdxf>"
        
        if not os.path.isdir(repxdxf):
            os.mkdir(repxdxf) # Créer le répertoire du dico
        
        return strFich


    
    def __doWB(self, dicRev, cp1, cp2):
        u"""
        Convertit le pyDictionnaire inverse en chaîne WB
        """
        
        strFich = ""
        
        self.progr.setValue(0)
        
        wc = 1
        for k, x in dicRev.items():
        # k = nouvelle entrée, x = nouvelle traduction
            
            self.progr.setValue(wc)
            
            k = k.decode('utf-8')
            try:
                k = k.encode(cp2)
            except:
                print map(ord,k)
                return "! Erreur d'encodage '"+ cp2 +"' (chp1/" + str(wc) + ") ! "
            
            k = k[:30].ljust(31,'\0')
            
            x = x.decode('utf-8')
            try:    
                x = x.encode(cp1) 
            except:
                print map(ord,x)
                return "! Erreur d'encodage '"+ cp1 +"' (chp2/" + str(wc) + ")! "
            
            x = x [:50].ljust(53,'\0')
                
            strFich += k + x
            
            wc += 1
        
        return strFich
    
    
    
    def __doDict(self, dicRev, fichpath, dicInfo, flagAddInfo=False):
        u"""
        Convertit le pyDictionnaire inverse en chaînes Dict
        Enregistre le *ifo et le *idx et retourne la chaîne du *dict
        
        dicInfo     : pyDictionnaire des infos/traductions longues associées (non inversées)
        flagAddInfo : flag permettant la prise en compte ou non de dicInfo
        """
        
        radicfich = os.path.splitext(fichpath)[0]   # Radical des différents fichiers Dict
      
        
        # --------------------------------------
        # --- Scanner le pyDictionnaire         
        # --------------------------------------
        
        decal = 0
        wc = 0
        strdict = []
        stridx = []
        
        for k, x in dicRev.items():
            
            wc += 1
            
            if flagAddInfo and dicInfo[k] != "":
                x = x + "\n-------\n" + dicInfo[k]
            
            strdict.append(x)                       # Ajouter la traduction dans le *.dict
            stridx.append(k + '\0')                 # Ajouter l'entrée dans le *.idx
            
            stridx.append(NumToStr32(decal))        # Ajouter le décalage (image chaîne en 32 bits) dans le *.idx
            stridx.append(NumToStr32(len(x)))       # Ajouter la longueur de la traduction (image chaîne en 32 bits) dans le *.idx
            
            decal = decal + len(x)                  # Décalage (offset) de la traduction suivante
            
        
        # ---------------------------------------------
        # --- Enregistrer le fichier *idx              
        # ---------------------------------------------
        
        ff = open(radicfich + ".idx",'wb')          # Enregistrer le fichier *.idx
        ff.write("".join(stridx))
        ff.close()
        
        
        # ------------------------------------------------
        # --- Construire et enregistrer le fichier *.ifo  
        # ------------------------------------------------
        
        t ="StarDict's dict ifo file\nversion=2.4.2\n[options]\nbookname=" + os.path.basename(radicfich) +\
        "\nwordcount=" + str(wc) +\
        "\nidxfilesize=" + str(os.path.getsize(radicfich + ".idx")) +\
        "\nsynwordcount=\nsametypesequence=m\nidxoffsetbits=32\nauthor=\nemail=\nwebsite=\ndescription=\ndate=\n"
        
        try:
            t = t.encode('utf-8')
        except:
            return "! Echec d'encodage utf-8 dans le fichier *.ifo !"
        
        ff = open(radicfich + ".ifo",'wb')
        ff.write(t)
        ff.close()
        
        # ---------------------------------------------
        # --- Retourner la chaîne *dict                
        # ---------------------------------------------
        
        strdict = "".join(strdict)
        
        return strdict
    
    

class WindowUnicode(Toplevel):
    u"""
    Fenêtre d'affichage des caractères Unicode d'une police donnée
    Permet la confection d'une chaîne qui est copiée dans le presse-papiers
    """

    def __init__(self, ifont=0):
        u"""
        Mise en place de l'interface graphique
        ifont : 1 = police langue 1 / 2 = police langue 2
        """
        
        
        Toplevel.__init__(self)
        
        self.title("Table de caractères UNICODE")
        self.resizable(width=NO, height=NO)
        self.focus_set()
        
        # Placement relatif à la fenêtre principale
        left = str(int(app.geometry().split('+')[1])+20)
        top = str(int(app.geometry().split('+')[2])+40)
        self.geometry('+' + left + '+' + top)
        
        
        # Liste des sections
        self.lst_mnu = lib.myscrlst.ScrolledList(self, width=35, height=35, onclic=self.seeTable)
        self.lst_mnu.grid(row=0, column=0, rowspan=10)
        
        
        # Frame des options de police
        fr_font=Frame(self, borderwidth=2, relief=GROOVE)
        fr_font.grid(row=0, column=1, pady=5)
        
        self.vFnt = IntVar()
        opt_lg1 = Radiobutton(fr_font, text="Police de la langue source", variable=self.vFnt, value=1)
        opt_lg1.grid(row=0, column=0, sticky='W', padx=10)
        opt_lg2 = Radiobutton(fr_font, text="Police de la langue cible", variable=self.vFnt, value=2)
        opt_lg2.grid(row=0, column=1, sticky='W', padx=10)
        if ifont > 0:
            self.vFnt.set(ifont)
        opt_lg1.bind('<ButtonRelease-1>', self.swapLang)
        opt_lg2.bind('<ButtonRelease-1>', self.swapLang)
        
        
        Label(self, width=90, height=32).grid(row=1, column=1) # dimensionneur
        
        # Frame droit central
        self.fr_D = Frame(self)
        self.fr_D.grid(row=1, column=1, sticky='N')
        Label(self.fr_D, width=90).grid(row=0, column=0, sticky='N') # dimensionneur
        Label(self.fr_D, text="Si cette police ne prend pas en charge les caractères désirés,\nvous devez choisir une autre police d'affichage par défaut (Options)\nou modifier les polices spécifiques du dictionnaire (Propriétés du dictionnaire)", justify=LEFT, background='black', foreground='white').grid(row=0, column=0, pady=5)
        
        # Frame des caractères
        self.fr_chr = Frame(self.fr_D, borderwidth=2, relief=GROOVE)
        self.fr_chr.grid(row=1, column=0, padx=4, pady=4)
        Label(self.fr_chr, text="Choisissez une sous-table dans la liste", width=70, height=10).grid(row=0, column=0, rowspan=10, columnspan=10) # dimensionneur
        
        # Frame de saisie
        fr_edit = Frame(self)
        fr_edit.grid(row=2, column=1, sticky='S')
        
        Label(fr_edit, text="Double-cliquez sur un caractère pour l'ajouter ").grid(row=0, column=0)
        
        # Case de la chaîne
        self.txt_str = Entry(fr_edit, width=25)
        self.txt_str.grid(row=0, column=1, sticky='S')
        
        Button(fr_edit, text=" Copier ", command=self.copyStr).grid(row=0, column=2)
        Button(fr_edit, image=app.imglst.hlp, compound=LEFT, text=" Aide", width=40, padx=8,command=lambda: app.showHelp("chartab")).grid(row=0, column=3, padx=10)
        
        # ----
        
        self.loadUnicodeMenu() # Remplir la liste-menu
        
        return


    def swapLang(self,event):
        u"""
        Clic sur options de changement de langue entrées/notices
        """
        # Compenser décalage évènement release et modif de la tkVariable
#        if self.vFnt.get() == 2:
#            self.vFnt.set(1)
#        else:
#            self.vFnt.set(2)
        
        # ---
        self.seeTable(self.lst_mnu.getselectedindex(), self.lst_mnu)
        
        
        return


    def loadUnicodeMenu(self):

        # --------------------------------------------------
        # --- "0000 FFFF Plan multilingue de base (BMP)"    
        # --------------------------------------------------
        
        self.lst_mnu.add("0000 007F Latin de base")
        #self.lst_mnu.add("0080 009F Non-utilisé")
        self.lst_mnu.add("00A0 00FF Supplément Latin-1")
        self.lst_mnu.add("0100 017F Latin étendu A")
        self.lst_mnu.add("0180 024F Latin étendu B")
        self.lst_mnu.add("0250 02AF Alphabet phonétique international")
        self.lst_mnu.add("02B0 02FF Lettres modificatives avec chasse")
        self.lst_mnu.add("0300 036F Diacritiques")
        self.lst_mnu.add("0370 03FF Grec et copte")
        self.lst_mnu.add("0400 04FF Cyrillique")
        self.lst_mnu.add("0500 052F Supplément cyrillique")
        self.lst_mnu.add("0530 058F Arménien")
        self.lst_mnu.add("0590 05FF Hébreu")
        self.lst_mnu.add("0600 06FF Arabe")
        self.lst_mnu.add("0700 074F Syriaque")
        self.lst_mnu.add("0780 07BF Thâna")
        self.lst_mnu.add("0900 097F Dévanâgarî")
        self.lst_mnu.add("0980 09FF Bengali")
        self.lst_mnu.add("0A00 0A7F Gourmoukhî")
        self.lst_mnu.add("0A80 0AFF Goudjerate")
        self.lst_mnu.add("0B00 0B7F Oriya")
        self.lst_mnu.add("0B80 0BFF Tamoul")
        self.lst_mnu.add("0C00 0C7F Télougou")
        self.lst_mnu.add("0C80 0CFF Kannara")
        self.lst_mnu.add("0D00 0D7F Malayalam")
        self.lst_mnu.add("0D80 0DFF Singhalais")
        self.lst_mnu.add("0E00 0E7F Thai")
        self.lst_mnu.add("0E80 0EFF Lao")
        self.lst_mnu.add("0F00 0FFF Tibétain")
        self.lst_mnu.add("1000 109F Birman")
        self.lst_mnu.add("10A0 10FF Géorgien")
        self.lst_mnu.add("1100 11FF Jamos hangûl")
        self.lst_mnu.add("1200 137F Éthiopien")
        self.lst_mnu.add("13A0 13FF Chérokî")
        self.lst_mnu.add("1400 167F Syllabaires autochtones canadiens unifiés")
        self.lst_mnu.add("1680 169F Ogam")
        self.lst_mnu.add("16A0 16FF Runes")
        self.lst_mnu.add("1700 171F Tagalog ou tagal")
        self.lst_mnu.add("1720 173F Hanounóo")
        self.lst_mnu.add("1740 175F Bouhide")
        self.lst_mnu.add("1760 177F Tagbanoua")
        self.lst_mnu.add("1780 17FF Khmer")
        self.lst_mnu.add("1800 18AF Mongol")
        self.lst_mnu.add("1900 194F Limbou")
        self.lst_mnu.add("1950 197F Taï-le")
        self.lst_mnu.add("19E0 19FF Symboles khmers")
        self.lst_mnu.add("1D00 1D7F Supplément phonétique")
        self.lst_mnu.add("1E00 1EFF Latin étendu additionnel")
        self.lst_mnu.add("1F00 1FFF Grec étendu")
        self.lst_mnu.add("2000 206F Ponctuation générale")
        self.lst_mnu.add("2070 209F Exposants et indices")
        self.lst_mnu.add("20A0 20CF Symboles monétaires")
        self.lst_mnu.add("20D0 20FF Signes combinatoires pour symboles")
        self.lst_mnu.add("2100 214F Symboles de type lettre")
        self.lst_mnu.add("2150 218F Formes numérales")
        self.lst_mnu.add("2190 21FF Flèches")
        self.lst_mnu.add("2200 22FF Opérateurs mathématiques")
        self.lst_mnu.add("2300 23FF Signes techniques divers")
        self.lst_mnu.add("2400 243F Pictogrammes de commande")
        self.lst_mnu.add("2440 245F Reconnaissance optique de caractères")
        self.lst_mnu.add("2460 24FF Alphanumériques cerclés")
        self.lst_mnu.add("2500 257F Filets")
        self.lst_mnu.add("2580 259F Pavés")
        self.lst_mnu.add("25A0 25FF Formes géométriques")
        self.lst_mnu.add("2600 26FF Symboles divers")
        self.lst_mnu.add("2700 27BF Casseau")
        self.lst_mnu.add("27C0 27EF Divers symboles mathématiques - A")
        self.lst_mnu.add("27F0 27FF Supplément A de flèches")
        self.lst_mnu.add("2800 28FF Combinaisons Braille")
        self.lst_mnu.add("2900 297F Supplément B de flèches")
        self.lst_mnu.add("2980 29FF Divers symboles mathématiques-B")
        self.lst_mnu.add("2A00 2AFF Opérateurs mathématiques supplémentaires")
        self.lst_mnu.add("2B00 2BFF Divers symboles et flèches")
        self.lst_mnu.add("2D30 2D6F Alphabet Tifinagh et néo-Tifinagh")
        self.lst_mnu.add("2E80 2EFF Formes supplémentaires des clés CJC")
        self.lst_mnu.add("2F00 2FDF Clés chinoises (K'ang-hsi ou Kangxi)")
        self.lst_mnu.add("2FF0 2FFF Description idéophonographique")
        self.lst_mnu.add("3000 303F Symboles et ponctuation CJC")
        self.lst_mnu.add("3040 309F Hiragana")
        self.lst_mnu.add("30A0 30FF Katakana")
        self.lst_mnu.add("3100 312F Bopomofo")
        self.lst_mnu.add("3130 318F Jamos de compatibilité hangûls")
        self.lst_mnu.add("3190 319F Kanboun")
        self.lst_mnu.add("31A0 31BF Bopomofo étendu")
        self.lst_mnu.add("31F0 31FF Extension phonétique katakana")
        
        # Sections suivantes masquées car la plupart sont trop grosses à charger...
        
        #self.lst_mnu.add("3200 32FF Lettres et mois CJC cerclés")
        #self.lst_mnu.add("3300 33FF Compatibilité CJC")
        #self.lst_mnu.add("3400 4DB5 Supplément A aux idéophonogrammes unifiés CJC")
        #self.lst_mnu.add("4DC0 4DFF Hexagrammes du Classique des mutations ou Yi-king")
        #self.lst_mnu.add("4E00 9FA5 Idéophonogrammes unifiés CJC")
        #self.lst_mnu.add("A000 A48F Syllabaire yi des Monts frais")
        #self.lst_mnu.add("A490 A4CF Clés yi")
        #self.lst_mnu.add("AC00 D7A3 Hangûl")
        
        #~ 'D800 DB7F Demi-zone haute ; non-caractères, points de code invalides isolément")
        #~ '---------------D800 à D83F : codets hauts utilisés en UTF-16 pour les points de code du plan multilingue supplémentaire")
        #~ '---------------D840 à D87F : codets hauts utilisés en UTF-16 pour les points de code du plan idéographique supplémentaire")
        #~ '---------------D880 à DB3F : codets hauts utilisés en UTF-16 pour les points de code des plans supplémentaires réservés")
        #~ '---------------D840 à D87F : codets hauts utilisés en UTF-16 pour les points de code du plan spécial supplémentaire")
        #~ 'DB80 DBFF Partie à usage privé de la demi-zone haute non-caractères, points de code invalides isolément
        #~ '---------------DB80 à DBBF : codets hauts utilisés en UTF-16 pour les points de code de la zone supplémentaire A à usage privé
        #~ '---------------DBC0 à DBFF : codets hauts utilisés en UTF-16 pour les points de code de la zone supplémentaire B à usage privé
        
        #~ 'DC00 DFFF Demi-zone basse ; non-caractères, points de code invalides isolément
        #~ '---------------DC80 à DFFD : codets bas utilisés en UTF-16 pour des points de code assignés aux caractères valides ou réservés des plans supplémentaires (assignés, réservés ou à usage privé)
        #~ '---------------DFFE à DFFF : codets bas pouvant être utilisés en UTF-16 pour la représentation de points de code assignés aux non-caractères en fin de chaque plan, lorsque le codet haut est le dernier assigné dans la demi-zone haute pour chaque plan supplémentaire (assigné, réservé ou à usage privé)
        
        #~ 'E000 F8FF Zone à usage privé
        
        #self.lst_mnu.add("F900 FAFF Idéogrammes de compatibilité CJC")
        self.lst_mnu.add("FB00 FB4F Formes de présentation alphabétiques")
        self.lst_mnu.add("FB50 FDFF Formes A de présentation arabes")
        #self.lst_mnu.add("FDD0 FDEF   non-caractères")
        self.lst_mnu.add("FE00 FE0F Sélecteurs de variante")
        self.lst_mnu.add("FE20 FE2F Demi-signes combinatoires")
        self.lst_mnu.add("FE30 FE4F Formes de compatibilité CJC")
        self.lst_mnu.add("FE50 FE6F Petites variantes de forme")
        self.lst_mnu.add("FE70 FEFF Formes B de présentation arabes")
        self.lst_mnu.add("FF00 FFEF Formes de demi et pleine chasse")
        self.lst_mnu.add("FFF0 FFFD Caractères spéciaux")
        #self.lst_mnu.add("FFFE FFFF non-caractères")
        
        # --------------------------------------------------------------
        # --- "10000 1FFFF Plan multilingue supplémentaire (SMP)"
        # --------------------------------------------------------------
        
        #~ 'le reste est inaffichable car le code dépasse la capacité de ChrW()
        
        
        #self.lst_mnu.add("10000 1007F Syllabaire linéaire B ou syllabaire mycénien")
        #self.lst_mnu.add("10080 100FF Idéogrammes du linéaire B")
        #self.lst_mnu.add("10100 1013F Nombres égéens")
        #self.lst_mnu.add("10300 1032F Alphabet italique")
        #self.lst_mnu.add("10330 1034F Gotique")
        #self.lst_mnu.add("10380 1039F Ougaritique")
        #self.lst_mnu.add("10400 1044F Déséret")
        #self.lst_mnu.add("10450 1047F Shavien")
        #self.lst_mnu.add("10480 104AF Osmanya")
        #self.lst_mnu.add("10800 1083F Syllabaire chypriote")
        #self.lst_mnu.add("1D000 1D0FF Symboles musicaux byzantins")
        #self.lst_mnu.add("1D100 1D1FF Symboles musicaux occidentaux")
        #self.lst_mnu.add("1D300 1D35F Symboles du Classique du mystère suprême")
        #self.lst_mnu.add("1D400 1D7FF Symboles mathématiques alphanumériques")
        #self.lst_mnu.add("1FFFE 1FFFF non-caractères")
        
        # --------------------------------------------------------------
        # --- "20000 2FFFF Plan idéographique supplémentaire (SIP)"
        # --------------------------------------------------------------
        
        #self.lst_mnu.add("20000 2A6D6 Supplément B aux idéogrammes unifiés CJC")
        #self.lst_mnu.add("2F800 2FA1F Supplément aux idéogrammes de compatibilité CJC")
        #self.lst_mnu.add("2FFFE 2FFFF non-caractères")
        
        # --------------------------------------------------------------
        # --- "30000 DFFFF Plans supplémentaires réservés"
        # --------------------------------------------------------------
        
        #self.lst_mnu.add("3FFFE 3FFFF non-caractères")
        #self.lst_mnu.add("4FFFE 4FFFF non-caractères")
        #self.lst_mnu.add("5FFFE 5FFFF non-caractères")
        #self.lst_mnu.add("6FFFE 6FFFF non-caractères")
        #self.lst_mnu.add("7FFFE 7FFFF non-caractères")
        #self.lst_mnu.add("8FFFE 8FFFF non-caractères")
        #self.lst_mnu.add("9FFFE 9FFFF non-caractères")
        #self.lst_mnu.add("AFFFE AFFFF non-caractères")
        #self.lst_mnu.add("BFFFE BFFFF non-caractères")
        #self.lst_mnu.add("CFFFE CFFFF non-caractères")
        #self.lst_mnu.add("DFFFE DFFFF non-caractères")
        
        # --------------------------------------------------------------
        # --- "E0000 EFFFF Plan spécial supplémentaire (SSP)"
        # --------------------------------------------------------------
        
        #self.lst_mnu.add("E0000 E007F Étiquettes")
        #self.lst_mnu.add("E0100 E01EF Supplément de sélecteurs de variante")
        #self.lst_mnu.add("EFFFE EFFFF   non-caractères")
        
        # --------------------------------------------------------------
        # --- "F0000 10FFFF Plans supplémentaires à usage privé"
        # --------------------------------------------------------------
        
        #self.lst_mnu.add("F0000 FFFFD Zone supplémentaire A à usage privé")
        #self.lst_mnu.add("FFFFE FFFFF non-caractères")
        #self.lst_mnu.add("100000 10FFFD Zone supplémentaire B à usage privé")
        #self.lst_mnu.add("10FFFE 10FFFF non-caractères")
        
        
        
        return


    def seeTable(self, i, sender):
        u"""
        Afficher la sous-table UNICODE  cliquée
        """
        
        
        # --- Clic sur noeud racine > ne rien faire
        #~ if xxxxxx:
            #~ afficher = "Choisissez une sous-table"
            #~ return
        
        
        # --- Extraire les codes Unicode
        
        codes = self.lst_mnu.getitem(i).split()
        c1 = int(codes[0],16)
        c2 = int(codes[1],16)
        
        
        # --------------------------------------
        # --- Charger les étiquettes            
        # --------------------------------------
        
        # Déterminer la police
        fnt = ""
        if self.vFnt.get() == 2:
            fnt = app.txt_notice['font']
        else:
            fnt = app.txt_search['font']

        # Effacer les étiquettes antérieures
        self.fr_chr.destroy()
        self.fr_D.update()
        
        # Charger les nouvelles étiquettes
        self.fr_chr = Frame(self.fr_D, borderwidth=2, relief=GROOVE)
        self.fr_chr.grid(row=1, column=0, padx=4, pady=4, ipadx=2, ipady=2)
        
        
        row = 0
        col = -1
        for code in range(c1,c2+1):
            col += 1
            if col > 31:
                row += 1
                col = 0
            
            w = Label(self.fr_chr, borderwidth=1, relief=SOLID, width=1, height=1, bg='white', font=fnt)
            w.grid(row=row, column=col, sticky='w', ipady=3, ipadx=3)
            w.bind('<Double-1>', self.dbClicOnChr)
            try:
                w['text'] = unichr(code)
            except:
                w['text'] =""
        
        return


    def dbClicOnChr(self,event):
        u"""
        Double-clic sur un caractère Unicode > l'ajouter à la case de texte
        """
        
        self.txt_str.insert(END, event.widget['text'])
        
        return


    def copyStr(self):
        u"""
        Copier le contenu  de la zone de texte dans le presse-papiers
        """
        
        self.txt_str.clipboard_clear()
        self.txt_str.clipboard_append(self.txt_str.get())
        
        return




class WindowMAIN(Tk):
    u"""
    Fenêtre principale de l'application
    """

    def __init__(self, version):
        u"""
        CREER LA FENETRE PRINCIPALE DE L'APPLICATION
        """
        
        # Objet répertoire des données d'application (en fonction de la plateforme)
        self.appdata = AppData("linguae")
        
        # Objet fichier ini 
        self.ini = Inifile(os.path.join(self.appdata.path, "linguae.ini"))
        
        # Objet Aide (on l'initialise avec le chemin complet du fichier d'aide)
        self.hlp = lib.adjutor.Adjutor(os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])),"linguae.adj"))
        
        
        
        # ============================================
        # === Interface graphique                     
        # ============================================
        
        Tk.__init__(self)
        
        self.title("Linguae")
        
        # Taille et position de la fenêtre : on applique les valeurs mémorisées dans l'ini
        # ou sinon des valeurs par défaut
        self.geometry(self.ini.get('Geometry','900x650+30+30')) 
        
        
        self.imglst = lib.myimg.ImgList()       # Compiler les images de l'interface
        
        self.version = version                  # Version du logiciel (string)
        self.dic = None                         # objet dictionnaire courant (sera chargé par .loadDico)
        
        
        self.__makeToolBar()                    # Construction de la barre d'outils
        
        
        # ======================================
        # === Panneau redimensionnable général  
        # ======================================
        
        self.panM=PanedWindow(self, orient=HORIZONTAL, sashpad=4, sashwidth=6, sashrelief=RAISED)
        self.panM.pack(expand=True,fill=BOTH)
        
        
        # ============================================
        # === Frame des entrées de dico (à gauche)    
        # ============================================
        
        fr_dico = Frame(self.panM, borderwidth=0, padx=2, pady=2)
        self.panM.add(fr_dico)
        
        
        # -----------------------------------
        # --- Frame de recherche + drapeaux  
        # -----------------------------------
        
        fr_search = Frame(fr_dico, borderwidth=0)
        fr_search.pack(fill=X)
        
        # Case du drapeau 1
        self.lab_img1=Label(fr_search)
        self.lab_img1.grid(row=0, padx=3, pady=6, column=0)
        
        # Case de recherche
        self.txt_search = Entry(fr_search, width=25, bg="white", borderwidth=2, highlightthickness=0)
        self.txt_search.bind('<KeyRelease>', self.onTheFlySearch)
        self.txt_search.grid(row=0, padx=3, pady=6, column=1)
        self.txt_search.ib=lib.infobulle.InfoBulle(parent=self.txt_search, text="Recherche rapide : tapez un début de mot", temps=400)
        
        # Case du drapeau 2
        self.lab_img2=Label(fr_search)
        self.lab_img2.grid(row=0, padx=3, pady=6, column=2)
        
        
       
        # ---------------------------------
        # --- Liste des entrées du dico    
        # ---------------------------------
        
        r=1
        
        # nb : augmenter selectborderwidth permet d'espacer les mots dans la liste
        #      width se comporte en largeur mini de visibilité de la Scrollbar
        self.lst_dico = Listbox(fr_dico, selectborderwidth=3, width=5, borderwidth=2, bg="white", highlightthickness=0)
        self.lst_dico.bind('<ButtonRelease-1>', self.selDisplayData)
        self.lst_dico.pack(expand=True, fill=BOTH, side=LEFT)
        
        self.bind('<Up>', self.listDicoUpDown)
        self.bind('<Down>', self.listDicoUpDown)
        self.bind('<Prior>', self.listDicoUpDown)
        self.bind('<Next>', self.listDicoUpDown)
        self.bind('<MouseWheel>', self.scrollWheel)
        
        # Scrollbar des  entrées du dico
        self.scr = Scrollbar(fr_dico, command=self.lst_dico.yview)
        self.lst_dico['yscrollcommand'] = self.scr.set
        self.scr.pack(side=LEFT, fill=Y)
        
        
        # ====================================================
        # === Frame global Droit                              
        # ====================================================
        
        wOngl = 160
        hOngl = 20
        
        fr_D = Frame(self.panM, pady=2, padx=2)
        self.panM.add(fr_D)
        
        
        # --------------------------------------------------
        # --- Frame des faux-onglets (boutons)              
        # --- et frame dde leur conteneur cible d'affichage 
        # --------------------------------------------------
        
        r=0
        fr_ongl = Frame(fr_D)
        fr_ongl.pack(fill=X)
        
        # Onglets
        self.ongl_0 = Button(fr_ongl, text='Traduction & Définition', image=self.imglst.puceS, compound=LEFT, height=hOngl, width=wOngl, borderwidth=0, background='white')
        self.ongl_0['font']=("Helvetica",9,"normal")
        self.ongl_0.pack(side=LEFT, anchor=W, padx=3)
        
        self.ongl_plugin = Button(fr_ongl, text='(greffon : aucun)', image=self.imglst.puceX, compound=LEFT, height=hOngl, width=wOngl, borderwidth=0, foreground='white', background='#B4B4B4')
        self.ongl_plugin['font']=("Helvetica",9,"normal")
        self.ongl_plugin.pack(side=LEFT, padx=3)
        
        # Frame conteneur cible d'affichage
        self.fr_tabs = Frame(fr_D, borderwidth=2, relief='ridge', padx=4, pady=4)
        self.fr_tabs.pack(expand=True, fill=BOTH)
        
        
        # ------------------------------------------------------------
        # --- Frame des Traductions & Définitions (panneau/onglet 1)  
        # ------------------------------------------------------------
        
        fr_def = Frame(self.fr_tabs)
        fr_def.pack(expand=True, fill=BOTH)
        
        
        # Case de la notice
        # (la largeur affectée se comporte en largeur minimale) 
        self.txt_notice=ScrolledText(fr_def, highlightthickness=0, width=20, borderwidth=1, relief=SOLID, background='white', wrap=WORD, state='disabled')
        self.txt_notice.pack(side=LEFT, expand=True, fill=BOTH)
        
        # Styles d'affichage
        # (les styles de police sont dans appliquerFont)
        self.txt_notice.tag_config("normal", lmargin1=5, lmargin2=5, spacing1=5)
        self.txt_notice.tag_config("miroir", lmargin1=5, lmargin2=5, spacing1=5, spacing3=5, borderwidth=2, relief=SOLID, background="yellow")
        self.txt_notice.tag_config("short", lmargin1=5, lmargin2=5, spacing1=5, spacing3=2, foreground="maroon")
        self.txt_notice.tag_config("long", lmargin1=5, lmargin2=5, spacing1=5, foreground="navy")
        self.txt_notice.tag_config("phon", lmargin1=5, lmargin2=5, spacing1=5, foreground="grey")
        self.txt_notice.tag_config("u", underline=1)
        
        
        # --- Frame des listes latérales
        
        fr_lst=Frame(fr_def, padx=3)
        fr_lst.pack(side=RIGHT, fill=BOTH)
        
        # Bouton audio
        self.bt_audio = Button(fr_lst, width=120, compound=LEFT, text=" Ecouter ", image=self.imglst.audio, borderwidth=1, state='disabled', command=self.playWord, font=("Helvetica",8,"normal"), pady=2)
        self.bt_audio.grid(column=0)
        
        # Bouton des illustrations
        self.bt_img = Button(fr_lst, width=120, compound=LEFT, text=" Illustrations ", borderwidth=1, image=self.imglst.photo, state='disabled', command=self.showImage, font=("Helvetica",8,"normal"), pady=2)
        self.bt_img.grid(column=0, pady=3)
        
        wlst = 23
        # Liste des racines
        Label(fr_lst, text='Racines :', font=("Helvetica",8,"normal")).grid(column=0, sticky='W')
        self.lst_rac=lib.myscrlst.ScrolledList(fr_lst, height=4, width=wlst, onclic=self.seeRelation)
        self.lst_rac.grid(column=0, padx=2)
        self.lst_rac.ib=lib.infobulle.InfoBulle(parent=self.lst_rac, text="Cliquez sur une racine pour afficher sa notice", temps=400)
        self.lst_rac.ib.deactivate()
        
        # Liste des synonymes
        Label(fr_lst, text='Synonymes :', font=("Helvetica",8,"normal")).grid(column=0, sticky='W')
        self.lst_syn=lib.myscrlst.ScrolledList(fr_lst, height=4, width=wlst, onclic=self.seeRelation)
        self.lst_syn.grid(column=0, padx=2)
        self.lst_syn.ib=lib.infobulle.InfoBulle(parent=self.lst_syn, text="Cliquez sur un synonyme pour afficher sa notice", temps=400)
        self.lst_syn.ib.deactivate()
        
        # Liste des antonymes
        Label(fr_lst, text='Antonymes :', font=("Helvetica",8,"normal")).grid(column=0, sticky='W')
        self.lst_ant=lib.myscrlst.ScrolledList(fr_lst, height=4, width=wlst, onclic=self.seeRelation)
        self.lst_ant.grid(column=0, padx=2)
        self.lst_ant.ib=lib.infobulle.InfoBulle(parent=self.lst_ant, text="Cliquez sur un antonyme pour afficher sa notice", temps=400)
        self.lst_ant.ib.deactivate()
        
        # Liste des "voir aussi"
        Label(fr_lst, text='Voir aussi :', font=("Helvetica",8,"normal")).grid(column=0, sticky='W')
        self.lst_see=lib.myscrlst.ScrolledList(fr_lst, height=4, width=wlst, onclic=self.seeRelation)
        self.lst_see.grid(column=0, padx=2)
        self.lst_see.ib=lib.infobulle.InfoBulle(parent=self.lst_see, text="Cliquez sur un mot pour afficher sa notice", temps=400)
        self.lst_see.ib.deactivate()
        
        # Bouton de retour
        self.bt_back=Button(fr_lst, image=self.imglst.back, borderwidth=1, command=self.backToPreviousIdx, state='disabled')
        self.bt_back.grid(column=0, pady=5)
        lib.infobulle.InfoBulle(parent=self.bt_back, text="Retourner au mot précédent")
        
        
        # ---------------------------------------------------
        # --- Frame du plugin (panneau/onglet)               
        #             Emplacement du greffon = Widget        
        #     (le plugin est chargé par le menu des plugins) 
        # ---------------------------------------------------
        
        # le Frame est construit mais non affiché (non "packé")
        # il se sera lors d'une action sur les onglets
        self.fr_plugin  = Frame(self.fr_tabs)
        
        Message(self.fr_plugin, text="\nAucun greffon n'est actuellement chargé.\n\nPour charger un greffon, utilisez le menu 'Greffons'.\n\nPour ajouter un greffon à Linguae, copiez son fichier dans le sous-dossier 'Plugins' de l'application.", justify=LEFT, width=250).grid(row=0, column=0)
        
        
        # -------------------------
        # --- Gestion des onglets  
        # -------------------------
        
        listOngl = [ (self.ongl_0, fr_def), (self.ongl_plugin, self.fr_plugin )]
        self.ongl_0['command'] = lambda: onglClick(0, listOngl, self.imglst, self, 'p')
        self.ongl_plugin['command'] = lambda: onglClick(1, listOngl, self.imglst, self, 'p')
        
        
        # ---------
        self.__makeStatusBar()              # Construction de la barre de statut
        self.__makeMainMenu()               # Construction de la barre de menus
        
        
        # Barre de progression principale (masquée = non packée)
        # elle sera ensuite affichée et masquée par showProgr() et hideProgr()
        self.progr = lib.myprogbr.ProgressBar(self)
        
        
        
        
        # ======================================================
        # === Suite au chargement de l'interface graphique...   
        # ======================================================
        
        
        # --- Appliquer à l'interface les polices mémorisées dans l'ini   
        self.appliquerFont(self.ini.get('dicofont', '<defaut>?9?normal'), self.ini.get('traducfont','<defaut>?9?normal'))
        
        
        # --- Placement par défaut de la coulisse de redimensionnement
        self.panM.sash_place(0, int(self.ini.get('Xsash', '260')), 0)
        
        
        # --------------------------------------------------------------
        # -- Contrôler les nouvelles versions en mode échec silencieux  
        # --------------------------------------------------------------
        
        if self.ini.get("CheckNewVersion","1") == "1":
            self.__checkProgVersion(mute=True)
        
        
        # ---------------------------------------
        # --- Rouvrir le dernier plugin ouvert   
        # ---------------------------------------
        
        if self.ini.get("OpenLastPlugin","1") == "1":
            f = self.ini.get('lastopenplugin',"")
            if f:
                self.runPlugin(f)
        
        
        
        return
        
    
    def showProgr(self):
        u"""
        Affiche la barre de progression de la fenêtre principale
        à la place de la barre de statut
        """
        self.fr_status.pack_forget()
        self.progr.setWidth(self.winfo_width()-20)
        
        self.progr.pack(side=BOTTOM, fill=X)
        
        return
    
    
    def hideProgr(self):
        u"""
        Masque la barre de progression de la fenêtre principale
        et réaffiche la barre de statut
        """
        self.progr.pack_forget()
        self.fr_status.pack(side=BOTTOM,fill=X)
        
        return
    
    
    
    def __makeStatusBar(self):
        u"""
        Mise en place de la barre de statut
        """
        
        self.fr_status=Frame(self)
        self.fr_status.pack(side=BOTTOM,fill=X)
        
        # case de l'index
        self.status0 = Label(self.fr_status, text="...", width=6, borderwidth=1, relief=SUNKEN)
        self.status0.pack(side=LEFT, padx=2)
        lib.infobulle.InfoBulle(parent=self.status0, text="Index de l'entrée sélectionnée")
        
        # case de l'ID
        self.status3 = Label(self.fr_status, text="...", width=6, borderwidth=1, relief=SUNKEN)
        self.status3.pack(side=LEFT, padx=2)
        lib.infobulle.InfoBulle(parent=self.status3, text="Identifiant de l'entrée sélectionnée")
        
        # Case de l'encodage
        self.status1 = Label(self.fr_status, text="(aucun dictionnaire chargé)", width=22, borderwidth=1, relief=SUNKEN)
        self.status1.pack(side=LEFT, padx=2)
        lib.infobulle.InfoBulle(parent=self.status1, text="Encodages de l'entrée sélectionnée")
        
        # Case de l'indicateur de protection
        self.status4 = Label(self.fr_status, text="", image=self.imglst.caden0, width=18, borderwidth=1, relief=SUNKEN)
        self.status4.pack(side=LEFT, padx=2)
        self.status4.ib=lib.infobulle.InfoBulle(parent=self.status4, text="Dictionnaire non protégé")
        
        # Case de la version du programme
        status2 = Label(self.fr_status, text="Linguae "+ self.version, borderwidth=1, relief=SUNKEN)
        status2.pack(side=LEFT, padx=2, expand=True, fill=X)
        
        # Poignée de redimensionnement
        Label(self.fr_status, image=self.imglst.poign).pack(side=LEFT)
        
        self.status0['font']="Helvetica 8 normal"
        self.status1['font']="Helvetica 8 normal"
        status2['font']="Helvetica 8 normal"
        self.status3['font']="Helvetica 8 normal"
        
        return


    def scrollWheel(self, event):
        u"""
        Scroll avec la roulette de souris
        """
        
        if event.delta <0:
            self.lst_dico.yview_scroll(3, 'units')
        else:
            self.lst_dico.yview_scroll(-3, 'units')
        
        return


    def listDicoUpDown(self, event):
        u"""
        Action sur les touches haut-bas :
        déplacer le curseur dans la liste des entrées
        """
        
        try:
            i = int(self.lst_dico.curselection()[0])   # index du mot en sélection
        except:
            return
        
        if event.keysym == "Up":
            # Correction d'une différence de comportement entre Tkinter 2.5 et 2.6
            if str(hex(sys.hexversion))[2:5] == "206":
                v = 0
            else:
                v = -1
        elif event.keysym == "Down":
            # Correction d'une différence de comportement entre Tkinter 2.5 et 2.6
            if str(hex(sys.hexversion))[2:5] == "206":
                v = 0
            else:
                v = 1
        elif event.keysym == "Prior":
            v = -20
        elif event.keysym == "Next":
            v = 20
        else: return
        
        self.lst_dico.selection_clear(0, END)
        
        i= i + int(v)
        
        if i < 0: i = 0
        if i >= self.lst_dico.size(): i = self.lst_dico.size()-1
        
        self.lst_dico.activate(i)
        self.lst_dico.selection_set(i)
        self.lst_dico.see(i)
        self.selDisplayData(None)
        
        return



    def playWord(self):
        u"""
        Ecouter la prononciation du mot en sélection (appel par clic sur le bouton)
        """
        
        # Librairie audio Snack activée ?
        if not self.snackisactive:
            tkMessageBox.showwarning(parent=self,
                    title=u"Ecouter la prononciation",
                    message=u"Les fonctions audio de Linguae sont désactivées.\n\nVérifiez l'installation correcte de la librairie audio 'Snack'.")
            return
        # ---
        
        id = self.status3['text']
        
        mp3 = os.path.join(self.dic.path + "_res","mp3",id + ".mp3")
        
        # -------------------
        # --- Ecouter        
        # -------------------
        
        try:
            self.mysound = tkSnack.Sound()
            self.mysound.read(mp3) 
            self.mysound.play()
        except:
            tkMessageBox.showwarning(parent=self,
                title=u"Ecouter la prononciation",
                message=u"Echec de la lecture du fichier audio !")
            try:
                self.mysound.stop()
            except:
                pass
            return
        
        return


    def __checkDicVersion(self, mute=False):
        
        u"""
        Contrôle la disponibilité d'une nouvelle version téléchargeable du dictionnaire courant.
        Pour cela on recherche un petit fichier d'information réferencé par l'attribut .verUrl de l'objet dico
        
        NB : ce contrôle ne peut se faire de manière fiable par accés direct au fichier dictionnaire en ligne
             et lecture de son numéro de version car ce fichier est très souvent protégé par des
             dispositifs anti-leeching/hotlinking.
        
        Si 'mute' est True :
            - L'échec de connexion est silencieux
            - Le résultat de la recherche est silencieux s'il n'y a pas de nouvelle version disponible
        """
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Vérifier la version du dictionnaire",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        
        # ---------------------------------------------------------------
        # --- Récupérer le n° de la version disponible sur le Web        
        # ---------------------------------------------------------------
        
        # --- Ouvrir l'url du mini-fichier d'info
        
        try:
            lignes = urllib.urlopen(self.dic.verUrl).readlines()
        except:
            if not mute:
                tkMessageBox.showwarning(parent=self,
                        title="Rechercher une nouvelle version du dictionnaire",
                        message="Echec de la connexion Internet pour ce dictionnaire")
            return
        
        newvlit = ""
        newv = ""
        
        for ligne in lignes:
            if ligne.startswith("$latbotVer"):
                
                newvlit = ligne.split('"')[1]           # libellé littéral (avec mentions beta ou autres)
                newv = newvlit.split()[0].split(".")    # liste des composants numériques
                
                break
        
        # ------------------------------------------------
        # --- Comparer avec le n° de la version locale    
        # ------------------------------------------------
        
        oldv = self.dic.dicVersionNumber.split()[0].split(".") # liste des composants numériques
        
        flagnew = False
        
        if int(newv[0]) > int(oldv[0]):
            flagnew = True
        elif int(newv[0]) == int(oldv[0]):
            if int(newv[1]) > int(oldv[1]):
                flagnew = True
        
        if not flagnew: # Rien de neuf
        
            if mute:
                return
            else:
                tkMessageBox.showinfo(parent=self,
                        title="Rechercher une nouvelle version du dictionnaire",
                        message="La version utilisée est la plus récente disponible.")
                return
        
        # ----------------------------------------
        # --- Avertissement de nouvelle version   
        # ----------------------------------------
        
        r = tkMessageBox.askyesno(parent=self,
                title="Rechercher une nouvelle version du dictionnaire",
                message="Une nouvelle version est disponible en téléchargement\n\nVersion locale : " + self.dic.dicVersionNumber \
                        +"\nVersion téléchargeable : " + newvlit \
                        +"\n\nDésirez-vous vous connecter au site Web du dictionnaire ?")
        
        if r:
            self.toWebDic()# Connexion au site web du dictionnaire courant   
        
        
        return



    
    def __checkProgVersion(self, mute=False):
        
        u"""
        Contrôle la disponibilité d'une nouvelle version en ligne du programme.
        
        Si 'mute' est True :
            - L'échec de connexion est silencieux
            - Le résultat de la recherche est silencieux s'il n'y a pas de nouvelle version disponible
        """
        
        # --------------------------------------------------------
        # --- Récupérer le n° de la nouvelle version sur le Web   
        # --------------------------------------------------------
        
        try:
            lignes = urllib.urlopen('http://stalikez.info/linguae/inc/version.ver').readlines()
        except:
            if not mute:
                tkMessageBox.showwarning(parent=self,
                        title="Rechercher une nouvelle version",
                        message="Echec de la connexion Internet")
            return
        
        newvlit = ""
        newv = ""
        
        for ligne in lignes:
            if ligne.startswith("$linguaeVer"):
                
                newvlit = ligne.split('"')[1]           # libellé littéral (avec mentions beta ou autres
                newv = newvlit.split()[0].split(".")    # liste des composants numériques
                
                break
        
        # ------------------------------------------------
        # --- Comparer avec le n° de la version locale    
        # ------------------------------------------------
        
        oldv = self.version.split()[0].split(".") # liste des composants numériques
        
        flagnew = False
        
        if int(newv[0]) > int(oldv[0]):
            flagnew = True
        elif int(newv[0]) == int(oldv[0]):
            if int(newv[1]) > int(oldv[1]):
                flagnew = True
            elif int(newv[2]) > int(oldv[2]):
                flagnew = True
        
        if not flagnew: # Rien de neuf
        
            if mute:
                return
            else:
                tkMessageBox.showinfo(parent=self,
                        title="Rechercher une nouvelle version",
                        message=u"La version utilisée est la plus récente disponible :\n\n" + self.version)
                return
        
        # ----------------------------------------
        # --- Avertissement de nouvelle version   
        # ----------------------------------------
        
        r = tkMessageBox.askyesno(parent=self,
                title="Rechercher une nouvelle version",
                message="Une nouvelle version est disponible en téléchargement\n\nVersion locale : " + self.version \
                        +"\nVersion téléchargeable : " + newvlit \
                        +"\n\nDésirez-vous vous connecter au site Web de Linguae pour la télécharger ?")
        
        if r:
            self.toWebLing(cible="maj") # Connexion à la page de téléchargement du site web du programme   
        
        
        return
    
    

    def __makeMainMenu(self):
        
        u"""
        Construction de la barre de menu de la fenêtre principale
        """
        
        self.menubar = Menu(self)
	
	
        # Forcer la police en "normal" (car vilain "gras" par défaut avec anciens Tkinter/Linux)
        try:
		strfont = str(self.menubar['font'])
		
		if strfont.startswith("{"):     # nom de police à termes multiples
		    mnufont = strfont.split("}")
		    mnufont  = (mnufont[0][1:] , int(mnufont[1].split()[0]), "normal")
		else:                           # nom de police à terme unique
		    mnufont = strfont.split()
		    mnufont  = (mnufont[0], mnufont[1], "normal")
        except: 
            # erreur générée avec Tkinter/Linux récent ( ['font'] retourne TkFont et non un descripteur de police)
            # > on laisse la police par défaut car elle n'est plus en "gras"
            pass
		
        
        self.menubar['font'] = mnufont
        
        
        # -------------------------
        # --- Menu 'Fichier'       
        # -------------------------
        
        self.m_file = Menu(self.menubar, tearoff=0, font=mnufont)
        
        self.m_file.add_command(label="Ouvrir un dictionnaire...", compound=LEFT, image=self.imglst.open, command=self.openDicFromDialog)
        self.m_file.add_command(label="Créer un nouveau dictionnaire...", compound=LEFT, image=self.imglst.open, command=self.showOpenNewDic)
        self.m_file.add_command(label="Propriétés...", compound=LEFT, image=self.imglst.propr, command=self.showProperties)
        self.m_file.add_separator()
        self.m_file.add_command(label="Importer...", compound=LEFT, image=self.imglst.imprt, command=self.showImport)
        self.m_file.add_command(label="Exporter...", compound=LEFT, image=self.imglst.export, command=self.showExport)
        self.m_file.add_command(label="Construire le dictionnaire inverse...", compound=LEFT, image=self.imglst.reverse, command=self.showReverse)
        self.m_file.add_command(label="Voir le dictionnaire inverse...", compound=LEFT, image=self.imglst.seerev, command=self.openReverseDic)
        self.m_file.add_separator()
        self.m_file.add_command(label="Quitter", compound=LEFT, image=self.imglst.exit, command=self.queryUnload)
        
        self.menubar.add_cascade(label="Fichier", underline=0, menu=self.m_file)
        
        
        # -------------------------
        # --- Menu 'Dictionnaires' 
        # -------------------------
        x=0
        self.m_dic = Menu(self.menubar, tearoff=0, title="mDic", font=mnufont)
        self.m_dic.add_command(label='(Aucun dictionnaire)', state='disabled', command=self.doNothing) # une commande est nécessaire pour permettre vidage du menu par .delete (bug Python 2.6)
        self.menubar.add_cascade(label="Dictionnaires", underline=0, menu=self.m_dic)
        
        self.updateMenuDic()
        
        
        
        # -------------------------
        # --- Menu 'Chercher'      
        # -------------------------
        
        self.menubar.add_command(label="Chercher", underline=0, command=self.showFind, font=mnufont)
        
        
        # -------------------------
        # --- Menu 'Edition'       
        # -------------------------
        
        self.m_edit = Menu(self.menubar, tearoff=0, font=mnufont)
        
        self.m_edit.add_command(label="Trier le dictionnaire...", compound=LEFT, image=self.imglst.sortcr, command=self.showSort)
        self.m_edit.add_separator()
        self.m_edit.add_command(label="Editer l'entrée de dictionnaire...", compound=LEFT, image=self.imglst.edit, command=self.showEdit)
        self.m_edit.add_command(label="Supprimer l'entrée...", compound=LEFT, image=self.imglst.suppr, command=self.editDel)
        self.m_edit.add_command(label="Déplacer l'entrée...", compound=LEFT, image=self.imglst.wmove, command=self.editMove)
        self.m_edit.add_command(label="Ajouter une nouvelle entrée...", compound=LEFT, image=self.imglst.add, command=self.showEditAdd)
        self.m_edit.add_separator()
        self.m_edit.add_command(label="Table des caractères...", compound=LEFT, image=self.imglst.carspec, command=self.showUnicode)
        self.m_edit.add_separator()
        self.m_edit.add_command(label="Modifier les icônes du dictionnaire...", compound=LEFT, image=self.imglst.setic, command=self.showSetIcon)
        
        self.menubar.add_cascade(label="Edition", underline=0, menu=self.m_edit)
        
        
        # -------------------------
        # --- Menu 'Options'       
        # -------------------------
        
        self.m_opt = Menu(self.menubar, tearoff=0, font=mnufont)
        self.vPv2br = IntVar()
        self.m_opt.add_checkbutton(label='Passer à la ligne les " ; "', variable=self.vPv2br)
        self.vPv2br.set(1)
        self.m_opt.add_separator()
        self.m_opt.add_command(label="Transparence...", compound=LEFT, image=self.imglst.rien, command=self.showAlpha)
        pf = sys.platform.lower()
        if pf[:3] == 'win': # Windows
            pass
        elif pf[:6] == 'darwin' or pf[:5] == 'linux':# --- Unix (MacosX, Linux, )
            pass
        
        # Autres plateformes : on utilise le répertoire de l'application
        # (en attente de documentation sur les chemins précis)
        # (autres valeurs possibles : 'mac', 'sunos5', 'PalmOS3', 'hp-ux11')
        else:
            
            repAppData = os.path.dirnam
        
        
        
        self.m_opt.add_separator()
        self.m_opt.add_command(label="Préférences...", compound=LEFT, image=self.imglst.tools, command=self.showOptions)
        
        
        self.menubar.add_cascade(label="Options", underline=0, menu=self.m_opt)
        
        
        # -------------------------
        # --- Menu Outils          
        # -------------------------
        
        self.m_tool = Menu(self.menubar, tearoff=0, font=mnufont)
        self.m_tool.add_command(label="Aide à la traduction...", compound=LEFT, image=self.imglst.traduc, command=self.showTranslate)
        self.m_tool.add_separator()
        self.m_tool.add_command(label="Contruire un dictionnaire par héritage...", compound=LEFT, image=self.imglst.herit, command=self.showHerit)
        self.m_tool.add_separator()
        self.m_tool.add_command(label="Convertir un gif en base64...", compound=LEFT, image=self.imglst.rien, command=self.convGifToBase64)
        
        
        self.menubar.add_cascade(label="Outils", underline=1, menu=self.m_tool)
        
        
        # -------------------------
        # --- Menu Plugins         
        # -------------------------
       
        self.m_plug = Menu(self.menubar, tearoff=0, font=mnufont)
        self.m_plug.add_command(label="Aucun", compound=LEFT, image=self.imglst.plugnone, command=self.stopPlugin)
        
        # Chemin du répertoire des plugins
        dirplug = os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])),'plugins')
        
        # Scanner le répertoire des plugins
        if os.path.isdir(dirplug):
            n=0
            self.imgplug=[] # liste des icônes des plugins
            for f in os.listdir(dirplug):
                if f != "__init__.py" and f.endswith(".py"):
                    f = f.replace(".py","")                       # Masquer l'extension
                    t = f.capitalize()
                    
                    # Importer immédiatement le plugin (nécessaire pour accèder à son icône)
                    try:
                        exec ("import plugins." + f) in globals()
                        self.imgplug.append(PhotoImage(data= eval("plugins." + f +".Widget.codeImage")))
                    except:
                        self.imgplug.append(None)
                    
                    # ajouter le plugin au menu
                    self.m_plug.add_command(label=t, compound=LEFT, image=self.imgplug[n], command= lambda arg=f: self.runPlugin(arg))
                    
                    n += 1
        
        
        self.menubar.add_cascade(label="Greffons", underline=0, menu=self.m_plug)
        
        # -------------------
        # --- Menu Aide      
        # -------------------
        #(certains menus sont activés par LoadDico et l'instanciation de la fenêtre d'info)
        self.m_hlp = Menu(self.menubar, tearoff=0, font=mnufont)
        self.m_hlp.add_command(label="A propos de Linguae...", compound=LEFT, image=self.imglst.info, command=self.showAbout)
        self.m_hlp.add_command(label="Aide de Linguae...", compound=LEFT, image=self.imglst.hlp, command=self.showHelp)
        self.m_hlp.add_command(label="Site Web de Linguae...", compound=LEFT, image=self.imglst.web, command=self.toWebLing)
        self.m_hlp.add_command(label="Rechercher une nouvelle version de Linguae...", compound=LEFT, image=self.imglst.chknewver, command=self.__checkProgVersion)
        
        self.m_hlp.add_separator()
        
        self.m_hlp.add_command(label="A propos de ce dictionnaire...", compound=LEFT, image=self.imglst.info, command=self.showAboutDic, state='disabled')
        self.m_hlp.add_command(label="Contacter l'auteur de ce dictionnaire...", compound=LEFT, image=self.imglst.mail, command=self.sendMailDic, state='disabled')
        self.m_hlp.add_command(label="Site Web de ce dictionnaire...", compound=LEFT, image=self.imglst.web, command=self.toWebDic, state='disabled')
        self.m_hlp.add_command(label="Rechercher une nouvelle version de ce dictionnaire...", compound=LEFT, image=self.imglst.chknewver, command=self.__checkDicVersion, state='disabled')
        
        self.m_hlp.add_separator()
        self.m_hlp.add_command(label="Chercher des dictionnaires sur le Web...", compound=LEFT, image=self.imglst.web, command=self.toWebPolyglotte)
        
        self.menubar.add_cascade(label="Aide", underline=0, menu=self.m_hlp)
        
        # ---
        
        self['menu'] = self.menubar
        
        
        return


    def doNothing(self):
        pass


    def scanrepdic(self, rep, m_parent, mnufont):
        u"""
        Scannage résursif d'un répertoire contenant des dictionnaire
        avec complémentation parallèle du menu 'Dictionnaires'
        retourne le nombre d'éléments (dictionnaire ou dossier) trouvés
        """
        
        lstfich = os.listdir(rep)    # liste des entrées (fichiers et dossiers) du répertoire lié au menu
        
        lstfich.sort(lambda x,y: cmp(x.lower(),y.lower())) # tri alphabétique insensible à la casse
        
        n=0
        for f in lstfich:
            
            if os.path.isfile(os.path.join(rep,f)): # c'est un fichier
            
                ext = os.path.splitext(f)[1].lower()
                
                if ext == ".ling":
                    m_parent.add_command(label=f, compound=LEFT, image=self.imglst.dicling, command=lambda arg1=f, arg2=m_parent : self.openDicFromMenuDic(arg1, arg2))
                    n += 1
                elif ext == ".dict":
                    m_parent.add_command(label=f, compound=LEFT, image=self.imglst.dicdict, command=lambda arg1=f, arg2=m_parent : self.openDicFromMenuDic(arg1, arg2))
                    n += 1
                elif ext in (".dz", ".gz"):
                    if "dict." in f:
                        m_parent.add_command(label=f, compound=LEFT, image=self.imglst.dicdict, command=lambda arg1=f, arg2=m_parent : self.openDicFromMenuDic(arg1, arg2))
                        n += 1
                elif ext == ".xdxf":
                    m_parent.add_command(label=f, compound=LEFT, image=self.imglst.dicxdxf, command=lambda arg1=f, arg2=m_parent : self.openDicFromMenuDic(arg1, arg2))
                    n += 1
                elif ext == ".wb":
                    m_parent.add_command(label=f, compound=LEFT, image=self.imglst.dicwb, command=lambda arg1=f, arg2=m_parent : self.openDicFromMenuDic(arg1, arg2))
                    n += 1
                elif ext in (".csv", ".tab", ".txt"):
                    m_parent.add_command(label=f, compound=LEFT, image=self.imglst.diccsv, command=lambda arg1=f, arg2=m_parent : self.openDicFromMenuDic(arg1, arg2))
                    n += 1
                elif ext == ".ini":
                    m_parent.add_command(label=f, compound=LEFT, image=self.imglst.dicini, command=lambda arg1=f, arg2=m_parent : self.openDicFromMenuDic(arg1, arg2))
                    n += 1
                    
            else: # c'est un répertoire
                
                if not f.endswith("_res"): # Sauter les répertoires de ressources associées aux dicos
                
                    # Créer un sous-menu
                    m = Menu(m_parent, tearoff=0, title=m_parent['title'] + '|' + f, font=mnufont)
                    m_parent.add_cascade(label=f, compound=LEFT, image=self.imglst.folder, menu=m)
                    n += 1
                    
                    # scannage récursif de ce sous-répertoire
                    subrep = os.path.join(rep,f)              
                    self.scanrepdic(subrep, m, mnufont)
         
         
        if n == 0:
            m_parent.add_command(label='(Aucun dictionnaire)', state='disabled')       
                
        return n
    

    def updateMenuDic(self):
        u"""
        Met à jour le menu des dictionnaires
        à partir du contenu du répertoire des dictionnaires
        """
        
        mnufont = self.menubar['font']
        
        
        # -------------------------
        # --- Vider le menu        
        # -------------------------
        
        self.m_dic.delete(0, END)
        
        flagvide = True
        
        # ------------------------------------------
        # --- Définir le dossier des dictionnaires  
        # ------------------------------------------
        
        dp = self.ini.get('DicDefaultFolder','')
        
        if dp == "" or not os.path.isdir(dp):
            self.m_dic.add_command(label='(chemin non paramétré : voir "Options")', state='disabled')
            return
        
        
        # --------------------------------------------
        # --- Scanner le dossier des dictionnaires    
        # --------------------------------------------
        
        
        self.scanrepdic(dp, m_parent=self.m_dic, mnufont=mnufont) # (procédure récursive)
       
        return



    def __makeToolBar(self):
        u"""
        Constuire la barre d'outils
        de la fenêtre principale
        """
        x = 1
        y = 3
        ix = 1
        iy = 1
        
        tb = Frame(self, borderwidth=1, relief=RAISED)
        #tb.grid(row=0, column=0, columnspan=10, sticky='WE')
       
        
        B1 = Button(tb, image=self.imglst.open, borderwidth=1, relief=FLAT, command=self.openDicFromDialog, height=20, width=20)
        B1.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B1, text="Ouvrir un dictionnaire", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        B2=Button(tb, image=self.imglst.propr, borderwidth=1, relief=FLAT, command=self.showProperties, height=20, width=20)
        B2.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B2, text="Propriétés du dictionnaire", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        B3=Button(tb, image=self.imglst.export, borderwidth=1, relief=FLAT, command=self.showExport, height=20, width=20)
        B3.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B3, text="Exporter le dictionnaire", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        Label(tb, image=self.imglst.toolsep).pack(side=LEFT) # séparateur
        
        B11=Button(tb, image=self.imglst.seerev, borderwidth=1, relief=FLAT, command=self.openReverseDic, height=20, width=20)
        B11.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B11, text="Ouvrir le dictionnaire inverse", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        Label(tb, image=self.imglst.toolsep).pack(side=LEFT) # séparateur
        
        B4=Button(tb, image=self.imglst.find, borderwidth=1, relief=FLAT, command=self.showFind, height=20, width=20)
        B4.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B4, text="Chercher dans les entrées de dictionnaire", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        Label(tb, image=self.imglst.toolsep).pack(side=LEFT) # séparateur
        
        B5=Button(tb, image=self.imglst.edit, borderwidth=1, relief=FLAT, command=self.showEdit, height=20, width=20)
        B5.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B5, text="Editer l'entrée de dictionnaire", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        B7=Button(tb, image=self.imglst.add, borderwidth=1, relief=FLAT, command=self.showEditAdd, height=20, width=20)
        B7.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B7, text="Ajouter une nouvelle entrée de dictionnaire", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        B8=Button(tb, image=self.imglst.suppr, borderwidth=1, relief=FLAT, command=self.editDel, height=20, width=20)
        B8.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B8, text="Supprimer l'entrée de dictionnaire sélectionnée", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        B10=Button(tb, image=self.imglst.sortcr, borderwidth=1, relief=FLAT, command=self.showSort, height=20, width=20)
        B10.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B10, text="Trier le dictionnaire", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        B9=Button(tb, image=self.imglst.carspec, borderwidth=1, relief=FLAT, command=self.showUnicode, height=20, width=20)
        B9.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B9, text="Afficher la table des caractères", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        Label(tb, image=self.imglst.toolsep).pack(side=LEFT) # séparateur
        
        B6=Button(tb, image=self.imglst.hlp, borderwidth=1, relief=FLAT, command=self.showHelp, height=20, width=20)
        B6.pack(padx=x, pady=y, side=LEFT, ipadx=ix, ipady=iy)
        lib.infobulle.InfoBulle(parent=B6, text="Voir l'aide de Linguae", arg_enter=self.enterMenuButton, arg_leave=self.leaveMenuButton)
        
        tb.pack(fill=X)
        
        return


    def toWebLing(self, cible=None):
        
        u"""
        Ouvrir la page web de Linguae dans le navigateur par défaut.
        
        'cible' détermine la page cible du site
        
        """
        
        # --------------------------------------------------
        # --- Connexion pour Mise à jour du programme       
        # --------------------------------------------------
        
        if cible == "maj":
            
            # Ouverture de la page de téléchargement
            # Le n° de la version actuellement utilisée est envoyée au serveur par l'url
            try:
                webbrowser.open('http://stalikez.info/linguae/dwnld.php?ex=' + self.version.split()[0])
            except:
                tkMessageBox.showwarning(parent=self,
                        title="Télécharger une nouvelle version de Linguae",
                        message="Echec de la connexion Internet")
        
        
        # ---------------------------------------------------
        # --- Connexion pour simple consultation du site     
        # ---------------------------------------------------
        
        else:
            # Ouverture de la page d'accueil
            try:
                webbrowser.open('http://stalikez.info/linguae')
            except:
                tkMessageBox.showwarning(parent=self,
                        title="Connexion au site Web de Linguae",
                        message="Echec de la connexion")
        
        return
    
    
    
    def toWebPolyglotte(self):
        
        u"""
        Ouvrir la page web de Polyglotte dans le navigateur par défaut
        """
        
        r = tkMessageBox.askyesno(parent=self,
                    title="Dictionnaires sur le Web",
                    message=u"Linguae va vous connecter au site du Projet POLYGLOTTE.\n\nLe but de celui-ci est, entre autres, de regrouper et centraliser un maximum de liens vers des dictionnaires numériques d'accès libre.\n\nConsultez en particulier la section 'Dictionnaires bilingues' de ce site.\n\n\nContinuer ?")
        if r == False: return
        
        try:
            webbrowser.open('http://polyglotte.tuxfamily.org')
        except:
            pass
        
        return
    
    
    def toWebDic(self):
        
        u"""
        Ouvrir la page Web du dictionnaire dans le navigateur par défaut
        """
        
        try:
            webbrowser.open(self.dic.dicUrl)
        except:
            pass
        
        return


    def sendMailDic(self):
        u"""
        Envoyer un e-mail à l'auteur du dictionnaire
        """
        
        targetAddr = self.dic.contactAuthor.strip()
        obj = "About your dictionary "
        
        if self.dic.dicName != "":
            obj += self.dic.dicName
        
        pf = sys.platform.lower()
        
        if pf[:3] == 'win':  
            os.startfile('mailto:'+ targetAddr + "?subject=" + obj)
        else:
            tkMessageBox.showwarning(parent=self,
                    title="Linguae",
                    message="Fonctionnalité non disponible sur cette plateforme.\n\nVeuillez contacter l'auteur à \n" + targetAddr +"\nà l'aide de votre logiciel de courrier habituel.")
        
        return
    

    def enterMenuButton(self, wButton):
        u""" Entrée de la souris sur un bouton de menu """
        
        wButton['relief']=RAISED 
        
        return


    def leaveMenuButton(self, wButton):
        u""" sortie de la souris d'un bouton de menu """
        
        wButton['relief']=FLAT
        
        return


    def backToPreviousIdx(self):
        u"""
        Affiche dans l'interface le mot (+ notice) précédemment affiché
        """
        
        # Récupérer l'index précédent dans l'objet dico
        
        idx = self.dic.previousIdx
        
        # Afficher mot et notice
        
        self.lst_dico.selection_clear(0, END)
        
        self.lst_dico.activate(idx)
        self.lst_dico.selection_set(idx)
        self.lst_dico.see(idx)
        self.selDisplayData(None)
        
        self.bt_back['state']='disabled'
        
        return


    def editMove(self):
        u"""
        Déplacer l'entrée de dictionnaire en sélection
        """
        
        # --- Contrôles
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Supprimer une entrée de dictionnaire",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        try:
            isource = int(app.lst_dico.curselection()[0]) # index du mot en sélection
        except:
            tkMessageBox.showinfo(parent=self,
                    title=u"Déplacer une entrée de dictionnaire",
                    message=u"Sélectionnez d'abord une entrée !")
            return
        
        # --- Demander l'index cible
        
        imax = self.dic.wordcount - 1
        
        icible = tkSimpleDialog.askstring(parent=self,
                title="Déplacer une entrée",
                prompt="Index actuel de l'entrée : "+ str(isource) +"\n\nIndiquez le nouvel index désiré\ndans l'intervalle suivant :\n\n   0 - " + str(imax) + "\n",
                initialvalue=str(isource))
        
        # --- Contrôles de la valeur fournie
        
        if not icible:
            return
        elif icible == isource:
            return
        
        flagbad = False 
        try:
            icible = int(icible)
        except:
            flagbad = True
            
        if icible >= imax:
            flagbad = True
        
        if flagbad:
            tkMessageBox.showwarning(parent=self,
                    title="Valeur incorrecte",
                    message="L'index cible doit être un chiffre entre 0 et " + str(imax))
            return
        
        # --- Déplacer
        
        self['cursor']="watch"
        
        i = self.dic.moveindic(isource, icible)
        
        # --- Sélectionner l'entrée déplacée
        if not i is None:
            self.dic.lstbox.activate(i)
            self.dic.lstbox.selection_set(i)
            self.dic.lstbox.see(i)
            self.selDisplayData(None)
        
        self['cursor']=""
        
        return
    

    def editDel(self):
        u"""
        Supprimer l'entrée de dictionnaire en sélection
        """
        
        # --- Contrôles
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Supprimer une entrée de dictionnaire",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        try:
            i = int(self.lst_dico.curselection()[0]) # index du mot en sélection
        except:
            tkMessageBox.showinfo(parent=self,
                    title=u"Supprimer une entrée de dictionnaire",
                    message=u"Sélectionnez d'abord une entrée !")
            return
        
        # --- Confirmer la suppression
        # (ne pas afficher le libellé de l'entrée : possibilité de problèmes d'encodages insolubles...)
        r = tkMessageBox.askokcancel(default='cancel',
                title=u"Confirmer la suppression de l'entrée",
                message=u"Etes-vous sûr de vouloir supprimer l'entrée d'index n° "+ str(i) + u" ?\nLa suppression sera irréversible.\n\nConfirmer ?")
        
        if not r: return
        
        # ------------------------------------
        # --- Aiguillage de la suppression    
        # ------------------------------------
        
        app['cursor'] = "watch"
        
        i = int(self.lst_dico.curselection()[0])    # index du mot en sélection à supprimer
        ID = self.status3['text']                   # ID du mot en sélection à supprimer
        
        if app.dic.type.startswith("ling"):
            self.editDelLing(i)
        
        elif app.dic.type.startswith("wb"):
            self.editDelWB(i)
        
        elif app.dic.type.startswith("dict"):
            self.editDelDict(i)
        
        elif app.dic.type.startswith("xdxf"):
            self.editDelCSVorINIorXDXF(i)
        
        elif app.dic.type.startswith("csv"):
            self.editDelCSVorINIorXDXF(i)
        
        elif app.dic.type.startswith("ini"):
            self.editDelCSVorINIorXDXF(i)
        
        
        # ------------------------------------------------
        # --- Rafraîchit l'affichage des données          
        # ------------------------------------------------
        # (affiche l'entrée suivante puisque i a été supprimé)
        
        self.dic.lstbox.update()
        if i > self.lst_dico.size()-1 : i == self.lst_dico.size()-1
        
        self.dic.lstbox.activate(i)
        self.dic.lstbox.selection_set(i)
        self.dic.lstbox.see(i)
        self.selDisplayData(None)
        
        
        # -----------------------------------------------------------------------
        # --- Effacement des ressources externes associées à l'entrée supprimée  
        # -----------------------------------------------------------------------
        
        
        # --- mp3 (prononciation)
        
        fichmp3 = os.path.join(self.dic.path + "_res","mp3",ID + ".mp3")
        try   : os.chmod(fichmp3, stat.S_IWRITE)       # Supprimer l'attribut lecture seule
        except: pass
    
        try   : os.remove(fichmp3)      # Supprimer le fichier
        except: pass
        
        # --- illustrations gif
        
        lf = glob.glob(os.path.join(self.dic.path + "_res", "gif", ID + "_*.gif"))
        for f in lf:
            try   : os.remove(f)
            except: pass
        
        
        # ---
        
        app['cursor'] = ""
        
        return



    def editDelLing(self, i):
        u"""
        Supprimer l'entrée courante d'un dictionnaire Ling
        """
        
        # Confectionner la nouvelle chaîne preling du dictionnaire en sautant
        # l'entrée à supprimer
        strPreling = self.dic.toStrPreling(progr=None, iEdit=i, contentEdit="---")
        
        # Confectionner la nouvelle chaîne Ling
        
        strLing = DicObject.strPrelingToStrLing(strPreling=strPreling)
        
        # Enregistre le nouveau fichier
        fichpath = self.dic.path
        app.dic.file.close()
        ff = open(fichpath, 'wb')
        ff.write(strLing)
        ff.close()
        
        # Recharge le dictionnaire avec le nouveau fichier
        self.loadDico(fichpath, showSplash=False)
        
        
        return


    def editDelWB(self, i):
        u"""
        Supprimer l'entrée courante d'un dictionnaire WB
        """
        
        # Mise à jour de la liste des entrées dans l'interface
        app.lst_dico.delete(i)
        
        
        # Confectionner la nouvelle chaîne du fichier
        
        fichpath = self.dic.path
        self.dic.file.seek(0)
        part1 = self.dic.file.read(84*i)
        self.dic.file.seek(84*(i+1))
        part2 = self.dic.file.read(os.path.getsize(fichpath))
        
        strfile = part1 + part2
        
        
        # Enregistre le nouveau fichier WB
        fichpath = self.dic.path
        app.dic.file.close()
        ff = open(fichpath, 'wb')
        ff.write(strfile)
        ff.close()
        
        
        # Recharge le dictionnaire avec le nouveau fichier
        cp1 = app.dic.encoding1
        cp2 = app.dic.encoding2
        self.loadDico(fichpath, cp1=cp1, cp2=cp2, showSplash=False)
        
        return ""


    def editDelDict(self, i):
        u"""
        Supprimer l'entrée courante d'un dictionnaire Dict
        """
        
        if self.dic.type in ("dictdz", "dictgz"):
            app.hideProgr()
            tkMessageBox.showinfo(parent=self,
                    title=u"Trier le dictionnaire",
                    message=u"Ce dictionnaire est au format DICT compressé, il ne peut être modifié.\n\nPour l'éditer, vous devrez d'abord le convertir au format DICT non compressé\nou tout autre format géré par Linguae (Menu 'Fichier' > 'Exporter').")
            return
        
        
        # Confectionner les nouvelles chaînes du dictionnaire Dict en sautant
        # l'entrée à suuprimer
        strs = self.dic.toStrsDict(progr=None, iEdit=i, contentEdit="---")
        
        
        # si strs est une chaine et non un tupple => problème !
        if type(strs) is type("xxx"):
            return strs
        
        
        # ------------------------------------------------------
        # --- Enregistrer les nouveaux fichiers *.idx et *.dict
        # ------------------------------------------------------
        
        radicfich = os.path.splitext(app.dic.path)[0]     # Radical des différents fichiers Dict
        
        app.dic.file.close()
        
        ff = open(radicfich + '.idx', 'wb')   # Enregistrer le *idx
        ff.write(strs[0])
        ff.close()
        
        ff = open(radicfich + '.dict', 'wb')  # Enregistrer le *dict
        ff.write(strs[1])
        ff.close()
        
        
        # ------------------------------------------------
        # --- Reconstruire le fichier *.ifo               
        # ------------------------------------------------
        lg = ""
        strifo = ""
        ff = open(radicfich + '.ifo', 'rU')
        
        for ligne in ff:
            
            if ligne.startswith("wordcount="):
                lg = "wordcount=" + strs[2] + "\n"
            elif ligne.startswith("idxfilesize="):
                lg = "idxfilesize=" + str(os.path.getsize(radicfich + ".idx")) + "\n"
            else:
                lg = ligne
                
            strifo += lg 
        
        ff.close()
        ff = open(radicfich + '.ifo', 'wb')   # Enregistrer le *ifo
        ff.write(strifo)
        ff.close()
        
        
        
        # --------------------------------------------------------------
        # --- Recharger le dictionnaire avec le nouveau fichier Dict    
        # --------------------------------------------------------------
        
        app.loadDico(radicfich + '.dict', showSplash=False)
        
        
        return ""



    def editDelCSVorINIorXDXF(self, i):
        u"""
        Supprimer l'entrée courante d'un dictionnaire CSV, INI, XDXF
        """
        
        
        # -----------------------------------------------------
        # --- Confectionner les blocs du fichier, à conserver  
        # -----------------------------------------------------
        
        fichpath = app.dic.path
        
        ii = i-1
        if ii<0: ii=0
        
        app.dic.file.seek(0)
        part1 = app.dic.file.read(app.dic.datamap[ii][0] + app.dic.datamap[ii][1])
        
        
        app.dic.file.seek(app.dic.datamap[i][0] + app.dic.datamap[i][1])
        part2 = app.dic.file.read(os.path.getsize(fichpath)) # lit jusqu'au bout du fichier
        
        
        # -------------------------------------------------
        # --- Mise à jour du fichier                       
        # -------------------------------------------------
        
        # Confectionner la nouvelle chaîne du fichier
        strfile = part1 + part2
        
        # Enregistrer le fichier
        app.dic.file.close()
        app.dic.file = open(fichpath, 'wb')
        app.dic.file.write(strfile)
        app.dic.file.close()
        app.dic.file = open(fichpath, 'rb')
        
        
        # Recharge le dictionnaire avec le nouveau fichier
        cp = app.dic.encoding1
        app.loadDico(fichpath, cp1=cp, showSplash=False)
        
        
        return ""


    def openReverseDic(self):
        u"""
        Charge le dictionnaire inverse
        """
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Ouvrir le dictionnaire inverse",
                    message=u"Chargez d'abord un dictionnaire de référence !")
            return
        
        
        # ------------------------------------------------------------------------------
        # --- Si le nom de fichier du dico inverse est inscrit dans le dico courant :   
        # ------------------------------------------------------------------------------
        
        if app.dic.reverseDicFileName != "":
            
            revfichpath = os.path.join(os.path.dirname(app.dic.path), app.dic.reverseDicFileName)
        
        
        
        # ------------------------------------------------------------------------------
        # --- Si le nom de fichier du dico courant se termine par "_rev"                
        # ------------------------------------------------------------------------------
        
        elif os.path.splitext(app.dic.path)[0].endswith("_rev"):
            
            revfichpath = os.path.splitext(app.dic.path)[0][:-4] + os.path.splitext(app.dic.path)[1]
        
        
        
        # -----------------------------------------------------------
        # --- Tenter de deviner le nom de fichier du dico inverse    
        # -----------------------------------------------------------
        
        else:
            
            f = os.path.splitext(os.path.basename(app.dic.path))
            
            fichname = f[0]
            fichext = f[1]
            revfich = ""
            revfichpath = ""
            flagtrouve = False
            
            for sp in (" - ", "-", " _ ", "_", "  ", " "):
                p = fichname.find(sp)
                if p>0:
                    
                    revfich = fichname[p+1:]+ sp + fichname[:p ] + fichext
                    revfichpath = os.path.join(os.path.dirname(app.dic.path), revfich)
                    if os.path.isfile(revfichpath): 
                        flagtrouve = True
                        break
            
            # On quitte si nom impossible à deviner 
            if not flagtrouve:
                tkMessageBox.showwarning(parent=self,
                        title=u"Ouvrir le dictionnaire inverse",
                        message=u"Le dictionnaire inverse est introuvable !\n\nNB : il est possible de construire automatiquement un dictionnaire inverse\nen utilisant la commande appropriée du menu 'Fichier'.") 
                return
        
        
        # -----------------------------------------
        # --- Rechercher les encodages si besoin   
        # -----------------------------------------
        
        ext = os.path.splitext(revfichpath)[1].lower() # Extension du fichier du dico
        
        ret = None
        cp1 = ""
        cp2 = ""
        flagcancel = False
        
        if not ext in ('.dict', '.ifo', '.gz', '.dz', '.ling', '.xdxf'):
            ret = self.getEncodings(revfichpath)
            cp1 = ret[0]
            cp2 = ret [1]
            flagcancel = ret[2]
        
        # -----------------------------
        # --- Ouvrir le dico inverse   
        # -----------------------------
        
        self.loadDico(revfichpath, cp1=cp1, cp2=cp2, showSplash=False)
        
        
        # --------------------------------------------
        # --- Mémoriser les encodages si nécessaire   
        # --------------------------------------------
        
        if not ret is None:
            encfile = Inifile(os.path.join(self.appdata.path, "encodings"))
            
            k = os.path.normpath(revfichpath).lower()
            encfile.write(k, cp1 + "|" + cp2)
            
            encfile = None
        
        
        return
    

    def openDicFromMenuDic(self, fichname, m_parent):
        
        u"""
        Charge le dictionnaire cliqué dans le menu "Dictionnaires"
        Affiche la fenêtre de choix des encodages si besoin
        
        Arguments :
            fichname : libellé du dico dans le menu
            m_parent : objet Menu contenant la ligne du dico
        """
        
        dicospath = self.ini.get('DicDefaultFolder','') # dossier des dictionnaires
        
        
        # ------------------------------------------------------------------------
        # --- Extraire l'arborescence du dico à partir du menu "Dictionnaires"    
        # ------------------------------------------------------------------------
      
        path = dicospath
        
        if m_parent['title'] != "mDic":
            
            pathitems = m_parent['title'].split("|")
            
            for i in range(1,len(pathitems)):   # on néglige le premier élément ("mDic")
                path = os.path.join(path, pathitems[i])
        
        fichpath = os.path.join(path, fichname) # chemin du dico à ouvrir
        
        
        # -----------------------------------------
        # --- Rechercher les encodages si besoin   
        # -----------------------------------------
        
        ext = os.path.splitext(fichpath)[1].lower() # Extension du fichier du dico
        
        ret = None
        cp1 = ""
        cp2 = ""
        flagcancel = False
        
        if not ext in ('.dict', '.ifo', '.gz', '.dz', '.ling', '.xdxf'):
            ret = self.getEncodings(fichpath)
            cp1 = ret[0]
            cp2 = ret [1]
            flagcancel = ret[2]
        
        
        # -------------------------------
        # --- Charger le dictionnaire    
        # -------------------------------
        
        if flagcancel:
            encfile = None
            return
        
        self.loadDico(fichpath, cp1=cp1, cp2=cp2)
        
        
        # --------------------------------------------
        # --- Mémoriser les encodages si nécessaire   
        # --------------------------------------------
        
        if not ret is None:
            encfile = Inifile(os.path.join(self.appdata.path, "encodings"))
            
            k = os.path.normpath(fichpath).lower()
            encfile.write(k, cp1 + "|" + cp2)
            
            encfile = None
        
        return



    def showOpenNewDic(self):
        u"""
        Affiche (instancie) la fenêtre de création d'un nouveau dico
        """
        
        self.f_newdic = WindowNewDic()   # NB : fenêtre modale
        
        
        return


    def appliquerFont(self, strFontDico, strFontTraduc):

        flagDicoDef=False
        flagTraducDef=False
        
        # ----------------------------------------------
        # --- Splittage des chaînes des paramètres      
        # ----------------------------------------------
        
        fntDico = strFontDico.split("?")
        fntTraduc = strFontTraduc.split("?")
        
        try:
            fntDico=(fntDico[0], int(fntDico[1]), fntDico[2])
        except:       # le splittage a échoué
            flagDicoDef = True
        
        try:
            fntTraduc=(fntTraduc[0], int(fntTraduc[1]), fntTraduc[2])
        except:       # le splittage a échoué
            flagTraducDef = True
        
        
        # -----------------------------------------------------------
        # --- Appliquer les polices en cas de succès du splittage    
        # -----------------------------------------------------------
        
        if not flagDicoDef:
            try:
                self.lst_dico['font'] = fntDico
                self.lst_rac.setFont(fntDico)
                self.lst_syn.setFont(fntDico)
                self.lst_see.setFont(fntDico)
                self.lst_ant.setFont(fntDico)
            
            except:
                flagDicoDef = True
        
        if not flagTraducDef:
            try:
                self.txt_notice['font'] = fntTraduc
                
                self.txt_notice.tag_config("b", font=(fntTraduc[0], int(fntTraduc[1]), "bold"))
                self.txt_notice.tag_config("i", font=(fntTraduc[0], int(fntTraduc[1]), "italic"))
                self.txt_notice.tag_config("small", font=(fntTraduc[0], int(fntTraduc[1])-2, fntTraduc[2]))
                self.txt_notice.tag_config("big", font=(fntTraduc[0], int(fntTraduc[1])+3, fntTraduc[2]))
                
            except:
                flagTraducDef = True
        
        
        # ------------------------------------------------
        # --- Echecs : appliquer les polices par défaut   
        # ------------------------------------------------
        
        if flagDicoDef:
            strFontDico="<defaut>?9?normal"
            
            self.lst_dico['font'] =('<defaut>', 9, 'normal')
            self.lst_rac.setFont(('<defaut>', 9, 'normal'))
            self.lst_syn.setFont(('<defaut>', 9, 'normal'))
            self.lst_see.setFont(('<defaut>', 9, 'normal'))
            self.lst_ant.setFont(('<defaut>', 9, 'normal'))
        
        if flagTraducDef:
            strFontTraduc = "<defaut>?9?normal"
            
            self.txt_notice['font'] = ('<defaut>', 9, 'normal')
            
            self.txt_notice.tag_config("b", font=('<defaut>', 9, 'bold'))
            self.txt_notice.tag_config("i", font=('<defaut>', 9, 'italic'))
            self.txt_notice.tag_config("small", font=('<defaut>', 7, 'normal'))
            self.txt_notice.tag_config("big", font=('<defaut>', 12, 'normal'))
        
        
        # -------------------------------------------
        # --- Mémoriser les polices dans l'ini       
        # -------------------------------------------
        
        self.ini.write('dicofont', strFontDico)
        self.ini.write('traducfont', strFontTraduc)
        
        
        return


    def loadDico(self, fichPath, cp1="", cp2="", showSplash=True):
        
        u"""
        1) Lit le ou les fichiers du dictionnaire :
          - charge les entrées d'index dans la listBox
          - crée l'objet app.dic et sa séquence 'app.dic.datamap' en mémoire (liste de tuples contenant
            les décalages et tailles des données)
        
        2) Ouvre et laisse ouvert le fichier des données du dico,
            accessible par l'objet fichier app.dic.file
            
        'showSplash' : booléen déterminant si la fenêtre des infos&statut s'affiche au chargement
        """
        
        self['cursor'] = "watch"
        
        # --------------------------------------
        # --- Réinitialiser l'interface         
        # --------------------------------------
        
        self.title("Linguae")                               # Barre de titre sans nom de fichier
        self.txt_notice['state'] = 'normal'
        self.txt_notice.delete("1.0", END)                  # Vider la case de la notice
        self.txt_notice['state'] = 'disabled'
        self.bt_audio['state']='disabled'                   # Désactiver le bouton "Ecouter"
        self.status1['text'] = ""                           # Vider la case "encodages" de la StatusBar
        self.status0['text'] = ""                           # Vider la case "index" de la StatusBar
        self.status3['text'] = ""                           # Vider la case "ID" de la StatusBar
        self.status4['image'] = self.imglst.caden0          # Icône de dico non protégé
        self.status4.ib.libel("Dictionnaire non protégé")
        self.lab_img1['image'] = None                       # Effacer l'image de langue 1
        self.lst_rac.clear()                                # Vider la liste des racines
        self.lst_syn.clear()                                # Vider la liste des synonymes
        self.lst_see.clear()                                # Vider la liste des voir-aussi
        self.lst_ant.clear()                                # Vider la liste des voir-aussi
        try   : self.greffon.Display("")                    # Envoyer une chaîne vide au plugin
        except: pass
        
        self.progr.setValue(0)
        self.showProgr()                                    # Afficher la barre de progression
        
        try:
            self.f_transl.terminate()                       # Ferme une eventuelle fenêtre de traduction en cours
        except:
            pass
        
        
        # --------------------------------------------------------------------------
        # --- Réinitialiser l'objet dictionnaire (l'initialisation charge le dico)  
        # --------------------------------------------------------------------------
        
        self.dic = DicObject(fichPath, self.lst_dico, self.progr, cp1=cp1, cp2=cp2)
        
        if self.dic.file is None: # Le chargement a échoué
            self.dic = None
            self['cursor'] = ""
            tkMessageBox.showwarning(parent=self,
                    title=u'Linguae',
                    message=u"Echec du chargement du dictionnaire :\n\n" + fichPath)
            return
        
        
        # -----------------------------------------------------------
        # --- Mise à jour de l'interface avec les données du dico    
        # -----------------------------------------------------------
        
        self.status1['text'] = self.dic.encoding1 + " | " + self.dic.encoding2    # Noter l'encodage du fichier dans la StatusBar
        
        self.ini.write("lastopenfile", fichPath)                # Mémoriser dans l'ini le chemin du fichier ouvert
        
        self.title("Linguae [" + self.dic.path + "]")           # Inscrire le chemin du dico dans la barre de titre
        
        if self.dic.protected1 != "" or self.dic.protected2 != "" :  
            self.status4['image'] = self.imglst.caden1          # Icône de protection du dico
            self.status4.ib.libel("Dictionnaire protégé")
        
        self.ic1 = PhotoImage(data=self.dic.img1)
        self.lab_img1['image'] = self.ic1                       # Afficher l'image de langue 1
        if self.dic.langName1 != "":                            # Infobulle de l'image de langue 1
            t = "Langue source : " + self.dic.langName1
            self.lab_img1.ib=lib.infobulle.InfoBulle(parent=self.lab_img1, text=t, temps=400)
        
        self.ic2 = PhotoImage(data=self.dic.img2)
        self.lab_img2['image'] = self.ic2                       # Afficher l'image de langue 2
        if self.dic.langName2 != "":                            # Infobulle de l'image de langue 2
            t = "Langue cible : " + self.dic.langName2
            self.lab_img2.ib=lib.infobulle.InfoBulle(parent=self.lab_img2, text=t, temps=400)
        
        if self.dic.dicUrl != "":                               # Activer le menu "url du dico"
            self.m_hlp.entryconfigure(7, state='normal')
        else:
            self.m_hlp.entryconfigure(7, state='disabled')
        
        if self.dic.contactAuthor != "":                        # Activer le menu "mail à auteur du dico"
            self.m_hlp.entryconfigure(6, state='normal')
        else:
            self.m_hlp.entryconfigure(6, state='disabled')
        
        if self.dic.verUrl != "":                               # Activer le menu "chercher nouvelle version du dico"
            self.m_hlp.entryconfigure(8, state='normal')
        else:
            self.m_hlp.entryconfigure(8, state='disabled')
        
        
        self.hideProgr()                                        # Masquer la barre de progression
        
        
        # ----------------------------------------------------------------------
        # --- Griser l'arrière-plan des widgets inutiles en fonction du format  
        # ----------------------------------------------------------------------
        
        # --- D'abord tout blanchir
        self.lst_rac.config(background='white')
        self.lst_syn.config(background='white')
        self.lst_see.config(background='white')
        self.lst_ant.config(background='white')
        
        # --- Griser
        if self.dic.type.startswith("wb") or self.dic.type.startswith("csv") or self.dic.type.startswith("ini"):
            self.lst_rac.config(background=self['background'])
            self.lst_syn.config(background=self['background'])
            self.lst_see.config(background=self['background'])
            self.lst_ant.config(background=self['background'])
        elif self.dic.type.startswith("dict"):
            self.lst_rac.config(background=self['background'])
            self.lst_syn.config(background=self['background'])
            self.lst_see.config(background=self['background'])
            self.lst_ant.config(background=self['background'])
        
        
        # ---------------------------------------------------
        # --- Comportement du point-virgule suivant format   
        # ---------------------------------------------------
        
        if self.dic.type.startswith("wb") or self.dic.type.startswith("ling"):
            self.vPv2br.set(1)
        else:
            self.vPv2br.set(0)
        
        
        # ----------------------------------------------------------------
        # --- Affichage optionnel du splash des Infos & Statut du dico    
        # ----------------------------------------------------------------
        
        if showSplash:
            self.f_info = WindowInfo()   # NB : fenêtre modale
        
        # ---
        
        self['cursor'] = ""
        
        return



    def onTheFlySearch(self, event):
        u"""
        Chercher le début du mot dans la Listbox des entrées
        à l'aide de la case de recherche
        """
        
        self.txt_search.update()
        
        motif = self.txt_search.get().lower()       # Motif tapé dans la case (en minuscule)
        
        self.lst_dico.selection_clear(0, END)       # Désélectionner tout dans la Listbox
        
        
        # Chercher l'index par l'objet dico
        
        i = self.dic.idxfind(motif=motif, dom='idx', decal=0, debut=True, ignorecase=True, ignorelig=True)
        
        # Sélectionner dans la listbox
        if not i is None:
            self.lst_dico.activate(i)
            self.lst_dico.selection_set(i)
            self.lst_dico.see(i)
            self.selDisplayData(event)
            self.txt_search['foreground']='black'
        else:
            self.lst_dico.selection_clear(0, END)   # Désélectionner tout dans la liste
            self.lst_dico.see(0)                    # Afficher le début de la liste
            self.txt_search['foreground']='red'     # Motif incorrect en rouge dans la TextBox
        
        return



    def getEncodings(self, fichpath, flagforce=False):
        u"""
        Recherche si des encodages sont mémorisés pour le dictionnaire de chemin fichpath
        si rien : affiche la boîte de dialogue de demande des encodages
        
        si 'flagforce' est vrai les encodages sont demandés même s'ils sont mémorisés
        
        Retourne un tuple (cp1, cp2, flagcancel)
        """
        
        # -------------------------------------------------------------------
        # --- Rechercher si des encodages ont été antérieurement mémorisés   
        # -------------------------------------------------------------------
        
        cp1 = ""
        cp2 = ""
        flagcancel = False
        
        
        # Nb : les encodages sont mémorisés dans le fichier "encodings"
        # de type ini simplifié, présent dans Appdata
        
        
        encfile = Inifile(os.path.join(self.appdata.path, "encodings"))
        
        k = os.path.normpath(fichpath).lower()
        
        try:    k = k.encode('utf-8')
        except: pass
        
        cps = encfile.get(k,"") # recupère les encodages du dico
        
        if cps != "":
            cps = cps.split("|")
            cp1 = cps[0]
            cp2 = cps[1]
        
        
        
        # ---------------------------------------------------------
        # --- Afficher la Fenêtre de choix des encodages           
        # ---------------------------------------------------------
        
        if flagforce or cps == "":
        
            
            # Afficher (fenêtre modale bloquante)
            f_enc = WindowEncodeOpen(cp1=cp1, cp2=cp2)
            
            # Récupérer les données de la fenêtre
            cp1 = f_enc.cp1
            cp2 = f_enc.cp2
            flagcancel = f_enc.annuler
            
            # Détruire la fenêtre
            f_enc.destroy()
        
        
        return (cp1, cp2, flagcancel)



    def openDicFromLastOpen(self):
        
        u"""
        Rouvrir le dernier dico ouvert (son chemin a été mémorisé dans l'ini)
        """
        
        if self.ini.get("OpenLastFile","1") != "1":
            return
        
        # --------------------------------------------
        # --- Récupérer le chemin du fichier du dico  
        # --------------------------------------------
        
        fichpath = self.ini.get('lastopenfile',"")
        
        if fichpath == "":
            return
        
        if not os.path.isfile(fichpath):
            return
        
        
        # -----------------------------------------
        # --- Rechercher les encodages si besoin   
        # -----------------------------------------
        
        ext = os.path.splitext(fichpath)[1].lower() # Extension du fichier du dico
        
        ret = None
        cp1 = ""
        cp2 = ""
        flagcancel = False
        
        if not ext in ('.dict', '.ifo', '.gz', '.dz', '.ling', '.xdxf'):
            ret = self.getEncodings(fichpath)
            cp1 = ret[0]
            cp2 = ret [1]
            flagcancel = ret[2]
        
        
        # ----------------------------------------
        # --- Tenter de rouvrir le dico           
        # ----------------------------------------
        
        if flagcancel:
            return
        
        try:
            self.loadDico(fichpath, cp1=cp1, cp2=cp2, showSplash=True) # NB : le splash active le menu "à propos du dico"
        except:
            self['cursor'] = ""
            tkMessageBox.showwarning(parent=self,
                    title=u"Linguae",
                    message=u"Echec de la réouverture automatique d'un dictionnaire.\n\n" + fichpath + "\n\n" + str(sys.exc_info()[0]) + " : " + str(sys.exc_info()[1]))
            return
        
        
        # --------------------------------------------
        # --- Mémoriser les encodages si nécessaire   
        # --------------------------------------------
        
        if not ret is None:
            encfile = Inifile(os.path.join(self.appdata.path, "encodings"))
            
            k = os.path.normpath(fichpath).lower()
            encfile.write(k, cp1 + "|" + cp2)
            
            encfile = None
        
        
        
        return



    def openDicFromDialog(self):
        
        u"""
        - Affiche la boîte standard de sélection de fichier (choix du dico à charger)
        - Affiche, si besoin, la fenêtre de choix des encodages
        - Appelle la procédure de chargement de dictionnaire
        """
        
        
        # -------------------------------------
        # --- Boîte de séllection de fichier   
        # -------------------------------------
        
        dirSearch = self.ini.get("DicDefaultPath",os.path.expanduser('~'))
        
        fichpath = tkFileDialog.askopenfilename(parent=self,
                    title=u"Ouvrir un dictionnaire",
                    filetypes=(
                    ('Fichiers Linguae','*.ling'),
                    ('Fichiers Stardict','*.ifo *.dict'),
                    ('Fichiers Stardict compressés','*.ifo.gz *.dict.dz *.dict.gz'),
                    ('Fichiers WB','*.wb'),
                    ('Fichiers XDXF','*.xdxf'),
                    ('Fichiers délimités','*.csv *.tab *.txt'),
                    ('Fichiers ini','*.ini'),
                    ('Fichiers tous formats supportés','*.ling *.ifo *.dict *.ifo.gz *.dict.dz *.dict.gz *.wb *.xdxf *.csv *.tab *.txt *.ini'),
                    ('Tous les fichiers','*.*')),
                    initialdir = dirSearch
                    )
        
        if not fichpath: return   # L'utilisateur a cliqué 'Annuler' dans la boîte de dialogue
        
        
        
        # -----------------------------------------
        # --- Rechercher les encodages             
        # -----------------------------------------
        
        ext = os.path.splitext(fichpath)[1].lower() # Extension du fichier du dico
        
        ret = None
        cp1 = ""
        cp2 = ""
        flagcancel = False
        
        if not ext in ('.dict', '.ifo', '.gz', '.dz', '.ling', '.xdxf'):
            ret = self.getEncodings(fichpath, flagforce=True)
            cp1 = ret[0]
            cp2 = ret [1]
            flagcancel = ret[2]
        
        
        
        # ---------------------------------
        # --- Chargement du dictionnaire   
        # ---------------------------------
        
        if flagcancel:
            return
        
        # Charge le dictionnaire dans l'interface
        self.loadDico(fichpath, cp1=cp1, cp2=cp2)
        
        
        
        # --------------------------------------------
        # --- Mémoriser les encodages si nécessaire   
        # --------------------------------------------
        
        if not ret is None:
            encfile = Inifile(os.path.join(self.appdata.path, "encodings"))
            
            k = os.path.normpath(fichpath).lower()
            encfile.write(k, cp1 + "|" + cp2)
            
            encfile = None
        
        return




    def runPlugin(self,f):
        u"""
        Charge un plugin
        f est le nom du fichier dans le répertoire des plugins
        """
        
        if f=='(none)':
            self.stopPlugin(memo=True)
            return
        
        self.stopPlugin(memo=False)                                     # Clore le plugin actif (sans mémo dans ini)
        
        try:
            exec ("import plugins." + f) in globals()                   # Importer le module du plugin
        except:
            tkMessageBox.showwarning(parent=self,
                    title=u"Echec du chargement d'un plugin",
                    message=u"Le plugin/greffon '" + unicode(f) + u"' n'a pas pu être chargé.")
            return

        # ------------------------------------
        # --- Instancier le plugin            
        # ------------------------------------
        
        self.greffon = eval("plugins." + f + ".Widget(self.fr_plugin, getlist=self.getList, getfile=self.getfile, getAppDataPath=self.getAppDataPath, getdata=self.senddata)")
        
        self.greffon.grid(row=0, column=0, padx=6, pady=3)              # Afficher le plugin
        
        self.ongl_plugin['text'] = self.greffon.GetTitle()              # Afficher le titre du greffon dans l'onglet
        
        self.ini.write("lastopenplugin", f)                             # Mémoriser dans l'ini le plugin actif
        
        return



    def stopPlugin(self, memo=True):
        u"""
        Décharge le plugin actif
        """
        
        try:
            exec("app.greffon.destroy()") in globals()
        except:
            pass
        
        self.ongl_plugin['text']="(greffon : aucun)"
        
        if memo : self.ini.write("lastopenplugin", "(none)")            # Mémoriser dans l'ini de dernier plugin ouvert
        
        return



    def selDisplayData(self, event):
        u"""
        Afficher les données de l'entrée de dictionnaire en sélection
        """
        
        # -----------------------------
        # --- Récupérer les données    
        # -----------------------------
        
        self.lst_dico.update()
        
        try:
            i = int(self.lst_dico.curselection()[0])    # Index du mot en sélection
        except:
            return
        
        if i <0 : return
        
        data = self.dic.getdata(i)                      # Récupère les données dans un pyDictionnaire
        
        mot = data['mot']
        
        # --------------------------------
        # --- Envoi du mot au plugin      
        # --------------------------------
        
        try:
            self.greffon.Display(self.lst_dico.get(i))
        except:
            pass
        
        # ----------------------------------
        # --- Affichage des données         
        # ----------------------------------
        
        self.status0['text'] = str(i)                   # Afficher l'index dans la barre de statut
        
        # --- Activer les cases d'affichages pour permettre l'écriture
        
        self.txt_notice['state'] = 'normal'
        
        # --- Effacer les contenus antérieurs
        
        self.txt_notice.delete("1.0", END)
        self.lst_rac.clear()
        self.lst_syn.clear()
        self.lst_see.clear()
        self.lst_ant.clear()
        self.status3['text']="" # ID
        
        
        # --- Afficher la répétition du mot
        
        if self.dic.type.startswith('ini'):  
            if mot.startswith("["):             # titre de section INI
                self.txt_notice.insert(END, mot + "\n", "miroir")
            else:
                sec = ""
                for s in range(i, -1, -1):      # Rechercher le titre de la section
                    if self.dic.lstbox.get(s).startswith("["):
                        sec = self.dic.lstbox.get(s)
                        break
                self.txt_notice.insert(END, sec + "  " + mot.strip() + "\n", "miroir")
        else:
            mot = mot.split("[", 1)
            self.txt_notice.insert(END, "<b>" + mot[0] + "</b>", "miroir")
            if len(mot) > 1 :
                self.txt_notice.insert(END, "[" + mot[1] + "\n", "miroir")
            else:
                self.txt_notice.insert(END, "\n", "miroir")
        
        # --- Phonétique
        t = data['phon'].strip()
        if t != "" and t[0:2] != "[(":
            self.txt_notice.insert(END, "[ " + t + " ]\n", "phon")
        
        
        # --- Traductions courtes 
        
        t = entConv(data['short']) # convertir éventuelles entités AVANT le remplacement des ';'
        
        if self.vPv2br.get():
            t = t.replace(";","\n")
        t = t.replace("\n ","\n")
        
        self.txt_notice.insert(END, t + "\n", "short")
        
        
        # --- Traductions longues et infos
        
        t = entConv(data['long'])
        
        if t != "" and t[0:2] !="[(":
            self.txt_notice.insert(END, "-"*30 + "\n", "normal")
            self.txt_notice.insert(END, t + "\n", "long")
        
        
        # --- Formater le contenu des balises de la notice
        
        tagsConv(self.txt_notice, btag="b", itag="i", utag="u", bigtag="big", smalltag="small")
        
        
        # --- Liste des Racines
        
        if data['rac'] == "":
            self.lst_rac.ib.deactivate()                # Désactiver l'infobulle si liste vide
        elif data['rac'].startswith("[("):
            self.lst_rac.add(data['rac'])               # Texte "non géré par ce format"
        else:
            chps = data['rac'].split(";")               # Chaîne des IDs des Racines
            for chp in chps:
                try:
                    idx = self.dic.IDs[chp]             # ID > Index
                    self.lst_rac.add(self.dic.lstbox.get(idx) + " "*50 + chp)  # Index > texte de l'entrée
                except KeyError:
                    self.lst_rac.add("??? : " + chp)
            self.lst_rac.ib.activate()                  # Activer l'infobulle
        

        
        # --- Liste des Synonymes
        
        if data['syn'] == "":
            self.lst_syn.ib.deactivate()                # Désactiver l'infobulle si liste vide
        elif data['syn'].startswith("[("):
            self.lst_syn.add(data['syn'])               # texte "non géré par ce format"
        else:
            chps = data['syn'].split(";")               # Chaîne des IDs des synonymes
            for chp in chps:
                try:
                    idx = self.dic.IDs[chp]             # ID > Index
                    self.lst_syn.add(self.dic.lstbox.get(idx) + " "*50 + chp)   # Index > texte de l'entrée
                except KeyError:
                    self.lst_syn.add("??? : " + chp)
            self.lst_syn.ib.activate()                  # Activer l'infobulle
        
        
        # --- Liste des voir-aussi
        
        if data['see'] == "":
            self.lst_see.ib.deactivate()                # Désactiver l'infobulle si liste vide
        elif data['see'].startswith("[("):
            self.lst_see.add(data['see'])               # texte "non géré par ce format"
        else:
            chps = data['see'].split(";")               # Chaîne des IDs des voir-aussi
            for chp in chps:
                try:
                    idx = self.dic.IDs[chp]             # ID > Index
                    self.lst_see.add(self.dic.lstbox.get(idx) + " "*50 + chp)   # Index > texte de l'entrée
                except:
                    self.lst_see.add("??? : " + chp)
            self.lst_see.ib.activate()                  # Activer l'infobulle
        
        
        # --- Liste des antonymes
        
        if data['ant'] == "":
            self.lst_ant.ib.deactivate()                # Désactiver l'infobulle si liste vide
        elif data['ant'].startswith("[("):
            self.lst_ant.add(data['ant'])               # texte "non géré par ce format"
        else:
            chps = data['ant'].split(";")               # Chaîne des IDs des antonyme
            for chp in chps:
                try:
                    idx = self.dic.IDs[chp]             # ID > Index
                    self.lst_ant.add(self.dic.lstbox.get(idx) + " "*50 + chp)   # Index > texte de l'entrée
                except:
                    self.lst_ant.add("??? : " + chp)
            self.lst_ant.ib.activate()           
        
        
        # --- désactiver les cases après leur remplissage
        self.txt_notice['state'] = 'disabled'
        
        
        # Mise à jour de la mémorisation des index (pour navigation en arrière)
        self.dic.previousIdx = self.dic.curIdx
        self.dic.curIdx = i
            
            
        # -------------------------------------------
        # --- Recherche de ressources associées      
        # -------------------------------------------
        
        
        self.bt_audio['state']='disabled'
        self.bt_img['state']='disabled'
        
        id = data['ID']
        self.dic.curID = id
        
        if id != "":
            
            # --- Case de l'ID (statusbar)
            self.status3['text'] = id                       # ID de l'entrée
            
            
            # --- Audio ?
            mp3 = os.path.join(self.dic.path + "_res","mp3",id + ".mp3")
            if os.path.isfile(mp3):
                self.bt_audio['state']='normal'
                
             
            # --- Images ?
            
            lf = glob.glob(os.path.join(self.dic.path + "_res", "gif", id + "_*.gif"))
            if len(lf) > 0:
                self.bt_img['state']='normal'
        
        
        
        return

    
    def convGifToBase64(self):
        u"""
        Conversion d'un fichier gif en texte base64,
        le texte est ensuite copié dans le presse-papiers
        """
        
        # -----------------------------------------
        # --- Message d'intro                      
        # -----------------------------------------
        
        r = tkMessageBox.askyesno(parent=self,
                    title="Convertir un gif en base64",
                    message="Cet outil permet de convertir un fichier au format Gif (format binaire) en format texte suivant le codage standard \"base64\". Ce format texte est celui-ci accepté par Linguae pour insérer des icônes de dictionnaire dans un fichier texte d'importation d'un dictionnaire.\n\nContinuer ?")
        
        if r == False:
            return
        
        # -------------------------------------
        # --- Choisir le fichier à convertir   
        # -------------------------------------
          
        fichpath = tkFileDialog.askopenfilename(
                        title="Convertir un fichier binaire Gif en chaîne base64",
                        filetypes=(
                            ('Fichiers GIF','*.gif'),
                            ('Tous les fichiers','*.*')
                            )
                        )
        
        # L'utilisateur a cliqué 'Annuler' dans la boîte de dialogue
        if not fichpath:
            return   
        
        
        # ---------------------------------
        # --- Lire la chaîne binaire       
        # ---------------------------------
        
        ff = open(fichpath, "rb")
        strBin = ff.read() 
        ff.close()
        
        # le format de fichier est reconnu par son extension
        typefich = os.path.splitext(fichpath)[1][1:].lower()
        
        
        # -------------------------
        # --- Contrôles de format  
        # -------------------------
        
        if typefich == "gif":
            
            if strBin[:4] != "GIF8":
                tkMessageBox.showwarning(parent=self,
                        title="Convertir un Gif en chaîne base64",
                        message="Ce fichier n'est pas un fichier GIF standard")
                return
        else:
            tkMessageBox.showwarning(parent=self,
                        title="Convertir un Gif en chaîne base64",
                        message="Ce fichier n'est pas un fichier GIF !")
            return
        
        
        # ----------------------------------------
        # --- Convertir en chaîne texte base64    
        # ----------------------------------------
        
        t = base64.encodestring(strBin)
        
        
        # ----------------------------------------------------
        # --- Copier le texte base64 dans le presse-papiers   
        # ----------------------------------------------------
        
        self.clipboard_clear()
        self.clipboard_append(t)
        
        tkMessageBox.showinfo(parent=self,
                    title="Conversion effcetuée",
                    message="Le texte base64 de l'image a été copié dans le presse-papiers.")
        
        return
        

    def seeRelation(self,i,idw):
        u"""
        Affiche le mot en relation (racine, synonyme, voir-aussi) cliqué
        i est l'index dans la liste concernée
        idw est l'ID du widget cliqué
        """
        
        laListe = None # référence à la liste cliquée
        
        if idw == id(self.lst_rac):
            laListe = self.lst_rac
        elif idw == id(self.lst_syn):
            laListe = self.lst_syn
        elif idw == id(self.lst_see):
            laListe = self.lst_see
        elif idw == id(self.lst_ant):
            laListe = self.lst_ant
        
        txt = laListe.getitem(i)        # Libellé de la ligne de la liste de relation
        
        if txt.startswith("???"):       # La ligne pointe sur un ID inexistant
            tkMessageBox.showwarning(parent=self,
                    title=u"Voir la relation",
                    message=u"Cette relation pointe sur un identifiant qui est absent du dictionnaire.")
            return
        elif txt.startswith("[("):      # Non géré par ce format
            return
            
            
        # Extraire l'ID de la racine dans la liste concernée, en déduire l'index
        idx = self.dic.IDs[ laListe.getitem(i).split()[-1] ]
        
        # Afficher le mot du dictionnaire
        self.dic.lstbox.activate(idx)
        self.dic.lstbox.selection_set(idx)
        self.dic.lstbox.see(idx)
        self.selDisplayData(None)
        
        # Aciver le bouton de retour au mot précédent
        self.bt_back['state']='active'
        
        return


    def getfile(self):
        u"""
        Retourne l'objet fichier du dictionnaire
        
        NB : la référence à cette méthode est passée au greffon lors de son instanciation
        pour qu'il puisse ensuite récupèrer l'objet fichier
        """
        
        if not self.dic is None:
            return self.dic.file
        else:
            return None


    def getList(self):
        u"""
        Retourne le contenu de la liste des entrées
        
        NB : la référence à cette méthode est passée au greffon lors de son instanciation
        pour qu'il puisse ensuite récupèrer la liste
        """
        
        return self.lst_dico.get(0, END)


    def getAppDataPath(self):
        u"""
        Retourne le chemin du répertoire des données d'application
        
        NB : la référence à cette méthode est passée au greffon lors de son instanciation
        pour qu'il puisse ensuite récupèrer le chemin
        """
        
        
        return self.appdata.path
    

    def queryUnload(self):
        u"""
        Suite d'actions à effectuer avant la fermeture définitive de la fenêtre principale
        """
        
        # -------------------------------------------------------
        # --- Boîte de message de confirmation de fermeture      
        # -------------------------------------------------------
        
        # Echappatoire provisoirement annulé car bogue de tkinter2.6/Linux
        #r = tkMessageBox.askyesno(
        #        title="Quitter Linguae",
        #        message=u"Merci de signaler tout problème rencontré à :\n\n\tlinguae@stalikez.info\n\n\nQuitter ?")
        #if r == False: return
        tkMessageBox.showinfo(
                title="Quitter Linguae",
                message=u"Merci de signaler tout problème rencontré à :\n\n\tlinguae@stalikez.info")
        
        
        # ------------------------------------------
        # --- Détruire l'éventuelle fenêtre d'aide  
        # ------------------------------------------
        
        self.hlp.terminate() 
        
        
        # -------------------------------------------------------
        # --- Mémoriser la géométrie de l'interface dans l'ini   
        # -------------------------------------------------------
        
        # Taille et position de la fenêtre 
        if app.wm_state() == "zoomed":
            g = self.geometry().split("+")[0] + "+0+0"
        else:
            g = self.geometry()
        self.ini.write('Geometry', g)
        
        # Position de la coulisse
        self.ini.write('Xsash', str(self.panM.sash_coord(0)[0]))
        
        
        # ------------------------------------------------------------------------------
        # --- Nettoyer le fichier des encodings de ses entrées obsolètes (le réécrire)  
        # ------------------------------------------------------------------------------
        
        #    ..... (à faire) .....
        
        
        
        # -----------------------------------------------------------------------
        # --- Afficher un message "Ok" dans la console et fermer l'application   
        # -----------------------------------------------------------------------
        
        print "\n Linguae :\tAu revoir..."
        
        
        self.destroy()

    
    


    def senddata(self, i):
        u"""
        Retourne le contenu de l'entrée d'index 'i' du dico courant (s'il existe)
        
        NB : la référence à cette méthode est passée au greffon lors de son instanciation
        pour qu'il puisse ensuite récupèrer les données d'un mot quelconque
        """
        if app.dic is None:
            return None
        else:
            return self.dic.getdata(i)



    def showSort(self):
        u""" 
        Afficher la fenêtre de tri du dictionnaire
        """
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Trier le dictionnaire",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        WindowSort()      # Fenêtre des paramètres de tri (fenêtre modale)
        
        return


    def showFind(self):
        u"""
        Afficher (instancier ou activer) la fenêtre de recherche
        """
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Chercher dans le dictionnaire",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        
        try:
            self.f_find.focus_set()           # Tenter d'activer (fenêtre non modale)
        except:
            self.f_find = WindowFind()        # Echec de l'activation > instancier la fenêtre
        
        return


    def showImport(self):
        u"""
        Affiche (instancie) la fenêtre d'importation
        """
        
        self.f_import = WindowImport()      # NB : fenêtre modale
        
        return


    def showAlpha(self):
        u"""
        Affiche (instancie) la fenêtre du réglage de la transparence
        """
        
        self.f_alpha = WindowAlpha()   # NB : fenêtre modale
        
        return
    

    def showOptions(self):
        u"""
        Affiche (instancie) la fenêtre des réglages et options
        """
        
        self.f_opt = WindowOptions()   # NB : fenêtre modale
        
        return


    def showReverse(self):
        u"""
        Afficher la fenêtre de confection du dico inverse
        """
        
        # Avertisement de copyright
        r = tkMessageBox.askyesno(parent=self,
                title=u"Construire un dictionnaire inverse",
                message=u"ATTENTION, certains dictionnaires peuvent être sous licence et sujets à des droits d'auteur.\nLeur diffusion sous une autre forme que le fichier original est peut-être interdite.\n\nLa construction d'un dictionnaire inverse ne peut, dans ce cas, être effectuée que dans un cadre strictement privé\net sans aucune diffusion secondaire.\n\nVeuillez, en cas de doute, vous reporter aux licences respectives des dictionnaires qui ne vous sont pas personnels.\n\nContinuer ?")
        if r == False : return
        
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Construire un dictionnaire inverse",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        if self.dic.type == "":
            return
        elif self.dic.type.startswith("ini"):
            app.hideProgr()
            tkMessageBox.showinfo(parent=self,
                    title=u"Construire un dictionnaire inverse",
                    message=u"Du fait de la structure hiérarchique du format INI\nun dictionnaire inverse ne peut être construit")
            return
        
        
        self.f_reverse = WindowReverse()   # NB : fenêtre modale
        
        
        return


    def showProperties(self):
        u"""
        Afficher la fenêtre des propriétés du dictionnaire
        """
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Voir les propriétés",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        if self.dic.type == "": return
        
        self.f_propr = WindowProperties()   # NB : fenêtre modale
        
        return



    def showExport(self):
        u"""
        Afficher la fenêtre d'exportation
        """
        
        # Avertisement de copyright
        r = tkMessageBox.askyesno(parent=self,
                title=u"Exporter le dictionnaire courant",
                message=u"ATTENTION, certains dictionnaires peuvent être sous licence et sujets à des droits d'auteur.\nLeur diffusion sous une autre forme que le fichier original est peut-être interdite.\n\nL'exportation vers un autre format ne peut, dans ce cas, être effectuée que dans un cadre strictement privé\net sans aucune diffusion secondaire.\n\nVeuillez, en cas de doute, vous reporter aux licences respectives des dictionnaires qui ne vous sont pas personnels.\n\nContinuer ?")
        if r == False : return
        
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Exporter le dictionnaire",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        if self.dic.type == "": 
            tkMessageBox.showinfo(parent=self,
                    title=u"Exporter le dictionnaire",
                    message=u"Type de dictionnaire non spécifié. Exportation impossible !")
            return
        
        self.f_export = WindowExport()   # NB : fenêtre modale
        
        return


    def showEdit(self):
        u"""
        Afficher la fenêtre d'édition
        """
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                title=u"Editer une entrée de dictionnaire",
                message=u"Chargez d'abord un dictionnaire !")
            return
        
        # Si la fenêtre est restée ouverte, on la ferme
        try:
            self.f_edit.destroy()
        except:
            pass
        
        # On crée la fenêtre en lui passant l'index du mot courant
        try:
            i = int(app.lst_dico.curselection()[0]) # index du mot en sélection
        except:
            tkMessageBox.showinfo(parent=self,
                    title=u"Editer une entrée de dictionnaire",
                    message=u"Sélectionnez d'abord une entrée !")
            return
        
        app.f_edit = WindowEdit(i)
        
        return



    def showImage(self):
        u"""
        Afficher la fenêtre de visualisation des illustrations
        avec par défaut la première image
        """
        
        self.f_img = WindowImage(idximg=0)   # NB : fenêtre modale
        
        
        return


    def showEditAdd(self):
        u"""
        Afficher la fenêtre d'édition pour ajouter une entrée de dictionnaire
        """
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Ajouter une entrée de dictionnaire",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        
        # Si la fenêtre est restée ouverte, on la ferme
        try:
            self.f_edit.destroy()
        except:
            pass
        
        
        # On crée la fenêtre en lui passant le pseudo index '-1'(= ajouter une entrée)
        
        app.lst_dico.selection_clear(0,END)     # Désélectionner tout dans la liste
        
        app.f_edit = WindowEdit(-1)
        
        return


    def showSetIcon(self):
        u"""
        Affiche la fenêtre de modification des icônes du dictionnaire
        """
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Mofifier les icônes du dictionnaire",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        if not self.dic.type.startswith("ling"):
            tkMessageBox.showinfo(parent=self,
                    title=u"Mofifier les icônes du dictionnaire",
                    message=u"L'ajout ou la modification des icônes n'est possible que pour les dictionnaires au format '*.ling'.\n\nVous devez d'abord convertir le dictionnaire à ce format avant d'intégrer des icônes à ce dictionnaire.")
            return
        
        self.f_icon = WindowSetIcon()   # NB : fenêtre modale
        
        
        return


    def showHerit(self):
        u"""
        Afficher la fenêtre de construction d'un dictionnaire par héritage
        """
        
        if self.ini.get('DicDefaultFolder','') == "":
            tkMessageBox.showinfo(parent=self,
                    title=u"Construire un dictionnaire par héritage",
                    message=u"Cette fonction n'est disponible qu'après avoir défini un dossier des dictionnaires (voir : Options)")
            return
        
        
        self.f_herit = WindowHerit()   # NB : fenêtre modale
        
        return


    def showTranslate(self):
        u"""
        Afficher/activer la fenêtre d'aide à la traduction
        """
        
        if self.dic is None:
            tkMessageBox.showinfo(parent=self,
                    title=u"Aide à la traduction",
                    message=u"Chargez d'abord un dictionnaire !")
            return
        
        try:
            self.f_transl.focus_set()             # Tenter d'activer (fenêtre non modale)
        except:
            self.f_transl = WindowTranslate()     # Echec de l'activation > instancier la fenêtre
        
        
        return


    def showUnicode(self):
        u"""
        Afficher/activer la fenêtre des caractères Unicode
        """
        
        try:
            self.f_unic.focus_set()                 # Tenter d'activer (fenêtre non modale)
        except:
            self.f_unic = WindowUnicode(ifont=1)    # Echec de l'activation > instancier la fenêtre avec la police langue1
        
        return


    def showAbout(self):
        u"""
        Afficher la fenêtre "A propos de Linguae"
        """
        
        tkMessageBox.showinfo(parent=self,
                title=u"A propos de Linguae...",
                message=u"LINGUAE " + app.version +"\n\nGestionnaire de dictionnaires multiformat\n\n\
Site Web : http://linguae.stalikez.info\n\n\
Logiciel sous licence CeCILL (compatible GNU-GPL) :\n\
   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt\n\
   http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt\n\n\
Billig - 2008-2009")
        
        return


    def showAboutDic(self):
        u"""
        Afficher la fenêtre "A propos de ce dictionnaire"
        """
        
        self.f_info = WindowInfo()   # NB : fenêtre modale
        
        return


    def showHelp(self, contextID=""):
        u"""
        Afficher la fenêtre d'aide contextuelle
        """
        
        self.hlp.showHelp(contextID)
        
        return




# /////////////////////////////////////////////////////////////////
# /// MAIN ////////////////////////////////////////////////////////
# /////////////////////////////////////////////////////////////////


# Crée et affiche la fenêtre principale de l'application
app = WindowMAIN("0.0.30 Alpha")


# ---------------------------------------------
# --- Importer la bibliothèque audio Snack     
# ---------------------------------------------

try:
    import tkSnack
    # IMPORTANT : l'initialisation de Snack DOIT obligatoirement
    # survenir APRES celle de Tk() (=app) car Snack est une extension de Tk/Tcl
    tkSnack.initializeSnack(app)  
    app.snackisactive = True
    
except: # échec
    
    msg = u"Les fonctions audio de cette application nécessitent que la bibliothèque Snack (licence GNU-GPL) soit installée.\n"
    if sys.platform.lower()[:3] == 'win':        
        msg += u"\nSnack pour Windows est joint au paquetage de Linguae et son installation manuelle est très simple.\n"
    msg += u"\nToutes les fonctions audio de Linguae seront désactivées tant que cette bibliothèque ne sera pas intallée.\n\nVoulez-vous afficher l'aide pour plus de détails ?"

    r = tkMessageBox.askyesno(parent=app,
            title = u"Bibliothèque audio non accessible",
            message = msg)
    app.snackisactive = False
    if r: app.showHelp("dep")

# -----------
# -----------
# -----------


# Rouvrir le dernier dico ouvert
app.openDicFromLastOpen()


# Nécessaire car l'utilisation de la classe "infobulle" fait,
# pour d'obscures raisons, démarrer l'application sans focus...
app.focus_force()


# Procédure qui sera appelée à la tentative de fermeture de la fenêtre principale
app.protocol('WM_DELETE_WINDOW', app.queryUnload)


app.mainloop()
